
CREATE TYPE range_type AS (lower_bound bigint, upper_bound bigint);

CREATE OR REPLACE FUNCTION get_free_ids(min_id bigint, max_id bigint)
  RETURNS SETOF range_type AS
$BODY$
DECLARE
   ids_count bigint;	
   tbl record;
   cnt record;
   rec record;
   range range_type;
   tempstr varchar;
   tempint integer;
   tables varchar[] = '{}';
   prediction_q text = '';
   q text = '';
   id1 bigint;
   id2 bigint; 
BEGIN
   ids_count = max_id - min_id + 1;
   FOR tbl IN SELECT table_name FROM information_schema.tables WHERE table_schema = 'public' 
	and table_name not like '%!_!_%' escape '!' LOOP
         tables = array_append(tables, CAST(tbl.table_name as varchar));
   END LOOP;

   IF array_lower(tables,1) is null THEN
      range.lower_bound = min_id;
      range.upper_bound = max_id;
      RETURN NEXT range;
      RETURN;
   END IF;

   prediction_q = 'SELECT (';
   FOR i IN array_lower(tables,1)..array_upper(tables,1) LOOP
	prediction_q = prediction_q || ' (SELECT count(id) from ' || tables[i] || ' WHERE id >= ' || min_id || ' and id <= ' || max_id || ') ';
	IF i <> array_upper(tables,1) THEN
          prediction_q = prediction_q || '+';
	END IF;
   END LOOP;
   prediction_q = prediction_q || ') as cnt';

   EXECUTE prediction_q INTO cnt;
   IF cnt.cnt = ids_count THEN  
      RETURN; -- no free ids available
   END IF;

   -- if cnt = 0 then all ids in this range is available in all tables
   IF cnt.cnt = 0 THEN   
      range.lower_bound = min_id;
      range.upper_bound = max_id;
      RETURN NEXT range;
      RETURN;	  
   END IF;

   FOR i IN array_lower(tables,1)..array_upper(tables,1) LOOP
	q = q || ' (SELECT id from ' || tables[i] || ' WHERE id >= ' || min_id || ' and id <= ' || max_id || ') ';
	IF i <> array_upper(tables,1) THEN
          q = q || 'UNION';
	END IF;
   END LOOP;

   q = q || ' ORDER BY id ASC';
   
    id1 = 0; 
    FOR rec IN EXECUTE q LOOP
      IF id1 = 0 THEN
	id1 = rec.id;
	IF min_id < id1 THEN
	    range.lower_bound = min_id;
	    range.upper_bound = id1 - 1; 	
            RETURN NEXT range;
	END IF;
      ELSE 
	id2 = id1;
	id1 = rec.id; 

	IF (id2 + 1 <= id1 - 1) THEN
         range.lower_bound = id2 + 1;
         range.upper_bound = id1 - 1; 	
         RETURN NEXT range;
        END IF; 
      END IF;
    END LOOP;

    IF (id1 < max_id) THEN
        range.lower_bound = id1 + 1;
	range.upper_bound = max_id; 	
        RETURN NEXT range;
    END IF;

END;
$BODY$
  LANGUAGE 'plpgsql' VOLATILE;

