/*
** Module   : AmbiguousSymbolException.java
** Abstract : Exception caused by ambiguous function and variable lookups
**
** Copyright (c) 2005-2017, Golden Code Development Corporation.
**
** -#- -I- --Date-- -T- --JPRM-- ----------------Description-----------------
** 001 ECF 20050211 ADD   @19818 Created initial version. Used for ambiguous
**                               function lookups.
** 002 ECF 20050222 CHG   @20072 Changed superclass to SymbolException, since
**                               FunctionException is now obsolete.
** 003 ECF 20050430 CHG   @20951 Changed name from AmbiguousFunctionException
**                               to AmbiguousSymbolException. Reflects more
**                               generalized use.
*/
/*
** This program is free software: you can redistribute it and/or modify
** it under the terms of the GNU Affero General Public License as
** published by the Free Software Foundation, either version 3 of the
** License, or (at your option) any later version.
**
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU Affero General Public License for more details.
**
** You may find a copy of the GNU Affero GPL version 3 at the following
** location: https://www.gnu.org/licenses/agpl-3.0.en.html
** 
** Additional terms under GNU Affero GPL version 3 section 7:
** 
**   Under Section 7 of the GNU Affero GPL version 3, the following additional
**   terms apply to the works covered under the License.  These additional terms
**   are non-permissive additional terms allowed under Section 7 of the GNU
**   Affero GPL version 3 and may not be removed by you.
** 
**   0. Attribution Requirement.
** 
**     You must preserve all legal notices or author attributions in the covered
**     work or Appropriate Legal Notices displayed by works containing the covered
**     work.  You may not remove from the covered work any author or developer
**     credit already included within the covered work.
** 
**   1. No License To Use Trademarks.
** 
**     This license does not grant any license or rights to use the trademarks
**     Golden Code, FWD, any Golden Code or FWD logo, or any other trademarks
**     of Golden Code Development Corporation. You are not authorized to use the
**     name Golden Code, FWD, or the names of any author or contributor, for
**     publicity purposes without written authorization.
** 
**   2. No Misrepresentation of Affiliation.
** 
**     You may not represent yourself as Golden Code Development Corporation or FWD.
** 
**     You may not represent yourself for publicity purposes as associated with
**     Golden Code Development Corporation, FWD, or any author or contributor to
**     the covered work, without written authorization.
** 
**   3. No Misrepresentation of Source or Origin.
** 
**     You may not represent the covered work as solely your work.  All modified
**     versions of the covered work must be marked in a reasonable way to make it
**     clear that the modified work is not originating from Golden Code Development
**     Corporation or FWD.  All modified versions must contain the notices of
**     attribution required in this license.
*/

package com.goldencode.expr;

/**
 * Exception thrown when a lookup of a function or variable is ambiguous.
 * <p>
 * For a function lookup, this will only happen if more than one function
 * can possibly match a set of runtime arguments, and one or more of those
 * arguments is <code>null</code>, such that a type match for that argument
 * is ambiguous between alternative functions. This can occur between
 * overloaded functions in the same namespace, or between unqualified
 * functions when a cross-namespace lookup occurs.
 * <p>
 * For a variable lookup, this will only happen if the variable symbol is
 * unqualified in a user expression, and a cross-namespace lookup turns up
 * more than one match for that variable name.
 * <p>
 * This is an unchecked exception for performance reasons, so that the caller
 * is not required to set up an expensive try-catch block in all cases. This
 * is appropriate because this exception can easily be avoided by registering
 * only non-overloaded functions, or implementing client code such that an
 * ambiguous match cannot occur.
 */
public class AmbiguousSymbolException
extends SymbolException
{
   /**
    * Constructor which accepts an error message.
    *
    * @param   message
    *          Error message.
    */
   public AmbiguousSymbolException(String message)
   {
      super(message);
   }
}