/*
** Module   : ExpressionFlags.java
** Abstract : Flags which affect behavior of compiled expressions
**
** Copyright (c) 2004-2017, Golden Code Development Corporation.
**
** -#- -I- --Date-- -T- --JPRM-- ----------------Description-----------------
** 001 ECF 20071104 ADD  @39256  Created initial version. Flags which affect
**                               compile and runtime behavior of compiled
**                               expressions.
*/
/*
** This program is free software: you can redistribute it and/or modify
** it under the terms of the GNU Affero General Public License as
** published by the Free Software Foundation, either version 3 of the
** License, or (at your option) any later version.
**
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU Affero General Public License for more details.
**
** You may find a copy of the GNU Affero GPL version 3 at the following
** location: https://www.gnu.org/licenses/agpl-3.0.en.html
** 
** Additional terms under GNU Affero GPL version 3 section 7:
** 
**   Under Section 7 of the GNU Affero GPL version 3, the following additional
**   terms apply to the works covered under the License.  These additional terms
**   are non-permissive additional terms allowed under Section 7 of the GNU
**   Affero GPL version 3 and may not be removed by you.
** 
**   0. Attribution Requirement.
** 
**     You must preserve all legal notices or author attributions in the covered
**     work or Appropriate Legal Notices displayed by works containing the covered
**     work.  You may not remove from the covered work any author or developer
**     credit already included within the covered work.
** 
**   1. No License To Use Trademarks.
** 
**     This license does not grant any license or rights to use the trademarks
**     Golden Code, FWD, any Golden Code or FWD logo, or any other trademarks
**     of Golden Code Development Corporation. You are not authorized to use the
**     name Golden Code, FWD, or the names of any author or contributor, for
**     publicity purposes without written authorization.
** 
**   2. No Misrepresentation of Affiliation.
** 
**     You may not represent yourself as Golden Code Development Corporation or FWD.
** 
**     You may not represent yourself for publicity purposes as associated with
**     Golden Code Development Corporation, FWD, or any author or contributor to
**     the covered work, without written authorization.
** 
**   3. No Misrepresentation of Source or Origin.
** 
**     You may not represent the covered work as solely your work.  All modified
**     versions of the covered work must be marked in a reasonable way to make it
**     clear that the modified work is not originating from Golden Code Development
**     Corporation or FWD.  All modified versions must contain the notices of
**     attribution required in this license.
*/

package com.goldencode.expr;

/**
 * Expression flags that affect the compilation and runtime behavior of a
 * compiled expression.  These are used both by the {@link Compiler} and by
 * {@link Expression}.
 */
public interface ExpressionFlags
{
   /** No special runtime behavior; the default */
   public static final int RT_NONE = 0;
   
   /** Check for precision loss on narrowing conversion of integral values */ 
   public static final int RT_INT_NARROWING = 0x01;
   
   /** Check for precision loss on widening conversion to floating point */
   public static final int RT_FP_WIDENING = 0x02;
   
   /** Perform floating point rounding */
   public static final int RT_FP_ROUNDING = 0x04;
   
   /** Convert ASSIGN (<code>=</code>) to EQUALS (<code>==</code>) */
   public static final int RT_NO_ASSIGN = 0x08;
   
   /** Create a new instance of a compiled expression for each Expression */
   public static final int RT_CLONE = 0x10;
   
   /** Enhanced error reporting (slower) */
   public static final int RT_ERROR = 0x40;
   
   /** Debug trace reporting (verbose superset of RT_ERROR) */
   public static final int RT_DEBUG = 0x80;
}