/*
** Module   : Scope.java
** Abstract : interface used by symbol resolver to manage expression
**            compilation
**
** Copyright (c) 2005-2017, Golden Code Development Corporation.
**
** -#- -I- --Date-- -T- --JPRM-- ----------------Description-----------------
** 001 ECF 20050526 ADD   @21307 Created initial version. Allows symbol
**                               resolver to manage nested scopes during
**                               expression parsing/compilation.
*/
/*
** This program is free software: you can redistribute it and/or modify
** it under the terms of the GNU Affero General Public License as
** published by the Free Software Foundation, either version 3 of the
** License, or (at your option) any later version.
**
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU Affero General Public License for more details.
**
** You may find a copy of the GNU Affero GPL version 3 at the following
** location: https://www.gnu.org/licenses/agpl-3.0.en.html
** 
** Additional terms under GNU Affero GPL version 3 section 7:
** 
**   Under Section 7 of the GNU Affero GPL version 3, the following additional
**   terms apply to the works covered under the License.  These additional terms
**   are non-permissive additional terms allowed under Section 7 of the GNU
**   Affero GPL version 3 and may not be removed by you.
** 
**   0. Attribution Requirement.
** 
**     You must preserve all legal notices or author attributions in the covered
**     work or Appropriate Legal Notices displayed by works containing the covered
**     work.  You may not remove from the covered work any author or developer
**     credit already included within the covered work.
** 
**   1. No License To Use Trademarks.
** 
**     This license does not grant any license or rights to use the trademarks
**     Golden Code, FWD, any Golden Code or FWD logo, or any other trademarks
**     of Golden Code Development Corporation. You are not authorized to use the
**     name Golden Code, FWD, or the names of any author or contributor, for
**     publicity purposes without written authorization.
** 
**   2. No Misrepresentation of Affiliation.
** 
**     You may not represent yourself as Golden Code Development Corporation or FWD.
** 
**     You may not represent yourself for publicity purposes as associated with
**     Golden Code Development Corporation, FWD, or any author or contributor to
**     the covered work, without written authorization.
** 
**   3. No Misrepresentation of Source or Origin.
** 
**     You may not represent the covered work as solely your work.  All modified
**     versions of the covered work must be marked in a reasonable way to make it
**     clear that the modified work is not originating from Golden Code Development
**     Corporation or FWD.  All modified versions must contain the notices of
**     attribution required in this license.
*/

package com.goldencode.expr;

/**
 * Objects of this type are used by the {@link SymbolResolver} to partition
 * library objects and user variables registered by clients of the expression
 * engine. Whenever a worker library or user variable is registered, a
 * <code>Scope</code> instance must be provided by the registering client.
 * This instance is associated with the library or variable for the lifetime
 * of the resolver.
 * <p>
 * When and expression is parsed and compiled, the resolver's current scope
 * is requested via {@link SymbolResolver#getCurrentScope}. The instance of
 * <code>Scope</code> returned by that call is used as a key to resolve
 * library and variable references as they are encountered in the expression.
 * Only references contained within the current scope or within an enclosing
 * scope (or within its enclosing scope, and so on...) are correctly
 * resolved.
 * <p>
 * Scope is also used when resetting the values of variables in a variable
 * pool.
 *
 * @see  SymbolResolver#resetVariables
 */
public interface Scope
{
   /**
    * This method must return the object which represents the scope context
    * which encloses this scope, if this scope is nested within a broader
    * scope.
    *
    * @return  Enclosing scope object, or <code>null</code> if this scope
    *          represents the broadest scope available.
    */
   public Scope getEnclosingScope();
}