/*
** Module   : Acl.java
** Abstract : container for Access Control List
**
** Copyright (c) 2017, Golden Code Development Corporation.
** ALL RIGHTS RESERVED. Use is subject to license terms.
**
**           Golden Code Development Corporation
**                      CONFIDENTIAL
**
** -#- -I- --Date-- ----------------Description----------------------
** 001 SBI 20170225   Moved out of AclDef.
*/
/*
** This program is free software: you can redistribute it and/or modify
** it under the terms of the GNU Affero General Public License as
** published by the Free Software Foundation, either version 3 of the
** License, or (at your option) any later version.
**
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU Affero General Public License for more details.
**
** You may find a copy of the GNU Affero GPL version 3 at the following
** location: https://www.gnu.org/licenses/agpl-3.0.en.html
**
** Additional terms under GNU Affero GPL version 3 section 7:
**
**   Under Section 7 of the GNU Affero GPL version 3, the following additional
**   terms apply to the works covered under the License.  These additional terms
**   are non-permissive additional terms allowed under Section 7 of the GNU
**   Affero GPL version 3 and may not be removed by you.
**
**   0. Attribution Requirement.
**
**     You must preserve all legal notices or author attributions in the covered
**     work or Appropriate Legal Notices displayed by works containing the covered
**     work.  You may not remove from the covered work any author or developer
**     credit already included within the covered work.
**
**   1. No License To Use Trademarks.
**
**     This license does not grant any license or rights to use the trademarks
**     Golden Code, FWD, any Golden Code or FWD logo, or any other trademarks
**     of Golden Code Development Corporation. You are not authorized to use the
**     name Golden Code, FWD, or the names of any author or contributor, for
**     publicity purposes without written authorization.
**
**   2. No Misrepresentation of Affiliation.
**
**     You may not represent yourself as Golden Code Development Corporation or FWD.
**
**     You may not represent yourself for publicity purposes as associated with
**     Golden Code Development Corporation, FWD, or any author or contributor to
**     the covered work, without written authorization.
**
**   3. No Misrepresentation of Source or Origin.
**
**     You may not represent the covered work as solely your work.  All modified
**     versions of the covered work must be marked in a reasonable way to make it
**     clear that the modified work is not originating from Golden Code Development
**     Corporation or FWD.  All modified versions must contain the notices of
**     attribution required in this license.
*/


package com.goldencode.p2j.admin;

import java.io.Serializable;

import com.goldencode.p2j.security.Rights;

/**
 * The Access Control List definition links together the resource instance
 * with a list of account names that all get the equal permissions for
 * accessing this resource instance, which are specified in the rights
 * object.
 * <p>
 * The error flags are filled with the state bits when the ACL is prepared
 * on the server. 
 */
public class Acl
implements Serializable
{
   /** Server instance name (empty for shared) or null */
   public String origin;
   
   /** ACL identifier, which is unique within the origin */
   public int oid;
   
   /** Resource instance name */
   public String name;
   
   /** Flag telling exact match (true) vs regexp match (false) */
   public boolean exact;
   
   /** Array of subject IDs */
   public int[] subjects;
   
   /** Instance of the rights */
   public Rights rights;
   
   /** error flag - oid */
   public boolean errOid;
   
   /** error flag - name */
   public boolean errName;
   
   /** error flag - exact */
   public boolean errExact;
   
   /** error flag - subjects */
   public boolean errSubjs;
   
   /** error flag - rights */
   public boolean errRights;
}