/*
** Module   : ApplicationPresenter.java
** Abstract : MVP presenter.
**
** Copyright (c) 2017, Golden Code Development Corporation.
**
** -#- -I- --Date-- --------------------------------Description-----------------------------------
** 001 HC  20170612 Initial version.
*/
/*
** This program is free software: you can redistribute it and/or modify
** it under the terms of the GNU Affero General Public License as
** published by the Free Software Foundation, either version 3 of the
** License, or (at your option) any later version.
**
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU Affero General Public License for more details.
**
** You may find a copy of the GNU Affero GPL version 3 at the following
** location: https://www.gnu.org/licenses/agpl-3.0.en.html
** 
** Additional terms under GNU Affero GPL version 3 section 7:
** 
**   Under Section 7 of the GNU Affero GPL version 3, the following additional
**   terms apply to the works covered under the License.  These additional terms
**   are non-permissive additional terms allowed under Section 7 of the GNU
**   Affero GPL version 3 and may not be removed by you.
** 
**   0. Attribution Requirement.
** 
**     You must preserve all legal notices or author attributions in the covered
**     work or Appropriate Legal Notices displayed by works containing the covered
**     work.  You may not remove from the covered work any author or developer
**     credit already included within the covered work.
** 
**   1. No License To Use Trademarks.
** 
**     This license does not grant any license or rights to use the trademarks
**     Golden Code, FWD, any Golden Code or FWD logo, or any other trademarks
**     of Golden Code Development Corporation. You are not authorized to use the
**     name Golden Code, FWD, or the names of any author or contributor, for
**     publicity purposes without written authorization.
** 
**   2. No Misrepresentation of Affiliation.
** 
**     You may not represent yourself as Golden Code Development Corporation or FWD.
** 
**     You may not represent yourself for publicity purposes as associated with
**     Golden Code Development Corporation, FWD, or any author or contributor to
**     the covered work, without written authorization.
** 
**   3. No Misrepresentation of Source or Origin.
** 
**     You may not represent the covered work as solely your work.  All modified
**     versions of the covered work must be marked in a reasonable way to make it
**     clear that the modified work is not originating from Golden Code Development
**     Corporation or FWD.  All modified versions must contain the notices of
**     attribution required in this license.
*/
package com.goldencode.p2j.admin.client.application;

import javax.inject.Inject;

import com.goldencode.p2j.admin.client.widget.dialog.*;
import com.google.web.bindery.event.shared.EventBus;
import com.gwtplatform.mvp.client.Presenter;
import com.gwtplatform.mvp.client.View;
import com.gwtplatform.mvp.client.annotations.ProxyStandard;
import com.gwtplatform.mvp.client.presenter.slots.*;
import com.gwtplatform.mvp.client.proxy.Proxy;
import com.goldencode.p2j.admin.client.application.ApplicationPresenter.MyView;
import com.goldencode.p2j.admin.client.application.ApplicationPresenter.MyProxy;

/**
 * MVP presenter.
 */
public class ApplicationPresenter
extends Presenter<MyView, MyProxy>
implements ModalDialogs
{
   /**
    * View.
    */
   interface MyView extends View
   {
   }

   /**
    * Proxy.
    */
   @ProxyStandard
   interface MyProxy extends Proxy<ApplicationPresenter> {
   }

   /** Presenter's slot */
   public static final NestedSlot SLOT_MAIN = new NestedSlot();

   /** Slot for message dialogs */
   public static final PermanentSlot SLOT_MESSAGE_DLG = new PermanentSlot();

   /** Message dialog presenter */
   private MessageDialogPresenter messagePresenter;

   /**
    * Ctor.
    *
    * @param    eventBus
    *           Event bus.
    * @param    view
    *           View.
    * @param    proxy
    *           Proxy.
    * @param    messagePresenter
    *           Message dialog presenter.
    */
   @Inject
   ApplicationPresenter(EventBus eventBus,
                        MyView view,
                        MyProxy proxy,
                        MessageDialogPresenter messagePresenter)
   {
      super(eventBus, view, proxy, RevealType.Root);
      this.messagePresenter = messagePresenter;
   }

   /**
    * Lifecycle method called when binding the object.
    */
   @Override
   protected void onBind()
   {
      super.onBind();
      setInSlot(SLOT_MESSAGE_DLG, messagePresenter);
   }

   /**
    * Message dialog presenter getter.
    *
    * @return  an instance of {@link MessageDialogPresenter}.
    */
   @Override
   public MessageDialogPresenter getMessageDialogPresenter()
   {
      return messagePresenter;
   }
}