/*
** Module   : AcquiredLocksUIHandler.java
** Abstract : AcquiredLocksUIHandler defines business methods for views to manage acquired locks.
**
** Copyright (c) 2017, Golden Code Development Corporation.
**
** -#- -I- --Date-- ----------------Description----------------------
** 001 SBI 20170601 Created initial version.
*/
/*
** This program is free software: you can redistribute it and/or modify
** it under the terms of the GNU Affero General Public License as
** published by the Free Software Foundation, either version 3 of the
** License, or (at your option) any later version.
**
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU Affero General Public License for more details.
**
** You may find a copy of the GNU Affero GPL version 3 at the following
** location: https://www.gnu.org/licenses/agpl-3.0.en.html
** 
** Additional terms under GNU Affero GPL version 3 section 7:
** 
**   Under Section 7 of the GNU Affero GPL version 3, the following additional
**   terms apply to the works covered under the License.  These additional terms
**   are non-permissive additional terms allowed under Section 7 of the GNU
**   Affero GPL version 3 and may not be removed by you.
** 
**   0. Attribution Requirement.
** 
**     You must preserve all legal notices or author attributions in the covered
**     work or Appropriate Legal Notices displayed by works containing the covered
**     work.  You may not remove from the covered work any author or developer
**     credit already included within the covered work.
** 
**   1. No License To Use Trademarks.
** 
**     This license does not grant any license or rights to use the trademarks
**     Golden Code, FWD, any Golden Code or FWD logo, or any other trademarks
**     of Golden Code Development Corporation. You are not authorized to use the
**     name Golden Code, FWD, or the names of any author or contributor, for
**     publicity purposes without written authorization.
** 
**   2. No Misrepresentation of Affiliation.
** 
**     You may not represent yourself as Golden Code Development Corporation or FWD.
** 
**     You may not represent yourself for publicity purposes as associated with
**     Golden Code Development Corporation, FWD, or any author or contributor to
**     the covered work, without written authorization.
** 
**   3. No Misrepresentation of Source or Origin.
** 
**     You may not represent the covered work as solely your work.  All modified
**     versions of the covered work must be marked in a reasonable way to make it
**     clear that the modified work is not originating from Golden Code Development
**     Corporation or FWD.  All modified versions must contain the notices of
**     attribution required in this license.
*/


package com.goldencode.p2j.admin.client.application.home.console.acquiredlocks;

import java.util.Set;

import com.goldencode.p2j.admin.RecordInfo;
import com.goldencode.p2j.admin.client.application.home.ViewStateMachine.TransitionState;
import com.gwtplatform.mvp.client.UiHandlers;


/**
 * Defines business methods for views to manage acquired locks. The acquired locks table is
 * defined on the client side only and managed by the client. The unlocked records aren't
 * persistent between different client's sessions.
 */
public interface AcquiredLocksUIHandler
extends UiHandlers
{
   /**
    * Executes the action to lock new added records from the acquired locks table.
    */
   void lockRecords();
   
   /**
    * Executes the action to unlock locked records.
    */
   void unlockRecords();
   
   /**
    * Adds new unlocked record to the acquired locks table view.
    *  
    * @param    database
    *           The given database
    * @param    table
    *           The given table containing the target record 
    * @param    recordId
    *           The target record identification number
    */
   void addRecordToLocks(String database, String table, Long recordId);
   
   /**
    * Retrieves all tables from the given database asynchronously and refreshes the corresponding
    * widgets.
    * 
    * @param    databaseName
    *           The given database
    */
   void refreshTables(String databaseName);
   
   /**
    * Retrieves all managed databases asynchronously and refreshes the corresponding widgets.
    */
   void refreshDatabases();
   
   /**
    * Delete the selected unlocked records. If the selected record is locked, then it cann't be
    * removed from the acquired locks table. It is required to unlock records first and then
    * delete selected records.
    * 
    * @param    selected
    *           The set of selected records
    */
   void deleteSelectedRecords(Set<RecordInfo> selected);
   
   /**
    * Delete all unlocked records from the acquired locks table.
    */
   void deleteAllRecords();
   
   /**
    * Applies the given transitional state to the current state of the view state machine.
    *  
    * @param    transitionState
    *           The given transitional state 
    */
   void setCurrentState(TransitionState transitionState);
}
