/*
** Module   : IntegerValidator.java
** Abstract : Integer UI validator.
**
** Copyright (c) 2017, Golden Code Development Corporation.
**
** -#- -I- --Date-- --------------------------------Description-----------------------------------
** 001 HC  20170612 Initial version.
*/
/*
** This program is free software: you can redistribute it and/or modify
** it under the terms of the GNU Affero General Public License as
** published by the Free Software Foundation, either version 3 of the
** License, or (at your option) any later version.
**
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU Affero General Public License for more details.
**
** You may find a copy of the GNU Affero GPL version 3 at the following
** location: https://www.gnu.org/licenses/agpl-3.0.en.html
**
** Additional terms under GNU Affero GPL version 3 section 7:
**
**   Under Section 7 of the GNU Affero GPL version 3, the following additional
**   terms apply to the works covered under the License.  These additional terms
**   are non-permissive additional terms allowed under Section 7 of the GNU
**   Affero GPL version 3 and may not be removed by you.
**
**   0. Attribution Requirement.
**
**     You must preserve all legal notices or author attributions in the covered
**     work or Appropriate Legal Notices displayed by works containing the covered
**     work.  You may not remove from the covered work any author or developer
**     credit already included within the covered work.
**
**   1. No License To Use Trademarks.
**
**     This license does not grant any license or rights to use the trademarks
**     Golden Code, FWD, any Golden Code or FWD logo, or any other trademarks
**     of Golden Code Development Corporation. You are not authorized to use the
**     name Golden Code, FWD, or the names of any author or contributor, for
**     publicity purposes without written authorization.
**
**   2. No Misrepresentation of Affiliation.
**
**     You may not represent yourself as Golden Code Development Corporation or FWD.
**
**     You may not represent yourself for publicity purposes as associated with
**     Golden Code Development Corporation, FWD, or any author or contributor to
**     the covered work, without written authorization.
**
**   3. No Misrepresentation of Source or Origin.
**
**     You may not represent the covered work as solely your work.  All modified
**     versions of the covered work must be marked in a reasonable way to make it
**     clear that the modified work is not originating from Golden Code Development
**     Corporation or FWD.  All modified versions must contain the notices of
**     attribution required in this license.
*/
package com.goldencode.p2j.admin.client.widget;

import org.gwtbootstrap3.client.ui.form.validator.*;

/**
 * Integer validator. Checks to see if the value is a valid integer.
 *
 */
public class IntegerValidator
extends AbstractValidator<Object>
{
   /**
    * Defines the default constructor.
    */
   public IntegerValidator()
   {
      super("Field value must be a valid integer");
   }

   /**
    * Constructor. This overrides all validation message handling. Use this constructor for
    * field specific
    * custom validation messages.
    *
    * @param   invalidMessageOverride
    *          the invalid message override
    */
   public IntegerValidator(String invalidMessageOverride)
   {
      super(invalidMessageOverride);
   }

   /**
    * Constructor. Looks up the message using the messageKey and replacing arguments with
    * messageValueArgs.
    *
    * @param   messageKey
    *          the message key
    * @param   messageValueArgs
    *          the message value args
    */
   public IntegerValidator(String messageKey, Object[] messageValueArgs)
   {
      super(messageKey, messageValueArgs);
   }

   /** {@inheritDoc} */
   @Override
   public int getPriority() {
      return Priority.HIGH;
   }

   /** {@inheritDoc} */
   @Override
   public boolean isValid(Object value) {
      if (value == null) { return true; }

      String str = value.toString();

      try
      {
         Integer.parseInt(str);
         return true;
      }
      catch (NumberFormatException e)
      {
         return false;
      }
   }
}
