/*
** Module   : GroupsReportsBuilder.java
** Abstract : GroupsReportsBuilder is a PDF report builder to produce Group Accounts reports.
**
** Copyright (c) 2017, Golden Code Development Corporation.
**
** -#- -I- --Date-- ----------------Description----------------------
** 001 SBI 20170601 Created initial version.
*/
/*
** This program is free software: you can redistribute it and/or modify
** it under the terms of the GNU Affero General Public License as
** published by the Free Software Foundation, either version 3 of the
** License, or (at your option) any later version.
**
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU Affero General Public License for more details.
**
** You may find a copy of the GNU Affero GPL version 3 at the following
** location: https://www.gnu.org/licenses/agpl-3.0.en.html
** 
** Additional terms under GNU Affero GPL version 3 section 7:
** 
**   Under Section 7 of the GNU Affero GPL version 3, the following additional
**   terms apply to the works covered under the License.  These additional terms
**   are non-permissive additional terms allowed under Section 7 of the GNU
**   Affero GPL version 3 and may not be removed by you.
** 
**   0. Attribution Requirement.
** 
**     You must preserve all legal notices or author attributions in the covered
**     work or Appropriate Legal Notices displayed by works containing the covered
**     work.  You may not remove from the covered work any author or developer
**     credit already included within the covered work.
** 
**   1. No License To Use Trademarks.
** 
**     This license does not grant any license or rights to use the trademarks
**     Golden Code, FWD, any Golden Code or FWD logo, or any other trademarks
**     of Golden Code Development Corporation. You are not authorized to use the
**     name Golden Code, FWD, or the names of any author or contributor, for
**     publicity purposes without written authorization.
** 
**   2. No Misrepresentation of Affiliation.
** 
**     You may not represent yourself as Golden Code Development Corporation or FWD.
** 
**     You may not represent yourself for publicity purposes as associated with
**     Golden Code Development Corporation, FWD, or any author or contributor to
**     the covered work, without written authorization.
** 
**   3. No Misrepresentation of Source or Origin.
** 
**     You may not represent the covered work as solely your work.  All modified
**     versions of the covered work must be marked in a reasonable way to make it
**     clear that the modified work is not originating from Golden Code Development
**     Corporation or FWD.  All modified versions must contain the notices of
**     attribution required in this license.
*/


package com.goldencode.p2j.admin.server.reports;


import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;
import java.util.Map;
import java.util.stream.Stream;

import com.goldencode.p2j.admin.AdminServerImpl;
import com.goldencode.p2j.admin.TaggedName;
import com.goldencode.p2j.admin.shared.ReportRequest;
import com.goldencode.p2j.admin.shared.Reports;


/**
 * GroupsReportsBuilder is a PDF report builder to produce Group Accounts reports.
 */
class GroupsReportsBuilder
extends AccountsReportBuilder<Map.Entry<TaggedName, TaggedName[]>>
{

   /**
    * Creates the groups reports builder.
    * 
    * @param    request
    *           The report request
    */
   public GroupsReportsBuilder(ReportRequest request)
   {
      super(Reports.GROUPS.name(),
            request.getFilter(),
            request.getSelectedRows(),
            request.getReportParameters());
   }

   /**
    * Gets the report data stream.
    * 
    * @return   The report data stream
    */
   @Override
   protected Stream<Map.Entry<TaggedName, TaggedName[]>> getDataStream()
   {
      TaggedName[] groups = AdminServerImpl.listGroups();
      
      Stream<TaggedName> data;
      
      if (hasSelectedRows())
      {
         data = getSelectionStream(groups);
      }
      else
      {
         data = Arrays.asList(groups).stream();
      }

      return data.<Map.Entry<TaggedName, TaggedName[]>>map(group -> {
         TaggedName[] groupUsers = AdminServerImpl.listGroupUsers(group.getName());
         return new Map.Entry<TaggedName, TaggedName[]>()
         {

            @Override
            public TaggedName getKey()
            {
               return group;
            }

            @Override
            public TaggedName[] getValue()
            {
               return groupUsers;
            }

            @Override
            public TaggedName[] setValue(TaggedName[] value)
            {
               return value;
            }
         };
      });
   }

   /**
    * Gets the report column list.
    * 
    * @return   The report column list
    */
   @Override
   protected List<AbstractReportBuilder.ColumnInfo<Map.Entry<TaggedName, TaggedName[]>>> getColumns()
   {
      ArrayList<AbstractReportBuilder.ColumnInfo<Map.Entry<TaggedName, TaggedName[]>>> columns = new ArrayList<>();
      
      columns.add(new ColumnInfo<Map.Entry<TaggedName, TaggedName[]>>("Group ID", entry -> {
         return entry.getKey().getName();
      }));
      columns.add(new ColumnInfo<Map.Entry<TaggedName, TaggedName[]>>("Description", entry -> {
         return entry.getKey().getTag();
      }));
      columns.add(new ColumnInfo<Map.Entry<TaggedName, TaggedName[]>>("User ID", entry -> {
         if (entry.getValue() == null)
         {
            return "*** error ***";
         }
         
         ArrayList<String> uids = new ArrayList<String>(entry.getValue().length);
         Arrays.asList(entry.getValue()).forEach(group -> { uids.add(group.getName());});
         
         return uids.toString();
      }));
      columns.add(new ColumnInfo<Map.Entry<TaggedName, TaggedName[]>>("Person", entry -> {
         if (entry.getValue() == null)
         {
            return "*** error ***";
         }
         
         ArrayList<String> pids = new ArrayList<String>(entry.getValue().length);
         Arrays.asList(entry.getValue()).forEach(group -> { pids.add(group.getTag());});
         
         return pids.toString();
      }));
      
      return columns;
   }
}
