/*
** Module   : BufferBlockState.java
** Abstract : stores the state of buffer scope processing at the block level
**
** Copyright (c) 2005-2017, Golden Code Development Corporation.
**
** -#- -I- --Date-- --JPRM-- ----------------Description-----------------
** 001 GES 20050922   @22787 First version which stores the state of 
**                           buffer scope processing at the block level
**                           during pattern engine processing.
** 002 GES 20090518   @42374 Import change.
*/
/*
** This program is free software: you can redistribute it and/or modify
** it under the terms of the GNU Affero General Public License as
** published by the Free Software Foundation, either version 3 of the
** License, or (at your option) any later version.
**
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU Affero General Public License for more details.
**
** You may find a copy of the GNU Affero GPL version 3 at the following
** location: https://www.gnu.org/licenses/agpl-3.0.en.html
** 
** Additional terms under GNU Affero GPL version 3 section 7:
** 
**   Under Section 7 of the GNU Affero GPL version 3, the following additional
**   terms apply to the works covered under the License.  These additional terms
**   are non-permissive additional terms allowed under Section 7 of the GNU
**   Affero GPL version 3 and may not be removed by you.
** 
**   0. Attribution Requirement.
** 
**     You must preserve all legal notices or author attributions in the covered
**     work or Appropriate Legal Notices displayed by works containing the covered
**     work.  You may not remove from the covered work any author or developer
**     credit already included within the covered work.
** 
**   1. No License To Use Trademarks.
** 
**     This license does not grant any license or rights to use the trademarks
**     Golden Code, FWD, any Golden Code or FWD logo, or any other trademarks
**     of Golden Code Development Corporation. You are not authorized to use the
**     name Golden Code, FWD, or the names of any author or contributor, for
**     publicity purposes without written authorization.
** 
**   2. No Misrepresentation of Affiliation.
** 
**     You may not represent yourself as Golden Code Development Corporation or FWD.
** 
**     You may not represent yourself for publicity purposes as associated with
**     Golden Code Development Corporation, FWD, or any author or contributor to
**     the covered work, without written authorization.
** 
**   3. No Misrepresentation of Source or Origin.
** 
**     You may not represent the covered work as solely your work.  All modified
**     versions of the covered work must be marked in a reasonable way to make it
**     clear that the modified work is not originating from Golden Code Development
**     Corporation or FWD.  All modified versions must contain the notices of
**     attribution required in this license.
*/

package com.goldencode.p2j.convert;

import java.util.*;
import com.goldencode.ast.*;

/**
 * This class stores the state of buffer scope processing at the block level
 * during pattern engine processing.  Each instance stores state related to
 * a single buffer name and this state corresponds with a single block in the
 * tree.  All data is public since this is a simple container class.  It is
 * expected that a stack of these instances will be maintained for each
 * buffer name.
 */
public class BufferBlockState
{
   /** Block node associated with this instance. */
   public Aast blockNode = null;
   
   /** Block node associated with a pending rollup. */
   public Aast pendingRollupNode = null;
   
   /** Remembers the first weak reference of this block. */ 
   public BufferScopeTracker firstWeak = null;
   
   /** Remembers the last expanded scope of this block (or below). */ 
   public BufferScopeTracker lastExpanded = null;
   
   /** Tracks the most recent bind target for an unbound free reference. */ 
   public BufferScopeTracker bindTarget = null;
   
   /** Records the list of all scopes to be rolled up at this level. */
   public Set rollupList = new HashSet();   
}