/*
** Module   : MatchPhraseConstants.java
** Abstract : Constants used for name conversion operations.
**
** Copyright (c) 2005-2017, Golden Code Development Corporation.
**
** -#- -I- --Date-- --JPRM-- -----------------------------Description-----------------------------
** 001 ECF 20050428   @20934 Created initial version. Defines constants for name conversion types
**                           and for groupings of match phrase definitions in the match phrase
**                           dictionary.
** 002 OM  20170207          Added TYPE_PROPERTY for special case of DMO property names.
*/
/*
** This program is free software: you can redistribute it and/or modify
** it under the terms of the GNU Affero General Public License as
** published by the Free Software Foundation, either version 3 of the
** License, or (at your option) any later version.
**
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU Affero General Public License for more details.
**
** You may find a copy of the GNU Affero GPL version 3 at the following
** location: https://www.gnu.org/licenses/agpl-3.0.en.html
** 
** Additional terms under GNU Affero GPL version 3 section 7:
** 
**   Under Section 7 of the GNU Affero GPL version 3, the following additional
**   terms apply to the works covered under the License.  These additional terms
**   are non-permissive additional terms allowed under Section 7 of the GNU
**   Affero GPL version 3 and may not be removed by you.
** 
**   0. Attribution Requirement.
** 
**     You must preserve all legal notices or author attributions in the covered
**     work or Appropriate Legal Notices displayed by works containing the covered
**     work.  You may not remove from the covered work any author or developer
**     credit already included within the covered work.
** 
**   1. No License To Use Trademarks.
** 
**     This license does not grant any license or rights to use the trademarks
**     Golden Code, FWD, any Golden Code or FWD logo, or any other trademarks
**     of Golden Code Development Corporation. You are not authorized to use the
**     name Golden Code, FWD, or the names of any author or contributor, for
**     publicity purposes without written authorization.
** 
**   2. No Misrepresentation of Affiliation.
** 
**     You may not represent yourself as Golden Code Development Corporation or FWD.
** 
**     You may not represent yourself for publicity purposes as associated with
**     Golden Code Development Corporation, FWD, or any author or contributor to
**     the covered work, without written authorization.
** 
**   3. No Misrepresentation of Source or Origin.
** 
**     You may not represent the covered work as solely your work.  All modified
**     versions of the covered work must be marked in a reasonable way to make it
**     clear that the modified work is not originating from Golden Code Development
**     Corporation or FWD.  All modified versions must contain the notices of
**     attribution required in this license.
*/

package com.goldencode.p2j.convert;

/**
 * Constants for name conversion match entry groupings and conversion types.
 *
 * @see  MatchPhraseDictionary
 */
public interface MatchPhraseConstants
{
   /** Standard set of substring replacement definitions */
   public static final int STANDARD = 0x0001;
   
   /** Custom set of substring replacement definitions */
   public static final int CUSTOM = 0x0002;
   
   /** Set of substring tokens which must be preserved during conversion */
   public static final int PRESERVE = 0x0004;
   
   /** Entire set of substring replacement definitions */
   public static final int ALL = STANDARD | CUSTOM | PRESERVE;
   
   /** Indicates no extra conversion other than simple replacements */
   public static final int TYPE_SIMPLE = 0;
   
   /** Indicates conversion to a Java variable symbol name */
   public static final int TYPE_VARIABLE = 1;
   
   /** Indicates conversion to a Java method symbol name */
   public static final int TYPE_METHOD = 2;
   
   /** Indicates conversion to a Java "constant" symbol name */
   public static final int TYPE_CONSTANT = 3;
   
   /** Indicates conversion to a Java class symbol name */
   public static final int TYPE_CLASS = 4;
   
   /** Indicates conversion to an SQL database symbol name */
   public static final int TYPE_DATABASE = 5;
   
   /** Indicates conversion to an SQL database table symbol name */
   public static final int TYPE_TABLE = 6;
   
   /** Indicates conversion to an SQL table column symbol name */
   public static final int TYPE_COLUMN = 7;
   
   /** Indicates conversion to an SQL index symbol name */
   public static final int TYPE_INDEX = 8;
   
   /** Indicates conversion to a DMO property name */
   public static final int TYPE_PROPERTY = 9;
}