/*
** Module   :NodeProcessor.java
** Abstract :An interface for the tree processing.
**
** Copyright (c) 2005-2017, Golden Code Development Corporation.
**
** -#- -I- --Date-- -T- --JPRM-- ----------------Description-----------------
** 001 SIY 20050301 ADD  @20366  Created initial version
** 002 SIY 20050429 CHG  @21022  Fixed comments and formatting.
*/
/*
** This program is free software: you can redistribute it and/or modify
** it under the terms of the GNU Affero General Public License as
** published by the Free Software Foundation, either version 3 of the
** License, or (at your option) any later version.
**
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU Affero General Public License for more details.
**
** You may find a copy of the GNU Affero GPL version 3 at the following
** location: https://www.gnu.org/licenses/agpl-3.0.en.html
** 
** Additional terms under GNU Affero GPL version 3 section 7:
** 
**   Under Section 7 of the GNU Affero GPL version 3, the following additional
**   terms apply to the works covered under the License.  These additional terms
**   are non-permissive additional terms allowed under Section 7 of the GNU
**   Affero GPL version 3 and may not be removed by you.
** 
**   0. Attribution Requirement.
** 
**     You must preserve all legal notices or author attributions in the covered
**     work or Appropriate Legal Notices displayed by works containing the covered
**     work.  You may not remove from the covered work any author or developer
**     credit already included within the covered work.
** 
**   1. No License To Use Trademarks.
** 
**     This license does not grant any license or rights to use the trademarks
**     Golden Code, FWD, any Golden Code or FWD logo, or any other trademarks
**     of Golden Code Development Corporation. You are not authorized to use the
**     name Golden Code, FWD, or the names of any author or contributor, for
**     publicity purposes without written authorization.
** 
**   2. No Misrepresentation of Affiliation.
** 
**     You may not represent yourself as Golden Code Development Corporation or FWD.
** 
**     You may not represent yourself for publicity purposes as associated with
**     Golden Code Development Corporation, FWD, or any author or contributor to
**     the covered work, without written authorization.
** 
**   3. No Misrepresentation of Source or Origin.
** 
**     You may not represent the covered work as solely your work.  All modified
**     versions of the covered work must be marked in a reasonable way to make it
**     clear that the modified work is not originating from Golden Code Development
**     Corporation or FWD.  All modified versions must contain the notices of
**     attribution required in this license.
*/

package com.goldencode.p2j.directory;

/**
 * This interface is used internally by the <code>DirectoryService</code> to
 * perform tasks on the subtree or entire tree. The <code>visit()</code>
 * method of the object that implements this interface will be called for each
 * node of the tree until instance of some object is returned or all nodes are
 * visited.
 * <p>
 * There are several possible uses of this interface. The simplest one is a
 * search operation which is not supported by the back-end. In this case
 * object which implements this interface can just return nodeId once node
 * which meet search criteria. Or it can collect all matching node in internal
 * data where caller can retrieve it after traversing entire subtree. Other
 * use is subtree backup operation. If all nodes are visited in the "top-down"
 * order can just copy each visited node into external data storage or even
 * into other subtree of the same tree.
 * 
 * @author  SIY
 * @version 1.0
 */
interface NodeProcessor
{
   /**
    * This method in called for each visited node.
    * 
    * @param   service
    *          Remapper instance which is traversed
    * @param   nodeId
    *          Node Id
    * @return  an object instance if tree traversal should be stopped or
    *          <code>null</code> to continue operation.
    */
   public Object visit(Remapper service, String nodeId);
}