/*
** Module   : ProcessBuilderParameters.java
** Abstract : store the process parameters.
**
** Copyright (c) 2014-2017, Golden Code Development Corporation.
**
** -#- -I- --Date-- -------------------------------Description------------------------------------
** 001 MAG 20140224 First version.
*/
/*
** This program is free software: you can redistribute it and/or modify
** it under the terms of the GNU Affero General Public License as
** published by the Free Software Foundation, either version 3 of the
** License, or (at your option) any later version.
**
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU Affero General Public License for more details.
**
** You may find a copy of the GNU Affero GPL version 3 at the following
** location: https://www.gnu.org/licenses/agpl-3.0.en.html
** 
** Additional terms under GNU Affero GPL version 3 section 7:
** 
**   Under Section 7 of the GNU Affero GPL version 3, the following additional
**   terms apply to the works covered under the License.  These additional terms
**   are non-permissive additional terms allowed under Section 7 of the GNU
**   Affero GPL version 3 and may not be removed by you.
** 
**   0. Attribution Requirement.
** 
**     You must preserve all legal notices or author attributions in the covered
**     work or Appropriate Legal Notices displayed by works containing the covered
**     work.  You may not remove from the covered work any author or developer
**     credit already included within the covered work.
** 
**   1. No License To Use Trademarks.
** 
**     This license does not grant any license or rights to use the trademarks
**     Golden Code, FWD, any Golden Code or FWD logo, or any other trademarks
**     of Golden Code Development Corporation. You are not authorized to use the
**     name Golden Code, FWD, or the names of any author or contributor, for
**     publicity purposes without written authorization.
** 
**   2. No Misrepresentation of Affiliation.
** 
**     You may not represent yourself as Golden Code Development Corporation or FWD.
** 
**     You may not represent yourself for publicity purposes as associated with
**     Golden Code Development Corporation, FWD, or any author or contributor to
**     the covered work, without written authorization.
** 
**   3. No Misrepresentation of Source or Origin.
** 
**     You may not represent the covered work as solely your work.  All modified
**     versions of the covered work must be marked in a reasonable way to make it
**     clear that the modified work is not originating from Golden Code Development
**     Corporation or FWD.  All modified versions must contain the notices of
**     attribution required in this license.
*/
package com.goldencode.p2j.main;

import com.goldencode.p2j.security.SecurityManager;

/**
 * This structure holds process parameters to create and spawn a P2J client process. 
 */
public class ProcessBuilderParameters
extends ClientBuilderParameters
{
   /** P2J process name */
   private String process;
   
   /** Output file (optional) to redirect STDOUT */
   private String outputToFile;
   
   /** application server name if defined or null if not defined */
   private String appServer;

   /**
    * Constructor.
    * 
    * @param   options
    *          A ProcessBuilderOptions instance.
    */
   public ProcessBuilderParameters(ProcessBuilderOptions options)
   {
      super((ClientBuilderOptions) options);

      process      = options.getProcess();
      outputToFile = options.getOutputToFile();
      
      SecurityManager sm = SecurityManager.getInstance();
      appServer = sm.getAppserverForProcess(process);
   }
   
   /**
    * In case of processes returns the P2J process id. 
    */
   @Override
   public String getUser()
   {
      return process;
   }
   
   /**
    * Build a file name for outpuToFile option based on a template.
    * The place holders %userid%, %timestamp% and %appserver% are substituted 
    * with process id and current time in milliseconds and application server
    * name if an application server is defined.
    * 
    * %return  File name.
    */
   @Override
   public String getOutputToFile()
   {      
      if (outputToFile != null)
      {
         String timestamp = Long.valueOf(System.currentTimeMillis()).toString();
         
         outputToFile = outputToFile.replace("%userid%", process);
         outputToFile = outputToFile.replace("%timestamp%", timestamp);
         
         if (appServer != null)
         {
            outputToFile = outputToFile.replace("%appserver%", appServer);
         }
      }      

      return outputToFile;
   }   
}