/*
** Module   : AddressServer.java
** Abstract : address server abstraction layer
**
** Copyright (c) 2005-2017, Golden Code Development Corporation.
**
** -#- -I- --Date-- -T- --JPRM-- ----------------Description-----------------
** 001 SIY 20050120 ADD   @19490 Initial address partitioning is 16 upper
**                               bits for routing node address, lower 16
**                               bits - node address in the router's address
**                               space.
** 002 SIY 20050315 CHG   @20375 Fixed spelling and other minor cleanups.
** 003 GES 20061222 CHG   @31783 Reworked the interface to be more useful for
**                               its original intended purpose of being the
**                               central "clearinghouse" for all routing
**                               information.
*/
/*
** This program is free software: you can redistribute it and/or modify
** it under the terms of the GNU Affero General Public License as
** published by the Free Software Foundation, either version 3 of the
** License, or (at your option) any later version.
**
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU Affero General Public License for more details.
**
** You may find a copy of the GNU Affero GPL version 3 at the following
** location: https://www.gnu.org/licenses/agpl-3.0.en.html
** 
** Additional terms under GNU Affero GPL version 3 section 7:
** 
**   Under Section 7 of the GNU Affero GPL version 3, the following additional
**   terms apply to the works covered under the License.  These additional terms
**   are non-permissive additional terms allowed under Section 7 of the GNU
**   Affero GPL version 3 and may not be removed by you.
** 
**   0. Attribution Requirement.
** 
**     You must preserve all legal notices or author attributions in the covered
**     work or Appropriate Legal Notices displayed by works containing the covered
**     work.  You may not remove from the covered work any author or developer
**     credit already included within the covered work.
** 
**   1. No License To Use Trademarks.
** 
**     This license does not grant any license or rights to use the trademarks
**     Golden Code, FWD, any Golden Code or FWD logo, or any other trademarks
**     of Golden Code Development Corporation. You are not authorized to use the
**     name Golden Code, FWD, or the names of any author or contributor, for
**     publicity purposes without written authorization.
** 
**   2. No Misrepresentation of Affiliation.
** 
**     You may not represent yourself as Golden Code Development Corporation or FWD.
** 
**     You may not represent yourself for publicity purposes as associated with
**     Golden Code Development Corporation, FWD, or any author or contributor to
**     the covered work, without written authorization.
** 
**   3. No Misrepresentation of Source or Origin.
** 
**     You may not represent the covered work as solely your work.  All modified
**     versions of the covered work must be marked in a reasonable way to make it
**     clear that the modified work is not originating from Golden Code Development
**     Corporation or FWD.  All modified versions must contain the notices of
**     attribution required in this license.
*/

package com.goldencode.p2j.net;

/**
 * Provides an API for the implementation of a registry of the nodes in
 * the network, especially in regards to enabling effective multi-hop
 * routing.
 * <p>
 * <b>WARNING: this class is not yet done. It is more of an early example
 * at this point, but it probably does not easily provide for mapping an
 * entire network from a single location as is the original intent. Certainly
 * it would be very hard to make routing decisions based on this.  For these
 * reasons, this class will probably be changed in the future.</b>
 */
public interface AddressServer
{
   /**
    * Mask used to extract address of the node in the router's address space.
    * If resulting address is 0 then the address is an address of a router,
    * otherwise the address belongs to a leaf node.
    */
   public static final int NODE_MASK = 0x0000FFFF;

   /** Mask used to extract the router address from any valid node address. */
   public static final int ROUTER_MASK = 0xFFFF0000;

   /**
    * Add the given node descriptor into the registry of known/valid nodes.
    *
    * @param    node
    *           The node descriptor to add.
    */
   public void register(NodeInfo node);
   
   /**
    * Remove the given node descriptor from the registry of known/valid
    * nodes.
    *
    * @param    node
    *           The node descriptor to remove.
    */
   public void deregister(NodeInfo node);
   
   /**
    * Returns the node description associated with the specified name.
    * 
    * @param    name
    *           The name of the node to be found.  If set to <code>null</code>
    *           then the descriptor for the local node will be returned.
    *
    * @return   The descriptor for the given node with all fields known to
    *           address server filled or <code>null</code> if no matching
    *           node is found.
    */
   public NodeInfo lookupByName(String name);
   
   /**
    * Returns the node description associated with the specified address.
    * 
    * @param    addr
    *           The address of the node to be found.  If set to 0, then the
    *           the descriptor for the local node will be returned.
    *
    * @return   The descriptor for the given node with all fields known to
    *           address server filled or <code>null</code> if no matching
    *           node is found.
    */
   public NodeInfo lookupByAddress(int addr);
}