/*
** Module   : SerializeOptions.java
** Abstract : Options related to serialized temp-table data.
**
** Copyright (c) 2018, Golden Code Development Corporation.
**
** -#- -I- --Date-- ---------------------------------Description---------------------------------
** 001 CA  20180511 Created initial version.
*/
/*
** This program is free software: you can redistribute it and/or modify
** it under the terms of the GNU Affero General Public License as
** published by the Free Software Foundation, either version 3 of the
** License, or (at your option) any later version.
**
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU Affero General Public License for more details.
**
** You may find a copy of the GNU Affero GPL version 3 at the following
** location: https://www.gnu.org/licenses/agpl-3.0.en.html
** 
** Additional terms under GNU Affero GPL version 3 section 7:
** 
**   Under Section 7 of the GNU Affero GPL version 3, the following additional
**   terms apply to the works covered under the License.  These additional terms
**   are non-permissive additional terms allowed under Section 7 of the GNU
**   Affero GPL version 3 and may not be removed by you.
** 
**   0. Attribution Requirement.
** 
**     You must preserve all legal notices or author attributions in the covered
**     work or Appropriate Legal Notices displayed by works containing the covered
**     work.  You may not remove from the covered work any author or developer
**     credit already included within the covered work.
** 
**   1. No License To Use Trademarks.
** 
**     This license does not grant any license or rights to use the trademarks
**     Golden Code, FWD, any Golden Code or FWD logo, or any other trademarks
**     of Golden Code Development Corporation. You are not authorized to use the
**     name Golden Code, FWD, or the names of any author or contributor, for
**     publicity purposes without written authorization.
** 
**   2. No Misrepresentation of Affiliation.
** 
**     You may not represent yourself as Golden Code Development Corporation or FWD.
** 
**     You may not represent yourself for publicity purposes as associated with
**     Golden Code Development Corporation, FWD, or any author or contributor to
**     the covered work, without written authorization.
** 
**   3. No Misrepresentation of Source or Origin.
** 
**     You may not represent the covered work as solely your work.  All modified
**     versions of the covered work must be marked in a reasonable way to make it
**     clear that the modified work is not originating from Golden Code Development
**     Corporation or FWD.  All modified versions must contain the notices of
**     attribution required in this license.
*/

package com.goldencode.p2j.persist.serial;

/**
 * Options related to serialized temp-table data.
 */
public class TableSerializeOptions
{
   /** Name of field in serialized output (overrides table name). */
   private final String serializeName;
   
   /** Name of element or attribute representing field in XML output. */
   private final String xmlNodeName;
   
   /** The namespace URI used in serializing. */
   private final String namespaceUri;
   
   /** The namespace prefix used in serializing. */
   private final String namespacePrefix;
   
   /**
    * Constructor.
    * 
    * @param   serializeName
    *          Name of field in serialized output (overrides field name).
    * @param   xmlNodeName
    *          Name of element or attribute representing field in XML output.
    * @param   namespaceUri
    *          The namespace URI used in serializing.
    * @param   namespacePrefix
    *          The namespace prefix used in serializing.
    */
   public TableSerializeOptions(String serializeName,
                                String xmlNodeName,
                                String namespaceUri,
                                String namespacePrefix)
   {
      this.serializeName = serializeName;
      this.xmlNodeName = xmlNodeName;
      this.namespaceUri = namespaceUri;
      this.namespacePrefix = namespacePrefix;
   }
   
   /**
    * Get the name of the field in serialized output (overrides field name in table).
    * 
    * @return  Serialize name.
    */
   public String getSerializeName()
   {
      return serializeName;
   }
   
   /**
    * Get the node name of the field in XML output (overrides serialize-name and field name in
    * table).
    * 
    * @return  XML node name.
    */
   public String getXmlNodeName()
   {
      return xmlNodeName;
   }
   
   /**
    * Get the namespace prefix.
    * 
    * @return   The {@link #namespacePrefix}.
    */
   public String getNamespacePrefix()
   {
      return namespacePrefix;
   }
   
   /**
    * Get the namespace URI.
    * 
    * @return   The {@link #namespaceUri}.
    */
   public String getNamespaceUri()
   {
      return namespaceUri;
   }
}
