/*
** Module   : DetailRow.java
** Abstract : Detail report row data bean.
**
** Copyright (c) 2017, Golden Code Development Corporation.
**
** -#- -I- --Date-- ---------------------------------Description----------------------------------
** 001 ECF 20170521 First version.
*/
/*
** This program is free software: you can redistribute it and/or modify
** it under the terms of the GNU Affero General Public License as
** published by the Free Software Foundation, either version 3 of the
** License, or (at your option) any later version.
**
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU Affero General Public License for more details.
**
** You may find a copy of the GNU Affero GPL version 3 at the following
** location: https://www.gnu.org/licenses/agpl-3.0.en.html
** 
** Additional terms under GNU Affero GPL version 3 section 7:
** 
**   Under Section 7 of the GNU Affero GPL version 3, the following additional
**   terms apply to the works covered under the License.  These additional terms
**   are non-permissive additional terms allowed under Section 7 of the GNU
**   Affero GPL version 3 and may not be removed by you.
** 
**   0. Attribution Requirement.
** 
**     You must preserve all legal notices or author attributions in the covered
**     work or Appropriate Legal Notices displayed by works containing the covered
**     work.  You may not remove from the covered work any author or developer
**     credit already included within the covered work.
** 
**   1. No License To Use Trademarks.
** 
**     This license does not grant any license or rights to use the trademarks
**     Golden Code, FWD, any Golden Code or FWD logo, or any other trademarks
**     of Golden Code Development Corporation. You are not authorized to use the
**     name Golden Code, FWD, or the names of any author or contributor, for
**     publicity purposes without written authorization.
** 
**   2. No Misrepresentation of Affiliation.
** 
**     You may not represent yourself as Golden Code Development Corporation or FWD.
** 
**     You may not represent yourself for publicity purposes as associated with
**     Golden Code Development Corporation, FWD, or any author or contributor to
**     the covered work, without written authorization.
** 
**   3. No Misrepresentation of Source or Origin.
** 
**     You may not represent the covered work as solely your work.  All modified
**     versions of the covered work must be marked in a reasonable way to make it
**     clear that the modified work is not originating from Golden Code Development
**     Corporation or FWD.  All modified versions must contain the notices of
**     attribution required in this license.
*/

package com.goldencode.p2j.report.server;

/**
 * Bean which encapsulates a row of data for a detail report. This includes the ID of the match
 * found when the report was applied to the source code. It also includes the match text itself,
 * the IDs of the source file and associated AST, the source file name, and the line and column
 * number at which the match was detected in the source file.
 */
public class DetailRow
{
   /** Match ID */
   private long id = 0L;
   
   /** Source file ID */
   private long fid = 0L;
   
   /** AST ID */
   private long astid = 0L;
   
   /** Source file name */
   private String file = null;
   
   /** Line number of the start of the match */
   private int line = 0;
   
   /** Column number of the start of the match */
   private int col = 0;
   
   /** Match text extracted from the AST */
   private String text = null;
   
   /**
    * Default constructor.
    */
   public DetailRow()
   {
   }
   
   /**
    * Getter for the match ID in the report database.
    * 
    * @return  Match ID.
    */
   public long getId()
   {
      return id;
   }
   
   /**
    * Setter for the match ID in the report database.
    * 
    * @param   id
    *          Match ID.
    */
   public void setId(long id)
   {
      this.id = id;
   }
   
   /**
    * Getter for the source file ID in the report database.
    * 
    * @return  File ID.
    */
   public long getFid()
   {
      return fid;
   }
   
   /**
    * Setter for the source file ID in the report database.
    * 
    * @param   fid
    *          File ID.
    */
   public void setFid(long fid)
   {
      this.fid = fid;
   }
   
   /**
    * Getter for the AST ID.
    * 
    * @return  AST ID.
    */
   public long getAstid()
   {
      return astid;
   }
   
   /**
    * Setter for the AST ID.
    * 
    * @param   astid
    *          AST ID.
    */
   public void setAstid(long astid)
   {
      this.astid = astid;
   }
   
   /**
    * Getter for the source file name.
    * 
    * @return  Source file name, relative to the project root directory.
    */
   public String getFile()
   {
      return file;
   }
   
   /**
    * Setter for the source file name.
    * 
    * @param   file
    *          Source file name, relative to the project root directory.
    */
   public void setFile(String file)
   {
      this.file = file;
   }
   
   /**
    * Getter for the starting line number in the source file of the match.
    * 
    * @return  Starting line number of the match.
    */
   public int getLine()
   {
      return line;
   }
   
   /**
    * Setter for the starting line number in the source file of the match.
    * 
    * @param   line
    *          Starting line number of the match.
    */
   public void setLine(int line)
   {
      this.line = line;
   }
   
   /**
    * Getter for the starting column number in the source file of the match.
    * 
    * @return  Starting column number of the match.
    */
   public int getCol()
   {
      return col;
   }
   
   /**
    * Setter for the starting column number in the source file of the match.
    * 
    * @param   col
    *          Starting column number of the match.
    */
   public void setCol(int col)
   {
      this.col = col;
   }
   
   /**
    * Getter for the match text.
    * 
    * @return  Match text.
    */
   public String getText()
   {
      return text;
   }
   
   /**
    * Setter for the match text.
    * 
    * @param   text
    *          Match text.
    */
   public void setText(String text)
   {
      this.text = text;
   }
}
