/*
** Module   : FilterProfile.java
** Abstract : File filter profile.
**
** Copyright (c) 2017, Golden Code Development Corporation.
**
** -#- -I- --Date-- ---------------------------------Description----------------------------------
** 001 ECF 20170602 First version.
*/
/*
** This program is free software: you can redistribute it and/or modify
** it under the terms of the GNU Affero General Public License as
** published by the Free Software Foundation, either version 3 of the
** License, or (at your option) any later version.
**
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU Affero General Public License for more details.
**
** You may find a copy of the GNU Affero GPL version 3 at the following
** location: https://www.gnu.org/licenses/agpl-3.0.en.html
** 
** Additional terms under GNU Affero GPL version 3 section 7:
** 
**   Under Section 7 of the GNU Affero GPL version 3, the following additional
**   terms apply to the works covered under the License.  These additional terms
**   are non-permissive additional terms allowed under Section 7 of the GNU
**   Affero GPL version 3 and may not be removed by you.
** 
**   0. Attribution Requirement.
** 
**     You must preserve all legal notices or author attributions in the covered
**     work or Appropriate Legal Notices displayed by works containing the covered
**     work.  You may not remove from the covered work any author or developer
**     credit already included within the covered work.
** 
**   1. No License To Use Trademarks.
** 
**     This license does not grant any license or rights to use the trademarks
**     Golden Code, FWD, any Golden Code or FWD logo, or any other trademarks
**     of Golden Code Development Corporation. You are not authorized to use the
**     name Golden Code, FWD, or the names of any author or contributor, for
**     publicity purposes without written authorization.
** 
**   2. No Misrepresentation of Affiliation.
** 
**     You may not represent yourself as Golden Code Development Corporation or FWD.
** 
**     You may not represent yourself for publicity purposes as associated with
**     Golden Code Development Corporation, FWD, or any author or contributor to
**     the covered work, without written authorization.
** 
**   3. No Misrepresentation of Source or Origin.
** 
**     You may not represent the covered work as solely your work.  All modified
**     versions of the covered work must be marked in a reasonable way to make it
**     clear that the modified work is not originating from Golden Code Development
**     Corporation or FWD.  All modified versions must contain the notices of
**     attribution required in this license.
*/

package com.goldencode.p2j.report.server;

import java.util.*;
import com.fasterxml.jackson.annotation.*;

/**
 * Bean containing information about a profile used to filter source files to be included in
 * report results.
 */
public class FilterProfile
{
   /** Profile ID */
   private long id = -1L;
   
   /** Flag indicating whether this is the default profile (there can be only one) */
   private boolean isDefault;
   
   /** Name assigned to this profile */
   private String name;
   
   /** File filters associated with this profile */
   @JsonIgnore
   private final List<FileFilter> filters = new LinkedList<>();
   
   /**
    * Default constructor.
    */
   public FilterProfile()
   {
   }
   
   /**
    * Getter for profile ID.
    * 
    * @return  Profile ID.
    */
   public long getId()
   {
      return id;
   }
   
   /**
    * Setter for profile ID.
    * 
    * @param   id
    *          Profile ID.
    */
   public void setId(long id)
   {
      this.id = id;
   }
   
   /**
    * Getter for default flag.
    * 
    * @return  {@code True} if this profile is the default profile, else {@code false}.
    */
   public boolean isDefault()
   {
      return isDefault;
   }
   
   /**
    * Setter for default flag.
    * 
    * @param   isDefault
    *          {@code True} if this profile is the default profile, else {@code false}.
    */
   public void setDefault(boolean isDefault)
   {
      this.isDefault = isDefault;
   }
   
   /**
    * Getter for profile name.
    * 
    * @return  Profile name.
    */
   public String getName()
   {
      return name;
   }
   
   /**
    * Setter for profile name.
    * 
    * @param   name
    *          Profile name.
    */
   public void setName(String name)
   {
      this.name = name;
   }
   
   /**
    * Add a file filter.
    * 
    * @param   filter
    *          File filter to add
    */
   public void addFilter(FileFilter filter)
   {
      filters.add(filter);
   }
   
   /**
    * Get the list of filters.
    * 
    * @return  List of filters.
    */
   public List<FileFilter> getFilters()
   {
      return Collections.unmodifiableList(filters);
   }
}
