/*
** Module   : UserRecord.java
** Abstract : Object which represents a user record.
**
** Copyright (c) 2017, Golden Code Development Corporation.
**
** -#- -I- --Date-- --------------------------------Description----------------------------------
** 001 ECF 20170725 Created first version.
*/
/*
** This program is free software: you can redistribute it and/or modify
** it under the terms of the GNU Affero General Public License as
** published by the Free Software Foundation, either version 3 of the
** License, or (at your option) any later version.
**
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU Affero General Public License for more details.
**
** You may find a copy of the GNU Affero GPL version 3 at the following
** location: https://www.gnu.org/licenses/agpl-3.0.en.html
** 
** Additional terms under GNU Affero GPL version 3 section 7:
** 
**   Under Section 7 of the GNU Affero GPL version 3, the following additional
**   terms apply to the works covered under the License.  These additional terms
**   are non-permissive additional terms allowed under Section 7 of the GNU
**   Affero GPL version 3 and may not be removed by you.
** 
**   0. Attribution Requirement.
** 
**     You must preserve all legal notices or author attributions in the covered
**     work or Appropriate Legal Notices displayed by works containing the covered
**     work.  You may not remove from the covered work any author or developer
**     credit already included within the covered work.
** 
**   1. No License To Use Trademarks.
** 
**     This license does not grant any license or rights to use the trademarks
**     Golden Code, FWD, any Golden Code or FWD logo, or any other trademarks
**     of Golden Code Development Corporation. You are not authorized to use the
**     name Golden Code, FWD, or the names of any author or contributor, for
**     publicity purposes without written authorization.
** 
**   2. No Misrepresentation of Affiliation.
** 
**     You may not represent yourself as Golden Code Development Corporation or FWD.
** 
**     You may not represent yourself for publicity purposes as associated with
**     Golden Code Development Corporation, FWD, or any author or contributor to
**     the covered work, without written authorization.
** 
**   3. No Misrepresentation of Source or Origin.
** 
**     You may not represent the covered work as solely your work.  All modified
**     versions of the covered work must be marked in a reasonable way to make it
**     clear that the modified work is not originating from Golden Code Development
**     Corporation or FWD.  All modified versions must contain the notices of
**     attribution required in this license.
*/

package com.goldencode.p2j.report.server;

import java.sql.*;

/**
 * Information contained in a user database record.
 */
public class UserRecord
extends User
{
   /** Salt used to encrypt user's password */
   private final byte[] salt;
   
   /** User's encrypted password */
   private final byte[] password;
   
   /** Date of the last password change */
   private final Date passChanged;
   
   /**
    * Constructor.
    * 
    * @param   name
    *          Unique name.
    * @param   admin
    *          Does user have administrative privileges?
    * @param   salt
    *          Salt used to encrypt user's password.
    * @param   password
    *          User's encrypted password.
    * @param   passChanged
    *          Date of the last password change.
    */
   public UserRecord(String name, boolean admin, byte[] salt, byte[] password, Date passChanged)
   {
      // use a bogus ID (the purpose of this object is import/export, so ID is not needed)
      super(0L, name, admin);
      
      this.salt = salt;
      this.password = password;
      this.passChanged = passChanged;
   }
   
   /**
    * Get the salt used to encrypt the user's password.
    * 
    * @return  Salt.
    */
   public byte[] getSalt()
   {
      return salt;
   }
   
   /**
    * Get the user's encrypted password.
    * 
    * @return  Encrypted password.
    */
   public byte[] getPassword()
   {
      return password;
   }
   
   /**
    * Get the date of the last password change.
    * 
    * @return  Date of last password change.
    */
   public Date getPassChanged()
   {
      return passChanged;
   }
}