/*
** Module   : JasperDataSource.java
** Abstract : Data source for a FWD Jasper report.
**
** Copyright (c) 2018, Golden Code Development Corporation.
**
** -#- -I- --Date-- ---------------------------------Description----------------------------------
** 001 SVL 20180207 Created initial version.
** 002 SVL 20180209 This class now supports generic data sources.
*/
/*
** This program is free software: you can redistribute it and/or modify
** it under the terms of the GNU Affero General Public License as
** published by the Free Software Foundation, either version 3 of the
** License, or (at your option) any later version.
**
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU Affero General Public License for more details.
**
** You may find a copy of the GNU Affero GPL version 3 at the following
** location: https://www.gnu.org/licenses/agpl-3.0.en.html
**
** Additional terms under GNU Affero GPL version 3 section 7:
**
**   Under Section 7 of the GNU Affero GPL version 3, the following additional
**   terms apply to the works covered under the License.  These additional terms
**   are non-permissive additional terms allowed under Section 7 of the GNU
**   Affero GPL version 3 and may not be removed by you.
**
**   0. Attribution Requirement.
**
**     You must preserve all legal notices or author attributions in the covered
**     work or Appropriate Legal Notices displayed by works containing the covered
**     work.  You may not remove from the covered work any author or developer
**     credit already included within the covered work.
**
**   1. No License To Use Trademarks.
**
**     This license does not grant any license or rights to use the trademarks
**     Golden Code, FWD, any Golden Code or FWD logo, or any other trademarks
**     of Golden Code Development Corporation. You are not authorized to use the
**     name Golden Code, FWD, or the names of any author or contributor, for
**     publicity purposes without written authorization.
**
**   2. No Misrepresentation of Affiliation.
**
**     You may not represent yourself as Golden Code Development Corporation or FWD.
**
**     You may not represent yourself for publicity purposes as associated with
**     Golden Code Development Corporation, FWD, or any author or contributor to
**     the covered work, without written authorization.
**
**   3. No Misrepresentation of Source or Origin.
**
**     You may not represent the covered work as solely your work.  All modified
**     versions of the covered work must be marked in a reasonable way to make it
**     clear that the modified work is not originating from Golden Code Development
**     Corporation or FWD.  All modified versions must contain the notices of
**     attribution required in this license.
*/

package com.goldencode.p2j.reporting;

import com.goldencode.p2j.util.*;
import net.sf.jasperreports.engine.*;

/**
 * Data source for a FWD Jasper report.
 */
public interface JasperDataSource
extends JRDataSource
{
   /**
    * Get compiled Jasper report for the given report design file.
    *
    * @param  designFileName
    *         File name of the Jasper report design file.
    *
    * @return compiled Jasper report for the given report design file.
    */
   JasperReport getReport(String designFileName);

   /**
    * Executed before report is filled from the data source.
    *
    * @param  report
    *         Jasper report to be filled.
    *
    * @return <code>true</code> on success, <code>false</code> if the data source cannot provide
    *         data.
    */
   boolean reportStarted(JasperReport report);

   /**
    * Executed after report was filled from the data source.
    *
    * @param  error
    *         <code>true</code> if there were problems while getting data from the data source.
    *         <code>false</code> on success,
    */
   void reportFinished(boolean error);

   /**
    * Get underlying 4GL object which provides data for the report.
    *
    * @return handle to the underlying 4GL object which provides data for the report.
    */
   handle asProgressObject();
}
