/*
** Module   : DmoAsmTypes.java
** Abstract : Common constants used by DMO proxy implementations with ASM.
**
** Copyright (c) 2015-2017, Golden Code Development Corporation.
**
** -#- -I- --Date-- --------------------------------Description----------------------------------
** 001 ECF 20160219 Created initial version. Extracted from DmoAsmWorker.
*/
/*
** This program is free software: you can redistribute it and/or modify
** it under the terms of the GNU Affero General Public License as
** published by the Free Software Foundation, either version 3 of the
** License, or (at your option) any later version.
**
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU Affero General Public License for more details.
**
** You may find a copy of the GNU Affero GPL version 3 at the following
** location: https://www.gnu.org/licenses/agpl-3.0.en.html
** 
** Additional terms under GNU Affero GPL version 3 section 7:
** 
**   Under Section 7 of the GNU Affero GPL version 3, the following additional
**   terms apply to the works covered under the License.  These additional terms
**   are non-permissive additional terms allowed under Section 7 of the GNU
**   Affero GPL version 3 and may not be removed by you.
** 
**   0. Attribution Requirement.
** 
**     You must preserve all legal notices or author attributions in the covered
**     work or Appropriate Legal Notices displayed by works containing the covered
**     work.  You may not remove from the covered work any author or developer
**     credit already included within the covered work.
** 
**   1. No License To Use Trademarks.
** 
**     This license does not grant any license or rights to use the trademarks
**     Golden Code, FWD, any Golden Code or FWD logo, or any other trademarks
**     of Golden Code Development Corporation. You are not authorized to use the
**     name Golden Code, FWD, or the names of any author or contributor, for
**     publicity purposes without written authorization.
** 
**   2. No Misrepresentation of Affiliation.
** 
**     You may not represent yourself as Golden Code Development Corporation or FWD.
** 
**     You may not represent yourself for publicity purposes as associated with
**     Golden Code Development Corporation, FWD, or any author or contributor to
**     the covered work, without written authorization.
** 
**   3. No Misrepresentation of Source or Origin.
** 
**     You may not represent the covered work as solely your work.  All modified
**     versions of the covered work must be marked in a reasonable way to make it
**     clear that the modified work is not originating from Golden Code Development
**     Corporation or FWD.  All modified versions must contain the notices of
**     attribution required in this license.
*/

package com.goldencode.p2j.schema;

/**
 * Common constants needed when assembling DMO proxy method implementations with ASM.
 * 
 * @author ecf
 */
public interface DmoAsmTypes
{
   /** Prefix representing the P2J persist package in internal type form */
   public static final String PKG_PERSIST = "com/goldencode/p2j/persist/";
   
   /** Prefix representing the P2J util package in internal type form */
   public static final String PKG_UTIL = "com/goldencode/p2j/util/";
   
   /** Name of inner DMO interface */
   public static final String BUF = "Buf";
   
   /** Suffix of inner DMO interface when appended to enclosing interface name */
   public static final String BUF_SUFFIX = "$" + BUF;
   
   /** Fully qualified, internal type name of persistence <code>Buffer</code> interface */
   public static final String BUFFER_IFACE = PKG_PERSIST + "Buffer";
   
   /** Prefix used for annotation internal type descriptors */
   public static final String ANNOTATION_DESC_PREFIX = "L" + PKG_PERSIST;
   
   /** Constructor method name */
   public static final String METH_INIT = "<init>";
   
   /** Method name <code>add</code> */
   public static final String METH_ADD = "add";
   
   /** Method name <code>assign</code> */
   public static final String METH_ASSIGN = "assign";
   
   /** Method name <code>checkExtentRange</code> */
   public static final String METH_CHKXRNG = "checkExtentRange";
   
   /** Method name <code>get</code> */
   public static final String METH_GET = "get";
   
   /** Method name <code>intValue</code> */
   public static final String METH_INTVALUE = "intValue";
   
   /** Method name <code>isUnknown</code> */
   public static final String METH_ISUNK = "isUnknown";
   
   /** Method name <code>min</code> */
   public static final String METH_MIN = "min";
   
   /** Method name <code>size</code> */
   public static final String METH_SIZE = "size";
   
   /** Method name <code>outOfBoundsError</code> */
   public static final String METH_OOBE = "outOfBoundsError";
   
   /** Method name <code>setUnknown</code> */
   public static final String METH_SETUNK = "setUnknown";
   
   /** Internal type name for Java <code>ArrayList</code> */
   public static final String TYPE_ARRAYLIST = "java/util/ArrayList";
   
   /** Internal type name for P2J <code>date</code> */
   public static final String TYPE_BINARYDATA = PKG_UTIL + "BinaryData";
   
   /** Internal type name for P2J <code>character</code> */
   public static final String TYPE_CHARACTER = PKG_UTIL + "character";
   
   /** Internal type name for P2J <code>date</code> */
   public static final String TYPE_DATE = PKG_UTIL + "date";
   
   /** Internal type name for P2J <code>datetime</code> */
   public static final String TYPE_DATETIME = PKG_UTIL + "datetime";
   
   /** Internal type name for P2J <code>datetimetz</code> */
   public static final String TYPE_DATETIMETZ = PKG_UTIL + "datetimetz";
   
   /** Internal type name for P2J <code>decimal</code> */
   public static final String TYPE_DECIMAL = PKG_UTIL + "decimal";
   
   /** Internal type name for P2J <code>handle</code> */
   public static final String TYPE_HANDLE = PKG_UTIL + "handle";
   
   /** Internal type name for P2J <code>int64</code> */
   public static final String TYPE_INT64 = PKG_UTIL + "int64";
   
   /** Internal type name for P2J <code>integer</code> */
   public static final String TYPE_INTEGER = PKG_UTIL + "integer";
   
   /** Internal type name for Java <code>List</code> */
   public static final String TYPE_LIST = "java/util/List";
   
   /** Internal type name for P2J <code>logical</code> */
   public static final String TYPE_LOGICAL = PKG_UTIL + "logical";
   
   /** Internal type name for P2J <code>longchar</code> */
   public static final String TYPE_LONGCHAR = PKG_UTIL + "longchar";
   
   /** Internal type name for P2J <code>memptr</code> */
   public static final String TYPE_MEMPTR = PKG_UTIL + "memptr";
   
   /** Internal type name for P2J <code>NumberType</code> */
   public static final String TYPE_NUMBERTYPE = PKG_UTIL + "NumberType";
   
   /** Internal type name for P2J <code>raw</code> */
   public static final String TYPE_RAW = PKG_UTIL + "raw";
   
   /** Internal type name for P2J <code>recid</code> */
   public static final String TYPE_RECID = PKG_UTIL + "recid";
   
   /** Internal type name for P2J <code>rowid</code> */
   public static final String TYPE_ROWID = PKG_UTIL + "rowid";
   
   /** Internal type name for P2J <code>Text</code> */
   public static final String TYPE_TEXT = PKG_UTIL + "Text";
   
   /** Internal type name for P2J <code>Undoable</code> */
   public static final String TYPE_UNDOABLE = PKG_UTIL + "Undoable";
   
   /** Internal type name for P2J <code>Temporary</code> */
   public static final String TYPE_TEMPORARY = PKG_PERSIST + "Temporary";
   
   /** Internal type name for P2J <code>Persistable</code> */
   public static final String TYPE_PERSISTABLE = PKG_PERSIST + "Persistable";
   
   /** Internal type name for P2J <code>BufferImpl</code> */
   public static final String TYPE_BUFFERIMPL = PKG_PERSIST + "BufferImpl";
   
   /** Internal type name for Java <code>Math</code> */
   public static final String TYPE_MATH = "java/lang/Math";
   
   /** Internal type name for Java <code>Object</code> */
   public static final String TYPE_OBJECT = "java/lang/Object";
   
   /** Internal type name for Java <code>Serializable</code> */
   public static final String TYPE_SERIALIZABLE = "java/io/Serializable";
   
   /** Internal type descriptor for Java <code>Integer</code> */
   public static final String DESC_JAVA_INTEGER = "Ljava/lang/Integer;";
   
   /** Internal type descriptor for Java <code>String</code> */
   public static final String DESC_JAVA_STRING = "Ljava/lang/String;";
   
   /** Internal type descriptor for Java <code>ArrayList</code> */
   public static final String DESC_ARRAYLIST = "L" + TYPE_ARRAYLIST + ";";
   
   /** Internal type descriptor for date */
   public static final String DESC_DATE = "L" + TYPE_DATE + ";";
   
   /** Internal type descriptor for Java <code>List</code> */
   public static final String DESC_LIST = "L" + TYPE_LIST + ";";
   
   /** Internal form of default (no-arg) constructor method signature */
   public static final String SIG_DEF_INIT = "()V";
   
   /** Internal form of <code>BaseDataType.isUnknown</code> method signature */
   public static final String SIG_ISUNK = "()Z";
   
   /** Internal form of <code>BaseDataType.setUnknown</code> method signature */
   public static final String SIG_SETUNK = "()V";
   
   /** Internal form of <code>BufferImpl.outOfBoundsError</code> method signature */
   public static final String SIG_OOBE = "(I)V";
   
   /** Internal form of <code>BufferImpl.checkExtentRange</code> method signature */
   public static final String SIG_CHKXRNG = "(II)Z";
   
   /** Internal form of Java <code>List.add</code> method signature */
   public static final String SIG_ADD = "(L" + TYPE_OBJECT + ";)Z";
   
   /** Internal form of Java <code>List.get</code> method signature */
   public static final String SIG_GET = "(I)L" + TYPE_OBJECT + ";";
   
   /** Internal form of Java <code>Math.min</code> method signature */
   public static final String SIG_MIN = "(II)I";
   
   /** Internal form of Java <code>List.size</code> method signature */
   public static final String SIG_SIZE = "()I";
   
   /** Internal form of signature for a constructor which takes one <code>int</code> argument */
   public static final String SIG_INT_INIT = "(I)V";
   
   /** Internal form of <code>intValue</code> method signature */
   public static final String SIG_INTVALUE = "()I";
   
   /** Internal form of P2J <code>Undoable.assign</code> method signature */
   public static final String SIG_DMO_ASSIGN = "(L" + TYPE_UNDOABLE + ";)V";
   
   /** Name of the temp-table multiplex ID field */
   public static final String FLD_MULTIPLEX = "_multiplex";
}