/*
** Module   : MetaTableRelation.java
** Abstract : Representation of a database relation between two metaschema tables
**
** Copyright (c) 2013-2017, Golden Code Development Corporation.
**
** -#- -I- --Date-- --------------------------------Description----------------------------------
** 001 ECF 20131215 Created initial version.
*/
/*
** This program is free software: you can redistribute it and/or modify
** it under the terms of the GNU Affero General Public License as
** published by the Free Software Foundation, either version 3 of the
** License, or (at your option) any later version.
**
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU Affero General Public License for more details.
**
** You may find a copy of the GNU Affero GPL version 3 at the following
** location: https://www.gnu.org/licenses/agpl-3.0.en.html
** 
** Additional terms under GNU Affero GPL version 3 section 7:
** 
**   Under Section 7 of the GNU Affero GPL version 3, the following additional
**   terms apply to the works covered under the License.  These additional terms
**   are non-permissive additional terms allowed under Section 7 of the GNU
**   Affero GPL version 3 and may not be removed by you.
** 
**   0. Attribution Requirement.
** 
**     You must preserve all legal notices or author attributions in the covered
**     work or Appropriate Legal Notices displayed by works containing the covered
**     work.  You may not remove from the covered work any author or developer
**     credit already included within the covered work.
** 
**   1. No License To Use Trademarks.
** 
**     This license does not grant any license or rights to use the trademarks
**     Golden Code, FWD, any Golden Code or FWD logo, or any other trademarks
**     of Golden Code Development Corporation. You are not authorized to use the
**     name Golden Code, FWD, or the names of any author or contributor, for
**     publicity purposes without written authorization.
** 
**   2. No Misrepresentation of Affiliation.
** 
**     You may not represent yourself as Golden Code Development Corporation or FWD.
** 
**     You may not represent yourself for publicity purposes as associated with
**     Golden Code Development Corporation, FWD, or any author or contributor to
**     the covered work, without written authorization.
** 
**   3. No Misrepresentation of Source or Origin.
** 
**     You may not represent the covered work as solely your work.  All modified
**     versions of the covered work must be marked in a reasonable way to make it
**     clear that the modified work is not originating from Golden Code Development
**     Corporation or FWD.  All modified versions must contain the notices of
**     attribution required in this license.
*/

package com.goldencode.p2j.schema;

import java.util.*;
import com.goldencode.ast.*;

/**
 * A representation of a <em>primary-foreign key relation</em> between two Progress metaschema
 * tables. Relations between metaschema tables differ from relations between regular Progress
 * tables, in that traditional foreign keys are used.
 */
class MetaTableRelation
extends TableRelation
{
   /**
    * Constructor.
    * 
    * @param   left
    *          Left table. May not be <code>null</code>.
    * @param   right
    *          Right table. May not be <code>null</code>.
    * @param   key
    *          Set of names of the fields which comprise the join. In this case, the set will
    *          only contain a single value -- the name of the foreign key field.
    */
   public MetaTableRelation(Aast left, Aast right, Set<String> key)
   {
      super(left, right, right, key, MANY_TO_ONE);
   }
   
   /**
    * Get the database schema name for the given table AST.
    * <p>
    * This version override's the parent class to return the standard database qualifier for
    * metaschema tables:  <code>_meta</code>.
    * 
    * @param   table
    *          Target table AST.
    * 
    * @return  Name of the table's database schema.
    */
   @Override
   protected String getDatabase(Aast table)
   {
      return "_meta";
   }
   
   /**
    * Indicates whether this relation uses a real foreign key join (vs. a Progress-style, natural
    * join).
    * 
    * @return  <code>true</code>.
    */
   @Override
   protected boolean hasRealForeignKey()
   {
      return true;
   }
}