/*
** Module   : Binding.java
** Abstract : Represents associations between a rights and list of subjects.
**
** Copyright (c) 2004-2017, Golden Code Development Corporation.
**
** -#- -I- --Date-- -T- --JPRM-- ----------------Description-----------------
** 001 NVS 20050318 NEW   @20445 Created the initial version. This class 
**                               represents associations between a rights 
**                               object and a list of subjects. 
** 002 NVS 20060412 CHG   @25494 Made the class and some getters public. 
** 003 NVS 20090415 CHG   @41802 Binding now keeps directory object IDs of the
**                               objects it was constructed from.
** 004 NVS 20090624 CHG   @42841 Simplified due to move to new ACLs.
*/ 
/*
** This program is free software: you can redistribute it and/or modify
** it under the terms of the GNU Affero General Public License as
** published by the Free Software Foundation, either version 3 of the
** License, or (at your option) any later version.
**
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU Affero General Public License for more details.
**
** You may find a copy of the GNU Affero GPL version 3 at the following
** location: https://www.gnu.org/licenses/agpl-3.0.en.html
** 
** Additional terms under GNU Affero GPL version 3 section 7:
** 
**   Under Section 7 of the GNU Affero GPL version 3, the following additional
**   terms apply to the works covered under the License.  These additional terms
**   are non-permissive additional terms allowed under Section 7 of the GNU
**   Affero GPL version 3 and may not be removed by you.
** 
**   0. Attribution Requirement.
** 
**     You must preserve all legal notices or author attributions in the covered
**     work or Appropriate Legal Notices displayed by works containing the covered
**     work.  You may not remove from the covered work any author or developer
**     credit already included within the covered work.
** 
**   1. No License To Use Trademarks.
** 
**     This license does not grant any license or rights to use the trademarks
**     Golden Code, FWD, any Golden Code or FWD logo, or any other trademarks
**     of Golden Code Development Corporation. You are not authorized to use the
**     name Golden Code, FWD, or the names of any author or contributor, for
**     publicity purposes without written authorization.
** 
**   2. No Misrepresentation of Affiliation.
** 
**     You may not represent yourself as Golden Code Development Corporation or FWD.
** 
**     You may not represent yourself for publicity purposes as associated with
**     Golden Code Development Corporation, FWD, or any author or contributor to
**     the covered work, without written authorization.
** 
**   3. No Misrepresentation of Source or Origin.
** 
**     You may not represent the covered work as solely your work.  All modified
**     versions of the covered work must be marked in a reasonable way to make it
**     clear that the modified work is not originating from Golden Code Development
**     Corporation or FWD.  All modified versions must contain the notices of
**     attribution required in this license.
*/

package com.goldencode.p2j.security;

/**
 * Associates an instance of Rights interface with an array of numeric subject
 * IDs. These bindings are building blocks for the access control lists.
 */
public class Binding
{
   /** instance of the Rights interface */
   private Rights rights = null;

   /** array of numeric subject IDs */
   private int[] subjects = null;

   /**
    * Package private constructor. 
    *
    * @param rights
    *        an instance of the Rights interface 
    *
    * @param subjects
    *        array of numeric subject IDs 
    */
   Binding(Rights rights, int[] subjects)
   {
      this.rights   = rights;
      this.subjects = subjects;
   }
   
   /**
    * Gets the embedded Rights object.
    *
    * @return embedded Rights object
    */
   public Rights getRights()
   {
      return rights;
   }
   
   /**
    * Gets the embedded subjects.
    *
    * @return embedded subjects
    */
   public int[] getSubjects()
   {
      return subjects;
   }

   /**
    * Checks whether this binding matches the specified subject ID.
    *
    * @param  sid
    *         numeric subject ID to be checked
    *
    * @return <code>true</code> if it does
    */
   public boolean matches(int sid)
   {
      for (int i = 0; i < subjects.length; i ++)
         if (subjects[i] == sid || subjects[i] == -2)
            return true;

      return false;
   }
}