/*
** Module   : Cleanable.java
** Abstract : Defines an interface for tokens in the security context that
**            implement finalization.
**
** Copyright (c) 2004-2017, Golden Code Development Corporation.
**
** -#- -I- --Date-- --JPRM-- ------------------Description-------------------
** 001 NVS 20050504   @21069 Created the initial version.
** 002 OM  20121016          Added comment description about how to avoid 
**                           issue #1577.
** 003 EVL 20160222          Javadoc fixes to make compatible with Oracle Java 8 for Solaris 10.
*/ 
/*
** This program is free software: you can redistribute it and/or modify
** it under the terms of the GNU Affero General Public License as
** published by the Free Software Foundation, either version 3 of the
** License, or (at your option) any later version.
**
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU Affero General Public License for more details.
**
** You may find a copy of the GNU Affero GPL version 3 at the following
** location: https://www.gnu.org/licenses/agpl-3.0.en.html
** 
** Additional terms under GNU Affero GPL version 3 section 7:
** 
**   Under Section 7 of the GNU Affero GPL version 3, the following additional
**   terms apply to the works covered under the License.  These additional terms
**   are non-permissive additional terms allowed under Section 7 of the GNU
**   Affero GPL version 3 and may not be removed by you.
** 
**   0. Attribution Requirement.
** 
**     You must preserve all legal notices or author attributions in the covered
**     work or Appropriate Legal Notices displayed by works containing the covered
**     work.  You may not remove from the covered work any author or developer
**     credit already included within the covered work.
** 
**   1. No License To Use Trademarks.
** 
**     This license does not grant any license or rights to use the trademarks
**     Golden Code, FWD, any Golden Code or FWD logo, or any other trademarks
**     of Golden Code Development Corporation. You are not authorized to use the
**     name Golden Code, FWD, or the names of any author or contributor, for
**     publicity purposes without written authorization.
** 
**   2. No Misrepresentation of Affiliation.
** 
**     You may not represent yourself as Golden Code Development Corporation or FWD.
** 
**     You may not represent yourself for publicity purposes as associated with
**     Golden Code Development Corporation, FWD, or any author or contributor to
**     the covered work, without written authorization.
** 
**   3. No Misrepresentation of Source or Origin.
** 
**     You may not represent the covered work as solely your work.  All modified
**     versions of the covered work must be marked in a reasonable way to make it
**     clear that the modified work is not originating from Golden Code Development
**     Corporation or FWD.  All modified versions must contain the notices of
**     attribution required in this license.
*/

package com.goldencode.p2j.security;

/**
 * Defines an interface for tokens in the security context that
 * implement finalization.
 * <p>
 * This interface defines just one method: cleanup(). Tokens that need
 * finalization should be instances of a class that implements this interface.
 * <p>
 * When it is time to finalize the token, its cleanup() method is called.
 *  
 * @author NVS
 * @version 1.0
 */
public interface Cleanable
{
   /**
    * The only method declared by the interface. It is called when it is time 
    * to finalize the token.  
    * <p>
    * <strong>Note</strong><br>
    * Do not use <code>ContextLocal</code> references in the cleanup() method 
    * implementation as, at the time of this call, the context-local variables 
    * are already deleted from the SecurityContext (by the 
    * <code>SecurityContext.cleanup()</code> code).
    */
   public void cleanup();
}