/*
** Module   : ButtonImageDefinition.java
** Abstract : stores button specific image definition
**
** Copyright (c) 2015-2017, Golden Code Development Corporation.
**
** -#- -I- --Date-- ---------------------------------Description---------------------------------
** 001 EVL 20150206 Created initial version.
** 002 HC  20161018 Fixed abends in image processing and drawing when image offset was involved.
*/
/*
** This program is free software: you can redistribute it and/or modify
** it under the terms of the GNU Affero General Public License as
** published by the Free Software Foundation, either version 3 of the
** License, or (at your option) any later version.
**
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU Affero General Public License for more details.
**
** You may find a copy of the GNU Affero GPL version 3 at the following
** location: https://www.gnu.org/licenses/agpl-3.0.en.html
** 
** Additional terms under GNU Affero GPL version 3 section 7:
** 
**   Under Section 7 of the GNU Affero GPL version 3, the following additional
**   terms apply to the works covered under the License.  These additional terms
**   are non-permissive additional terms allowed under Section 7 of the GNU
**   Affero GPL version 3 and may not be removed by you.
** 
**   0. Attribution Requirement.
** 
**     You must preserve all legal notices or author attributions in the covered
**     work or Appropriate Legal Notices displayed by works containing the covered
**     work.  You may not remove from the covered work any author or developer
**     credit already included within the covered work.
** 
**   1. No License To Use Trademarks.
** 
**     This license does not grant any license or rights to use the trademarks
**     Golden Code, FWD, any Golden Code or FWD logo, or any other trademarks
**     of Golden Code Development Corporation. You are not authorized to use the
**     name Golden Code, FWD, or the names of any author or contributor, for
**     publicity purposes without written authorization.
** 
**   2. No Misrepresentation of Affiliation.
** 
**     You may not represent yourself as Golden Code Development Corporation or FWD.
** 
**     You may not represent yourself for publicity purposes as associated with
**     Golden Code Development Corporation, FWD, or any author or contributor to
**     the covered work, without written authorization.
** 
**   3. No Misrepresentation of Source or Origin.
** 
**     You may not represent the covered work as solely your work.  All modified
**     versions of the covered work must be marked in a reasonable way to make it
**     clear that the modified work is not originating from Golden Code Development
**     Corporation or FWD.  All modified versions must contain the notices of
**     attribution required in this license.
*/

package com.goldencode.p2j.ui;

import java.io.*;

/**
 * This works like a public data structure to store all image related options for up or down or
 * insensitive state of the button widget.  Used to unify option access. 
 */
public class ButtonImageDefinition
implements Externalizable
{
   /** X offset in pixel units inside the image data to start drawing image. */
   public int offsetX = 0;
   
   /** Y offset in pixel units inside the image data to start drawing image. */
   public int offsetY = 0;
   
   /** Row offset in character units inside the image data to start drawing image. */
   public double offsetRow = 0.0;
   
   /** Column offset in character units inside the image data to start drawing image. */
   public double offsetCol = 0.0;

   /** Image width in pixel units. */   
   public int widthPixels = 0;
   
   /** Image width in character units. */   
   public double widthChars = 0.0;
   
   /** Image height in pixel units. */   
   public int heightPixels = 0;
   
   /** Image height in character units. */   
   public double heightChars = 0.0;

   /** Button image file name. */
   public String name = null;

   /**
    * Default constructor (only used in deserialization).
    */
   public ButtonImageDefinition()
   {
   }

   /**
    * Replacement for the default object reading method. The latest state is read from the input
    * source.
    * 
    * @param    in
    *           The input source from which fields will be restored.
    *
    * @throws   IOException
    *           In case of I/O errors.
    * @throws   ClassNotFoundException
    *           If payload can't be instantiated.
    */
   @Override
   public void readExternal(ObjectInput in)
   throws IOException,
          ClassNotFoundException
   {
      offsetX      = in.readInt();
      offsetY      = in.readInt();
      offsetRow    = in.readDouble();
      offsetCol    = in.readDouble();
      widthPixels  = in.readInt();
      widthChars   = in.readDouble();
      heightPixels = in.readInt();
      heightChars  = in.readDouble();
      name         = (String) in.readObject();
   }

   /**
    * Replacement for the default object writing method. The latest state is written to the output
    * destination.
    * 
    * @param    out
    *           The output destination to which fields will be saved.
    *
    * @throws   IOException
    *           In case of I/O errors.
    */
   @Override
   public void writeExternal(ObjectOutput out)
   throws IOException
   {
      out.writeInt(offsetX);
      out.writeInt(offsetY);
      out.writeDouble(offsetRow);
      out.writeDouble(offsetCol);
      out.writeInt(widthPixels);
      out.writeDouble(widthChars);
      out.writeInt(heightPixels);
      out.writeDouble(heightChars);
      out.writeObject(name);
   }
}