/*
** Module   : ClientExports.java
** Abstract : interface which describes all UI-related API's exported
**            by client for use by server  
**
** Copyright (c) 2005-2018, Golden Code Development Corporation.
**
** -#- -I- --Date-- --JPRM-- ---------------------------Description----------------------------
** 001 SIY 20051130   @23585 Created initial version.
** 002 SIY 20051205   @24098 Changed signature for enable() and removed disable().
** 003 NVS 20051221   @24099 Added isChui() implementation to the client exports.
** 004 NVS 20060111   @24100 apply() now returns ScreenBuffer to server.
** 005 NVS 20060111   @24101 Added getActiveFrame() export.
** 006 NVS 20060114   @24102 Extended hide() functionality.
** 007 GES 20060117   @23952 PAUSE rework and fixes.
** 008 NVS 20060130   @24145 Client exports returning screen buffer, now
**                           take screen buffer as well. These are:
**                           waitFor(), apply().
** 009 SIY 20060130   @24153 Added pauseBeforeEnd() exported method.
** 010 SIY 20060201   @24189 Changed message() signature.
** 011 SIY 20060210   @24510 Added getSelection() method.
** 012 SIY 20060213   @24580 Added block tracking exports.
** 013 SIY 20060215   @24599 Changed messageBox() signature.
** 014 SIY 20060218   @24629 Changed signature of choose(), message() and messageBox() methods.
** 015 GES 20060214   @24682 Stream redirection and some general cleanup.
** 016 GES 20060226   @24757 Header and footer support.
** 017 NVS 20060302   @24850 processEvents() export now takes and returns the ScreenBuffer.
** 018 GES 20060322   @25185 Added helpers for terminal type processing.
** 019 GES 20060510   @26058 New flag passed on blockIterate().
** 020 SIY 20060517   @26231 Added blockRetry() method.
** 021 GES 20060522   @26437 Added stopEditingMode() method.
** 022 GES 20060531   @26813 Signature change to pass an additional flag
**                           to enable() to force focus change.
** 023 GES 20060614   @27189 Converted the status related signatures to
**                           String parms instead of character.
** 024 GES 20060623   @27555 Changed signatures for pause() and message()
**                           to allow the return of a key code so that
**                           LASTKEY can be properly maintained.
** 025 GES 20060627   @27604 Signature change to pass a screen buffer
**                           to enable() to allow truly dynamic down
**                           frame sizing.
** 026 GES 20060630   @27734 Signature change for some getters.
** 027 SIY 20060720   @28141 Removed getFrameIndex() and getFrameValue().
** 028 SIY 20060725   @28180 Added markForHide() method.
** 029 SIY 20060726   @28232 Changes for the new frame hiding approach.
** 030 GES 20060807   @28448 Moved message list content processing to
**                           server to reduce traffic over the protocol.
** 031 SIY 20060807   @28457 Removed markFrames() since it is now part of
**                           "out-of-bound" messaging.
** 032 GES 20060808   @28484 Shifted APIs to use Java primitives instead
**                           of the Progress compatible wrappers, where
**                           possible. Made some fixes to messageBox()
**                           to add missing function.  Major javadoc
**                           improvements.
** 033 NVS 20061005   @30165 refreshFrameWidget() export replaced the old setFrameValue().
** 034 NVS 20061017   @30466 pauseBeforeEnd() takes no parameter.
** 035 GES 20061030   @30787 Removed consumeEvent().
** 036 GES 20061102   @30890 Added forceDeregisterPageElement().
** 037 SIY 20061105   @30984 Changed signatures of Browse related methods.
**                           Added refreshStartRow() method.
** 038 GES 20061114   @31123 Enable apply to update the list of the
**                           currently active triggers.  This is
**                           especially important in the case where apply
**                           is used outside of a wait-for.
** 039 GES 20061127   @31408 Added a shared flag to destroyFrame() to
**                           ensure that more deeply nested frames don't
**                           exit scope too soon.
** 040 GES 20061206   @31594 Added promptFor() helper.
** 041 GES 20061211   @31661 Added versions of client exports which take
**                           a stream ID and which implement an inline
**                           switch of the output destination to that ID
**                           and then a restoration of the previous
**                           destination back before returning.
** 042 NVS 20070118   @31885 EventList class is used instead of WaitList.
** 043 SIY 20070119   @31901 Removed separate handling of shared frames.
** 044 NVS 20070426   @33242 Added version of enable() export and
**                           isEnabled() export to back up SENSITIVE flag
**                           functionality on the client.
** 045 NVS 20070716   @34536 Changed signature of the view() method.
** 046 NVS 20070723   @34675 Same change made to the signature of another
**                           view() method - added viewDisp parameter.
**                           Now the nature of the call, VIEW or DISPLAY,
**                           is known, not guessed.
** 047 NVS 20070830   @34978 down() calls to the client now pass the flag
**                           that tells conditional downs.
** 048 ECF 20080208   @37030 Added getSelectedRowIndex(). Gets the row
**                           index of a selected row in a browse widget.
** 049 NVS 20080307   @37408 Added setLocation() for dynamic ROW and COL
**                           widget attributes support.
** 050 NVS 20080320   @37619 enable() export has changed signature again.
** 051 NVS 20080326   @37685 startEditingMode() export has been removed
**                           and its functionality is now part of the
**                           modified promptFor() export.
** 052 GES 20080421   @38070 Added setFormat().
** 053 NVS 20080423   @38100 choose() export has changed signature.
** 054 GES 20080501   @38185 Added parameters to allow multiple calls to
**                           be consolidated into a single call.
** 055 CA  20080905   @39740 Added missing getter for VISIBLE attribute.
** 056 CA  20080915   @39807 getFrameLine was changed to recieve the
**                           pendingUpDown value as a parameter.
** 057 CA  20080930   @39989 Added getters for FRAME:COL/ROW and
**                           FRAME:WIDTH/HEIGHT-CHARS attributes.
** 058 CA  20081204   @40814 Added exports needed for implementing the
**                           browse ROW-DISPLAY events.
** 059 SVL 20081208   @40824 Changed signature of the message() method.
** 060 CA  20081210   @40843 Implemented CLOSE QUERY statement - on close,
**                           the browse widget will be notified to reset
**                           its state.
** 061 SIY 20081224   @40991 Added hideWindow() method.
** 062 SIY 20090213   @41311 Added 'force' parameter into refreshStartRow
**                           signature.
** 063 CA  20090326   @41723 Modified apply(...) signature to identify an
**                           APPLY call with or without explicit target.
** 064 SVL 20090612   @42677 Added resetBufferSizeManager() method.
** 065 CA  20091111   @44327 Added a statusDefault(String, boolean) export 
**                           which disables 63-char trimming, if the
**                           noLengthLimit flag is set.
** 066 SIY 20110112          Changes related to new color handling scheme.
** 067 SIY 20111101          Updated imports.
** 068 CA  20130822          Added support of appserver async requests. The changes are related to
**                           allow posting server events to the client, which will be processed
**                           on the next PROCESS EVENTS or UI-blocking statement.
** 069 CA  20130908          Added API to remove certain server events from the queue.
** 070 CA  20131001          Added API to change the numeric separators on client-side.
** 071 EVL 20131127          Adding support for batch mode.
** 072 EVL 20140115          Adding support for batch mode flag query.
** 073 VIG 20140601          Adding tab order management methods: moveBeforeTabItem,  
**                           getFirstTabItem, setFirstTabItem, getNextTabItem.
** 074 VIG 20140612          Changing signatures and adding java doc to methods
**                           moveBeforeTabItem and getFirstTabItem.
** 075 CA  20140711          Enhanced font support, for GUI clients.
** 076 VIG 20140815          Added windowId for IN WINDOW clause 
**                           to methods pause and pauseBeforeHide.
** 077 MAG 20140812          Implements COLOR-TABLE, for GUI clients.
** 078 HC  20140911          GUI coordinate support - data types refactoring.
** 079 CA  20140926          Added shared widget configuration support, to allow GUI/ChUI
**                           concrete implementation for the same widget ID.  Refactored the
**                           widget configuration classes: all fields were made public; these
**                           classes need to be as dumb as possible, all logic related to
**                           setting/getting a certain field should be at the widget or at the
**                           caller. Refs #2254
** 080 VIG 20140911          Added status* and message* methods for cases when IN WINDOW clause 
**                           is used.
** 081 CA  20141106          Added window ID to promptFor API.
** 082 CA  20141119          Refactored drawing code to enable multiple OS windows, for GUI
**                           support.  Misc changes/fixes related to drawing on multiple windows.
** 083 SVL 20141209          Declared some browse-related functions.
** 084 HC  20141207          Implemented runtime support for several SESSION attributes.
** 085 EVL 20141216          Adding SESSION:TOOLTIPS attribute handling.
** 086 GES 20150104          Added DELETE-LINE() method support.
** 087 HC  20150111          Fixed widget position change in a frame.
** 088 EVL 20150114          Adding API to set/load window icon.
** 089 CA  20150122          Added API to apply event to a non-widget resource.
** 090 HC  20150128          Added support for frame widget position changes induced by
**                           setting ROW/COL/X/Y attributes.
** 091 GES 20150204          Clear GO-PENDING when a NEXT statement is used in an EDITING block.
** 092 VIG 20150128          Added new method pushMenuDescription and destroyMenu.
** 093 SVL 20150212          Modified refreshStartRow signature.
** 094 CA  20150224          APIs which make the frame visible must return the frame's buffer.
** 095 SVL 20150305          Updated refresh() signature.
** 096 CA  20150305          CLEAR and CLEAR ALL APIs must update the frame's screen buffer.
** 097 EVK 20150310          Added SAVE-FILE and READ-FILE support for editor. 
** 098 SVL 20150331          Added some browse-related functions. refreshStartRow become
**                           refreshScrollRow with more options.
** 099 IAS 20150401          Added support for new EDITOR attributes/methods
** 100 SVL 20150411          Added select{Next|Previous}Row.
** 101 HC  20150323          Removed business logic from config classes. 
** 102 SVL 20150420          Added executePendingRefresh, notifyColumnsUpdated.
** 103 HC  20150508          Improvements to GUI window scrolling support.
** 104 SVL 20150508          Added isNewRow, insertRow, isCurrentRowModified.
** 105 IAS 20150623          Added 'debug' method.
** 106 CA  20150722          APIs which affect the row in a down frame (like down and 
**                           pushScreenDefinition, when there is a pending conditional down) need
**                           to return the widget configs for this current iteration, so that the
**                           server side configuration can be in sync, too.
** 107 SVL 20150722          Updated browse-related API.
** 108 CA  20150725          ClientExports.view APIs now return the frame's down configs, too.
** 109 EVL 20150826          Adding support for SCROLL-TO-ITEM() method for selection-list.
** 110 IAS 20150922          Adding support for MOVE-TO-TOP()/MOVE-TO-BOTTOM() methods.
** 111 CA  20151027          Added MOVE-AFTER-TAB-ITEM().
**     SVL 20151029          Added queryRepositioned.
**     CA  20151106          A fix for HIDDEN attribute: distinguish between HIDE statement and 
**                           setting the HIDDEN attribute to true.
** 112 HC  20151213          Improved DIALOG-BOX window parenting logic and related changes.
** 113 EVL 20160224          Javadoc fixes to make compatible with Oracle Java 8 for Solaris 10.
** 114 IAS 20160229          LOAD-MOUSE-POINTER and SET-WAIT-STATE support
** 115 IAS 20160302          Support for the APPL-ALERT-BOXES, DATA-ENTRY-RETURN, 
**                           and IMMEDIATE-DISPLAY SESSION attributes
** 116 IAS 20160314          DEBUG-ALERT support.
** 117 IAS 20160328          Client log support
** 118 SVL 20160406          Added moveColumn.
** 119 SBI 20160412          Added openURL(String path) to open the target page in the browser.
**     CA  20160418          Added infrastructure for message passing from converted code to a 
**                           customer-specific application or calls from the customer-specific 
**                           application into the converted code (i.e. invoke external programs,
**                           procedures or functions).  Can be used only when the P2J client runs 
**                           embedded in a customer-specific application.
** 120 SVL 20160419          moveColumn renamed to moveColumnClient.
** 121 SBI 20160516          Added exports for CLIPBOARD: getClipboardValue and setClipboardValue.
** 122 EVL 20160726          Changing usage of the screen buffers, now we process all editaable
**                           frame's screen buffers instead of the single current.
** 123 IAS 20160829          displayAndDown() operation support
** 124 HC  20160901          Error messages in GUI are always displayed in alert boxes.
** 125 CA  20170228          Added PUBLISH/SUBSCRIBE/UNSUBSCRIBE extensions for global support and
**                           for external applications.
** 126 OM  20170320          Added support for Themes.
** 127 OM  20170504          Support for MOVE-TO-EOF, SEARCH and INSERT-FILE for editor widgets.
** 128 EVL 20170609          Adding support for dropped file list handling.
** 129 SVL 20170813          Added scrollToCurrent(Selected)Row, calcBrowseDown,
**                           pushIntermediateRows.
** 130 OM  20170720          Updated chooseFont() signature.
** 131 HC  20170819          Added extension widget method GET-MOUSE-POSITION.
** 132 HC  20171003          Implemented SESSION:SUPPRESS-WARNINGS runtime support and the
**                           ability to handle (display or output depending on the runtime state)
**                           legacy warnings.
** 133 SBI 20161101          Added a tab item list as a parameter to the enable, view and display,
**                           displayAndDown, promptFor methods provided by ClientExports.
** 134 HC  20171024          Runtime support for SYSTEM-DIALOG PRINT-SETUP and OUTPUT TO PRINTER.
** 135 SVL 20171024          Added displayError(errmsg, noMessageBox) and
**                           displayErrorMessage(errmsg, noMessageBox).
** 136 OM  20171027          The minimum and maximum size font parrameters are now supported by
**                           chooseFont(). chooseColor() returns the selected RGB.
** 137 SVL 20171204          Added refreshCurrentRow.
** 138 CA  20180130          Replaced pushScreenDefinition() with pushWidgetAttr() for cases when
**                           business logic is assigning a widget attribute.
** 139 SBI 20180402          Added openMimeResource. 
** 140 CA  20180313          Added FWD extension SESSION:DISABLE-REDRAW.
** 141 CA  20180529          Added placeFrame.
** 142 EVL 20180621          Added pulse flag to inform client the frame needs pulse on close.
** 143 CA  20180710          Removed destroyMenu.
** 144 SVL 20180528          Added applyEnhancedConfigClient().
*/

/*
** This program is free software: you can redistribute it and/or modify
** it under the terms of the GNU Affero General Public License as
** published by the Free Software Foundation, either version 3 of the
** License, or (at your option) any later version.
**
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU Affero General Public License for more details.
**
** You may find a copy of the GNU Affero GPL version 3 at the following
** location: https://www.gnu.org/licenses/agpl-3.0.en.html
** 
** Additional terms under GNU Affero GPL version 3 section 7:
** 
**   Under Section 7 of the GNU Affero GPL version 3, the following additional
**   terms apply to the works covered under the License.  These additional terms
**   are non-permissive additional terms allowed under Section 7 of the GNU
**   Affero GPL version 3 and may not be removed by you.
** 
**   0. Attribution Requirement.
** 
**     You must preserve all legal notices or author attributions in the covered
**     work or Appropriate Legal Notices displayed by works containing the covered
**     work.  You may not remove from the covered work any author or developer
**     credit already included within the covered work.
** 
**   1. No License To Use Trademarks.
** 
**     This license does not grant any license or rights to use the trademarks
**     Golden Code, FWD, any Golden Code or FWD logo, or any other trademarks
**     of Golden Code Development Corporation. You are not authorized to use the
**     name Golden Code, FWD, or the names of any author or contributor, for
**     publicity purposes without written authorization.
** 
**   2. No Misrepresentation of Affiliation.
** 
**     You may not represent yourself as Golden Code Development Corporation or FWD.
** 
**     You may not represent yourself for publicity purposes as associated with
**     Golden Code Development Corporation, FWD, or any author or contributor to
**     the covered work, without written authorization.
** 
**   3. No Misrepresentation of Source or Origin.
** 
**     You may not represent the covered work as solely your work.  All modified
**     versions of the covered work must be marked in a reasonable way to make it
**     clear that the modified work is not originating from Golden Code Development
**     Corporation or FWD.  All modified versions must contain the notices of
**     attribution required in this license.
*/

package com.goldencode.p2j.ui;

import java.util.*;

import com.goldencode.p2j.ui.client.*;
import com.goldencode.p2j.ui.client.event.*;
import com.goldencode.p2j.util.*;

/**
 * Describes all UI-related API's exported by client for use by server.
 * <p>
 * This API is designed to maximize the work done on the server side. It
 * is intentionally biased against sending complex data structures to the
 * client, relying instead on passing the most simple form of data possible
 * to get the correct result.  In addition, the amount of data passed to the
 * client is minimized where possible.  Java primitives are used in 
 * preference to the Progress compatible wrapper classes.  All of these 
 * design policies are required to maximize the performance of the remote
 * UI as well as to keep the remote UI implementation as simple as possible.
 */
public interface ClientExports
{
   /**
    * Rings a bell on the terminal.
    * <p>
    * This method corresponds to the Progress statement:
    * <p>
    * <pre>
    * BELL
    * </pre>
    */
   public void bell();

   /**
    * Returns the terminal type.  This usually corresponds to the value
    * set as the <code>TERM</code> environment variable on UNIX or Linux
    * unless set via {@link #setTerminalType}.  On Windows it depends upon
    * whether the terminal is in graphical or character mode.  If it is
    * in character mode, the result depends on the monitor type.
    * <p>
    * This method corresponds to the Progress built-in function:
    * <p>
    * <pre>
    * TERMINAL
    * </pre>
    * 
    * @return   Terminal type for in this session.
    */
   public String getTerminalType();
   
   /**
    * Hides the specified widget. If pausing is in effect, the standard 
    * message "Press space bar to continue" is displayed on the status line 
    * and the user is required to press a key.
    * <p>
    * This method corresponds to the Progress statement <br>
    * <b>HIDE [widget] IN WINDOW wnd</b>
    * 
    * @param   widgetId
    *          ID of frame or a contained widget
    * @param   noPause
    *          <code>true</code> if no pause is required before hiding
    *          messages
    * @param   hidden
    *          If <code>true</code>, set the HIDDEN attribute.
    */
   public boolean hide(int widgetId, boolean noPause, boolean hidden);

   /**
    * Hides all widgets and messages displayed in the message area. If pausing
    * is in effect, the standard message "Press space bar to continue" is
    * displayed on the status line and the user is required to press a key.
    * <p>
    * This method corresponds to the Progress statement <br>
    * <b>HIDE ALL [NO_PAUSE] IN WINDOW wnd</b>
    * 
    * @param   noPause
    *          <code>true</code> if no pause is required before hiding
    *          messages
    * @param   windowId
    *          Id of target window or {@link WindowConfig#RESOLVE_WINDOW} for 
    *          current or default window.
    */
   public void hideAll(boolean noPause, int windowId);

   /**
    * Hides all messages displayed in the message area. If pausing is in
    * effect, the standard message "Press space bar to continue" is displayed
    * on the status line and the user is required to press a key.
    * <p>
    * This method corresponds to the Progress statement <br>
    * <b>HIDE MESSAGE [NO_PAUSE] IN WINDOW wnd</b>
    * 
    * @param   noPause
    *          <code>true</code> if no pause is required before hiding
    *          messages
    * @param   windowId
    *          Id of target window or {@link WindowConfig#RESOLVE_WINDOW} for 
    *          current or default window.
    */
   public void hideMessage(boolean noPause, int windowId);

   /**
    * Clears the type-ahead buffer if the input source is the keyboard.
    * <p>
    * This method corresponds to the Progress statement <br>
    * <b>INPUT CLEAR </b>
    */
   public void inputClear();

   /**
    * Checks if the client is running in batch mode.
    * 
    * @return  <code>true</code> for batch mode, <code>false</code> otherwise.
    * 
    */
   public boolean isInBatchMode();

   /**
    * Checks if the client is running in batch background mode.
    * 
    * @return  <code>true</code> for background mode, <code>false</code> otherwise.
    * 
    */
   public boolean isBatchInBackground();

   /**
    * Reports if the terminal is currently redirected to a stream.
    *
    * @return   <code>true</code> if the terminal is redirected to a stream,
    *           <code>false</code> if the terminal is interactive.
    */
   public boolean isRedirected();
   
   /**
    * Stops the associated client session by generating UnstoppableExitException.
    */
   public void stopBatchSession();

   /**
    * Changes the activation flag for batch mode driver if loaded.
    *
    * @param   on
    *          The new satus of the active flag for batch driver.
    */
   public void activateBatchMode(boolean on);
   
   /**
    * Delete the line of text in which the targeted editor's cursor currently resides.  Leave
    * the cursor at the leftmost position of the same line number (which will be the line
    * previously below this one if the cursor is not already on the last line) or at the
    * leftmost position of the last line (if there are no lines below this one).
    *
    * @param    id
    *           Widget id of the target editor.
    *
    * @return   <code>true</code> if the line was deleted.
    */
   public Boolean deleteEditorLine(int id);
   
   /**
    * Removes the currently selected text from an Editor, Fill-In or ComboBox.
    *
    * @param   id
    *          Widget id of the target editor.
    */
   public void removeEditSelection(int id);
  
   /**
    * Selects text in the targeted editor between start and end points.
    * 
    * @param    id
    *           Widget id of the target editor.
    * @param    start
    *           Starting offset.
    * @param    end
    *           Ending offset.
    *
    * @return   <code>true</code> if the operation succeeded.
    */
   public Boolean setEditorSelection(int id, int start, int end);
   
   /**
    * Sets the cursor line of the targeted editor
    *
    * @param    id
    *           Widget id of the target editor.
    * @param    line
    *           the new cursor line value
    *           
    * @return   <code>true</code> if the cursor line is valid and set, 
    *           null if no widget with given id is found
    */
   public Boolean setEditorCursorLine(int id, int line);

   /**
    * Sets the cursor position in the current line of the targeted editor
    *
    * @param    id
    *           Widget id of the target editor.
    * @param    pos
    *           the new cursor position value
    *           
    * @return   <code>true</code> if the cursor position is valid and set, 
    *           null if no widget with given id is found
    */
   public Boolean setEditorCursorChar(int id, int pos);

   /**
    * Sets the cursor offset of the targeted editor.
    *
    * @param   id
    *          Widget id of the target editor.
    *
    * @param   offset
    *          the new offset value. Use -1 to move the caret at the end (MOVE-TO-EOF). 
    *
    * @return  {@code true} if the cursor offset is valid and set, {@code null} if no widget 
    *          with given {@code id} is found.
    */
   public Boolean setEditorCursorOffset(int id, int offset);

   /**
    * Inserts string at the current cursor position of the targeted editor.
    *
    * @param    id
    *           Widget id of the target editor.
    * @param    txt
    *           The string to be inserted.
    *
    * @return  {@code true} if the string is inserted, {@code null} if no widget with given
    *          {@code id} is found.
    */
   public Boolean insertEditorString(int id, String txt);
   
   /**
    * Inserts the content of a file at the current cursor position of the targeted editor.
    *
    * @param   id
    *          Widget id of the target editor. Full or relative path may be used. In
    *          latter case, the {@code PROPATH} is used to find the file.
    * @param   filePath
    *          The file to be inserted.
    *
    * @return  {@code true} if the file content is inserted, {@code null} if no widget with given
    *          {@code id} is found.
    */
   public Boolean insertEditorFile(int id, String filePath);

   /**
    * Replace selection of the targeted editor with a given string.
    *
    * @param    id
    *           Widget id of the target editor.
    * @param    txt
    *           the replacement string
    *
    * @return   <code>true</code> if the selection is replaced, 
    *           null if no widget with given id is found
    */
   public Boolean replaceEditorSelectionText(int id, String txt);

   /**
    * Replace given substring of the targeted editor with a given string
    *
    * @param    id
    *           Widget id of the target editor.
    * @param    from
    *           the string to be replaced
    * @param    to
    *           the replacement string
    * @param    flag
    *           the replacement mode flag as described in 4GL EDITOR:REPLACE documentation 
    * 
    * @return   <code>true</code> if the replacement was performed, 
    *           null if no widget with given id is found
    */
   public Boolean replaceEditorString(int id, String from, String to, int flag);
   
   /**
    * Searches for a specified string starting from the current text caret position.
    *
    * @param   id
    *          Widget id of the target editor.
    * @param   pattern
    *          The string to be search. No wildcard allowed.
    * @param   mode
    *          a combinations of flags that specifies the type of search to be performed. The
    *          following values are accepted:
    *          <table summary="">
    *              <tr><th>Type of search</th><th>Flag value</th>
    *              <tr><td>EditorWidget.FIND_NEXT_OCCURRENCE</td><td>1</td></tr>
    *              <tr><td>EditorWidget.FIND_PREV_OCCURRENCE</td><td>2</td></tr>
    *              <tr><td>EditorWidget.FIND_CASE_SENSITIVE</td><td>4</td></tr>
    *              <tr><td>EditorWidget.FIND_WRAP_AROUND</td><td>16</td></tr>
    *              <tr><td>EditorWidget.FIND_SELECT</td><td>32</td></tr>
    *          </table>
    *
    * @return  if the text is found and caret moved, the method returns {@code yes}.
    */
   public Boolean searchInEditor(int id, String pattern, int mode);
   
   /**
    * Displays the error message via current client side error writer.  This does not use
    * the client UI code tools, usually meaning standard output channel and further client
    * termination.
    * <p>
    * The method takes care of rendering the error message directly and does not interact with
    * an error writer.
    *
    * @param   errmsg
    *          The error message to display
    * @param   noMessageBox
    *          <code>true</code> if error is always displayed in message area or logged rather
    *          than displayed as a message box.
    */
   public void displayError(String errmsg, boolean noMessageBox);

   /**
    * Displays the warning message via current client side error writer.  This does not use
    * the client UI code tools, usually meaning standard output channel and further client
    * termination.
    * <p>
    * The method takes care of rendering the error message directly and does not interact with
    * an error writer.
    *
    * @param   wrnmsg
    *          The warning message to display
    */
   public void displayWarning(String wrnmsg);

   /**
    * Displays the error message via current client side error writer.  This does not use
    * the client UI code tools, ususally meaning standard outpot channel anf further client
    * termination.  This is the redirected version of the displayError.  If the output is
    * redirected to file the message is writing to the file.
    *
    * @param   errmsg
    *          The error message to display
    */
   public void displayErrorRedirected(String errmsg);
   
   /**
    * Displays a message and sets or updates a variable using a specified
    * format for the temporary FillIn widget.
    * <p>
    * This method corresponds to the Progress statement <br>
    * <b>MESSAGE SET/UPDATE variable AUTO-RETURN FORMAT </b>
    * 
    * @param   text
    *          Text to be displayed.
    * @param   set
    *          <code>true</code> for SET, <code>false</code> for UPDATE,
    *          not used if var parameter is <code>null</code>. 
    * @param   var
    *          The variable to set or update.  <code>null</code> means no
    *          set or update will occur.
    * @param   auto
    *          <code>true</code> if the AUTO-RETURN option is specified. This
    *          is only valid if <code>var</code> is not <code>null</code>.
    * @param   format
    *          The format specification to use for the set or update.  This
    *          is only valid if <code>var</code> is not <code>null</code>.
    * @param   cs
    *          The color specification to use for the text.
    * @param   windowId
    *          Id of target window or {@link WindowConfig#RESOLVE_WINDOW} for 
    *          current or default window.
    *          
    * @return  The result of the input operation if input parameter
    *          <code>var</code> is not <code>null</code> and operation
    *          finished successfully and/or if any key was pressed during
    *          editing/pausing.   If no pause occurred and there was no input
    *          (editing) in process, then <code>null</code> will be returned.
    */
   public MessageReturnValue message(String       text,
                                     boolean      set,
                                     BaseDataType var, 
                                     boolean      auto,
                                     String       format,
                                     Color        cs,
                                     int          windowId);

   /**
    * Displays an error message. Depending on the UI mode (ChUI vs GUI) the message is rendered
    * as a regular window message or as a GUI ALERT-BOX.
    *
    * @param   text
    *          Error text to be displayed.
    */
   public void displayErrorMessage(String text);

   /**
    * Displays an error message. Depending on the UI mode (ChUI vs GUI) and
    * <code>noMessageBox</code> flag, the message is rendered as a regular window message or as a
    * GUI ALERT-BOX.
    *
    * @param   text
    *          Error text to be displayed.
    * @param   noMessageBox
    *          <code>true</code> if error is always displayed in message area or logged rather
    *          than displayed as a message box.
    */
   public void displayErrorMessage(String text, boolean noMessageBox);

   /**
    * Displays a warning message. Depending on the UI mode (ChUI vs GUI) the message is rendered
    * as a regular window message or as a GUI ALERT-BOX.
    *
    * @param   text
    *          Warning text to be displayed.
    */
   public void displayWarningMessage(String text);

   /**
    * Displays a list of text items and empty lines in an alert box and
    * sets/updates a <code>logical</code> variable as a result of the action
    * taken.  This is a modal dialog with the specified title and button
    * configuration. The message body is represented by a content array. Each
    * line of the message including any number of empty lines (skip) are
    * elements of the content array.  Text elements of the content array must
    * be <code>Strings</code>.  In addition, there may be any number of
    * <code>skip</code> elements which are encoded as a Java
    * <code>Integer</code> instance where the integral value is equivalent
    * to the number of lines to skip.
    * <p>
    * The value of the <code>var</code> being edited determines the default
    * button if it is not <code>null</code>.  <code>true</code> is the first
    * button, <code>false</code> is the second button and for a 3-button
    * dialog, <code>unknown</code> is the third button.
    * <p>
    * The result of the user's selection is returned and this result value
    * can be mapped to a button following the same mapping as used for
    * setting the default button (see above).
    * <p>
    * This method corresponds to the Progress statements:
    * <p>
    * <pre>
    * MESSAGE list VIEW-AS ALERT-BOX
    * MESSAGE list VIEW-AS ALERT-BOX SET/UPDATE var
    * ... and all other variants that include options such as title, buttons
    * and colors
    * </pre>
    * 
    * @param    list
    *           The list of message text components.
    * @param    var
    *           The variable to update if not <code>null</code>.
    * @param    alertType
    *           One of the ALERT_* constants specifying message type.
    * @param    buttons
    *           One of the BTN_* constants specifying used button set.
    * @param    title
    *           The dialog title.
    * @param    color
    *           Color scheme to use.
    * @param    windowId
    *           Id of target window or {@link WindowConfig#RESOLVE_WINDOW} for 
    *           current or default window.
    * @param    stack
    *           Progress stack trace
    *
    * @return   The edited result which can be mapped into a specified 
    *           button.
    */
   public logical messageBox(Object[] list,
                             logical  var,
                             int      alertType,
                             int      buttons,
                             String   title,
                             Color    color,
                             int      windowId,
                             String[] stack);

   /**
    * Move specified widget before the widget "before".
    * 
    * @param   frameId
    *          ID of the frame that contains widgets being moved.
    * @param   widget
    *          ID of the widget to move.
    * @param   before            
    *          ID of the widget before which the widget should be moved.
    *          
    * @return  TRUE - the widget was moved successfully, FALSE - otherwise.         
    */
   public logical moveBeforeTabItem(int frameId, int widget, int before);

   /**
    * Move specified widget after the widget "after".
    * 
    * @param   frameId
    *          ID of the frame that contains widgets being moved.
    * @param   widget
    *          ID of the widget to move.
    * @param   after
    *          ID of the widget after which the widget should be moved.
    *          
    * @return  TRUE - the widget was moved successfully, FALSE - otherwise.         
    */
   public logical moveAfterTabItem(int frameId, int widget, int after);

   /**
    * Update the tab item list for the given frame.
    * 
    * @param    frameId
    *           ID of the frame that changes its tab item list.
    * @param    tabItemList
    *           The array of widgets ids according to their tab order
    */
   public void updateTabItemList(int frameId, int[] tabItemList);

   /**
    * Defines a specific action for the key.
    * <p>
    * This method corresponds to the Progress statement <br>
    * <b>ON key-label key-function </b>
    * 
    * @param   keyLabel
    *          the label of the key
    * @param   keyFunction
    *          the action to take when the user presses the key
    */
   public void remapKey(String keyLabel, String keyFunction);

   /**
    * Suspends processing for the specified number of seconds or until the
    * user presses any key, whichever comes first. The type-ahead buffer is
    * cleared. The specified (or no) message appears on the status line.
    * <p>
    * This method corresponds to the Progress statement <br>
    * <b>PAUSE n MESSAGE/NO-MESSAGE IN WINDOW</b>
    * 
    * @param   seconds
    *          Pause duration in seconds. -1 is an indefinite wait.
    * @param   text
    *          Either <code>null</code> if NO-MESSAGE is in effect, or the
    *          message text.
    * @param   noMessage
    *          <code>true</code> if NO-MESSAGE is in effect. This suppresses
    *          the message display, even if an explicit message text was
    *          passed.
    * @param   windowId
    *          Id of target window or {@link WindowConfig#RESOLVE_WINDOW} for 
    *          current or default window.
    *
    * @return   The key code of the key pressed to clear the pause or -1
    *           if a timeout occurred (no key was pressed) or -2 if this
    *           was an immediate return due to a timeout of 0.
    */
   public int pause(int seconds, String text, boolean noMessage, int windowId);

   /**
    * Specifies the pause action the user must take whenever frames are hidden
    * automatically. The pause lasts the specified number of seconds or until
    * the user presses a key, whichever comes first. The specified (or no)
    * message appears on the status line.
    * <p>
    * This method corresponds to Progress statements:
    * <p>
    * <pre>
    * PAUSE seconds BEFORE-HIDE MESSAGE text
    * PAUSE seconds BEFORE-HIDE NO-MESSAGE
    * </pre>
    * 
    * @param    seconds
    *           Pause duration in seconds.
    * @param    text
    *           Either <code>null</code> if NO-MESSAGE is in effect or the
    *           message text to display.
    * @param    windowId
    *           Id of target window or {@link WindowConfig#RESOLVE_WINDOW} for 
    *           current or default window.
    */
   public void pauseBeforeHide(int seconds, String text, int windowId);
   
   /**
    * Processes all outstanding events without blocking for user input.
    * <p>
    * This method corresponds to the Progress statement <br>
    * <b>PROCESS EVENTS </b>
    *
    * @param   inbuf
    *          current screen buffer array for editable frames known to the server
    * 
    * @return  <code>ScreenBuffer</code> array with the current values
    */
   public ScreenBuffer[] processEvents(ScreenBuffer[] inbuf);

   /**
    * Register the specified ROW-DISPLAY events.
    *
    * @param   wids
    *          The id of the browse widget for which there are ROW-DISPLAY
    *          triggers.
    * @param   triggerIds
    *          The corresponding trigger ID for each widget.
    */
   public void registerRowDisplayEvents(int[] wids, int[] triggerIds);

   /**
    * Remove the ROW-DISPLAY triggers from the specified list.
    * 
    * @param   triggers
    *          The triggers to be removed, if they are associated to a 
    *          ROW-DISPLAY event.
    */
   public void removeRowDisplayEvents(int[] triggers);

   /**
    * Turns cursor display on or off.
    * <p>
    * This method corresponds to the Progress statements <br>
    * <b>PUT CURSOR </b> <br>
    * <b>PUT CURSOR OFF </b>
    * 
    * @param   onOff
    *          <code>true</code> if the cursor display is being turned on.
    */
   public void putCursor(boolean onOff);

   /**
    * Puts cursor to the specified screen position.
    * <p>
    * This method corresponds to the Progress statements <br>
    * <b>PUT CURSOR ROW COLUMN </b>
    * 
    * @param   row
    *          1-based screen row number
    * @param   column
    *          1-based screen column number
    */
   public void putCursor(double row, double column);

   /**
    * Puts text on the screen at the specified position and with the given
    * text attributes.  Such text is directly drawn on top of the Z-order
    * and portions of that text will be automatically removed as those 
    * portions are obscured by other drawing on the screen (be it other
    * calls to this method or any other drawing).  Any partial obscuring of
    * the output will leave any unobscured text on the screen until drawing
    * occurs over those screen positions.
    * <p>
    * This method corresponds to the Progress statement:
    * <p>
    * <pre>
    * PUT SCREEN text ROW y COLUMN x
    * </pre>
    * 
    * @param    text
    *           The text to display.
    * @param    row
    *           1-based screen row number.
    * @param    column
    *           1-based screen column number.
    * @param    color
    *           The color specification or -1 for the default color.
    */
   public void putScreen(String text, double row, double column, Color color);

   /**
    * Reads and returns one keystroke from the keyboard type-ahead buffer.
    * <p>
    * This method corresponds to the Progress statements <br>
    * <b>READKEY PAUSE n READKEY </b>
    * 
    * @param   seconds
    *          -1 means indefinite wait, 0 means no wait any positive values
    *          means the number of seconds to wait
    * @return  -1 if timed out, otherwise the event code
    */
   public int readKey(int seconds);

   /**
    * Sets the terminal type.  This will cause the terminal to potentially
    * change its configuration if the new type is different from the old
    * type.
    * <p>
    * This processing also clears the input buffer and resets any key
    * mappings to their default values.
    * <p>
    * This method corresponds to the Progress statement: <br>
    * <b>TERMINAL</b>
    * 
    * @param   term
    *          Terminal type name.  If <code>null</code>, this resets the
    *          terminal without changing the type.
    */
   public void setTerminalType(String term);
   
   /**
    * Specifies a new default status message for the running procedure when
    * not in INPUT mode (when editing).
    * <p>
    * This method is used to display a status message, which is trimmed to
    * 63 chars only if the <code>noLengthLimit</code> flag is not set.
    * 
    * @param    text
    *           The new status message.  <code>null</code> is the equivalent
    *           of the empty string.
    * @param    noLengthLimit
    *           flag indicating the status message should not be trimmed
    * @param    windowId
    *           Id of target window or {@link WindowConfig#RESOLVE_WINDOW} for 
    *           current or default window.
    */
   public void statusDefault(String text, boolean noLengthLimit, int windowId);

   /**
    * Specifies a new status message used when the user interface is blocked
    * on input.
    * <p>
    * This method corresponds to the Progress statement
    * <code>STATUS INPUT text</code>.
    * 
    * @param    text
    *           The new status message.  <code>null</code> is the equivalent
    *           of the empty string.
    * @param    windowId
    *           Id of target window or {@link WindowConfig#RESOLVE_WINDOW} for 
    *           current or default window.
    */
   public void statusInput(String text, int windowId);

   /**
    * Reverts the input status message to the standard input message.

    * @param    windowId
    *           Id of target window
    */
   public void statusInputRevert(int windowId);
   
   /**
    * Disables display of the input status message.
    * <p>
    * This method corresponds to the Progress statement
    * <code>STATUS INPUT OFF IN WINDOW wnd</code>.
    * 
    * @param   windowId
    *          Id of target window or {@link WindowConfig#RESOLVE_WINDOW} for 
    *          current or default window.
    */
   public void statusInputOff(int windowId);

   /**
    * Wait for specified list of events. <br>
    * This method corresponds to the Progress statement: <br>
    * <b>WAIT-FOR FOCUS widget PAUSE n </b>
    * 
    * @param   wc
    *          List of events to wait for.
    * @param   focusWidgetId
    *          Widget ID which is in focus.
    * @param   seconds
    *          Time to wait for event.
    * @param   inbuf
    *          currently editable frames screen buffer known to the server.
    * 
    * @return  <code>ScreenBuffer</code> array with the current values.
    */
   public ScreenBuffer[] waitFor(EventList      wc,
                                 int            focusWidgetId, 
                                 int            seconds,
                                 ScreenBuffer[] inbuf);

   /**
    * Refreshes the specified frame and widget with a new screen value.
    * 
    * @param   fid
    *          frame to refresh
    * @param   wid
    *          widget to refresh
    * @param   value
    *          new widget value
    * @return  string representation of the value
    */
   public String refreshFrameWidget(int fid, int wid, BaseDataType value);
   
   /**
    * Send array of frame/widget definitions to the client.
    * 
    * @param   sd
    *          Array of screen definitions.
    *          
    * @return  For each down frame, the configs for the widgets in the current row.  Otherwise
    *          <code>null</code> is returned.
    */
   public WidgetConfig[] pushScreenDefinition(ScreenDefinition[] sd);

   /**
    * Place the frame.
    * 
    * @param    frameId
    *           The frame's ID.
    */
   public void placeFrame(int frameId);

   /**
    * Push the specified widget attribute change to the client-side.
    * 
    * @param    frameVisible
    *           The current (after config change) visible frame state.
    * @param    wid
    *           The widget ID.
    * @param    fname
    *           The config field name.
    * @param    value
    *           The new config field value.
    */
   public void pushWidgetAttr(boolean frameVisible, int wid, String fname, Object value);
   
   /**
    * Push the specified widget attribute changes to the client-side.
    * 
    * @param    frameVisible
    *           The current (after config change) visible frame state.
    * @param    wid
    *           The widget ID.
    * @param    fnames
    *           The list of config field names.
    * @param    values
    *           The list of new config field values.
    */
   public void pushWidgetAttr(boolean frameVisible, int wid, String[] fnames, Object[] values);
   
   /**
    * Send array of menu element descriptions to the client.
    * 
    * @param   md
    *          Array of menu descriptions.
    */
   public void pushMenuDescription(MenuDescription[] md);
   
   /**
    * Pushes the window configuration instance down to the client.
    * 
    * @param   config
    *          instance of <code>WindowConfig</code>
    */
   public void pushWindow(WindowConfig config);

   /**
    * Remove the window instance from the client.
    * 
    * @param   window
    *          The window ID.
    */
   public void removeWindow(int window);
   
   /**
    * Returns the ID of the currently active frame.
    *
    * @return frame ID
    */
    public int getActiveFrame();
    
   /**
    * Returns the number of screen lines available for frame display (does
    * not include the message or status lines).
    * <p>
    * This method corresponds to the Progress built-in function:
    * <p>
    * <pre>
    * SCREEN-LINES
    * </pre>
    *
    * @return   The number of available screen lines.
    */
   public double getScreenLines();
   
   /**
    * Returns the number of screen lines available for message display.
    * <p>
    * This method corresponds to the Progress built-in function:
    * <p>
    * <pre>
    * MESSAGE-LINES
    * </pre>
    *
    * @return   The number of available lines in the message area.
    */
   public int getMessageLines();
   
   //----------------------------------------------------------------
   // Per-frame calls
   //----------------------------------------------------------------

   /**
    * Applies the encoded event to the widget currently in focus in the given
    * frame, or directly to the specified widget.
    * 
    * @param    widgetId
    *           ID of frame or a contained widget.
    * @param    eventCode
    *           Progress-compatible event code.
    * @param    inbuf
    *           The screen buffers for the currently editable frames if the widgetId is 
    *           from a widget that is a frame or is contained within a frame.
    *           May be <code>null</code> if the widget is not associated with
    *           any frame (e.g. the session window).
    * @param    eventList
    *           The list of the latest triggers (which may be more recent 
    *           than that used in any previous <code>WAIT-FOR</code>).
    * @param    explicit
    *           <code>true</code> if this APPLY call has an explicit target.
    *
    * @return   Updated data for the associated frame representing the current
    *           screen values of all widgets in that frame.  May be
    *           <code>null</code> if the frame has no data containing widgets
    *           or if the input screen buffer is <code>null</code> or is
    *           otherwise invalid.
    */
   public ScreenBuffer[] apply(int            widgetId,
                               int            eventCode,
                               ScreenBuffer[] inbuf,
                               EventList      eventList,
                               boolean        explicit);

   /**
    * Apply this event to a non-widget resource.
    * 
    * @param    resId
    *           The resource ID to which this events needs to be applied.
    * @param    eventCode
    *           Progress-compatible event code.
    * @param    eventList
    *           The list of the latest triggers (which may be more recent than that used in any 
    *           previous <code>WAIT-FOR</code>).
    */
   public void apply(long resId, int eventCode, EventList eventList);

   /**
    * Post the server event to the client side. The event will be executed only on the next 
    * PROCESS-EVENTS or UI-blocking statement.
    * 
    * @param    event
    *           The server event.
    */
   public void postServerEvent(ServerEvent event);

   /**
    * Remove from the server event queue all the events in the given list, identified by their id.
    * 
    * @param    eventIds
    *           The list of event IDs to be removed.
    */
   public void removeServerEvents(int[] eventIds);
   
   /**
    * Performs CHOOSE processing.
    * <p>
    * This method corresponds to this Progress statement: <br>
    * <b>CHOOSE COLOR </b>
    * 
    * @param   frameId
    *          ID of frame this request came for
    * @param   c
    *          CHOOSE statement options
    * @param   wl
    *          Event list. 
    *          
    * @return  Instance of ChooseData updated during processing of request.
    */
   public ChooseData choose(int frameId, ChooseData c, EventList wl);

   /**
    * Clears the data and colors for all fill-in fields of the frame. For down
    * frames, also clears side labels).
    * <p>
    * This method corresponds to this Progress statement: <br>
    * <b>CLEAR </b>
    * 
    * @param   frameId
    *          ID of frame this request came for
    * @param   noPause
    *          <code>true</code> if NO-PAUSE option was used
    *
    * @return   The screen buffer array for editable frames.
    */
   public ScreenBuffer[] clear(int frameId, boolean noPause);

   /**
    * Clears all occurrences for the down frame and resets the current display
    * position to the top of the frame.
    * <p>
    * This method corresponds to this Progress statement: <br>
    * <b>CLEAR ALL </b>
    * 
    * @param   frameId
    *          ID of frame this request came for
    * @param   noPause
    *          <code>true</code> if NO-PAUSE option was used
    *
    * @return   The screen buffer array for editable frames.
    */
   public ScreenBuffer[] clearAll(int frameId, boolean noPause);
   
   /**
    * Positions the cursor on a new line in a down frame.  When 
    * <code>lines</code> is negative, this operates as a move of the cursor
    * <code>up</code>.
    * <p>
    * This method corresponds to these Progress statements: <br>
    * <b>DOWN n </b>
    * <b>UP n </b>
    * 
    * @param    frameId
    *           ID of frame this request came for
    * @param    lines
    *           Number and direction of iterations of the data to move. If
    *           positive, the move is <code>down</code>, if negative the
    *           move is <code>up</code>.  0 is ignored.
    * @param    cond
    *           <code>true</code> means conditional down; lines is assumed
    *           to be 1
    * @param    sid
    *           The stream for which pending PUT or EXPORT output must be
    *           ended and written to the stream. -1 means that this is not
    *           necessary.
    *           
    * @return   If this is a down frame, the configs for the widgets in the new row.  Otherwise
    *           <code>null</code> is returned.
    */
   public WidgetConfig[] down(int frameId, int lines, boolean cond, int sid);

   /**
    * Positions the cursor on a new line in a down frame.  When 
    * <code>lines</code> is negative, this operates as a move of the cursor
    * <code>up</code>.
    * <p>
    * The terminal will be temporarily redirected to the given stream for the
    * duration of this method call.  This allows the caller to avoid 
    * bracketing this call with {@link #switchOutput} invocations, as the
    * same result is achieved internal to this method's operation.
    * <p>
    * This method corresponds to these Progress statements: <br>
    * <b>DOWN n </b>
    * <b>UP n </b>
    * 
    * @param    frameId
    *           ID of frame this request came for
    * @param    lines
    *           Number and direction of iterations of the data to move. If
    *           positive, the move is <code>down</code>, if negative the
    *           move is <code>up</code>.  0 is ignored.
    * @param    streamId
    *           The stream to be used as the temporary terminal redirection
    *           (only for the duration of this call).
    * @param    cond
    *           <code>true</code> means conditional down; lines is assumed
    *           to be 1
    * @param    endOut                
    *           <code>true</code> to force any pending PUT or EXPORT output
    *           to be ended for the given stream (so long as that stream is
    *           not the terminal).
    *           
    * @return   If this is a down frame, the configs for the widgets in the new row.  Otherwise
    *           <code>null</code> is returned.
    */             
   public WidgetConfig[] down(int frameId, int lines, int streamId, boolean cond, boolean endOut);

   /**
    * Scrolls the lines of the down frame up or down.
    * <p>
    * This method corresponds to this Progress statement: <br>
    * <b>SCROLL </b>
    * 
    * @param   frameId
    *          ID of frame this request came for
    * @param   fromCurrent
    *          <code>true</code> if FROM-CURRENT option was used
    * @param   upDown
    *          <code>true</code> if scrolling up
    */
   public void scroll(int frameId, boolean fromCurrent, boolean upDown);
   
   /**
    * Enables/disables input for all specified field-level widgets of the
    * frame.  This method also handles the following:
    * <p>
    * <ul>
    *    <li> Maintains the tab order of the specified frame if the latter 
    *         does NOT have KEEP-TAB-ORDER option set. Every widget
    *         listed for enabling is moved to the end of the tab order
    *         retaining the same order as specified in the list.  If a
    *         frame with x, y and z widgets gets an enable call for y and
    *         x (in that order), then the tab order will change from the
    *         default <code>x y z</code> to <code>z y x</code>. In the case
    *         where all widgets are enabled, no change to the tab order
    *         occurs.
    *    <li> Changes the currently focused widget to:
    *       <ul>
    *          <li> the first widget in the list on an enable where no
    *               widgets were previously enabled or where the 
    *               <code>force</code> parameter is set
    *          <li> the next enabled widget on a disable when there is at
    *               least one currently enabled widget that is not in the
    *               disable list 
    *       </ul>
    * </ul>
    * <p>
    * This method corresponds to these Progress statements: <br>
    * <b>DISABLE ALL</b> <br>
    * <b>DISABLE ALL EXCEPT</b> <br>
    * <b>ENABLE ALL</b> <br>
    * <b>ENABLE ALL EXCEPT</b>
    * 
    * @param    frameId
    *           ID of frame this request came for
    * @param    widgetId
    *           array of field level widget IDs to be enabled or
    *           <code>null</code>, which means all of them.
    * @param    sb
    *           The currently editable frames screen buffer array.
    * @param    enabled
    *           New state for widgets.
    * @param    windowId
    *           Id of target window or {@link WindowConfig#RESOLVE_WINDOW} for 
    *           current or default window.
    * @param    tabItemList
    *           The array of widgets ids according to their tab order.
    *           
    * @return   The screen buffer array for editable frames.
    */
   public ScreenBuffer[] enable(int            frameId,
                                int[]          widgetId,
                                ScreenBuffer[] sb,
                                boolean        enabled,
                                int            windowId,
                                int[]          tabItemList); 

   /**
    * Enables/disables input for the specified field-level widget.
    * This method preserves the current tab order of the frame.
    * <p>
    * This method corresponds to these Progress statement: <br>
    * <b>widget:SENSITIVE=true/false</b>
    * 
    * @param    frameId
    *           ID of frame this request came for
    * @param    widgetId
    *           widget ID to be enabled or disabled
    * @param    sb
    *           The currently editable frames screen buffer array.
    * @param    enabled
    *           New state of the widget
    * @param    windowId
    *           Id of target window or {@link WindowConfig#RESOLVE_WINDOW} for 
    *           current or default window.
    * @param    tabItemList
    *           The array of widgets ids according to their tab order.
    *           
    * @return   The screen buffer array for editable frames.
    */
   public ScreenBuffer[] enable(int            frameId,
                                int            widgetId,
                                ScreenBuffer[] sb,
                                boolean        enabled,
                                int            windowId, 
                                int[]          tabItemList); 

   /**
    * Underlines a set of widgets of the specified frame. If the target
    * frame is not a down frame, the frame is simply cleared.
    * 
    * @param    frameId
    *           The frame ID this request came for.
    * @param    widgetIds
    *           The array of field level widget IDs to be underlined.
    */
   public void underline(int frameId, int[] widgetIds);

   /**
    * Shows all fields of the specified frame on screen.
    * 
    * @param    widgetId
    *           The frame ID or widget this request came for.
    * @param    frameBuf
    *           The screen buffer array with current values of widgets for editable frames.
    * @param    widgetIds
    *           The array of field level widget IDs to be shown or
    *           <code>null</code>, which means all of them.
    * @param    viewDisp
    *           <code>true</code> if the call is for VIEW, <code>false</code>
    *           for DISPLAY.
    * @param    windowId
    *           Id of target window or {@link WindowConfig#RESOLVE_WINDOW} for 
    *           current or default window.
    * @param    tabItemList
    *           The array of widgets ids according to their tab order.
    *           
    * @return   The frames screen buffer array.
    */
   public ScreenBuffer[] view(int widgetId, ScreenBuffer[] frameBuf, int[] widgetIds,
                              boolean viewDisp, int windowId, int[] tabItemList);

   /**
    * Shows all fields of the specified frame on screen.
    * <p>
    * The terminal will be temporarily redirected to the given stream for the
    * duration of this method call.  This allows the caller to avoid 
    * bracketing this call with {@link #switchOutput} invocations, as the
    * same result is achieved internal to this method's operation.
    * 
    * @param    widgetId
    *           The frame ID or widget this request came for.
    * @param    frameBuf
    *           The screen buffer array with current values of widgets for editable frames.
    * @param    widgetIds
    *           The array of field level widget IDs to be shown or
    *           <code>null</code>, which means all of them.
    * @param    streamId
    *           The stream to be used as the temporary terminal redirection
    *           (only for the duration of this call).
    * @param    register
    *           <code>true</code> if the frame associated with this request
    *           must be registered as a page element with the given stream. 
    * @param    pageTop
    *           <code>true</code> if the page element registration is for
    *           a page top element, <code>false</code> for a page bottom
    *           element.  This parameter is ignored if <code>register</code>
    *           is <code>false</code>.
    * @param    viewDisp
    *           <code>true</code> if the call is for VIEW, <code>false</code>
    *           for DISPLAY.
    * @param    windowId
    *           Id of target window or {@link WindowConfig#RESOLVE_WINDOW} for 
    *           current or default window.
    * @param    tabItemList
    *           The array of widgets ids according to their tab order
    *           
    * @return   If this is a down frame, the configs for the widgets in the new row.  Otherwise
    *           <code>null</code> is returned.
    */
   public WidgetConfig[] view(int            widgetId,
                              ScreenBuffer[] frameBuf,
                              int[]          widgetIds,
                              int            streamId,
                              boolean        register,
                              boolean        pageTop,
                              boolean        viewDisp,
                              int            windowId,
                              int[]          tabItemList);
   
   /**
    * Support for the DISPLAY followed by DOWN combination.
    * 
    * @param    widgetId
    *           The frame ID or widget this request came for.
    * @param    frameBuf
    *           The screen buffer array with current values of widgets for editable frames.
    * @param    widgetIds
    *           The array of field level widget IDs to be shown or
    *           <code>null</code>, which means all of them.
    * @param    streamId
    *           The stream to be used as the temporary terminal redirection
    *           (only for the duration of this call).
    * @param    register
    *           <code>true</code> if the frame associated with this request
    *           must be registered as a page element with the given stream. 
    * @param    pageTop
    *           <code>true</code> if the page element registration is for
    *           a page top element, <code>false</code> for a page bottom
    *           element.  This parameter is ignored if <code>register</code>
    *           is <code>false</code>.
    * @param    windowId
    *           Id of target window or {@link WindowConfig#RESOLVE_WINDOW} for 
    *           current or default window.
    * @param    tabItemList
    *           The array of widgets ids according to their tab order.
    */
   public void displayAndDown(int            widgetId, 
                              ScreenBuffer[] frameBuf,
                              int[]          widgetIds,
                              int            streamId,
                              boolean        register,
                              boolean        pageTop,
                              int            windowId,
                              int[]          tabItemList);
   /**
    * Provides a composite helper to implement a single call to the client
    * which does an enable, wait-for, disable and hide message (in that
    * order).  This single call is more efficient than executing the calls
    * separately since the client has all the state needed AND since it
    * avoids extra round trips.
    * 
    * @param    frameId
    *           ID of frame this request came for
    * @param    widgetIds
    *           array of field level widget IDs to be enabled or
    *           <code>null</code>, which means all of them.
    * @param    sb
    *           The currently editable frames screen buffer array.
    * @param    seconds
    *           Time to wait for event.
    * @param    wc
    *           List of events to wait for.
    * @param    doIO
    *           tells to perform the requested operation if <code>true</code>.
    *           otherwise prepares the frame for editing session only.
    * @param    windowId
    *           Id of target window or {@link WindowConfig#RESOLVE_WINDOW} for 
    *           current or default window.
    * @param    tabItemList
    *           The array of widgets ids according to their tab order.
    *           
    * @return   <code>ScreenBuffer</code> array with the current values.
    */
   public ScreenBuffer[] promptFor(int            frameId,
                                   int[]          widgetIds,
                                   ScreenBuffer[] sb,
                                   int            seconds,
                                   EventList      wc,
                                   boolean        doIO,
                                   int            windowId,
                                   int[]          tabItemList);
   
   /**
    * Specifies the field to be put in focus during the next input operation.
    * <p>
    * This method corresponds to this Progress statement: <br>
    * <b>NEXT-PROMPT </b>
    * 
    * @param   frameId
    *          ID of frame this request came for
    * @param   widgetId
    *          widget to be put in focus
    */
   public void nextPrompt(int frameId, int widgetId);

   /**
    * Returns the column position of the upper-left corner of the frame
    * within its window (in coordinates that are relative to the containing
    * window).
    * <p>
    * This method corresponds to the Progress built-in function:
    * <p>
    * <pre>
    * FRAME-COL
    * </pre>
    * 
    * @param    frameId
    *           The ID of the frame for this request.
    * 
    * @return   The column position for the upper-left corner of the frame.
    */
   public double getFrameCol(int frameId);
   
   /**
    * Returns the number of iterations in a down frame.
    * <p>
    * This method corresponds to the Progress built-in function:
    * <p>
    * <pre>
    * FRAME-DOWN
    * </pre>
    * 
    * @param    frameId
    *           The ID of the frame for this request.
    * 
    * @return   The number of data iterations in the frame.
    */
   public int getFrameDown(int frameId);

   /**
    * Returns the current logical line number in a down frame.
    * <p>
    * This method corresponds to the Progress built-in function:
    * <p>
    * <pre>
    * FRAME-LINE
    * </pre>
    * 
    * @param    frameId
    *           The ID of the frame for this request.
    * @param    pendingUpDown
    *           The pending down value for this frame.
    * 
    * @return   1-based ordinal of the current data iteration of the frame.
    */
   public int getFrameLine(int frameId, int pendingUpDown);

   /**
    * Returns the row position of the upper-left corner of the frame
    * within its window (in coordinates that are relative to the containing
    * window).
    * <p>
    * This method corresponds to the Progress built-in function:
    * <p>
    * <pre>
    * FRAME-ROW
    * </pre>
    * 
    * @param    frameId
    *           The ID of the frame for this request.
    * 
    * @return   The row position for the upper-left corner of the frame.
    */
   public double getFrameRow(int frameId);
   
   /**
    * Gets the first widget in the tab order from the widgets of specified list.
    * Widgets from the list are assumed from the field-group for which this method is called. 
    * 
    * @param    frameId
    *           ID of the containing frame.
    * @param    widgetIDs 
    *           IDs of widgets in the target field-group.
    *           
    * @return   ID of the first widget in tab order.
    */
   public int getFirstTabItem(int frameId, List<Integer> widgetIDs);

   /**
    * Move specified widget to the first position in the tab order.
    * 
    * @param   frameId
    *          ID of the frame that contains widgets being moved.
    * @param   fgWidgetIds
    *          Target field-group widgets.
    * @param   widgetId
    *          ID of the widget to move.
    */
   public void setFirstTabItem(int frameId, List<Integer> fgWidgetIds, int widgetId);
   
   /**
    * Getter for the <b>NUM-SELECTED-ROWS </b> attribute.
    * 
    * @param   browseId
    *          ID of the browse widget this request came for
    * 
    * @return  current value of NUM-SELECTED-ROWS attribute
    */
   public int getNumSelectedRows(int browseId);
   
   /**
    * Gets the widget that is the next from specified in the tab order.
    * 
    * @param    frameId
    *           ID of the containing frame.
    * @param    widgetId
    *           The widget to get the next widget in the tab order.
    * 
    * @return   ID of the widget that is the next from specified in the tab order.
    */
   public int getNextTabItem(int frameId, int widgetId);

   /**
    * Retrieve the 0-based row index of the <i>n</i>th selected row in the
    * given browse widget.
    * 
    * @param   browseId
    *          ID of the browse widget associated with the request.
    * @param   n
    *          0-based position of the row in the list of selected rows. <code>-1</code> for the
    *          last selected row.
    *          
    * @return  0-based row index associated with the <i>n</i>th selected row in the
    *          browse, or -1 if the associated Browse widget could not be
    *          identified.
    *
    * @throws  com.goldencode.p2j.util.ErrorConditionException
    *          If <code>selIndex</code> is invalid or if there are no "available" rows.
    */
   public int getSelectedRowIndex(int browseId, int n);

   /**
    * Retrieve the row indexes of the selected rows in the given browse widget.
    *
    ** @param  browseId
    *          ID of the browse widget this request came for.
    *
    * @return  Array of 0-based row indexes of the selected rows in the browse.
    *
    * @throws  com.goldencode.p2j.util.ErrorConditionException
    *          If <code>selIndex</code> is invalid or if there are no "available" rows.
    */
   public int[] getSelectedRowIndexes(int browseId);

   /**
    * Implements the <b>SELECT-ALL() </b> <b>BROWSE </b> widget method.
    * 
    * @param   browseId
    *          ID of the browse widget this request came for
    */
   public void selectAll(int browseId);

   /**
    * Implements the <b>DESELECT-ROWS() </b> <b>BROWSE </b> widget method.
    * 
    * @param   browseId
    *          ID of the browse widget this request came for
    */
   public void deselectRows(int browseId);

   /**
    * Notify the client that columns have been added to the browse.
    *
    * @param   browseId
    *          ID of the browse widget this request came for.
    */
   public void notifyColumnsUpdated(int browseId);

   /**
    * Scrolls a browse (if necessary) to bring the currently selected row into view. If the browse
    * supports multiple selections, then SCROLL-TO-CURRENT-ROW() brings the most recently selected
    * row into view.<p>
    * The position of the scrolled row is the first row in the browse viewport, unless the current
    * row is already visible. In this case, the current row remains in the original position.
    *
    * @param   browseId
    *          ID of the browse widget this request came for.
    *
    * @throws ErrorConditionException
    *         if a numbered error occurs.
    */
   public void scrollToCurrentRow(int browseId);

   /**
    * Scrolls a browse (if necessary) to bring the specified selected row into view.
    *
    * @param  browseId
    *         ID of the browse widget this request came for.
    * @param  selectedRowIndex
    *         Selected row within the browse. A numbered list of selected rows, starting at 1 is
    *         maintained. When the <code>SCROLL-TO-SELECTED-ROW(n)</code> method is encountered,
    *         the list is iterated to find the <i>n</i>th selected row.
    *
    * @throws ErrorConditionException
    *         if a numbered error occurs.
    */
   public void scrollToSelectedRow(int browseId, int selectedRowIndex);

   /**
    * Refresh the current browse row using the provided data.
    *
    * @param browseId
    *        ID of the browse widget this request came for.
    * @param row
    *        Container with the new data for the current row. If an {@link BrowseRow#data} element
    *        is <code>null</code>, corresponding cell shouldn't be refreshed.
    */
   public void refreshCurrentRow(int browseId, BrowseRow row);

   /**
    * Refresh the browse cache and redisplay the browse.
    *
    * @param   browseId
    *          ID of the browse widget this request came for
    * @param   deleteCurrentRowMode
    *          <code>true</code> if this function is used for update on DELETE-CURRENT-ROW.
    *          <code>false</code> if this function is used for update on REFRESH.
    */
   public void refresh(int browseId, boolean deleteCurrentRowMode);

   /**
    * Refresh the browse after some of its rows were deleted.
    *
    * @param   browseId
    *          ID of the browse widget this request came for.
    * @param   rowIndex
    *          0-based index of the deleted row. <code>-1</code> if all of the selected rows were
    *          deleted.
    */
   public void refreshAfterDeletion(int browseId, int rowIndex);

   /**
    * Implements the SELECT-NEXT-ROW() widget method.<p>
    * Deselects all currently selected rows in a browse and selects the row after the deselected
    * row.<p>
    * This method also repositions the query to the new row and copies the record into the
    * database buffer. The browse automatically scrolls to the selected row if it is out of
    * view.<p>
    * This method is intended for use with a browse that supports the selection of a single row at
    * a time (MULTIPLE attribute is set to FALSE). If more than one row is selected when you
    * execute this method, all of the selected rows are deselected and the record after the last
    * selected row becomes the selected row.
    *
    * @param   browseId
    *          ID of the browse widget this request came for.
    *
    * @return  <code>true</code> on success.
    */
   public logical selectNextRow(int browseId);

   /**
    * Implements the SELECT-PREV-ROW() widget method.<p>
    * Deselects all currently selected rows in a browse and selects the row before the deselected
    * row.<p>
    * This method also repositions the query to the new row and copies the record into the
    * database buffer.The browse automatically scrolls to the selected row if it is out of
    * view.<p>
    * This method is intended for use with a browse that supports the selection of a single row at
    * a time (MULTIPLE attribute is set to FALSE). If more than one row is selected when you
    * execute this method, all of the selected rows are deselected and the record before the last
    * selected row becomes the selected row.
    *
    * @param   browseId
    *          ID of the browse widget this request came for.
    *
    * @return  <code>true</code> on success.
    */
   public logical selectPreviousRow(int browseId);

   /**
    * Implements the <b>SELECT-FOCUSED-ROW() </b> <b>BROWSE </b> widget
    * method.
    * 
    * @param   browseId
    *          ID of the browse widget this request came for
    */
   public void selectFocusedRow(int browseId);

   /**
    * Implements the <b>DESELECT-FOCUSED-ROW() </b> <b>BROWSE </b> widget
    * method.
    *
    * @param   browseId
    *          ID of the browse widget this request came for
    */
   public void deselectFocusedRow(int browseId);

   /**
    * Implements the DESELECT-SELECTED-ROW() widget method.<p> Deselects the <i>n</i>th selected
    * row in a browse.<p>
    * Progress maintains a numbered list of selected rows, starting at 1. When the
    * DESELECT-SELECTED-ROW(n) method is encountered, Progress searches this list to find
    * the <i>n</i>th selected row.
    *
    * @param   browseId
    *          ID of the browse widget this request came for.
    * @param   rowIndex
    *          1-based index of the row (in the list of selected rows) to deselect.
    *
    * @return  <code>true</code> if the row is successfully deselected.
    */
   public boolean deselectSelectedRow(int browseId, int rowIndex);

   /**
    * Implements the SELECT-ROW() widget method.<p>
    * Selects the specified row if it is currently in the browse viewport. In a single-select
    * browse, the previously selected row is deselected. No rows are deselected in a
    * multiple-select browse.
    *
    * @param   browseId
    *          ID of the browse widget this request came for.
    * @param   rowIndex
    *          1-based row index in the browse viewport.
    *
    * @return  <code>true</code> if the row with the given index exist in the browse viewport.
    */
   public boolean selectRow(int browseId, int rowIndex);

   /**
    * Implements the <b>IS-ROW-SELECTED()</b> <b>BROWSE </b> widget method.
    *
    * @param   browseId
    *          ID of the browse widget this request came for.
    * @param   rowIndex
    *          1-based row index in the browse viewport.
    *
    * @return  <code>true</code> if a specified row in the browse viewport is currently selected.
    */
   public boolean isRowSelected(int browseId, int rowIndex);

   /**
    * Implements the FOCUSED-ROW attribute getter.<p>
    *
    * @param   browseId
    *          ID of the browse widget this request came for.
    *
    * @return  the 1-based index of the focused row in the viewport or <code>-1</code> if browse
    *          doesn't exist.
    */
   public int getFocusedRow(int browseId);

   /**
    * Implements the client side of the MOVE-COLUMN() widget method.<p>
    * Repositions a column in a browse widget.
    *
    * @param  browseId
    *         ID of the browse widget this request came for.
    * @param  srcIndex
    *         0-based position of the column to be moved.
    * @param  destIndex
    *         0-based position to which the column is moved.
    */
   public void moveColumnClient(int browseId, int srcIndex, int destIndex);

   /**
    * Client-side part of the code which applies the given enhanced configuration to the browse.
    *
    * @param  browseId
    *         ID of the browse widget this request came for.
    * @param  config
    *         Enhanced browse config.
    */
   public void applyEnhancedConfigClient(int browseId, EnhancedBrowseConfig config);

   /**
    * Get number of rows in the browse viewport set either by DOWN or SIZE options.
    *
    * @param   browseId
    *          ID of the browse widget this request came for.
    *
    * @return  number of rows in the browse viewport.
    */
   public int getBrowsePageRowCount(int browseId);

   /**
    * Execute pending refresh when REFRESHABLE is turned on. Includes pending query repositions
    * and reopenings.
    *
    * @param   browseId
    *          ID of the browse widget this request came for.
    * @param   queryReopened
    *          <code>true</code> if there is a pending query reopening.
    * @param   repositionBeforeReopen
    *          <code>true</code> if the last pending reposition was <i>before</i> the last pending
    *          reopening.
    * @param   repositionedRow
    *          0-based index of the last row the query was repositioned to. <code>-1</code> if
    *          there is no repositioned row.
    */
   public void executePendingRefresh(int browseId,
                                     boolean queryReopened,
                                     boolean repositionBeforeReopen,
                                     int repositionedRow);

   /**
    * Refresh scroll row in the browse and redisplay. Unless specific parameters are passed, the
    * current row becomes the scroll row.
    *
    * @param   browseId
    *          ID of the browse widget this request came for.
    * @param   leaveOnRowInView
    *          If the current row is in the viewport, do not update the scroll row.
    * @param   preserveViewportRow
    *          Try to preserve the current viewport row (i.e. the row at the same position in
    *          the viewport becomes the current row after refresh).
    * @param   repositioned
    *          If the browse is refreshed because the query was repositioned.
    * @param   repositionError
    *          For reposition mode - if an error has happened during reposition.
    * @param   targetRepositionRow
    *          For reposition mode - row used as the target for reposition. If it was deleted,
    *          reposition with fetching may end up on a different row.
    */
   public void refreshScrollRow(int browseId,
                                boolean leaveOnRowInView,
                                boolean preserveViewportRow,
                                boolean repositioned,
                                boolean repositionError,
                                int targetRepositionRow);

   /**
    * Notify the browse that the underlying query has been repositioned..
    *
    * @param   browseId
    *          ID of the browse widget this request came for.
    * @param   repositionError
    *          For reposition mode - if an error has happened during reposition.
    * @param   targetRepositionRow
    *          For reposition mode - row used as the target for reposition. If it was deleted,
    *          reposition with fetching may end up on a different row.
    */
   public void queryRepositioned(int browseId, boolean repositionError, int targetRepositionRow);

   /**
    * Notify browse widget at client side that query was closed.
    * 
    * @param   browseId
    *          ID of the browse widget this request came for
    */
   public void queryClosed(int browseId);

   /**
    * Returns <code>true</code> if the current row is a new row and is not present in the
    * database.
    *
    * @param   browseId
    *          ID of the browse widget this request came for.
    *
    * @return  <code>true</code> if the current row is a new row and <code>false</code> otherwise.
    */
   public boolean isNewRow(final int browseId);

   /**
    * Returns <code>true</code> if the current browse row is modified.
    *
    * @param   browseId
    *          ID of the browse widget this request came for.
    *
    * @return  <code>true</code> if the current browse row is modified.
    */
   public boolean isCurrentRowModified(final int browseId);

   /**
    * Inserts a blank row in an updateable browse before or after the last selected row.
    * The blank row is a placeholder for a new record to be added through the browse.
    *
    * @param   browseId
    *          ID of the browse widget this request came for.
    * @param   after
    *          <code>true</code> if the row is added after the last selected row.
    *          <code>false</code> if the row is added before the last selected row.
    *
    * @return  <code>true</code> if the row was successfully added.
    */
   public boolean insertRow(final int browseId, final boolean after);

   /**
    * Calculate DOWN value for the browse basing on the given row height.
    *
    * @param  browseId
    *         ID of the browse widget this request came for.
    * @param  rowHeightPixels
    *         Row height in pixels.
    *
    * @return calculated DOWN value.
    */
   public int calcBrowseDown(final int browseId, int rowHeightPixels);

   /**
    * Push "intermediate" rows (i.e. the rows that were already fetched in the process of getting
    * new rows by
    * {@link BrowseWidget#getRows(int, int, boolean, int, ScreenBuffer[], RowDisplayStrategy)})
    * to the client, so these rows can be drawn if a pause occurs in ROW-DISPLAY trigger.
    *
    * @param  browseId
    *         ID of the browse widget this request came for.
    * @param  rows
    *         Set of rows to push.
    */
   public void pushIntermediateRows(final int browseId, BrowseRow[] rows);

   /**
    * Get ID of the widget which has the input focus.
    * 
    * @return   ID of the focused widget.
    */
   public int getFocus();
   
   /**
    * Sets the format specification for the specified widget and redraws the
    * widget.
    *
    * @param    widget
    *           The widget to modify.
    * @param    format
    *           The format specification.
    */
   public void setFormat(int widget, String format);
   
   /**
    * Set the new decimal and group separators.
    * 
    * @param    decSep
    *           The new decimal separator.
    * @param    groupSep
    *           The new group separator.
    */
   public void setNumericFormat(char decSep, char groupSep);
   
   /**
    * Register the frame as a header or footer from the stream that is the
    * current redirected terminal output destination.  The terminal must be
    * non-interactive for this method to process properly.  Subsequent
    * pagination operations on the stream will cause the header or footer to
    * render as needed, without any further explicit request from the server.
    * 
    * @param    frameId
    *           The id of the frame for this request.
    * @param    header
    *           <code>true</code> to register as a header, <code>false</code>
    *           to register as a footer.
    */
   public void registerPageElement(int frameId, boolean header);
   
   /**
    * Register the frame as a header or footer from the stream that is the
    * current redirected terminal output destination.  The terminal must be
    * non-interactive for this method to process properly.  Subsequent
    * pagination operations on the stream will cause the header or footer to
    * render as needed, without any further explicit request from the server.
    * 
    * @param    frameId
    *           The id of the frame for this request.
    * @param    header
    *           <code>true</code> to register as a header, <code>false</code>
    *           to register as a footer.
    * @param    pulse
    *           <code>true</code> to make possible pulse on close, <code>false</code>
    *           otherwise.
    */
   public void registerPageElement(int frameId, boolean header, boolean pulse);
   
   /**
    * Register the frame as a header or footer with the given stream as the
    * redirected terminal output destination.  Subsequent pagination
    * operations on the stream will cause the header or footer to render
    * as needed, without any further explicit request from the server.
    *
    * @param    frameId
    *           The id of the frame for this request.
    * @param    header
    *           <code>true</code> to register as a header, <code>false</code>
    *           to register as a footer.
    * @param    streamId
    *           The stream to associate with the frame as a page element.
    */
   public void registerPageElement(int frameId, boolean header, int streamId);
   
   /**
    * Deregister (remove) the frame as a header or footer from the stream 
    * that is the current redirected terminal output destination.  The
    * terminal must be non-interactive for this method to process properly.
    * 
    * @param    frameId
    *           The id of the frame for this request.
    * @param    header
    *           <code>true</code> to register as a header, <code>false</code>
    *           to register as a footer.
    */
   public void deregisterPageElement(int frameId, boolean header);
   
   /**
    * Deregister (remove) the frame as a header or footer from the given
    * stream.
    * <p>
    * This is called (among other places) during hide processing on the
    * server, which must be safe to be called at any time.  This is true,
    * even if we aren't redirected, if the frame is not visible and/or if
    * there was no page element ever registered for this frame (because it
    * was never viewed).
    *
    * @param    frameId
    *           The id of the frame for this request.
    * @param    header
    *           <code>true</code> to register as a header, <code>false</code>
    *           to register as a footer.
    * @param    streamId
    *           The stream to associate with the frame as a page element.
    */
   public void deregisterPageElement(int     frameId,
                                     boolean header,
                                     int     streamId);
                                     
   /**
    * Deregister (remove) the frame as a header or footer from all streams
    * in which it is registered.
    *
    * @param    frameId
    *           The id of the frame for this request.
    * @param    header
    *           <code>true</code> to register as a header, <code>false</code>
    *           to register as a footer.
    */
   public void forceDeregisterPageElement(int frameId, boolean header);
   
   /**
    * Marks the end of the editing block when triggered by any abnormal exit
    * from the block on the server.  This method does NOT need to be called
    * if the exit is naturally triggered via {@link #continueEditing}. This
    * method allows the client's state to be cleaned up in the abnormal
    * termination case.
    *           
    * @return   The screen buffer array for currently editable frames.
    */
   public ScreenBuffer[] stopEditingMode();
   
   /**
    * Conditionally blocks until a next UI event is available.
    */
   public void waitForNextKey();
   
   /**
    * Checks to see whether a pending GO event is about to end the editing block.
    *
    * @param    hadNext
    *           <code>true</code> if the executing business logic executed a <code>NEXT</code>
    *           statement causing the editing block to iterate. This will result in clearing
    *           any current <code>GO-PENDING</code> condition.  This is passed as a parameter
    *           here so that a separate trip down to the client is not needed to process the
    *           editing block iteration state update.
    *
    * @return   <code>true</code> if there is no <code>GO-PENDING</code> and the editing mode
    *           continues.
    */
   public boolean continueEditing(boolean hadNext);

   /**
    * Checks whether the CHUI is up and running on the client.
    *
    * @return   <code>true</code> if client runs in CHUI mode.
    */
   public boolean isChui();

   /**
    * Check the client type: embedded or not.
    * 
    * @return   <code>TRUE</code> if the client is running in embedded mode, 
    *           <code>FALSE</code> otherwise 
    */
   public boolean isEmbedded();

   /**
    * Sets the SESSION:TOOLTIPS attribute for this session.
    *
    * @param    tooltips
    *           The new value for session related TOOLTIPS attribute.
    */
   public void setSessionTooltips(boolean tooltips);
   
   /**
    * Activates tootip feature for particular widget.
    *
    * @param   wId
    *          The widget ID to activate tooltip.
    */
   public void activateTooltipForWidget(int wId);
   
   /**
    * Deactivates tootip feature for particular widget.
    *
    * @param   wId
    *          The widget ID to deactivate tooltip.
    */
   public void deactivateTooltipForWidget(int wId);
   
   /**
    * Destroy the frame and free associated resources as needed. If the frame
    * is a dialog, it will be hidden immediately, otherwise it will be marked
    * as out of scope and valid for being hidden and resources will be
    * released when frame will be hidden.
    *
    * @param   frameId
    *          The frame requiring cleanup.
    */
   public void destroyFrame(int frameId);
   
   /**
    * Checks whether a GO event is pending for this editing block.
    * <p>
    * This method corresponds to the Progress built-in function: <br>
    * <b>GO-PENDING </b>
    * 
    * @return  yes if an event has been applied that caused pending GO for this
    *          editing block.
    */
   public boolean isGoPending();
   
   /**
    * Perform pause before end of application.
    */
   public void pauseBeforeEnd();
   
   /**
    * Request indexes of selected items in RadioSet, ComboBox or SelectionList
    * widgets.
    * 
    * @param   widgetId
    *          Widget ID.
    * 
    * @return  array of 0-based indexes of selected items or <code>null</code>
    *          if specified widget does not exists or does not support such a
    *          request.
    */
   public int[] getSelection(int widgetId);
   
   /**
    * Redirect terminal output to the stream represented by <code>newId</code>
    * (or to the interactive terminal if <code>newId</code> is -1) and close/
    * cleanup resources for the pseudo-terminal associated with
    * <code>oldId</code> (unless <code>oldId</code> is -1 in which case 
    * nothing needs to be closed).
    *
    * @param   oldId
    *          The previously redirected stream for which the pseudo-terminal
    *          is to be closed or -1 for no close/cleanup action.
    * @param   newId
    *          The stream for which a pseudo-terminal is to be setup for
    *          redirection or -1 to remove redirections and activate the
    *          interactive terminal.
    */
   public void redirectOutput(int oldId, int newId);
   
   /**
    * Redirect terminal output to the stream represented by <code>newId</code>
    * (or to the interactive terminal if <code>newId</code> is -1) and return
    * the id of the current output destination.  This may be another
    * redirected terminal or it may be -1 if the interactive terminal is in 
    * use.
    * <p>
    * If redirection has not yet been established for the given stream, it
    * will be setup.
    *
    * @param    newId
    *           The stream for which a redirected terminal is to be setup for
    *           redirection or -1 to remove redirections and activate the
    *           interactive terminal.
    * @param    clean
    *           <code>true</code> if this stream will need deferred cleanup.
    * @param    endOut
    *           <code>true</code> to force any pending PUT or EXPORT output
    *           to be ended for the given stream (so long as that stream is
    *           not the terminal).
    *
    * @return   The id of the previous terminal or -1 if the previous 
    *           terminal was interactive.   
    */
   public int switchOutput(int newId, boolean clean, boolean endOut);
   
   /**
    * Called when a stream is closed to ensure that any terminal resources
    * associated with that id are cleaned up.  While this will be called
    * for any output stream close, actual cleanup processing is only 
    * triggered when the given stream id was used at least once via the
    * {@link #switchOutput} interface.
    * <p>
    * This should never be called when the redirection is currently active.
    *
    * @param    id
    *           The stream needing cleanup.
    */
   public void cleanupPseudoTerminal(int id);
   
   /**
    * Obtains the next widget in the frame's tab order from the client.
    *
    * @param    frameId
    *           frame where the widget is placed
    *
    * @param    widgetId
    *           target widget which is being queried
    *
    * @return   id of the widget or<code>-1</code>.
    */
   public int nextTabItem(int frameId, int widgetId);

   /**
    * Reset buffer size manager to its initial state. 
    */
   public void resetBufferSizeManager();
   
   /**
    * Get the PIXELS-PER-COLUMN value for this client session.
    * 
    * @return   See above.
    */
   public int getPixelsPerColumn();
   
   /**
    * Get the PIXELS-PER-ROW value for this client session.
    * 
    * @return   See above.
    */
   public int getPixelsPerRow();
   
   /**
    * Displays the {@code File Selection} dialog box and prompt the user to select a file with
    * configured constraints set by the options of the {@code SYSTEM-DIALOG PRINTER-SETUP}
    * statement.
    *
    * @param   windowId
    *          The id of the window that will be the parent for the dialog.
    * @param   path
    *          The initial selected filename, (only if {@code useFilename == true}).
    * @param   title
    *          The dialog's title (optional).
    * @param   findDirectories
    *          Allow only directories to be selected.
    * @param   saveAs
    *          This is a {@code Save As...} dialog.
    * @param   askOverwrite
    *          Prompt the user for overwriting if the file exists. Used only with {@code saveAs}
    *          option. 
    * @param   mustExist
    *          Enforce that the file should already exist.
    * @param   createTestFile
    *          Test writing permissions by creating a temporary file. 
    * @param   returnToStartDir
    *          Do not save the last location where the user navigated for subsequent calls.
    * @param   useFilename
    *          If {@code true}, use the {@code path} as the starting location.
    * @param   defaultExtension
    *          Configures the default extension for files to be selected.
    * @param   initialDir
    *          Configures the initial directory where the dialog is opened.
    * @param   filters
    *          A set of filters to be displayed to user.
    * @param   initialFilter
    *          The index of the filter to be active when the dialog opens.
    *
    * @return  If the user accepted the dialog and selected a certain file/dir, then the full
    *          absolute path is returned. Otherwise {@code null} is returned (user selected 
    *          {@code Cancel} button to close the dialog).
    */
   public String chooseFile(int windowId,
                            String path,
                            String title, boolean findDirectories,
                            boolean saveAs,
                            boolean askOverwrite,
                            boolean mustExist,
                            boolean createTestFile,
                            boolean returnToStartDir,
                            boolean useFilename,
                            String defaultExtension,
                            String initialDir,
                            String[] filters,
                            int initialFilter);
   
   /**
    * Change the specified color, in the user's environment.
    * This implements the <code>SYSTEM-DIALOG COLOR color-num</code> statement.
    *
    * @param    rgbColor
    *           The current RGB of the palette entry.
    * @param    windowId
    *           The window ID to which the edited font-table belongs.
    *
    * @return  the RGB color selected if the dialog was accepted and -1 if the user cancelled the
    *          dialog.
    */
   public int chooseColor(int rgbColor, int windowId);
   
   /**
    * Change the specified font, in the user's environment.
    * This implements the <code>SYSTEM-DIALOG FONT font-num</code> statement.
    *
    * @param   fontNum
    *          The font-table entry to be changed.
    * @param   windowId
    *          The id of the window that will be the parent for the dialog.
    * @param   ansiOnly
    *          Do not include graphic symbols in the set of fonts to be selected.
    * @param   fixedOnly
    *          Only allow mono-spaced fonts to be selected.
    * @param   minSize
    *          The minimum size of the font to be allowed to be selected.
    * @param   maxSize
    *          The maximum size of the font to be allowed to be selected.
    * 
    * @return  {@code true} if the dialog was accepted and {@code false} if the user cancelled the
    *          dialog.
    */
   public boolean chooseFont(int fontNum,
                             int windowId,
                             boolean ansiOnly,
                             boolean fixedOnly,
                             int minSize,
                             int maxSize);
   
   /**
    * Displays the {@code Print} dialog box and lets the user set the default print context for
    * subsequent print jobs.
    * <p>
    * This implements the {@code SYSTEM-DIALOG PRINTER-SETUP} statement.
    *
    * @param   windowId
    *          The id of the window that will be the parent for the dialog.
    * @param   copies
    *          The initial value of {@code Copies} field. Ignored if negative.
    * @param   printOptions
    *          The initial print options.
    *
    * @return  Returns the print options set in the print setup dialog or <code>null</code> if the dialog is
    *          canceled by the user.
    */
   public PrintOptions setupPrinter(int windowId, int copies, PrintOptions printOptions);
   
   /**
    * Calls the Microsoft Windows Help engine to display Windows Help topics, and the HTML Help
    * engine to display HTML Help topics.
    * @param   fileName
    *          Path to the help file.
    * @param   windowName
    *          The window name (optional, amy be null).
    * @param   action
    *          Help action. See {@link SystemHelpAction} for more information.
    * @param   params
    *          Parameters specific to current action.
    */
   public void systemHelp(String fileName,
                          String windowName,
                          SystemHelpAction action,
                          Object[] params);
   
   /**
    * Set the number of entries in the font-table.
    * 
    * @param    numEntries
    *           The new size of the font-table. 
    */
   public void adjustFontTableSize(int numEntries);
   
   /**
    * Get the number of entries in the font-table.
    * 
    * @return   See above.
    */
   public int fontTableSize();
   
   /**
    * Get the width of the specified text, in pixels, using the specified font.  
    * 
    * @param    txt
    *           The text.
    * @param    fontNum
    *           The entry in the font-table.
    *           
    * @return   See above.
    */
   public int getTextWidthPixels(String txt, int fontNum);
   
   /**
    * Get the height in pixels, of the specified font.
    * 
    * @param    fontNum
    *           The entry in the font-table.
    * 
    * @return   See above.
    */
   public int getFontHeightPixels(int fontNum);
   
   /**
    * Resolve the font key associated with the given font, in the current window's font-table.
    * 
    * @param    fontNum
    *           The entry in the font-table.
    * 
    * @return   See above.
    */
   public String getFontKey(int fontNum);
   
   /**
    * Inform the client-side that it can start sending the config updates.
    */
   public void activateConfigUpdates();

   /**
    * Returns display width in native units.
    * 
    * For multiple-display environments the display is selected based on the supplied window id.
    * 
    * @param    windowId
    *           A valid window id.
    * 
    * @return   The width in native units.
    */
   public int getDisplayWidth(int windowId);

   /**
    * Returns display height in native.
    * 
    * For multiple-display environments the display is selected based on the supplied window id.
    * 
    * @param    windowId
    *           A valid window id.
    * 
    * @return   The height in native.
    */
   public int getDisplayHeight(int windowId);
   
   /**
    * Returns the X coordinate of the application-usable display area. For example, a task bar is
    * not counted as the usable area.
    * 
    * For multiple-display environments the display is selected based on the supplied window id.
    * 
    * @param    windowId
    *           A valid window id.
    *
    * @return   The value in native units.
    */
   public int getDisplayWorkAreaX(int windowId);

   /**
    * Returns the Y coordinate of the application-usable display area. For example, a task bar is
    * not counted as the usable area.
    * 
    * For multiple-display environments the display is selected based on the supplied window id.
    * 
    * @param    windowId
    *           A valid window id.
    *
    * @return   The value in native units.
    */
   public int getDisplayWorkAreaY(int windowId);
   
   /**
    * Returns the width of the application-usable display area. For example, a task bar is
    * not counted as the usable area.
    * 
    * For multiple-display environments the display is selected based on the supplied window id.
    * 
    * @param    windowId
    *           A valid window id.
    *
    * @return   The value in native units.
    */
   public int getDisplayWorkAreaWidth(int windowId);
   
   /**
    * Returns the height of the application-usable display area. For example, a task bar is
    * not counted as the usable area.
    * 
    * For multiple-display environments the display is selected based on the supplied window id.
    * 
    * @param    windowId
    *           A valid window id.
    *
    * @return   The value in native units.
    */
   public int getDisplayWorkAreaHeight(int windowId);
   
   /**
    * Sets the new application icon for given window on the client side.
    * 
    * @param    windowId
    *           A valid window id.
    * @param    icon
    *           A new icon image array data.
    *
    * @return   <code>TRUE</code> if operation successful <code>FALSE</code> otherwise.
    */
   public boolean setWindowIcon(int windowId, byte[] icon);
   
   /**
    * Loads the new application icon for given window from the client side filesystem.
    * 
    * @param    windowId
    *           A valid window id.
    * @param    small
    *           Flag indicating usage of the small icon case(16x16 in size).
    * @param    name
    *           The file name for the icon to be loaded.
    *
    * @return   <code>TRUE</code> if operation successful <code>FALSE</code> otherwise.
    */
   public boolean loadWindowIcon(int windowId, boolean small, String name);
   
   /**
    * Sets the default window icon for all new windows on the client side.
    * 
    * @param    type
    *           Data type for second parameter.
    * @param    data
    *           Data to create new icon on client side.  Can be the byte array, filename string
    *           or nothing for predefined icon from p2j.jar client side file. 
    *
    * @return   <code>TRUE</code> if operation successful <code>FALSE</code> otherwise.
    */
   public boolean setWindowDefaultIcon(Window.IconDataType type, Object data);

   /**
    * Sets the new image for given button or image widgets on the client side.
    * 
    * @param    wId
    *           A valid wdget id.
    * @param    type
    *           Image type to load.  Up, Down or Insensitive for buttons, -1 for image.
    * @param    image
    *           A new image array data.
    *
    * @return   <code>TRUE</code> if operation successful <code>FALSE</code> otherwise.
    */
   public boolean setImage(int wId, int type, byte[] image);

   /**
    * Loads the new image for given button or image widgets from the client side filesystem.
    * 
    * @param    wId
    *           A valid wdget id.
    * @param    type
    *           Image type to load.  Up, Down or Insensitive for buttons, -1 for image.
    * @param    name
    *           The file name for the image to be loaded.
    *
    * @return   <code>TRUE</code> if operation successful <code>FALSE</code> otherwise.
    */
   public boolean loadImage(int wId, int type, String name);

   /**
    * Save the content in the widgets in the client side filesystem.
    *
    * @param    wId
    *           A valid editor widget id.
    * @param    name
    *           The file name to be written.
    *
    * @return   <code>TRUE</code> if operation successful <code>FALSE</code> otherwise.
    */
   public boolean editorSaveFile(int wId, String name);

   /**
    * Calls the client side to implement SCROLL-TO-ITEM() method for selection-list widget.
    *
    * @param    slId
    *           Widget ID of the selection list to make scrolling.
    * @param    index
    *           The 1-based index of an item to scroll.
    *
    * @return   <code>TRUE</code> in success, <code>FALSE</code> otherwise.
    */
   public boolean scrollToItem(int slId, int index);

   /**
    * Loads the file to the widget from the client side filesystem.
    *
    * @param    wId
    *           A valid editor widget id.
    * @param    name
    *           The file name to be loaded.
    *
    * @return   <code>TRUE</code> if operation successful <code>FALSE</code> otherwise.
    */
   public boolean editorReadFile(int wId, String name);
   
   /**
    * Get the THREE-D value for this client session.
    * 
    * @return   See above.
    */
   public boolean is3D();
   
   /**
    * Set the THREE-D value for this client session.
    * 
    * @param    value
    *           The value to set.
    */
   public void set3D(boolean value);
   
   /**
    * Set the global SESSION:DISABLE-REDRAW attribute.
    * This attribute is a FWD extension, not real 4GL!
    * 
    * @param    value
    *           New flag state.
    */
   public void setDisableRedraw(boolean value);

   /**
    * Utility method for debugging, can be used to include information from the server-side
    * to the client-side log
    *
    * @param id
    *        widget id
    * @param message
    *       String parameter
    */
   public void debug(int id, String message);

   /**
    * Move the widget to the top in z-order
    * 
    * @param id
    *          widget id
    *          
    * @return <code>Boolean.TRUE</code> if operation was successful
    *         null if no widget with given id is found
    */
   public Boolean moveToTop(int id);

   /**
    * Move the widget to the Bottom in z-order
    * 
    * @param id
    *          widget id
    *          
    * @return <code>Boolean.TRUE</code> if operation was successful
    *         null if no widget with given id is found
    */
   public Boolean moveToBottom(int id);

   /**
    * Set the pre-defined custom mouse pointer for a widget or reset to the default one
    * 
    * @param id
    *        the widget id
    * @param ptr
    *        the new custom mouse pointer or <code>null</code>
    *               
    * @return <code>true</code> if operation was successful and null if 
    *         the pre-defined cursor is not supported yet
    */
   public Boolean setMousePointer(int id, MousePointer ptr);

   /**
    * Set the custom mouse pointer for a widget or reset to the default one
    * 
    * @param id
    *        the widget id
    * @param data
    *        the pointer data
    * @param ptrName
    *        the pointer name
    *               
    * @return <code>true</code> if operation was successful       
    */
   public boolean setMousePointer(int id, byte[] data, String ptrName);

   /**
    * Load custom mouse pointer from a file
    * 
    * @param id
    *        widget id
    * @param fileName
    *        the file name of the pointer data
    * 
    * @return <code>true</code> if successful.
    */
   public boolean loadMousePointer(int id, String fileName);

   /**
    * Initiate/cancel wait state
    * 
    * @param ptr 
    *        new mouse pointer or <code>null</code> for cancel 
    * 
    * @return <code>true</code> if operation was successful       
    */
   public boolean setWaitState(MousePointer ptr);

   /**
    * Initiate wait state with a custom cursor from file
    * 
    * @param ptrName 
    *        the name of the file with new mouse pointer data 
    * 
    * @return <code>true</code> if operation was successful       
    */
   public boolean setWaitState(String ptrName);

   /**
    * Initiate wait state with a custom cursor from file
    * 
    * @param data 
    *        the new mouse pointer data 
    * 
    * @return <code>true</code> if operation was successful       
    */
   public boolean setWaitState(byte[] data);

   /**
    * Set the state of SESSION:IMMEDIATE-DISPLAY attribute.
    * 
    * @param   on
    *          The new state for SESSION:IMMEDIATE-DISPLAY attribute.
    */
   public void setImmediateDisplay(boolean on);

   /**
    * Set the state of SESSION:DATA-ENTRY-RETURN attribute.
    * 
    * @param   on
    *          The new state for SESSION:DATA-ENTRY-RETURN attribute.
    */
   public void setDataEntryReturn(boolean on);
   
   /**
    * Write message and stack trace to the client log
    * 
    * @param message
    *        the message
    * @param stack
    *        the Progress stack trace
    */
   public void clientLog(String message, String[] stack);

   /**
    * Write message and stack trace to the client log
    * 
    * @param message
    *        the message
    */
   public void logStackTrace(String message);

   /**
    * Opens the specified resource in a browser's window.
    * 
    * @param    mimeType
    *           The resource mime type.
    * @param    path
    *           The url path to the target resource.
    * @param    embedded
    *           The document is embedded in the template document.
    */
   public void openMimeResource(String mimeType, String path, boolean embedded);

   /**
    * This is a special API which is used to delegate a call from converted 4GL code 
    * (via _P2J_REMOTE_CALL_ functions) to the remote side.  Usually, this is used when P2J client
    * runs in embedded mode, i.e. the P2J screens are embedded in another customer-specific 
    * application.  The call will be remoted into the customer-specific application.
    * <p>
    * This API is expected to block waiting for a response.
    * 
    * @param    request
    *           The request data incoming from 4GL.
    * 
    * @return   The response from the customer-specific application, which will be sent back to 
    *           the converted 4GL code.
    */
   public UnformattedPayload remoteCall(UnformattedPayload request);

   /**
    * Send a PUBLISH notification to a remote application. 
    * 
    * @param    subId
    *           The subscriber's ID, known by the remote application.
    * @param    event
    *           The published event name.
    * @param    param
    *           The parameters for this event.
    * @param    delayed
    *           Flag indicating if the PUBLISH must be executed immediately (when <code>true</code>)
    *           or posted to the client's event queue, otherwise.
    */
   public void publish(long subId, String event, String[] param, boolean delayed);
   
   /**
    * Exposes the client's system clipboard to the server.
    * 
    * @return   The clipboard content.
    */
   public character getClipboardValue();

   /**
    * Copies a string value to the client's system clipboard.
    * 
    * @param    value
    *           The string to be stored in the client's system clipboard.
    */
   public void setClipboardValue(String value);
   
   /**
    * Obtain the default system palette for current theme from client. Calls
    * {@code ThemeManager.getCurrentTheme().getDefaultSystemPalette()} remotely on the client.
    *
    * @return  the default system palette for current theme from client.
    */
   public Map<Integer, ColorRgb> getThemeSystemPalette();

   /**
    * Returns current mouse cursor position relative to the provided widget id. If the widget id
    * equals to -1 or it is invalid, the method returns position relative to the screen origin.
    * <p>
    * The method is only supported in GUI mode.
    *
    * @param   widgetId
    *          A widget id.
    *
    * @return  mouse cursor position, x coordinate at index 0, y coordinate at index 1, never
    *          <code>null</code>
    */
   public int[] getMousePosition(int widgetId);

   /**
    * Terminates file dropping operation and clean up resources.
    * 
    * @return  <code>TRUE</code> in success, <code>FALSE</code> otherwise.
    */
   public boolean endFileDrop();
   
   /**
    * Gets the number of the dropped files.
    * 
    * @return  The integer number for dropped file list size.
    */
   public int getNumDroppedFiles();
   
   /**
    * Gets the name of the dropped file from internal 1-based list list.
    * 
    * @param   index
    *          1-based number of the file name to get.
    *          
    * @return  The name of the file with given index.
    */
   public String getDroppedFile(int index);
}
