/*
** Module   : ComboBoxConfig.java
** Abstract : stores combo box specific state
**
** Copyright (c) 2005-2017, Golden Code Development Corporation.
**
** -#- -I- --Date-- --JPRM-- ----------------Description-----------------
** 001 EVL 20050714   @21692 Creation based on general concept. Necessary
**                           modification to get compiled.
** 002 EVL 20050723   @21804 Adding one more attribute: innerLines.
** 003 SIY 20050831   @22491 Refactoring (see @@22394-22409) 
** 004 NVS 20051116   @23343 Class access changed to public. 
** 005 SIY 20051129   @23561 Moved item list member to parent class.
** 006 SIY 20060209   @24485 Major refactoring.
** 007 SIY 20060209   @24496 Fixed trap in setMaximumRowCount.
** 008 NVS 20060313   @25040 Moved format, setFormat() and getFormat() to
**                           the superclass, ControlSetConfig, so that
**                           format methods are available in SelectionList
**                           as well.
** 009 GES 20060803   @28397 Made externalizable for better performance.
**                           Many code standard cleanups.
** 010 EVL 20060809   @28497 Fix for DropDown line mismatch.
** 011 SIY 20061213   @31674 Encapsulated fields.
** 012 SIY 20100619          Switching to own implementation of character UI.
** 013 SIY 20100804          Changes in imports, some refactorings.
** 014 SIY 20111101          Moved from CHUI package. Minor 
**                           cleanups/refactorings in preparation to GUI client.
** 015 SIY 20111202          Updated imports.
** 016 CA  20140926          Added shared widget configuration support, to allow GUI/ChUI
**                           concrete implementation for the same widget ID.  Refactored the
**                           widget configuration classes: all fields were made public; these
**                           classes need to be as dumb as possible, all logic related to
**                           setting/getting a certain field should be at the widget or at the
**                           caller. Refs #2254
** 017 SVL 20141102          Added MAX-CHARS attribute.
** 018 GES 20141124          Fixed javadoc.
** 019 CA  20150105          Changes for SIDE-LABEL-HANDLE runtime support.
** 020 IAS 20150604          Added INNER-LINES and SELECTED-TEXT attributes' support
** 021 EVL 20150528          Adding different combo-box mode types.  Changing MAX-CHARS attribute
**                           storage type and default value to be comatible with current 4GL
**                           behaviour.
** 022 HC  20171024          Runtime support for SYSTEM-DIALOG PRINT-SETUP and OUTPUT TO PRINTER.
*/
/*
** This program is free software: you can redistribute it and/or modify
** it under the terms of the GNU Affero General Public License as
** published by the Free Software Foundation, either version 3 of the
** License, or (at your option) any later version.
**
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU Affero General Public License for more details.
**
** You may find a copy of the GNU Affero GPL version 3 at the following
** location: https://www.gnu.org/licenses/agpl-3.0.en.html
** 
** Additional terms under GNU Affero GPL version 3 section 7:
** 
**   Under Section 7 of the GNU Affero GPL version 3, the following additional
**   terms apply to the works covered under the License.  These additional terms
**   are non-permissive additional terms allowed under Section 7 of the GNU
**   Affero GPL version 3 and may not be removed by you.
** 
**   0. Attribution Requirement.
** 
**     You must preserve all legal notices or author attributions in the covered
**     work or Appropriate Legal Notices displayed by works containing the covered
**     work.  You may not remove from the covered work any author or developer
**     credit already included within the covered work.
** 
**   1. No License To Use Trademarks.
** 
**     This license does not grant any license or rights to use the trademarks
**     Golden Code, FWD, any Golden Code or FWD logo, or any other trademarks
**     of Golden Code Development Corporation. You are not authorized to use the
**     name Golden Code, FWD, or the names of any author or contributor, for
**     publicity purposes without written authorization.
** 
**   2. No Misrepresentation of Affiliation.
** 
**     You may not represent yourself as Golden Code Development Corporation or FWD.
** 
**     You may not represent yourself for publicity purposes as associated with
**     Golden Code Development Corporation, FWD, or any author or contributor to
**     the covered work, without written authorization.
** 
**   3. No Misrepresentation of Source or Origin.
** 
**     You may not represent the covered work as solely your work.  All modified
**     versions of the covered work must be marked in a reasonable way to make it
**     clear that the modified work is not originating from Golden Code Development
**     Corporation or FWD.  All modified versions must contain the notices of
**     attribution required in this license.
*/

package com.goldencode.p2j.ui;

import java.io.*;

/**
 * Stores combo-box specific state.
 */
public class ComboBoxConfig
extends ControlSetConfig
{
   /** The Progress AUTO-ZAP attribute. */
   public boolean autoZap = true;

   /** The number of text lines in the widget. */
   public int innerLines = 0;
   
   /** The number of assignments to INNER-LINES since the last activation */
   public int innerLinesAssignments = 0;
   
   /** The max value of INNER-LINES */
   public int innerLinesLimit = 0;

   /** The adjustement of  innerLinesLimit after the widget activation*/
   public int innerLinesLimitDelta = 0;

   /** MAX-CHARS attribute. */
   public short maxChars = 0;
   
   /** Working mode for combo-box. */
   public static enum Mode
   {
      SIMPLE, DROP_DOWN, DROP_DOWN_LIST
   }
   
   /** Current combo-box mode. */
   public Mode mode = Mode.DROP_DOWN_LIST;
   
   /** Index of the selected item */
   public int selectedIndex = -1;
   
   /**
    * Default constructor (only used in de-serialization).
    */
   public ComboBoxConfig()
   {
      possibleSideLabel = true;
   }

   /**
    * Create a new config and associate it with the given widget.
    * 
    * @param    id
    *           The ID of the widget to which this configuration belongs.
    */
   public ComboBoxConfig(int id)
   {
      super(id);

      possibleSideLabel = true;
   }

   /**
    * Create a new config and associate it with the given widget.
    * 
    * @param    id
    *           The ID of the widget to which this configuration belongs.
    */
   public ComboBoxConfig(WidgetId id)
   {
      super(id);

      possibleSideLabel = true;
   }

   /**
    * Set new values from the provided configuration instance.
    * 
    * @param    config
    *           The instance from which to take values.
    */
   @Override
   public void applyConfig(WidgetConfig config)
   {
      super.applyConfig(config);
      
      ComboBoxConfig cfg = (ComboBoxConfig) config;
      
      autoZap = cfg.autoZap;
      innerLines = cfg.innerLines;
      maxChars = cfg.maxChars;
      mode = cfg.mode;
      innerLinesAssignments = cfg.innerLinesAssignments;
      innerLinesLimit = cfg.innerLinesLimit;
      innerLinesLimitDelta = cfg.innerLinesLimitDelta;
      selectedIndex = cfg.selectedIndex;
   }

   /**
    * Replacement for the default object reading method. The latest
    * state is read from the input source.  The parent's state is
    * read first.
    * 
    * @param    in
    *           The input source from which fields will be restored.
    *
    * @throws   IOException
    *           In case of I/O errors.
    * @throws   ClassNotFoundException
    *           If payload can't be instantiated.
    */
   @Override
   public void readExternal(ObjectInput in)
   throws IOException,
          ClassNotFoundException
   {
      super.readExternal(in);
      innerLines     = in.readInt();
      autoZap        = in.readBoolean();
      maxChars       = in.readShort();
      innerLinesAssignments = in.readInt();
      innerLinesLimit = in.readInt();
      innerLinesLimitDelta = in.readInt();
      selectedIndex = in.readInt();
      mode           = (Mode) in.readObject();
   }

   /**
    * Replacement for the default object writing method. The latest
    * state is written to the output destination.  The parent's state is
    * written first.
    * 
    * @param    out
    *           The output destination to which fields will be saved.
    *
    * @throws   IOException
    *           In case of I/O errors.
    */
   @Override
   public void writeExternal(ObjectOutput out)
   throws IOException
   {
      super.writeExternal(out);
      out.writeInt(innerLines);
      out.writeBoolean(autoZap);
      out.writeShort(maxChars);
      out.writeInt(innerLinesAssignments);
      out.writeInt(innerLinesLimit);
      out.writeInt(innerLinesLimitDelta);
      out.writeInt(selectedIndex);
      out.writeObject(mode);
   }
}