/*
** Module   : MenuItemInterface.java
** Abstract : Define all MENU-ITEM-specific attributes and methods available via a handle instance.
**
** Copyright (c) 2015-2017, Golden Code Development Corporation.
**
** -#- -I- --Date-- ---------------------------------Description---------------------------------
** 001 VIG 20150818 Created initial version.
** 002 SVL 20170120 Added TOGGLE-BOX attribute.
** 003 CA  20170929 Fixed ACCELERATOR and TOGGLE-BOX attribute getter return type.
**                  Added MNEMONIC and PREPROCESSED-LABEL FWD extension attributes.
*/
/*
** This program is free software: you can redistribute it and/or modify
** it under the terms of the GNU Affero General Public License as
** published by the Free Software Foundation, either version 3 of the
** License, or (at your option) any later version.
**
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU Affero General Public License for more details.
**
** You may find a copy of the GNU Affero GPL version 3 at the following
** location: https://www.gnu.org/licenses/agpl-3.0.en.html
** 
** Additional terms under GNU Affero GPL version 3 section 7:
** 
**   Under Section 7 of the GNU Affero GPL version 3, the following additional
**   terms apply to the works covered under the License.  These additional terms
**   are non-permissive additional terms allowed under Section 7 of the GNU
**   Affero GPL version 3 and may not be removed by you.
** 
**   0. Attribution Requirement.
** 
**     You must preserve all legal notices or author attributions in the covered
**     work or Appropriate Legal Notices displayed by works containing the covered
**     work.  You may not remove from the covered work any author or developer
**     credit already included within the covered work.
** 
**   1. No License To Use Trademarks.
** 
**     This license does not grant any license or rights to use the trademarks
**     Golden Code, FWD, any Golden Code or FWD logo, or any other trademarks
**     of Golden Code Development Corporation. You are not authorized to use the
**     name Golden Code, FWD, or the names of any author or contributor, for
**     publicity purposes without written authorization.
** 
**   2. No Misrepresentation of Affiliation.
** 
**     You may not represent yourself as Golden Code Development Corporation or FWD.
** 
**     You may not represent yourself for publicity purposes as associated with
**     Golden Code Development Corporation, FWD, or any author or contributor to
**     the covered work, without written authorization.
** 
**   3. No Misrepresentation of Source or Origin.
** 
**     You may not represent the covered work as solely your work.  All modified
**     versions of the covered work must be marked in a reasonable way to make it
**     clear that the modified work is not originating from Golden Code Development
**     Corporation or FWD.  All modified versions must contain the notices of
**     attribution required in this license.
*/

package com.goldencode.p2j.ui;

import com.goldencode.p2j.util.*;

/**
 * Define all MENU-ITEM-specific widget attributes and methods which can be accessed via a handle
 * instance.  Attributes and methods shared by multiple widget types will be defined elsewhere.
 */
public interface MenuItemInterface
extends MnemonicInterface
{
   /**
    * Gets this menu-item accelerator - KEYLABEL of the short key assigned to this menu-item.
    * 
    * @return  See above.
    */
   @LegacyAttribute(name = "ACCELERATOR")
   public character getAccelerator();
   
   /**
    * Sets this menu-item accelerator - KEYLABEL of the short key assigned to this menu-item.
    * 
    * @param  accelerator
    *         The valid key-label, that will be target accelerator.
    */
   @LegacyAttribute(name = "ACCELERATOR", setter = true)
   public void setAccelerator(String accelerator);

   /**
    * Sets this menu-item accelerator - KEYLABEL of the short key assigned to this menu-item.
    * 
    * @param  accelerator
    *         The valid key-label, that will be target accelerator.
    */
   @LegacyAttribute(name = "ACCELERATOR", setter = true)
   public void setAccelerator(character accelerator);

   /**
    * Gets the TOGGLE-BOX attribute value.
    *
    * @return TOGGLE-BOX attribute value: <code>true</code> if the menu-item should have a
    *         checkbox.
    */
   @LegacyAttribute(name = "TOGGLE-BOX")
   public logical isToggleBox();

   /**
    * Sets the TOGGLE-BOX attribute of the menu-item.
    *
    * @param    toggleBox
    *           <code>true</code> if the menu-item should have a checkbox.
    */
   @LegacyAttribute(name = "TOGGLE-BOX", setter = true)
   public void setToggleBox(boolean toggleBox);

   /**
    * Sets the TOGGLE-BOX attribute of the menu-item.
    *
    * @param    toggleBox
    *           <code>true</code> if the menu-item should have a checkbox.
    */
   @LegacyAttribute(name = "TOGGLE-BOX", setter = true)
   public void setToggleBox(logical toggleBox);
}