/*
** Module   : Button.java
** Abstract : push button implementation
**
** Copyright (c) 2005-2017, Golden Code Development Corporation.
**
** -#- -I- --Date-- --JPRM-- ----------------------------Description------------------------------
** 001 EVL 20050124   @19398 Creation based on CHARVA JButton class.
**                           Necessary modification to get compiled.
** 002 EVL 20050203   @19559 Rollback the #001 approach. Only draw() and
**                           probably processKeyEvent() need to be
**                           replaced.
** 003 EVL 20050207   @19738 Changes to apply SPACEBAR as selector key.
** 004 EVL 20050215   @19829 Adding simulation of Progress behavior for.
**                           focus moving by arrow keys.
** 005 EVL 20050221   @19908 Enhanced moving focus by arrow keys.
** 006 EVL 20050222   @19917 Width dynamic change handling.
** 007 EVL 20050411   @20670 Changing location detection to the
**                           getLocation call instead of
**                           getLocationOnScreen to be able to get the
**                           coordinates relative to the Button's parent
**                           container which can be inside the view-port.
** 008 EVL 20050601   @21373 Changing the inner variable names to sync
**                           with the rest of the project.
** 009 EVL 20050601   @21374 Adding the Progress specific methods.
** 010 EVL 20050602   @21392 Syntax changes to the javadoc comments.
** 011 EVL 20050607   @21435 Syntax for comments checking.
** 012 EVL 20050607   @21438 Removing the visible member. It is completely
**                           duplicated in CHARVA's Component superclass
**                           so we do not need to double this work. Also
**                           the methods to work with the HIDDEN attribute
**                           are modified to reflect this change.
** 013 EVL 20050610   @21483 Rework the getting next/previous siblings for
**                           the button widget.
** 014 EVL 20050614   @21484 Rework the getting next/previous tab item for
**                           the button widget.
** 015 EVL 20050615   @21510 Adoption the attribute to the CHARVA model.
**                           The attributes TAB-POSITION, ROW, COLUMN,
**                           HELP, FRAME-ROW, FRAME-COLUMN are touched.
** 016 EVL 20050729   @21915 Removing the ugly tabulation from source.
** 017 EVL 20050822   @22199 Using the P2JFrame/Container interfaces to
**                           detect the class.
** 018 SIY 20050831   @22487 Refactoring (see @@22394-22409) 
** 019 SIY 20050907   @22624 Fixed use of @inheritDoc tag in javadoc, 
**                           fixed constructor, fixed automatic resizing.
** 020 SIY 20050920   @22826 Added support for SELECTED attribute. 
**                           Using ColorTable for the colors, added
**                           support for PFCOLOR attribute.  
** 021 SIY 20051129   @23557 Commented out call to setParentAttribute().
** 022 SIY 20060131   @24197 Use ThinClient special method instead of
**                           direct calls to Window.message().
** 023 SIY 20060209   @24491 Cleanups and updates to accommodate changes
**                           in xxxConfig classes.
** 024 SIY 20060215   @24595 Refactoring and fixes to make it working 
**                           properly in AlertBox.
** 025 NVS 20060227   @24863 Key actions KA_* are used instead of virtual
**                           keys VK_* for all editing operations.
** 026 SIY 20060515   @26144 Encapsulated access to label and help fields.
** 027 GES 20060803   @28393 Major cleanup and refactoring.
** 028 GES 20060808   @28487 Fixed cursor processing to generate next or
**                           previous focus movement.
** 029 NVS 20060823   @28785 Cursor display is turned off while the button
**                           has focus.
** 030 SIY 20060824   @28836 Added LabeledWidget interface support.
** 031 SIY 20060824   @28842 Removed getP2JType() method.
** 032 NVS 20060824   @28872 Cursor left/right/up/down keys are processed
**                           like in Progress.
** 033 GES 20060825   @28916 Fixed event processing for alert boxes that
**                           was broken in H032.  Fixed drawing of the
**                           mnemonic highlight in a selected button.
** 034 GES 20061117   @31248 Fixed button drawing to properly account for
**                           all possible explicitly set widths. The
**                           getWidth() and getHeight() methods needed
**                           to be implemented to account for explicitly
**                           set width/height such that the layout of
**                           buttons was fixed.
** 035 NVS 20061204   @31543 Implemented TAB and BACK_TAB keys processing
**                           to make the widgets behave uniformly.
** 036 NVS 20061204   @31551 Switched to using a private widgetFocused
**                           flag instead of config.selected
** 037 GES 20061208   @31636 Added safer cursor visibility processing
**                           during focus gain/loss.
** 038 SIY 20061227   @31749 Fixed drawing of focused buttons in some 
**                           special cases.
** 039 NVS 20070426   @33249 Enabled state is checked instead of SENSITIVE
**                           attribute.
** 040 SIY 20070807   @34804 Added isColumnFixed() method.
** 041 SVL 20071221   @36528 Label is shown/hidden together with the
**                           widget. 
** 042 NVS 20080401   @37772 Added isFocusable() method implementation.
** 043 GES 20080421   @38059 I18N charset support.
** 044 NVS 20081226   @40997 Drawing takes into consideration highlighted
**                           state instead of focused. They are not equal
**                           (focused button is not highlighted while out
**                           of wait-for).
** 045 CA  20090115   @41127 Fixed invalid cursor state caused by 
**                           consecutive FOCUS_GAINED events: when the 
**                           button is already marked as 'focused', it
**                           must not overwrite the 'curvis' flag. 
** 046 SIY 20100619          Switching to own implementation of character UI.
** 047 SIY 20100804          Changes in imports, some refactorings.
** 048 SIY 20100822          Refactored AttributeContainer support.
** 049 SIY 20100831          Changes related to refactoring of FrameHelper.
** 050 SIY 20100915          locateFrame() is moved to UiUtils.
** 054 SIY 20101122          Reduced class hierarchy. Switching from KeyEvent 
**                           to KeyInput.
** 055 SIY 20110112          Changes related to new color handling scheme.
** 056 SIY 20111101          Updated imports, minor cleanups/refactorings.
** 057 SIY 20111202          Refactored and moved from CHUI package.
** 058 EVL 20140528          Adding implementation stubs for two new methods.
** 059 EVL 20140618          Refactored moving the code common to all button-like widgets to
**                           the new base class AbstractButton. Currently Button and ToggleBox
**                           are used this new class as the base class.
** 060 CA  20140805          Added generics to force the abstract widget classes to not use
**                           implementation-specific code: the abstract widgets will have access
**                           only to the base APIs provided by the OutputManager.
** 061 HC  20140911          GUI coordinate support - data types refactoring.
** 062 CA  20140926          Added shared widget configuration support, to allow GUI/ChUI
**                           concrete implementation for the same widget ID.  Refactored the
**                           widget configuration classes: all fields were made public; these
**                           classes need to be as dumb as possible, all logic related to
**                           setting/getting a certain field should be at the widget or at the
**                           caller. Refs #2254
** 063 HC  20141021          Implemented window move/position support.
** 064 CA  20150115          Added missed logic in setLabelInstance, related to side-label runtime 
**                           support (behavior common to all LabeledWidget implementations).
** 065 HC  20150128          Added support for frame widget position changes induced by
**                           setting ROW/COL/X/Y attributes.
** 066 CA  20150202          Side-labels needs to track the HIDDEN state, too.
** 067 HC  20150214          Improved positioning of side labels.
** 068 CA  20150223          Allow the label to be removed.
** 069 EVL 20150223          Fix for NPE for dynamic widget with text label not yet defined.
** 070 HC  20150323          Removed business logic from config classes.
** 071 HC  20150612          Implemented GUI ALERT-BOX.
** 072 CA  20150722          Added postprocessConfig - required additional processing, as layout  
**                           is performed only on the first frame definition push.
** 073 HC  20150913          Fixed a memory leak caused by label config not being unregistered.
** 074 CA  20151017          Implicit button state is disabled. 
**     HC  20151020          Fixed an abend in GUI ALERT-BOX caused by wrong button
**                           initialization.
** 075 HC  20151023          Fixed GUI ALERT-BOX keyboard input processing. 
**                           Button's width/height can have its chars size smaller than one (but 
**                           greater than zero).
** 076 VIG 20160215          Added labelInfo, containing mnemonic information, preprocessed label,
**                           Removed underscore flag and corresponding setter, fixed extra CHOOSE.
** 077 VIG 20160627          MnemonicInfo.init() call is renamed to MnemonicInfo.prepare().
** 078 HC  20161018          Added an initial support for rendering of mnemonic labels to all
**                           labeled widgets.
** 079 CA  20171127          Removed an unnecessary repaint.
*/
/*
** This program is free software: you can redistribute it and/or modify
** it under the terms of the GNU Affero General Public License as
** published by the Free Software Foundation, either version 3 of the
** License, or (at your option) any later version.
**
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU Affero General Public License for more details.
**
** You may find a copy of the GNU Affero GPL version 3 at the following
** location: https://www.gnu.org/licenses/agpl-3.0.en.html
** 
** Additional terms under GNU Affero GPL version 3 section 7:
** 
**   Under Section 7 of the GNU Affero GPL version 3, the following additional
**   terms apply to the works covered under the License.  These additional terms
**   are non-permissive additional terms allowed under Section 7 of the GNU
**   Affero GPL version 3 and may not be removed by you.
** 
**   0. Attribution Requirement.
** 
**     You must preserve all legal notices or author attributions in the covered
**     work or Appropriate Legal Notices displayed by works containing the covered
**     work.  You may not remove from the covered work any author or developer
**     credit already included within the covered work.
** 
**   1. No License To Use Trademarks.
** 
**     This license does not grant any license or rights to use the trademarks
**     Golden Code, FWD, any Golden Code or FWD logo, or any other trademarks
**     of Golden Code Development Corporation. You are not authorized to use the
**     name Golden Code, FWD, or the names of any author or contributor, for
**     publicity purposes without written authorization.
** 
**   2. No Misrepresentation of Affiliation.
** 
**     You may not represent yourself as Golden Code Development Corporation or FWD.
** 
**     You may not represent yourself for publicity purposes as associated with
**     Golden Code Development Corporation, FWD, or any author or contributor to
**     the covered work, without written authorization.
** 
**   3. No Misrepresentation of Source or Origin.
** 
**     You may not represent the covered work as solely your work.  All modified
**     versions of the covered work must be marked in a reasonable way to make it
**     clear that the modified work is not originating from Golden Code Development
**     Corporation or FWD.  All modified versions must contain the notices of
**     attribution required in this license.
*/

package com.goldencode.p2j.ui.client;

import java.util.ArrayList;

import com.goldencode.p2j.ui.*;
import com.goldencode.p2j.ui.chui.*;
import com.goldencode.p2j.ui.client.event.*;
import com.goldencode.p2j.ui.client.event.listener.*;
import com.goldencode.p2j.ui.client.widget.*;
import com.goldencode.p2j.util.*;

/**
 * An implementation of a "push button" with a text label.
 */
public abstract class Button<O extends OutputManager<?>>
extends AbstractButton<O, ButtonConfig>
implements LabeledWidget<O>,
           MnemonicTrait<O>
{
   /** Command to generate. */
   private String command;

   /** Container for listeners. */
   private ArrayList<ActionListener> listeners = new ArrayList<ActionListener>();

   /** Label instance associated with widget. */
   private Label<O> label = null;
   
   /** Mnemonic information */
   private MnemonicInfo mnemonic = null;

   /**
    * This constructor creates a button with a given text label.
    * 
    * @param   id
    *          The widget's ID.
    * @param   owner
    *          The parent container, frame or dialog which will include this
    *          button.
    * @param   text
    *          The string to be used as the button text.
    */
   public Button(WidgetId id, Container<O> owner, String text)
   {
      initButtonText(text);
      
      config = ConfigManager.getInstance().resolveWidgetConfig(id, ButtonConfig.class);
      
      // width() may require the button to be attached to the widget tree 
      if (owner != null)
      {
         owner.add(this);
      }
      
      config.name = text; // direct access
      config.widthChars = Coordinate.scale(width()); // direct access
      config.help = null;
      addFocusListener(this);
   }

   /**
    * Special constructor used to restore component using given configuration.
    * <p>
    * Call {@link #initialize} to finish the widget initialization.
    * 
    * @param   id
    *          The widget's ID.
    * @param   cfg
    *          Configuration data.
    */
   public Button(WidgetId id, ButtonConfig cfg)
   {
   }
   
   /**
    * Replacement API for the logic which should exist in the widget's c'tor, but the logic 
    * can't be executed as it depends on the widget being registered into the official registry.
    * <p>
    * This API is called after the widget is created and added to the registry.
    * 
    * @param    id
    *           The widget's ID.
    * @param    cfg
    *           The config used to initialize this widget.
    */
   @Override
   public void initialize(WidgetId id, ButtonConfig cfg)
   {
      initButtonText("");
      
      if (cfg != null)
      {
         config = (ButtonConfig) ConfigManager.getInstance().resolveWidgetConfig(id, cfg);
         ConfigManager.getInstance().replaceConfig(config, cfg);
      }
      else
      {
         // we are happy with the default config
         cfg = config;
      }

      String lbl = ((ButtonConfig) cfg).label;
      
      if (lbl != null)
         setText(lbl);
      addFocusListener(this);
      
      // not in edit mode by default
      // use direct field access to avoid repainting
      disable();
   }

   /**
    * Initialize button text.
    * 
    * @param   text
    *          Button text.
    */
   protected void initButtonText(String text) 
   {
      setTextLabel(text);
      this.command = text;
   }

   /**
    * Add action listener.
    * 
    * @param   listener
    *          Listener to add.
    */
   public void addListener(ActionListener listener)
   {
      listeners.add(listener);
   }

   /**
    * Check if passes key does match underscored character in the text.
    * 
    * @param   keyEvent
    *          Event to check.
    * 
    * @return  <code>true</code> if match is found.
    *  
    */
   public boolean checkKey(KeyInput keyEvent)
   {
      Character mnem = getMnemonicCharacter(false).orElse(null);
      if (mnem != null && keyEvent.id() == EventType.KEY_TYPED)
      {
         char kc = Utils.toChar(keyEvent.keyCode());
         
         if (Character.toLowerCase(mnem) == Character.toLowerCase(kc))      
            return true;
      }
      
      return false;
   }
   
   /**
    * Provide access to the attributes stored in the widget.
    * 
    * @return  Reference to attribute container.
    */
   @SuppressWarnings("unchecked")
   @Override
   public ButtonConfig config()
   {
      return config;
   }

   /**
    * Return current value of COLON option.
    * 
    * @return  Current value of the option column.
    */
   @Override
   public double getColon()
   {
      return ConfigHelper.getColon(config);
   }

   /**
    * Return text of the column label.
    * 
    * @return  text of the column label.
    */
   @Override
   public String getColumnLabelText()
   {
      return null;
   }

   /**
    * Return reference to the instance of the <code>Label</code> attached to
    * the widget.
    * 
    * @return  Reference to <code>Label</code> instance.
    */
   @Override
   public Label<O> getLabelInstance()
   {
      return label;
   }

   /**
    * Return text of the label.
    * 
    * @return  text of the label.
    */
   @Override
   public String getLabelText()
   {
      return null;
   }
   
   /**
    * Return TO column.
    * 
    * @return  Column assigned via TO option.
    */
   @Override
   public double getTo()
   {
      return ConfigHelper.getTo(config);
   }
   
   /**
    * Calculate the height of this widget.
    *
    * @return    The height in rows.
    */
   @Override
   public double height()
   {
      return config.heightChars <= 0 ? 1 : config.heightChars;
   }
   
   /**
    * Get current state of flag which is forces presence of the label
    * regardless from the frame NO-LABELS option.
    * 
    * @return  Current state of the flag.
    */
   @Override
   public boolean isForceLabel()
   {
      return config.forceLabel;
   }

   /**
    * Get current state of NO-LABELS flag.
    * 
    * @return  Current state of the flag.
    */
   @Override
   public boolean isNoLabels()
   {
      return !config.labels;
   }

   /**
    * Get widget alignment policy.
    * 
    * @return  <code>true</code> if widget is formatted for right alignment.
    */
   @Override
   public boolean isRightAligned()
   {
      return false;
   }

   /**
    * Process event and dispatch it to specific method.
    * 
    * @param   event
    *          Event to process.
    */
   @Override
   public void processEvent(Event event)
   {
      super.processEvent(event);

      if (event instanceof KeyInput)
      {
         KeyInput keyEvent = (KeyInput) event;
         
         if ((!keyEvent.isConsumed()) &&
             keyEvent.keyCode() == Key.VK_ENTER && isEnabled())
         {
            EventManager.postEvent(new ActionEvent(this, getCommand()));
            keyEvent.consume();
         }
         
         return;
      }
      
      if (event instanceof ActionEvent)
      {
         for (ActionListener listener : listeners)
         {
            listener.onAction(((ActionEvent) event));
         }
      }
   }

   /**
    * Processes key events occurring on this object by dispatching them
    * to any registered KeyListener objects.
    *
    * @param    keyEvent
    *           The event to be handled.
    */
   @Override
   public void processKeyEvent(KeyInput keyEvent)
   {
      if (handleHotkeys(keyEvent))
         return;
      
      super.processKeyEventCommon(keyEvent);
   }

   /**
    * Handle space and keys that match the highlighted char in button text.
    * 
    * @param   ki
    *          Input event.
    *          
    * @return  <code>true</code> if key is handled.
    */
   private boolean handleHotkeys(KeyInput ki)
   {
      boolean returnNow = false;
      int key = ki.actionCode();
      
      Character mnem = getMnemonicCharacter(false).orElse(null);
      
      // changing the key code to VK_ENTER will cause the button to be
      // "selected" (pressed) downstream
      if (key == ' ')
      {
         ki.setKeyCode(Key.VK_ENTER);
         return returnNow;
      }
      else if (mnem != null && ki.id() == EventType.KEY_TYPED)
      {
         // test for the mnemonic or accelerator character that is 
         // highlighted on the button
         if (checkKey(ki))
         {
            EventManager.postEvent(new ActionEvent(this, getCommand()));
            return returnNow;
         }
      }
      
      // do not check for underscore >= 0 here, GUI ALERT-BOX may have buttons with no underscore
      if (ki.id() == EventType.KEY_TYPED)
      {
         AlertBox<?> aBox = parent(AlertBox.class);
         if (aBox != null)
         {
            // alert boxes check for mnemonic characters across the
            // entire set of buttons (and if found will manually force
            // the selection of that button)
            if (aBox.findButton(ki))
               returnNow  = true;
         }
      }
         
      return returnNow;
   }

   /**
    * Process basic events - TAB, BACK-TAB and ENTER key event.
    * 
    * @param   event
    *          Key event to process.
    */
   @Override
   protected void handleBasicEvents(KeyInput event)
   {
      super.processKeyEvent(event);

      if (event.isConsumed())
      {
         return;
      }

      int key = event.keyCode();
      if (key == '\t')
      {
         parent().nextFocus();
         return;
      }
      else if (key == Key.VK_BACK_TAB)
      {
         parent().prevFocus();
         return;
      }

      if (super.isEnabled() && (key == Key.VK_ENTER))
      {
         EventManager.postEvent(new ItemEvent(this));
      }
   }
   
   /**
    * Request focus.
    */
   @Override
   public void requestFocus()
   {
      if (isEnabled())
         super.requestFocus();
   }

   /**
    * Get command associated with button.
    * 
    * @return  Command text.
    */
   public String getCommand()
   {
      return config.autoGo ? Keyboard.GO : config.autoEndKey ? Keyboard.ENDKEY : command;
   }

   /**
    * Set command associated with button.
    * 
    * @param    command  
    *           Command text.
    */
   public void setCommand(String command)
   {
      this.command = command;
   }

   /**
    * Assign new instance of <code>Label</code> to the widget.
    * 
    * @param   label
    *          Reference to new <code>Label</code> instance.
    */
   @Override
   public void setLabelInstance(Label<O> label)
   {
      if (label != this.label && this.label != null)
      {
         this.label.destroy();
      }
      
      if (label == null)
      {
         this.label = null;
         return;
      }

      int frameId = ThinClient.frameIdFromWidgetId(this.config.id.asInt());
      
      // link the label to the frame of this widget
      label.linkTo(frameId, config().sideLabelId);
      
      this.label = label;
      this.label.setVisible(isVisible());
      this.label.setHidden(hidden());
   }

   /**
    * Force NO-LABELS flag.
    * 
    * @param   labels
    *          New value of the flag.
    */
   @Override
   public void setNoLabels(boolean labels)
   {
      config.labels = !labels;
   }

   /**
    * Set the new text for the component.
    * 
    * @param   text
    *          New text value.
    */
   @Override
   public void setText(String text)
   {
      setTextLabel(text);
      
      if (config.autoResize)
      {
         double w = Coordinate.scale(textLabel().length() + 2);
         boolean repaint = Double.compare(w, config.widthChars) != 0;
         if (repaint)
         {
            repaint();
         }
         
         config.widthChars = w; // direct access

         if (repaint)
         {
            repaint();
         }
      }
   }

   /**
    * Set button label.
    * 
    * @param   label
    *          Label text.
    */
   public void setTextLabel(String label)
   {
      super.setTextLabel(label);
      
      if (label != null && !label.trim().isEmpty())
      {
         updateMnemonic(null, label, false);
      }
      else
      {
         // clear the mnemonic
         updateMnemonic(null, null, false);
      }
   }
   
   /**
    * Get label text.
    * 
    * @return  label text.
    */
   public String textLabel()
   {
      return getMnemonicLabel(false).orElse(super.textLabel());
   }

   /**
    * Shows or hides the button and its label.
    *
    * @param    visible
    *           <code>true</code> to make the widget visible, 
    *           <code>false</code> to hide the widget.
    */
   @Override
   public void setVisible(boolean visible)
   {
      super.setVisible(visible);

      if (label != null)
         label.setVisible(visible);
   }
   
   /**
    * Tracks the hidden state of the widget and attached label.
    * 
    * @param    hidden
    *           New hidden state.
    */
   @Override
   public void setHidden(boolean hidden)
   {
      super.setHidden(hidden);
      
      if (label != null)
         label.setHidden(hidden);
   }
   
   /**
    * Calculate the width of this widget.
    *
    * @return    The width in characters.
    */
   @Override
   public double width()
   {
      // safe, comparison of scaled doubles
      boolean max = (config.autoResize || config.widthChars <= 0);
      // button text label
      String labelText = textLabel();
      // text label can be null for dynamic widget we need to consider this case
      return max && labelText != null ? labelText.length() + 2 : config.widthChars;
   }
   
   /**
    * Checking if the widget was made visible at least once.
    * 
    * @return  <code>true</code> if the widget was visible,
    *          <code>false</code> otherwise.
    */
   @Override
   public boolean wasVisible()
   {
      return true;
   }
   
   /**
    * Remember the widget was made visible at least once.
    */
   @Override
   public void setWasVisible()
   {
      return;
   }

   /**
    * The method is called after the configuration associated with 
    * the implementor changes. The parameter points to a reference
    * representing the original configuration state. This reference
    * can be used to detect what configuration fields changed and so
    * optimize any processing related to the configuration change.
    * 
    * @param    beforeUpdate
    */
   @Override
   public void afterConfigUpdate(ButtonConfig beforeUpdate)
   {
      super.afterConfigUpdateBase(beforeUpdate);
      
      setTextLabel(config().label);
   }
   
   /**
    * Additional processing required when widget configuration is updated AFTER the frame layout
    * has been performed.
    */
   @Override
   public void postprocessConfig()
   {
      super.postprocessConfig();
      
      if (label != null)
      {
         label.postprocessConfig(this);
      }
   }
   
   /**
    * Destroy this widget.
    */
   @Override
   public void destroy()
   {
      if (label != null)
      {
         label.destroy();
         label = null;
      }
      
      super.destroy();
   }
   /**
    * Set the mnemonics support for this button.
    */
   public void setSupportsMnemonic()
   {
      // Do nothing. Just let descendants ability to override this method.
   }

   /**
    * Check whether this button supports mnemonics. Intended to be overridden.
    * CHUI: buttons do not support mnemonics except the ones in ALERT-BOX.
    * GUI: all buttons support mnemonics. 
    * 
    * @return   <code>true</code> if this button supports mnemonic.
    */
   public boolean isMnemonicSupported()
   {
      return false;
   }

   /**
    * MnemonicInfo instance getter.
    * <p>
    * The implementors should simply return the value of the mnemonic class field. No
    * additional processing logic is advised.
    *
    * @return   See above.
    */
   public MnemonicInfo getMnemonic()
   {
      return mnemonic;
   }

   /**
    * MnemonicInfo instance setter.
    * <p>
    * The implementors should simply assign the parameter value to the mnemonic class
    * field. No additional processing logic is advised.
    */
   public void setMnemonic(MnemonicInfo mnemonic)
   {
      this.mnemonic = mnemonic;
   }
}