/*
** Module   : FontMetricsHelper.java
** Abstract : provides an API for font metrics
**
** Copyright (c) 2015-2017, Golden Code Development Corporation.
**
** -#- -I- --Date-- ---------------------------------Description---------------------------------
** 001 GES 20150427 Created initial version.
** 002 CA  20160324 Added stringWidths().
*/
/*
** This program is free software: you can redistribute it and/or modify
** it under the terms of the GNU Affero General Public License as
** published by the Free Software Foundation, either version 3 of the
** License, or (at your option) any later version.
**
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU Affero General Public License for more details.
**
** You may find a copy of the GNU Affero GPL version 3 at the following
** location: https://www.gnu.org/licenses/agpl-3.0.en.html
** 
** Additional terms under GNU Affero GPL version 3 section 7:
** 
**   Under Section 7 of the GNU Affero GPL version 3, the following additional
**   terms apply to the works covered under the License.  These additional terms
**   are non-permissive additional terms allowed under Section 7 of the GNU
**   Affero GPL version 3 and may not be removed by you.
** 
**   0. Attribution Requirement.
** 
**     You must preserve all legal notices or author attributions in the covered
**     work or Appropriate Legal Notices displayed by works containing the covered
**     work.  You may not remove from the covered work any author or developer
**     credit already included within the covered work.
** 
**   1. No License To Use Trademarks.
** 
**     This license does not grant any license or rights to use the trademarks
**     Golden Code, FWD, any Golden Code or FWD logo, or any other trademarks
**     of Golden Code Development Corporation. You are not authorized to use the
**     name Golden Code, FWD, or the names of any author or contributor, for
**     publicity purposes without written authorization.
** 
**   2. No Misrepresentation of Affiliation.
** 
**     You may not represent yourself as Golden Code Development Corporation or FWD.
** 
**     You may not represent yourself for publicity purposes as associated with
**     Golden Code Development Corporation, FWD, or any author or contributor to
**     the covered work, without written authorization.
** 
**   3. No Misrepresentation of Source or Origin.
** 
**     You may not represent the covered work as solely your work.  All modified
**     versions of the covered work must be marked in a reasonable way to make it
**     clear that the modified work is not originating from Golden Code Development
**     Corporation or FWD.  All modified versions must contain the notices of
**     attribution required in this license.
*/

package com.goldencode.p2j.ui.client;

/**
 * Provides an API for font metrics.
 */
public interface FontMetricsHelper
{
   /**
    * Report the advance width of the given character.  Advance width measures the length of the 
    * character's baseline.
    *
    * @param    ch
    *           The character to size.
    *
    * @return   The width.
    */
   public int charWidth(char ch);
   
   /**
    * Report the advance widths of the first 256 characters in the font.  Advance width measures 
    * the length of the character's baseline.
    *
    * @return   The array of widths.
    */
   public int[] getWidths();
   
   /**
    * Report the advance width of the given text.  Advance width measures the length of the 
    * text's baseline.  This may not be the same as the sum of the advance widths for each
    * character in the text.
    *
    * @param    text
    *           The string to size.
    *
    * @return   The total width.
    */
   public int stringWidth(String text);
   
   /**
    * Report the height of a line of text.  This measures the length between the baseline of
    * two sequential lines of text.
    *
    * @return   The line height.
    */
   public int getHeight();
   
   /**
    * Get the intermediate widths of this text (for each substring starting from index 0, ending 
    * on indices 1 .. text.length - 1).
    *
    * @param    text
    *           The text.
    * 
    * @return   See above.
    */
   public int[] stringWidths(String text);
}