/*
** Module   : FrameData.java
** Abstract :
**
** Copyright (c) 2010-2017, Golden Code Development Corporation.
**
** -#- -I- --Date-- --JPRM-- ----------------Description-----------------
** 001 SIY 20100915          Created initial version
** 033 SIY 20111101          Moved from CHUI package. Minor 
**                           cleanups/refactorings in preparation to GUI client.
** 034 CA  20140926          Added shared widget configuration support, to allow GUI/ChUI
**                           concrete implementation for the same widget ID.  Refactored the
**                           widget configuration classes: all fields were made public; these
**                           classes need to be as dumb as possible, all logic related to
**                           setting/getting a certain field should be at the widget or at the
**                           caller. Refs #2254
** 035 HC  20150323          WidgetId.getId() renamed to asInt().
*/
/*
** This program is free software: you can redistribute it and/or modify
** it under the terms of the GNU Affero General Public License as
** published by the Free Software Foundation, either version 3 of the
** License, or (at your option) any later version.
**
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU Affero General Public License for more details.
**
** You may find a copy of the GNU Affero GPL version 3 at the following
** location: https://www.gnu.org/licenses/agpl-3.0.en.html
** 
** Additional terms under GNU Affero GPL version 3 section 7:
** 
**   Under Section 7 of the GNU Affero GPL version 3, the following additional
**   terms apply to the works covered under the License.  These additional terms
**   are non-permissive additional terms allowed under Section 7 of the GNU
**   Affero GPL version 3 and may not be removed by you.
** 
**   0. Attribution Requirement.
** 
**     You must preserve all legal notices or author attributions in the covered
**     work or Appropriate Legal Notices displayed by works containing the covered
**     work.  You may not remove from the covered work any author or developer
**     credit already included within the covered work.
** 
**   1. No License To Use Trademarks.
** 
**     This license does not grant any license or rights to use the trademarks
**     Golden Code, FWD, any Golden Code or FWD logo, or any other trademarks
**     of Golden Code Development Corporation. You are not authorized to use the
**     name Golden Code, FWD, or the names of any author or contributor, for
**     publicity purposes without written authorization.
** 
**   2. No Misrepresentation of Affiliation.
** 
**     You may not represent yourself as Golden Code Development Corporation or FWD.
** 
**     You may not represent yourself for publicity purposes as associated with
**     Golden Code Development Corporation, FWD, or any author or contributor to
**     the covered work, without written authorization.
** 
**   3. No Misrepresentation of Source or Origin.
** 
**     You may not represent the covered work as solely your work.  All modified
**     versions of the covered work must be marked in a reasonable way to make it
**     clear that the modified work is not originating from Golden Code Development
**     Corporation or FWD.  All modified versions must contain the notices of
**     attribution required in this license.
*/

package com.goldencode.p2j.ui.client;

import com.goldencode.p2j.ui.*;

/**
 * Frame value/widget ID container.
 */
public class FrameData
{
   /** Frame value. */
   private String value = null;
   
   /** Widget ID. */
   private WidgetId id = null;

   /**
    * Construct an instance for given value and widget ID.
    * 
    * @param   value
    *          New frame value.
    * @param   id
    *          New widget ID.
    */
   public FrameData(String value, WidgetId id)
   {
      StringBuilder buf = new StringBuilder(value);
      
      // strip trailing spaces
      
      while (buf.length() > 0)
      {
         if (buf.charAt(buf.length() - 1) != ' ')
            break;
         
         buf.setLength(buf.length() - 1);
      }
      
      this.value = buf.toString();
      this.id = id;
   }

   /**
    * Get widget ID.
    * 
    * @return  Returns the id.
    */
   public int getId()
   {
      return id == null ? -1 : id.asInt();
   }

   /**
    * Get frame value.
    * 
    * @return  Returns the value.
    */
   public String getValue()
   {
      return value;
   }

   /**
    * Provide string representation of the object content.
    * 
    * @see  java.lang.Object#toString()
    */
   @Override
   public String toString()
   {
      return " value = " + value + " widget id = " + id;
   }
}