/*
** Module   : NativeDimension.java
** Abstract : Area dimension descriptor.
**
** Copyright (c) 2014-2017, Golden Code Development Corporation.
**
** -#- -I- --Date-- ----------------Description-----------------
** 001 HC  20140816 Created initial version
** 002 HC  20151103 Added toString().
** 003 OM  20170517 Added hashCode() and equals() methods.
** 004 OM  20171027 toString() changed to a more compact form.
*/

/*
** This program is free software: you can redistribute it and/or modify
** it under the terms of the GNU Affero General Public License as
** published by the Free Software Foundation, either version 3 of the
** License, or (at your option) any later version.
**
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU Affero General Public License for more details.
**
** You may find a copy of the GNU Affero GPL version 3 at the following
** location: https://www.gnu.org/licenses/agpl-3.0.en.html
** 
** Additional terms under GNU Affero GPL version 3 section 7:
** 
**   Under Section 7 of the GNU Affero GPL version 3, the following additional
**   terms apply to the works covered under the License.  These additional terms
**   are non-permissive additional terms allowed under Section 7 of the GNU
**   Affero GPL version 3 and may not be removed by you.
** 
**   0. Attribution Requirement.
** 
**     You must preserve all legal notices or author attributions in the covered
**     work or Appropriate Legal Notices displayed by works containing the covered
**     work.  You may not remove from the covered work any author or developer
**     credit already included within the covered work.
** 
**   1. No License To Use Trademarks.
** 
**     This license does not grant any license or rights to use the trademarks
**     Golden Code, FWD, any Golden Code or FWD logo, or any other trademarks
**     of Golden Code Development Corporation. You are not authorized to use the
**     name Golden Code, FWD, or the names of any author or contributor, for
**     publicity purposes without written authorization.
** 
**   2. No Misrepresentation of Affiliation.
** 
**     You may not represent yourself as Golden Code Development Corporation or FWD.
** 
**     You may not represent yourself for publicity purposes as associated with
**     Golden Code Development Corporation, FWD, or any author or contributor to
**     the covered work, without written authorization.
** 
**   3. No Misrepresentation of Source or Origin.
** 
**     You may not represent the covered work as solely your work.  All modified
**     versions of the covered work must be marked in a reasonable way to make it
**     clear that the modified work is not originating from Golden Code Development
**     Corporation or FWD.  All modified versions must contain the notices of
**     attribution required in this license.
*/

package com.goldencode.p2j.ui.client;

/**
 * Area dimension descriptor.
 * The coordinate values are expressed in units native to UI type.
 */
public class NativeDimension
{
   /** Area height. */
   public int height;
   /** Area width.*/
   public int width;

   /**
    * Constructor.
    * 
    * @param   width
    *          Area width.
    * @param   height
    *          Area height.
    */
   public NativeDimension(int width, int height)
   {
      this.width = width;
      this.height = height;
   }
   
   /**
    * Create a copy of this dimension object.
    * 
    * @return  dimension object with the same with and height.
    */
   public NativeDimension copy()
   {
      return new NativeDimension(width, height);
   }
   
   /**
    * Checks whether {@code o} represents a {@code NativeDimension} with same values.
    * 
    * @param   o
    *          Another object to test for equality.
    *
    * @return  {@code true} if the object passed in represents a {@code NativeDimension} with same
    *          values for {@code height} and {@code width}.
    */
   @Override
   public boolean equals(Object o)
   {
      if (this == o)
      {
         return true;
      }
      if (o == null || getClass() != o.getClass())
      {
         return false;
      }
      
      NativeDimension that = (NativeDimension) o;
      
      return height == that.height && width == that.width;
   }
   
   /**
    * Returns a hash code value for the object. See {@link Object#hashCode()} for further
    * information.
    *
    * @return  the hash code value for the object.
    */
   @Override
   public int hashCode()
   {
      return 31 * height + width;
   }
   
   /**
    * Returns a string representation of the object.
    * 
    * @return  See above.
    */
   @Override
   public String toString()
   {
      return width + "x" + height;
   }
}
