/*
** Module   : ChuiClient.java
** Abstract : applet container for the ChuiSimulator class
**
** Copyright (c) 2011-2017, Golden Code Development Corporation.
**
** -#- -I- --Date-- ---------------------------------Description---------------------------------
** 001 GES 20110921 Created initial version.
** 002 MAG 20140508 Move common code to AbstractClient.
** 003 CA  20140805 Added ChUI specific client initialization.
** 004 GES 20150506 Modifications to match code moved from ChuiSimulator so that it could be
**                  shared for Swing and Web clients.
** 005 OM  20170214 Use proper Java Window type.
** 006 OM  20170510 Quit client when this window is closed.
*/
/*
** This program is free software: you can redistribute it and/or modify
** it under the terms of the GNU Affero General Public License as
** published by the Free Software Foundation, either version 3 of the
** License, or (at your option) any later version.
**
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU Affero General Public License for more details.
**
** You may find a copy of the GNU Affero GPL version 3 at the following
** location: https://www.gnu.org/licenses/agpl-3.0.en.html
** 
** Additional terms under GNU Affero GPL version 3 section 7:
** 
**   Under Section 7 of the GNU Affero GPL version 3, the following additional
**   terms apply to the works covered under the License.  These additional terms
**   are non-permissive additional terms allowed under Section 7 of the GNU
**   Affero GPL version 3 and may not be removed by you.
** 
**   0. Attribution Requirement.
** 
**     You must preserve all legal notices or author attributions in the covered
**     work or Appropriate Legal Notices displayed by works containing the covered
**     work.  You may not remove from the covered work any author or developer
**     credit already included within the covered work.
** 
**   1. No License To Use Trademarks.
** 
**     This license does not grant any license or rights to use the trademarks
**     Golden Code, FWD, any Golden Code or FWD logo, or any other trademarks
**     of Golden Code Development Corporation. You are not authorized to use the
**     name Golden Code, FWD, or the names of any author or contributor, for
**     publicity purposes without written authorization.
** 
**   2. No Misrepresentation of Affiliation.
** 
**     You may not represent yourself as Golden Code Development Corporation or FWD.
** 
**     You may not represent yourself for publicity purposes as associated with
**     Golden Code Development Corporation, FWD, or any author or contributor to
**     the covered work, without written authorization.
** 
**   3. No Misrepresentation of Source or Origin.
** 
**     You may not represent the covered work as solely your work.  All modified
**     versions of the covered work must be marked in a reasonable way to make it
**     clear that the modified work is not originating from Golden Code Development
**     Corporation or FWD.  All modified versions must contain the notices of
**     attribution required in this license.
*/

package com.goldencode.p2j.ui.client.chui.driver.swing;

import java.util.logging.*;
import java.lang.reflect.*;
import javax.swing.*;
import com.goldencode.p2j.ui.client.driver.swing.*;
import com.goldencode.p2j.ui.client.chui.driver.*;
import com.goldencode.p2j.util.LogHelper;

/**
 * Provides the containing frame for the content pane when used as an application.
 */
@SuppressWarnings("serial")
public class ChuiClient
extends AbstractClientFrame
{
   /** Logger. */
   private static final Logger LOG = LogHelper.getLogger(ChuiClient.class.getName());
   
   /** Frame title. */
   private static String title = "P2J ChUI Client";
   
   /** Simulator instance. */
   private ChuiSimulator sim = null;
   
   /**
    * Construct the new frame by instantiating the {@link ChuiSimulator} as
    * the content pane and initializing it using the given <code>options</code>.
    *
    * @param    options
    *           The configuration parameters for the content pane.
    */
   public ChuiClient(TerminalOptions options)
   {
      super(title); // no parent, this is a top-level window shown in taskbar
      
      sim = new ChuiSimulator(this, this, options);
      
      init(sim.getContentPane(), false);
   }
   
   /**
    * Initialize the application frame by creating a new instance of this
    * class and of the content pane which is an instance of
    * {@link ChuiSimulator}.
    *
    * @param    options
    *           The configuration parameters for the content pane.
    *
    * @return   The simulator instance. It will only be <code>null</code> if a catastrophic
    *           failure has occurred.
    */
   public static ChuiSimulator load(final TerminalOptions options)
   {
      // lame hack because of Java lack of parameter passing by reference (only by value)
      ChuiClient[] client = new ChuiClient[1];
      
      try
      {
         // instantiate the application's main frame
         SwingUtilities.invokeAndWait(() -> client[0] = new ChuiClient(options));
      }
      catch (InterruptedException | InvocationTargetException exc)
      {
         if (LOG.isLoggable(Level.SEVERE))
         {
            LOG.log(Level.SEVERE, "Could not load ChuiClient instance.", exc);
         }
         
         return null;
      }
      
      return client[0].sim;
   }

   /**
    * Initialize.
    * 
    * @param   pane
    *          Content pane.
    * @param   resizable
    *          Resizable attribute.
    */
   protected void init(JComponent pane, boolean resizable)
   {
      super.init(pane, resizable);
      setDefaultCloseOperation(WindowConstants.EXIT_ON_CLOSE); // quit client on close of window
      
      forceResize();
   }
}
