/*
** Module   : Event.java
** Abstract : Basic event class.
**
** Copyright (c) 2010-2018, Golden Code Development Corporation.
**
** -#- -I- --Date-- --JPRM-- ----------------Description-----------------
** 001 SIY 20100629          Created initial version
** 002 SIY 20100804          Changes in imports, some refactorings.
** 003 IAS 20150801          WIDGET-LEAVE/WIDGER-ENTER support.
** 004 HC  20180530          Moved consumed flag from KeyInput to Event.
*/
/*
** This program is free software: you can redistribute it and/or modify
** it under the terms of the GNU Affero General Public License as
** published by the Free Software Foundation, either version 3 of the
** License, or (at your option) any later version.
**
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU Affero General Public License for more details.
**
** You may find a copy of the GNU Affero GPL version 3 at the following
** location: https://www.gnu.org/licenses/agpl-3.0.en.html
** 
** Additional terms under GNU Affero GPL version 3 section 7:
** 
**   Under Section 7 of the GNU Affero GPL version 3, the following additional
**   terms apply to the works covered under the License.  These additional terms
**   are non-permissive additional terms allowed under Section 7 of the GNU
**   Affero GPL version 3 and may not be removed by you.
** 
**   0. Attribution Requirement.
** 
**     You must preserve all legal notices or author attributions in the covered
**     work or Appropriate Legal Notices displayed by works containing the covered
**     work.  You may not remove from the covered work any author or developer
**     credit already included within the covered work.
** 
**   1. No License To Use Trademarks.
** 
**     This license does not grant any license or rights to use the trademarks
**     Golden Code, FWD, any Golden Code or FWD logo, or any other trademarks
**     of Golden Code Development Corporation. You are not authorized to use the
**     name Golden Code, FWD, or the names of any author or contributor, for
**     publicity purposes without written authorization.
** 
**   2. No Misrepresentation of Affiliation.
** 
**     You may not represent yourself as Golden Code Development Corporation or FWD.
** 
**     You may not represent yourself for publicity purposes as associated with
**     Golden Code Development Corporation, FWD, or any author or contributor to
**     the covered work, without written authorization.
** 
**   3. No Misrepresentation of Source or Origin.
** 
**     You may not represent the covered work as solely your work.  All modified
**     versions of the covered work must be marked in a reasonable way to make it
**     clear that the modified work is not originating from Golden Code Development
**     Corporation or FWD.  All modified versions must contain the notices of
**     attribution required in this license.
*/

package com.goldencode.p2j.ui.client.event;

import com.goldencode.p2j.ui.client.widget.*;

/**
 * Basic event class.
 */
public abstract class Event
{
   /** Event type ID. */
   private final int id;
   
   /** Event source. */
   private final Widget source;

   /** Companion widget, used for ENTER/LEAVE events. */
   private final Widget other;

   /** If <code>true</code>, the keyboard event was processed and consumed. */
   private boolean consumed = false;

   /**
    * Constructor.
    * 
    * @param   source
    *          Event source.
    * @param   id
    *          Event type ID.
    */
   public Event(Widget source, int id)
   {
      this(source, null, id);
   }

   /**
    * Constructor.
    * 
    * @param   source
    *          Event source.
    * @param   other
    *          Event companion widget.
    * @param   id
    *          Event type ID.
    */
   public Event(Widget source, Widget other, int id)
   {
      this.source = source;
      this.other = other;
      this.id = id;
   }

   /**
    * Get event ID.
    * 
    * @return  event ID.
    */
   public int id()
   {
      return id;
   }

   /**
    * Source widget for this event.
    * 
    * @return  source widget.
    */
   public Widget source()
   {
      return source;
   }

   /**
    * Companion widget for this event.
    * 
    * @return  companion widget.
    */
   public Widget other()
   {
      return other;
   }

   /**
    * Mark event as consumed.
    */
   public void consume()
   {
      consumed = true;
   }

   /**
    * Check if event is already consumed.
    *
    * @return  <code>true</code> if event is consumed.
    */
   public boolean isConsumed()
   {
      return consumed;
   }
}