/*
** Module   : Key.java
** Abstract : Key codes.
**
** Copyright (c) 2010-2017, Golden Code Development Corporation.
**
** -#- -I- --Date-- ---------------------------------Description----------------------------------
** 001 SIY 20100629 Created initial version
** 002 SIY 20100804 Changes in imports, some refactorings.
** 003 GES 20110920 Added F21 through F24.
** 004 CA  20141123 Added code for CTRL-BREAK key.
** 005 CA  20141206 Added codes for text selection in GUI: SHIFT-CURSOR-RIGHT,SHIFT-CURSOR-LEFT, 
**                  SHIFT-HOME, SHIFT-END.
** 006 VIG 20150221 Added space action code.
** 007 CA  20150615 Added more keys related to GUI EDITOR support.
** 008 VIG 20150818 Added VK_ALT.
** 009 EVL 20151119 Adding ALT modified UP and DOWN key codes.
*/
/*
** This program is free software: you can redistribute it and/or modify
** it under the terms of the GNU Affero General Public License as
** published by the Free Software Foundation, either version 3 of the
** License, or (at your option) any later version.
**
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU Affero General Public License for more details.
**
** You may find a copy of the GNU Affero GPL version 3 at the following
** location: https://www.gnu.org/licenses/agpl-3.0.en.html
** 
** Additional terms under GNU Affero GPL version 3 section 7:
** 
**   Under Section 7 of the GNU Affero GPL version 3, the following additional
**   terms apply to the works covered under the License.  These additional terms
**   are non-permissive additional terms allowed under Section 7 of the GNU
**   Affero GPL version 3 and may not be removed by you.
** 
**   0. Attribution Requirement.
** 
**     You must preserve all legal notices or author attributions in the covered
**     work or Appropriate Legal Notices displayed by works containing the covered
**     work.  You may not remove from the covered work any author or developer
**     credit already included within the covered work.
** 
**   1. No License To Use Trademarks.
** 
**     This license does not grant any license or rights to use the trademarks
**     Golden Code, FWD, any Golden Code or FWD logo, or any other trademarks
**     of Golden Code Development Corporation. You are not authorized to use the
**     name Golden Code, FWD, or the names of any author or contributor, for
**     publicity purposes without written authorization.
** 
**   2. No Misrepresentation of Affiliation.
** 
**     You may not represent yourself as Golden Code Development Corporation or FWD.
** 
**     You may not represent yourself for publicity purposes as associated with
**     Golden Code Development Corporation, FWD, or any author or contributor to
**     the covered work, without written authorization.
** 
**   3. No Misrepresentation of Source or Origin.
** 
**     You may not represent the covered work as solely your work.  All modified
**     versions of the covered work must be marked in a reasonable way to make it
**     clear that the modified work is not originating from Golden Code Development
**     Corporation or FWD.  All modified versions must contain the notices of
**     attribution required in this license.
*/

package com.goldencode.p2j.ui.client.event;

/**
 * Key codes.
 */
public interface Key
{
   // Key codes adjusted to fit P2J needs
   public static final int  VK_DELETE         = 127;
   public static final int  VK_F1             = 301;
   public static final int  VK_F2             = 302;
   public static final int  VK_F3             = 303;
   public static final int  VK_F4             = 304;
   public static final int  VK_F5             = 305;
   public static final int  VK_F6             = 306;
   public static final int  VK_F7             = 307;
   public static final int  VK_F8             = 308;
   public static final int  VK_F9             = 309;
   public static final int  VK_F10            = 310;
   public static final int  VK_F11            = 311;
   public static final int  VK_F12            = 312;
   public static final int  VK_F13            = 313;
   public static final int  VK_F14            = 314;
   public static final int  VK_F15            = 315;
   public static final int  VK_F16            = 316;
   public static final int  VK_F17            = 317;
   public static final int  VK_F18            = 318;
   public static final int  VK_F19            = 319;
   public static final int  VK_F20            = 320;
   public static final int  VK_F21            = 321;
   public static final int  VK_F22            = 322;
   public static final int  VK_F23            = 323;
   public static final int  VK_F24            = 324;
   public static final int  VK_CURSOR_UP      = 501;
   public static final int  VK_CURSOR_DOWN    = 502;
   public static final int  VK_CURSOR_RIGHT   = 503;
   public static final int  VK_CURSOR_LEFT    = 504;
   public static final int  VK_HOME           = 505;
   public static final int  VK_END            = 506;
   public static final int  VK_PAGE_UP        = 507;
   public static final int  VK_PAGE_DOWN      = 508;
   public static final int  VK_BACK_TAB       = 509;
   public static final int  VK_INSERT         = 510;
   public static final int  VK_CTRL_BREAK     = 541;
   public static final int  VK_CTRL_AT        = 0;
   public static final int  VK_CTRL_A         = 1;
   public static final int  VK_CTRL_B         = 2;
   public static final int  VK_CTRL_C         = 3;
   public static final int  VK_CTRL_D         = 4;
   public static final int  VK_CTRL_E         = 5;
   public static final int  VK_CTRL_F         = 6;
   public static final int  VK_CTRL_G         = 7;
   public static final int  VK_BELL           = 7;
   public static final int  VK_BACKSPACE      = 8;
   public static final int  VK_TAB            = 9;
   public static final int  VK_CTRL_J         = 10;
   public static final int  VK_CTRL_K         = 11;
   public static final int  VK_CTRL_L         = 12;
   public static final int  VK_ENTER          = 13;
   public static final int  VK_CTRL_N         = 14;
   public static final int  VK_CTRL_O         = 15;
   public static final int  VK_CTRL_P         = 16;
   public static final int  VK_CTRL_Q         = 17;
   public static final int  VK_CTRL_R         = 18;
   public static final int  VK_CTRL_S         = 19;
   public static final int  VK_CTRL_T         = 20;
   public static final int  VK_CTRL_U         = 21;
   public static final int  VK_CTRL_V         = 22;
   public static final int  VK_CTRL_W         = 23;
   public static final int  VK_CTRL_X         = 24;
   public static final int  VK_CTRL_Y         = 25;
   public static final int  VK_CTRL_Z         = 26;
   public static final int  VK_ESCAPE         = 27;
   public static final int  VK_CTRL_BCKSL     = 28;
   public static final int  VK_CTRL_RBR       = 29;
   public static final int  VK_CTRL_CARET     = 30;
   public static final int  VK_CTRL_SUB       = 31;
   public static final int  VK_SPACE          = 32;

   // GUI navigation in EDITOR
   public static final int  VK_CTRL_CURSOR_RIGHT  = 2551;
   public static final int  VK_CTRL_CURSOR_LEFT   = 2552;
   public static final int  VK_CTRL_HOME          = 2553;
   public static final int  VK_CTRL_END           = 2554;

   // GUI text selection keys 
   public static final int  VK_SHIFT_CURSOR_UP    = 1013;
   public static final int  VK_SHIFT_CURSOR_DOWN  = 1014;
   public static final int  VK_SHIFT_CURSOR_RIGHT = 1015;
   public static final int  VK_SHIFT_CURSOR_LEFT  = 1016;
   public static final int  VK_SHIFT_HOME         = 1017;
   public static final int  VK_SHIFT_END          = 1018;
   public static final int  VK_SHIFT_PAGE_UP      = 1019;
   public static final int  VK_SHIFT_PAGE_DOWN    = 1020;
   
   public static final int  VK_ALT                 = 1024;
   public static final int  VK_ALT_CURSOR_UP       = 1525;
   public static final int  VK_ALT_CURSOR_DOWN     = 1526;
 
   public static final int  VK_CTRL_SHIFT_CURSOR_RIGHT = 3063;
   public static final int  VK_CTRL_SHIFT_CURSOR_LEFT  = 3064;
   public static final int  VK_CTRL_SHIFT_HOME         = 3065;
   public static final int  VK_CTRL_SHIFT_END          = 3066;

   public static final int  VK_CTRL_DEL                = 2175;
   public static final int  VK_CTRL_ENTER              = 2061;
}