/*
** Module   : WindowWorkSpace.java
** Abstract : window workspace widget
**
** Copyright (c) 2014-2017, Golden Code Development Corporation.
**
** -#- -I- --Date-- ---------------------------------Description---------------------------------
** 001 MAG 20141010 Created initial version.
** 002 CA  20141119 Refactored drawing code to enable multiple OS windows, for GUI support.  Misc
**                  changes/fixes related to drawing on multiple windows.
** 003 EVL 20141126 Content pane GUI fix.
** 004 EVL 20141127 Inserting this container into mouse source chain.
** 005 EVL 20141206 Fix for scrollbar not responding regression.
** 006 GES 20150425 Match class name change for GuiOptions to GuiConstants.
** 007 HC  20150831 Scrolling support overhaul.
** 008 HC  20151124 Fixed wrong workspace background color when its window is THREE-D.
** 009 SBI 20160328 Added getColorResolver().
** 010 OM  20170320 Added support for Themes.
** 011 HC  20170629 Implemented support of WINDOW:DISABLE-REDRAW attribute, the FWD extension to
**                  4GL.
*/
/*
** This program is free software: you can redistribute it and/or modify
** it under the terms of the GNU Affero General Public License as
** published by the Free Software Foundation, either version 3 of the
** License, or (at your option) any later version.
**
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU Affero General Public License for more details.
**
** You may find a copy of the GNU Affero GPL version 3 at the following
** location: https://www.gnu.org/licenses/agpl-3.0.en.html
** 
** Additional terms under GNU Affero GPL version 3 section 7:
** 
**   Under Section 7 of the GNU Affero GPL version 3, the following additional
**   terms apply to the works covered under the License.  These additional terms
**   are non-permissive additional terms allowed under Section 7 of the GNU
**   Affero GPL version 3 and may not be removed by you.
** 
**   0. Attribution Requirement.
** 
**     You must preserve all legal notices or author attributions in the covered
**     work or Appropriate Legal Notices displayed by works containing the covered
**     work.  You may not remove from the covered work any author or developer
**     credit already included within the covered work.
** 
**   1. No License To Use Trademarks.
** 
**     This license does not grant any license or rights to use the trademarks
**     Golden Code, FWD, any Golden Code or FWD logo, or any other trademarks
**     of Golden Code Development Corporation. You are not authorized to use the
**     name Golden Code, FWD, or the names of any author or contributor, for
**     publicity purposes without written authorization.
** 
**   2. No Misrepresentation of Affiliation.
** 
**     You may not represent yourself as Golden Code Development Corporation or FWD.
** 
**     You may not represent yourself for publicity purposes as associated with
**     Golden Code Development Corporation, FWD, or any author or contributor to
**     the covered work, without written authorization.
** 
**   3. No Misrepresentation of Source or Origin.
** 
**     You may not represent the covered work as solely your work.  All modified
**     versions of the covered work must be marked in a reasonable way to make it
**     clear that the modified work is not originating from Golden Code Development
**     Corporation or FWD.  All modified versions must contain the notices of
**     attribution required in this license.
*/

package com.goldencode.p2j.ui.client.gui;

import com.goldencode.p2j.ui.chui.ThinClient;
import com.goldencode.p2j.ui.client.*;
import com.goldencode.p2j.ui.client.ScrollBar.Bars;
import com.goldencode.p2j.ui.client.gui.driver.*;
import com.goldencode.p2j.ui.client.widget.*;

/**
 * Window work area.  A scrollable container.
 */
public class WindowWorkSpace
extends ScrollPaneGuiImpl
{     
   /** Max workspace size */
   public static Dimension maxSize;
   
   /**
    * Constructor.
    * 
    * @param   bars
    *          Container scroll bars.
    */
   public WindowWorkSpace(Bars bars)
   {
      super(new NativeScrollContainer<GuiOutputManager>());
      
      // scroll bars
      this.setScrollBars(bars);
      int msgLines = ThinClient.getInstance().getMessageLines();
      maxSize = new Dimension(GuiConstants.DEF_COLUMNS, GuiConstants.DEF_ROWS - msgLines);   
   }
   
   /**
    * Get workspace content pane.
    * 
    * @return  content pane.
    */
   public Container<GuiOutputManager> getContentPane() 
   {
      return (Container<GuiOutputManager>) this.viewport().getScrollWidget();
   }

   /**
    * Widget drawing method.
    */
   @Override
   public void draw()
   {
      // draw only when widget is visible  and not redraw-disabled
      if (!isVisible() || isRepaintDisabled())
      {
         return;
      }
      
      // resolve the bg color once and only after the parent window has been realized
      if (config().sysbgcolor == -1)
      {
         config().sysbgcolor = window().isThreeD() 
                               ? SysColor.COLOR_3DFACE
                               : SysColor.COLOR_APPWORKSPACE;
      }
      
      super.draw();
   }

   /**
    * Returns true if the widget repaint is disabled. This method is needed to support the
    * WINDOW:DISABLE-REDRAW attribute.
    *
    * @return   See above.
    */
   @Override
   public boolean isRepaintDisabled()
   {
      Window w = window();
      return w != null && w.config().disableRedraw;
   }

   /**
    * Provides a protected access to the internal color resolver instance.
    * 
    * @return   The color resolver.
    */
   protected GuiColorResolver getColorResolver()
   {
      return gc;
   }
}