/*
** Module   : p2j.module.js
** Abstract : Defines module navigation widget.
**
** Copyright (c) 2018, Golden Code Development Corporation.
**
** -#- -I- --Date-- -------------------------------Description-----------------------------------
** 001 CA  20180419 Created initial version.
*/
/*
** This program is free software: you can redistribute it and/or modify
** it under the terms of the GNU Affero General Public License as
** published by the Free Software Foundation, either version 3 of the
** License, or (at your option) any later version.
**
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU Affero General Public License for more details.
**
** You may find a copy of the GNU Affero GPL version 3 at the following
** location: https://www.gnu.org/licenses/agpl-3.0.en.html
** 
** Additional terms under GNU Affero GPL version 3 section 7:
** 
**   Under Section 7 of the GNU Affero GPL version 3, the following additional
**   terms apply to the works covered under the License.  These additional terms
**   are non-permissive additional terms allowed under Section 7 of the GNU
**   Affero GPL version 3 and may not be removed by you.
** 
**   0. Attribution Requirement.
** 
**     You must preserve all legal notices or author attributions in the covered
**     work or Appropriate Legal Notices displayed by works containing the covered
**     work.  You may not remove from the covered work any author or developer
**     credit already included within the covered work.
** 
**   1. No License To Use Trademarks.
** 
**     This license does not grant any license or rights to use the trademarks
**     Golden Code, FWD, any Golden Code or FWD logo, or any other trademarks
**     of Golden Code Development Corporation. You are not authorized to use the
**     name Golden Code, FWD, or the names of any author or contributor, for
**     publicity purposes without written authorization.
** 
**   2. No Misrepresentation of Affiliation.
** 
**     You may not represent yourself as Golden Code Development Corporation or FWD.
** 
**     You may not represent yourself for publicity purposes as associated with
**     Golden Code Development Corporation, FWD, or any author or contributor to
**     the covered work, without written authorization.
** 
**   3. No Misrepresentation of Source or Origin.
** 
**     You may not represent the covered work as solely your work.  All modified
**     versions of the covered work must be marked in a reasonable way to make it
**     clear that the modified work is not originating from Golden Code Development
**     Corporation or FWD.  All modified versions must contain the notices of
**     attribution required in this license.
*/

define(function ()
{
   "use strict";
   
   var me = {};

   /** The ID of the DOM element where to attach the module. */
   var moduleContainer;
   
   /** The callback executed when a event is performed. */
   var moduleEventCallback;
   
   /** The module object. */
   var pmodule;

   /** The animation speed (millis). */
   var speed = 200;

   /** Left-side padding for children. */
   var padding = 15;
   
   /**
    * Initialize the module.
    *
    * @param    {string} vmoduleContainer
    *           The ID of the DOM element where to attach the module.
    * @param    {function} vmoduleEventCallback
    *           The callback executed when an event is performed.
    */
   me.init = function(vmoduleContainer, vmoduleEventCallback)
   {
      moduleContainer = $("#" + vmoduleContainer);
      moduleEventCallback = vmoduleEventCallback;
   };

   /*
    * Destroy the module.
    */
   me.destroy = function()
   {
      moduleContainer.empty();
   };

   /*
    * Create a module and attach it, with the specified details.
    *
    * @param   {array} jsonModule
    *          The array with module specifications.
    */
   me.create = function(jsonModule)
   {
      me.destroy();
   
      buildModuleTree(moduleContainer, jsonModule, 1);

      if (moduleEventCallback)
      {
         moduleContainer.find("a").click(function(evt)
         {
            var a = $(this);
            if (a.attr("module"))
            {
               moduleEventCallback(a.attr("module"), a.attr("window-title"));
            }
         });
      }
   };

   /**
    * Build the collapsable module tree.
    * 
    * @param    {Object} mparent
    *           The parent element where children are attached.
    * @param    {array} jsonModule
    *           The array tree with module specifications.
    * @param    {integer} lvl
    *           The current nesting level.
    */
   function buildModuleTree(mparent, jsonModule, lvl)
   {
      if (jsonModule == undefined)
      {
         return;
      }

      var pList = $("<ul/>");
      mparent.append(pList);

      for (var i = 0; i < jsonModule.length; i++)
      {
         var cfg = jsonModule[i];
         
         var chLi = $("<li/>");
         var chA;
         if (cfg["items"])
         {
            chA = $("<a href='#'><i class='material-icons'></i>" + cfg["label"] + "</a>");
         }
         else
         {
            chA = $("<a href='#'>" + cfg["label"] + "</a>");
         }
         chA.css("padding-left", lvl * padding + (cfg["items"] ? 0 : padding));
         chA.attr("window-title", cfg["label"]);

         chLi.append(chA);
         pList.append(chLi);   

         if (cfg["items"])
         {
            chA.attr("data-option", "off");
            chA.find("i").text("keyboard_arrow_right");

            chA.unbind('click').on("click", function() 
            {
               var a = $(this);
               a.parent().parent().find("a[data-option='on']").parent("li").children("ul").slideUp(speed / 1.2, function() 
               {
                  $(this).parent("li").children("a").attr("data-option", "off");
                  $(this).parent("li").children("a").find("i").text("keyboard_arrow_right");
               })
               if (a.attr("data-option") == "off") 
               {
                  a.parent("li").children("ul").slideDown(speed, function()
                  {
                     $(this).parent("li").children("a").attr("data-option", "on");
                     $(this).parent("li").children("a").find("i").text("keyboard_arrow_down");
                  });
               }
               if (a.attr("data-option") == "on") 
               {
                  a.attr("data-option", "off");
                  a.find("i").text("keyboard_arrow_right");
                  a.parent("li").children("ul").slideUp(speed, function()
                  {
                     $(this).parent("li").children("a").attr("data-option", "off");
                     $(this).parent("li").children("a").find("i").text("keyboard_arrow_right");
                  });
               }
            });

            buildModuleTree(chLi, cfg["items"], lvl + 1);
         }
         else
         {
            chA.attr("module", cfg["module"]);
         }
      }
   };
   
   return me;
});
