/*
 ** Module   : MaterialTheme.java
 ** Abstract : Material theme manager. Designed to work with web embedded clients.
 **
 ** Copyright (c) 2018, Golden Code Development Corporation.
 **
 ** -#- -I- --Date-- ---------------------------------Description---------------------------------
 ** 001 OM  20180814 First version.
 */

/*
 ** This program is free software: you can redistribute it and/or modify
 ** it under the terms of the GNU Affero General Public License as
 ** published by the Free Software Foundation, either version 3 of the
 ** License, or (at your option) any later version.
 **
 ** This program is distributed in the hope that it will be useful,
 ** but WITHOUT ANY WARRANTY; without even the implied warranty of
 ** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 ** GNU Affero General Public License for more details.
 **
 ** You may find a copy of the GNU Affero GPL version 3 at the following
 ** location: https://www.gnu.org/licenses/agpl-3.0.en.html
 **
 ** Additional terms under GNU Affero GPL version 3 section 7:
 **
 **   Under Section 7 of the GNU Affero GPL version 3, the following additional
 **   terms apply to the works covered under the License.  These additional terms
 **   are non-permissive additional terms allowed under Section 7 of the GNU
 **   Affero GPL version 3 and may not be removed by you.
 **
 **   0. Attribution Requirement.
 **
 **     You must preserve all legal notices or author attributions in the covered
 **     work or Appropriate Legal Notices displayed by works containing the covered
 **     work.  You may not remove from the covered work any author or developer
 **     credit already included within the covered work.
 **
 **   1. No License To Use Trademarks.
 **
 **     This license does not grant any license or rights to use the trademarks
 **     Golden Code, FWD, any Golden Code or FWD logo, or any other trademarks
 **     of Golden Code Development Corporation. You are not authorized to use the
 **     name Golden Code, FWD, or the names of any author or contributor, for
 **     publicity purposes without written authorization.
 **
 **   2. No Misrepresentation of Affiliation.
 **
 **     You may not represent yourself as Golden Code Development Corporation or FWD.
 **
 **     You may not represent yourself for publicity purposes as associated with
 **     Golden Code Development Corporation, FWD, or any author or contributor to
 **     the covered work, without written authorization.
 **
 **   3. No Misrepresentation of Source or Origin.
 **
 **     You may not represent the covered work as solely your work.  All modified
 **     versions of the covered work must be marked in a reasonable way to make it
 **     clear that the modified work is not originating from Golden Code Development
 **     Corporation or FWD.  All modified versions must contain the notices of
 **     attribution required in this license.
 */

package com.goldencode.p2j.ui.client.gui.theme;

import com.goldencode.p2j.ui.*;
import com.goldencode.p2j.ui.client.*;
import com.goldencode.p2j.ui.client.gui.*;
import com.goldencode.p2j.ui.client.gui.driver.*;
import com.goldencode.p2j.ui.client.widget.*;

import javax.imageio.*;
import java.awt.Color;
import java.awt.image.*;
import java.io.*;
import java.util.*;

import static com.goldencode.p2j.ui.BrowseColumnWidgetType.TOGGLE_BOX;
import static com.goldencode.p2j.ui.client.SysColor.*;
import static com.goldencode.p2j.ui.client.gui.BrowseColumnGuiImpl.*;
import static com.goldencode.p2j.ui.client.gui.BrowseGuiImpl.*;

public class MaterialTheme
//implements Theme
extends Windows10Theme
{
   /** The palette entry for background color, by default WHITE. */
   private static final int BACKGROUND = 0;
   
   /** The palette entry for foreground color (text), by default BLACK. */
   private static final int FOREGROUND = 1;
   
   /** The palette entry for normal widget components, by default BLUE-GREEN-ish */
   private static final int NORMAL = 2;
   
   /** The palette entry for highlighted widget components, by default ORANGE-ish*/
   private static final int HIGHLIGHTED = 3;
   
   /** The palette entry for disabled widgets, by default some kind of GREY. */
   private static final int DISABLED = 4;
   
   /**
    * The default basic palette. Other related colors can be obtained using {@link #brighter}
    * and {@link #darker} APIs.
    */
   private final Color[] DEFAULT_PALETTE = {
         new Color(0xFFFFFF),    // BACKGROUND
         new Color(0x424242),    // FOREGROUND
         new Color(0x4C9BCE),    // NORMAL
         new Color(0xFB8D00),    // HIGHLIGHTED
         new Color(0x9B9B9B),    // DISABLED    item
   };
   
   /** The color palette that is currently in use. */
   private final Color[] pal;
   
   /** Cached map with default system colors. Legacy compatibility. */
   private Map<Integer, ColorRgb> defaultColors = null;
   
   /** Cached map with font table. Legacy compatibility. */
   private Map<String, String> defaultFontTable = null;
   
   /** Cached map with font aliases. Legacy compatibility. */
   private Map<String, String> defaultFontAliases = null;
   
   /** Cache with read /decoded images. */
   private Map<String, ImageWrapper<BufferedImage>> imgCache = new HashMap<>();
   
   /**
    * The constructor initializes the working palette. 
    */
   public MaterialTheme()
   {
      pal = new Color[DEFAULT_PALETTE.length];
      System.arraycopy(DEFAULT_PALETTE, 0, pal, 0, DEFAULT_PALETTE.length);
   }
   
   /**
    * Get the name of this {@code Theme}.
    *
    * @return  the name of this theme.
    */
   @Override
   public String getName()
   {
      return "material";
   }
   
   /**
    * Obtain the location of binary resources (like pixmaps and sprites) used by this theme. The 
    * location is relative to @Theme.class@ location so that the resources could be loaded easily
    * using @getResourceAsStream()@.
    *
    * @return  the location of binary resources used by this theme.
    */
   @Override
   public String getBinaryResourceLocation()
   {
      return "windows10"; // as temporary hack until all [Theme] methods are implemented
   }
   
   /**
    * Obtain an image resource specific to this {@code Theme}. The image can be cached for
    * performance reasons. It will be automatically released when {@link #releaseResources()} is
    * called, when a new {@code Theme} is loaded and this one is removed.
    *
    * @param   name
    *          The name of the image as path within jar bundle.
    * @param   w
    *          The desired with of the returned image. This is only a hint. If available, an
    *          image with exact width is returned. One need to check the size of the returned
    *          {@link ImageWrapper} to know the exact value.  
    * @param   h
    *          The desired with of the returned image. This is only a hint. If available, an
    *          image with exact width is returned. One need to check the size of the returned
    *          {@code ImageWrapper} to know the exact value.  
    * @param   antialias
    *          if {@code true} and image is scaled to meet the size hints, an smooth scaling
    *          algorithm will be used, otherwise a faster one will be used but the image quality
    *          will be lower. If image is not scaled the parameter is ignored.
    *
    * @return  an {@code ImageWrapper} containing the requested image resource if it can be found
    *          and loaded and {@code null} otherwise.
    */
   @Override
   public ImageWrapper getImage(String name, int w, int h, boolean antialias)
   {
      return getPngImage(getBinaryResourceLocation() + "/" + name + ".png");
   }
   
   /**
    * Check if the theme is mouse hover sensitive. Mouse hover sensitive themes need to repaint
    * the components when the mouse hovers over them (ie. without clicking them).
    *
    * @return  always {@code true}. This theme is mouse hover sensitive.
    */
   @Override
   public boolean isMouseHoverSensitive()
   {
      return true;
   }
   
   /**
    * Check whether this is a classic theme. A classic theme has a low color palette and usually
    * does not have animations which makes it better suited for low UI resources. Usually classic
    * themes are not mouse hover sensitive.
    *
    * @return  Always {@code false}.
    */
   @Override
   public boolean isClassic()
   {
      return false;
   }
   
   /**
    * Checks whether this theme is flat. 
    *
    * @return {@code true} if this {@code Theme} is flat.
    */
   public boolean isFlat()
   {
      return true;
   }
   
   /**
    * Obtain the default system palette for this theme.
    *
    * @return  the default system palette for this theme.
    */
   @Override
   public Map<Integer, ColorRgb> getDefaultSystemPalette()
   {
      if (defaultColors == null)
      {
         Map<Integer, ColorRgb> pal = new HashMap<>();
         pal.put(COLOR_SCROLLBAR, new ColorRgb(new Color(0xD1D3D4)));
         pal.put(COLOR_BACKGROUND, new ColorRgb(new Color(0x3A6EA5)));
         pal.put(COLOR_ACTIVECAPTION, new ColorRgb(new Color(0x2982CC)));
         pal.put(COLOR_INACTIVECAPTION, new ColorRgb(new Color(0xFFFFFF)));
         pal.put(COLOR_MENU, new ColorRgb(new Color(0xF2F2F2)));
         pal.put(COLOR_WINDOW, new ColorRgb(new Color(0xFFFFFF)));
         pal.put(COLOR_WINDOWFRAME, new ColorRgb(new Color(0x646464)));
         pal.put(COLOR_MENUTEXT, new ColorRgb(new Color(0x424242)));
         pal.put(COLOR_WINDOWTEXT, new ColorRgb(new Color(0x424242)));
         pal.put(COLOR_CAPTIONTEXT, new ColorRgb(new Color(0xFFFFFF)));
         pal.put(COLOR_ACTIVEBORDER, new ColorRgb(new Color(0x3C8CCD)));
         pal.put(COLOR_INACTIVEBORDER, new ColorRgb(new Color(0xA5A6A6)));
         pal.put(COLOR_APPWORKSPACE, new ColorRgb(new Color(0xA5A2A5)));
         pal.put(COLOR_HIGHLIGHT, new ColorRgb(new Color(0x4C9BCE)));
         pal.put(COLOR_HIGHLIGHTTEXT, new ColorRgb(new Color(0xFFFFFF)));
         pal.put(COLOR_3DFACE, new ColorRgb(new Color(0xD1D3D4)));
         pal.put(COLOR_3DSHADOW, new ColorRgb(new Color(0xA5A2A5)));
         pal.put(COLOR_GRAYTEXT, new ColorRgb(new Color(0x808080)));  /* not used ? */
         pal.put(COLOR_BTNTEXT, new ColorRgb(new Color(0x424242)));
         pal.put(COLOR_INACTIVECAPTIONTEXT, new ColorRgb(Color.BLUE)); // new Color(0xDAD0C8)));
         pal.put(COLOR_BTNHIGHLIGHT, new ColorRgb(new Color(0xFFFFFF)));
         pal.put(COLOR_3DDKSHADOW, new ColorRgb(new Color(0x404040))); /* not used ? */
         pal.put(COLOR_3DLIGHT, new ColorRgb(Color.GREEN)); // new Color(0xDAD0C8))); /* not used ? */
         pal.put(COLOR_INFOTEXT, new ColorRgb(new Color(0x000000)));
         pal.put(COLOR_INFOBK, new ColorRgb(new Color(0xFFFFFF)));
         pal.put(COLOR_BUTTONALTERNATEFACE, new ColorRgb(new Color(0xB5B5B5)));
         pal.put(COLOR_HOTLIGHT, new ColorRgb(new Color(0x000080)));
         pal.put(COLOR_GRADIENTACTIVECAPTION, new ColorRgb(new Color(0xA6CAF0)));
         pal.put(COLOR_GRADIENTINACTIVECAPTION, new ColorRgb(Color.CYAN)); // new Color(0xC0C0C0)));
         pal.put(COLOR_MENUHILIGHT, new ColorRgb(new Color(0x4C9BCE)));
         pal.put(COLOR_MENUBAR, new ColorRgb(new Color(0xFFFFFF)));
         pal.put(COLOR_TOOLTIPBK, new ColorRgb(new Color(0xFFFFE7)));
         pal.put(COLOR_FILLIN_BG, new ColorRgb(new Color(0xFFFFFF)));
         pal.put(COLOR_FILLIN_FG, new ColorRgb(new Color(0x424242)));
         pal.put(COLOR_CARET, new ColorRgb(new Color(0x424242)));
         
         defaultColors = Collections.unmodifiableMap(pal);
      }
      
      return defaultColors;
   }
   
   /**
    * Obtain the map of standard fonts specific to current {@code Theme}, to which a key defaults,
    * if not overridden in the directory.
    *
    * @return  the map of standard fonts specific to this {@code Theme}. 
    */
   @Override
   public Map<String, String> getStandardFontTable()
   {
      if (defaultFontTable == null)
      {
         defaultFontTable = new HashMap<>();
         
         defaultFontTable.put("default-font", "MS Sans Serif, size=9");
         defaultFontTable.put("default-fixed-font", "Courier New, size=9");
         
         // standard fonts
         defaultFontTable.put("font0", "Courier New, size=9");
         defaultFontTable.put("font1", "Segoe UI, size=10");
         defaultFontTable.put("font2", "Courier New, size=9");
         defaultFontTable.put("font3", "Courier New, size=9");
         defaultFontTable.put("font4", "Segoe UI, size=9");
         defaultFontTable.put("font5", "Segoe UI, size=11");
         defaultFontTable.put("font6", "Segoe UI, size=10, bold");
         defaultFontTable.put("font7", "Segoe UI, size=10");
         
         // system fonts
         defaultFontTable.put(FontTable.WINDOW_TITLE_FONT, "Segoe UI, size=12");
         defaultFontTable.put(FontTable.ALERT_BOX_FONT, "Segoe UI, size=10");
         defaultFontTable.put(FontTable.STACK_TRACE_WINDOW_FONT, "Segoe UI, size=9");
         defaultFontTable.put(FontTable.MENU_FONT, "Segoe UI, size=10");
         defaultFontTable.put(FontTable.SORT_NUMBER_FONT, "Segoe UI, size=8");
         
         // freeze the map
         defaultFontTable = Collections.unmodifiableMap(defaultFontTable);
      }
      return defaultFontTable;
   }
   
   /**
    * Obtain the map of default font aliases specific to current {@code Theme}, to which a key
    * defaults, if not overridden in the directory.
    *
    * @return  the map of standard font aliases specific to this {@code Theme}. 
    */
   @Override
   public Map<String, String> getDefaultFontAliases()
   {
      if (defaultFontAliases == null)
      {
         defaultFontAliases = new HashMap<>();
   
         defaultFontAliases.put("default-font", "Lato, size=9");
         defaultFontAliases.put("default-fixed-font", "Liberation Mono, size=9");
         
         // standard fonts
         defaultFontAliases.put("font0", "Liberation Mono, size=9");
         defaultFontAliases.put("font1", "DejaVu Sans, size=12, bold");
         defaultFontAliases.put("font2", "Liberation Mono, size=9");
         defaultFontAliases.put("font3", "Liberation Mono, size=9");
         defaultFontAliases.put("font4", "Liberation Sans, size=9");
         defaultFontAliases.put("font5", "Open Sans, size=12");
         defaultFontAliases.put("font6", "Open Sans, size=9, bold");
         defaultFontAliases.put("font7", "DejaVu Sans, size=12, bold");
         
         // system fonts
         defaultFontAliases.put(FontTable.WINDOW_TITLE_FONT, "Open Sans, size=12");
         defaultFontAliases.put(FontTable.ALERT_BOX_FONT, "Open Sans, size=10");
         defaultFontAliases.put(FontTable.STACK_TRACE_WINDOW_FONT, "Open Sans, size=9");
         defaultFontAliases.put(FontTable.MENU_FONT, "Open Sans, size=10");
         defaultFontAliases.put(FontTable.SORT_NUMBER_FONT, "Open Sans, size=8");
         
         // freeze the map
         defaultFontAliases = Collections.unmodifiableMap(defaultFontAliases);
      }
      return defaultFontAliases;
   }
   
//   @Override
//   public int getPrebuiltArrowImageSize()
//   {
//      return 0;
//   }
//   
//   @Override
//   public int getTitleBarHeight(boolean maximized)
//   {
//      return 0;
//   }
//   
//   @Override
//   public NativeInsets getWindowBorderInsets(boolean modal)
//   {
//      return null;
//   }
//   
//   @Override
//   public int getCaptionButtonHeight(CaptionButtonType type, boolean maximized)
//   {
//      return 0;
//   }
//   
//   @Override
//   public int getCaptionButtonWidth(CaptionButtonType type, boolean dialog)
//   {
//      return 0;
//   }
//   
//   @Override
//   public int getCaptionButtonsAlignment(CaptionButtonType type)
//   {
//      return 0;
//   }
//   
//   @Override
//   public int getWindowTitleAlignment()
//   {
//      return 0;
//   }
//   
//   @Override
//   public int getWindowIconAlignment()
//   {
//      return 0;
//   }
//   
//   @Override
//   public int getSpaceBeforeButton(CaptionButtonType type)
//   {
//      return 0;
//   }
//   
//   @Override
//   public double layoutTitleBar(TopLevelWindow window,
//                                Widget icon,
//                                Label title,
//                                CaptionButton[] captionButtons,
//                                double width,
//                                CoordinatesConversion c)
//   {
//      return 0;
//   }
//   
//   @Override
//   public Font getTitleBarFont()
//   {
//      return null;
//   }
//   
//   @Override
//   public Color getTitleBarTextColor(boolean active)
//   {
//      return null;
//   }
//   
//   @Override
//   public void drawCaptionButton(CaptionButton button,
//                                 GuiDriver gd,
//                                 int x,
//                                 int y,
//                                 int width,
//                                 int height)
//   {
//      
//   }
//   
//   @Override
//   public void drawWindowTitleBar(GuiDriver gd,
//                                  int x,
//                                  int y,
//                                  int width,
//                                  int height,
//                                  boolean active)
//   {
//      
//   }
//   
//   @Override
//   public void drawWindowTitle(GuiDriver gd,
//                               GuiFontResolver gf,
//                               int x,
//                               int y,
//                               int width,
//                               int height,
//                               String title,
//                               boolean active)
//   {
//      
//   }
   
   @Override
   public void drawWindowBorder(GuiDriver gd,
                                int x,
                                int y,
                                int width,
                                int height,
                                NativeInsets border,
                                boolean active)
   {
      gd.setColor(pal[active ? HIGHLIGHTED : NORMAL]);
      gd.drawRect(x, y, width - 1, height - 1);
      gd.drawRect(x + border.left, y + border.top, 
                  width - border.left - border.right - 1, 
                  height - border.top - border.bottom - 1);
   }
   
//   @Override
//   public void drawStatusBar(GuiDriver gd,
//                             GuiColorResolver gc,
//                             int x,
//                             int y,
//                             int width,
//                             int height,
//                             String text,
//                             GuiFontResolver.FontCache cache,
//                             NativePoint txtOrig,
//                             NativeDimension txtDim,
//                             boolean resizable)
//   {
//      
//   }
   
   /**
    * Renders borders of a top-level window.
    *
    * @param   button
    *          The {@link ButtonGuiImpl} to be rendered.
    * @param   gd
    *          The {@code GuiDriver} used for drawing operations.
    * @param   gc
    *          The {@link GuiColorResolver} to be used for color selection.
    * @param   gf
    *          The {@code GuiFontResolver} to be used.
    * @param   x
    *          The horizontal location (in pixel units) of the widget to be painted/component.
    * @param   y
    *          The vertical location (in pixel units) of the widget to be painted/component.
    * @param   width
    *          The width of the painted widget/component. 
    * @param   height
    *          The height of the painted widget/component.
    */
   @Override
   public void drawButton(ButtonGuiImpl button,
                          GuiDriver gd,
                          GuiColorResolver gc,
                          GuiFontResolver gf,
                          int x,
                          int y,
                          int width,
                          int height)
   {
      ButtonConfig bcfg =   button.config();
      boolean flat =        bcfg.flatButton;
      boolean highlighted = button.highlighted() && !flat;
      boolean isDefault =   button.isDefaultButton();
      boolean mouseOver =   button.isMouseOver();
      boolean pressed =     button.isPressed() && mouseOver; // even is pressed, mouse exited bounds
      
      // draw button text
      FontDetails fnt = gf.font().font;
      gd.setGuiFont(fnt);
      int fontNum = gf.resolveFontNum();
      MnemonicInfo labelInfo = button.getMnemonic();
      String label = (labelInfo == null) ? button.textLabel() : labelInfo.getPrepLabel();
      
      label = label.toUpperCase(); // TODO: optional
      
      // do not simplify the line below, this makes the font height to be up nearest even number
      int tY = y + (height - 1) / 2;
      int tX = x + (width - button.getTextWidthNative(label, fontNum)) / 2;
      gd.setFontStyle(fnt.style);
      
      if (!button.isEnabled()) // disabled button
      {
         // for flat buttons only the text is displayed
         if (!flat)
         {
            gd.setColor(brighter(DISABLED)); // button fill
            gd.fillRect(x, y, width, height);
         }
      }
      else if (pressed)
      {
         gd.setColor(brighter(isDefault ? HIGHLIGHTED : NORMAL)); // button fill
         gd.fillRect(x, y, width, height);
      }
      else if (flat)
      {
         if (mouseOver)
         {
            gd.setColor(brighter(isDefault ? HIGHLIGHTED : NORMAL, .67f)); // button fill
            gd.fillRect(x, y, width, height);
         }
      }
      else // not flat
      {
         gd.setColor(pal[isDefault ? HIGHLIGHTED : NORMAL]); // button fill
         gd.fillRect(x, y, width, height);
      }
      
      // draw button content
      if (button.hasImages())
      {
         ImageGuiImpl img = null;
         
         // draw button body images
         if (pressed)
         {
            // determine what image to draw
            ImageGuiImpl imgDown = button.getImgDown();
            if (imgDown != null)
            {
               img = imgDown;
            }
            else
            {
               ImageGuiImpl imgUp = button.getImgUp();
               if (imgUp != null)
               {
                  img = imgUp;
               }
            }
         }
         else
         {
            // determine what image to draw
            ImageGuiImpl imgUp = button.getImgUp();
            if (button.isEnabled() && imgUp != null)
            {
               img = imgUp;
            }
            else
            {
               ImageGuiImpl imgDisabled = button.getImgDisabled();
               if (!button.isEnabled() && imgDisabled != null)
               {
                  img = imgDisabled;
               }
            }
         }
         
         // if we found it, draw!
         if (img != null)
         {
            img.draw();
         }
      }
      else
      {
         // draw text:
         if (!button.isEnabled())
         {
            gd.setColor(pal[DISABLED]);
         }
         else if (flat)
         {
            gd.setColor(pal[NORMAL]);
         }
         else
         {
            gd.setColor(pal[mouseOver ? BACKGROUND : FOREGROUND]);
         }
         
         if (labelInfo != null && labelInfo.getMnemIdx() != null)
         {
            NativePoint tp = new NativePoint(tX, tY);
            gd.drawStringWithMnemonic(label, tp, labelInfo, gf, button, true);
         }
         else
         {
            // no mnemonic available
            gd.drawStringCentered(label, tX, tY);
         }
      }
      
      // focused button needs special highlight
      if (button.isEnabled() && highlighted && !button.config().noFocus)
      {
         // set dotted line style
         gd.setLineStroke(LineStroke.DOTTED_SMALL);
         
         gd.setColor(pressed ? brighter(FOREGROUND)   // pressed button selection dotted line
                             : pal[FOREGROUND]);      // normal button selection dotted line
         gd.drawRect(x + 2, y + 2, width - 5, height - 5);
         
         // restore default line style
         gd.setLineStroke(LineStroke.DEFAULT);
      }
   }
   
   /**
    * Renders a GUI toggle box widget.
    * 
    * @param   cb
    *          The {@link ToggleBoxGuiImpl} to be rendered.
    * @param   gd
    *          The {@code GuiDriver} used for drawing operations.
    * @param   gc
    *          The {@link GuiColorResolver} to be used for color selection.
    * @param   gf
    *          The {@code GuiFontResolver} to be used.
    * @param   x
    *          The horizontal location (in pixel units) of the widget to be painted/component.
    * @param   y
    *          The vertical location (in pixel units) of the widget to be painted/component.
    * @param   width
    *          The width of the painted widget/component. 
    * @param   height
    *          The height of the painted widget/component.
    * @param   pressed
    *          {@code true} if the checkbox is pressed (the mouse button is down on the button).
    * @param   hover
    *          {@code true} if mouse is hovering over the checkbox. Some styles are mouse hovering
    */
   public void drawToggleBox(ToggleBoxGuiImpl cb,
                             GuiDriver gd,
                             GuiColorResolver gc,
                             GuiFontResolver gf,
                             int x,
                             int y,
                             int width,
                             int height,
                             boolean pressed,
                             boolean hover)
   {
      String text = cb.getMnemonicLabel(false).orElse(cb.config().label);
      ToggleBoxConfig config = cb.config();
      GuiFontResolver.FontCache fntCache = gf.font();
      boolean enabled = cb.isEnabled();
      boolean highlighted = cb.highlighted(); // toggle-box state flags
      boolean indeterminate = false;
      
      // clear the area
      gd.setColor(gc.bgColor);
      gd.fillRect(x, y, width, height);
      
      int imgS = Math.min(height, width) - 4; // the size of the toggle rectangle, 2px left around
      int imgX = x + 1;
      int imgY = y + (height - imgS) / 2;
      
      gd.setLineStroke(LineStroke.SPECIFIED_WIDTH, 2);
      
      if (config.checked || indeterminate)
      {
         if (enabled)
         {
            if (pressed)
            {
               gd.setColor(brighter(NORMAL));
            }
            else if (hover)
            {
               gd.setColor(brighter(NORMAL, .51f));
            }
            else
            {
               gd.setColor(pal[NORMAL]);
            }
            gd.fillRoundRect(imgX, imgY, imgS, imgS, 4);
         }
         
         gd.setColor(pal[BACKGROUND]);
         drawCheckmark(gd, imgX, imgY, imgS, imgS, !indeterminate);
      }
      else
      {
         if (enabled && (pressed || hover))
         {
            if (hover)
            {
               gd.setColor(brighter(NORMAL, .51f));
            }
            else
            {
               // pressed
               gd.setColor(brighter(NORMAL));
            }
            gd.fillRoundRect(imgX + 1, imgY + 1, imgS - 2, imgS - 2, 4);
         }
         
         if (!enabled)
         {
            gd.setColor(pal[DISABLED]);
         }
         else
         {
            gd.setColor(brighter(FOREGROUND));
         }
         
         gd.drawRoundRect(imgX, imgY, imgS - 1, imgS - 1, 4);
      }
      
      // restore line width:
      gd.setLineStroke(LineStroke.DEFAULT, 1);
      
      // draw the text to the right of the checkbox
      if (text != null && text.length() > 0)
      {
         int xt = imgX + imgS + getCheckBoxSpace(); // space between checkbox and label
         int yt = y + height / 2;
         
         gd.setGuiFont(fntCache.font);
         gd.setFontStyle(fntCache.font.style);
         
         // draw the light text first!
         gd.setColor(pal[enabled ? FOREGROUND : DISABLED]);
         
         MnemonicInfo info = cb.getMnemonic();
         if (info != null && info.getMnemIdx() != null)
         {
            NativePoint tp = new NativePoint(xt, yt);
            gd.drawStringWithMnemonic(text, tp, info, gf, cb, true);
         }
         else
         {
            gd.drawStringCentered(text, xt, yt);
         }
         
         // focused toggle-box needs special highlight
         if (highlighted)
         {
            int font = gf.resolveFontNum();
            int textWidth = cb.getTextWidthNative(text, font) + 1;
            int textHeight = cb.getTextHeightNative(text, font) + 2;
            yt = y + (height - textHeight) / 2;
            
            gd.setColor(pal[FOREGROUND]);
            gd.setLineStroke(LineStroke.DOTTED_SMALL_THIN);
            gd.drawRect(xt - 1, yt, textWidth, textHeight);
            gd.setLineStroke(LineStroke.DEFAULT);
         }
      }
   }
   
   /**
    * Renders a GUI radio button widget.
    * @param  rb
    *         The {@link RadioButtonGuiImpl} to be rendered.
    * @param  gd
    *         The {@code GuiDriver} used for drawing operations.
    * @param   gc
    *         The {@link GuiColorResolver} to be used for color selection.
    * @param  gf
    *         The {@code GuiFontResolver} to be used.
    * @param  x
    *         The horizontal location (in pixel units) of the widget to be painted/component.
    * @param  y
    *         The vertical location (in pixel units) of the widget to be painted/component.
    * @param  width
    *         The width of the painted widget/component. 
    * @param  height
    *         The height of the painted widget/component.
    * @param  focused
    *         {@code true} if the radio button have the focus
    * @param  hover
    *         {@code true} if mouse is hovering over the checkbox. Some styles are mouse hovering
    */
   @Override
   public void drawRadioButton(RadioButtonGuiImpl rb,
                               GuiDriver gd,
                               GuiColorResolver gc,
                               GuiFontResolver gf,
                               int x,
                               int y,
                               int width,
                               int height,
                               boolean focused,
                               boolean hover)
   {
      // first draw the check mark image
      boolean isEnabled = rb.isEnabled() && rb.parent().isEnabled();
      boolean pressed = false; // TODO: add support for this
      
      int imgS = Math.min(height, width) - 4; // the size of the toggle rectangle, 1px left around
      int imgX = x + 1;
      int imgY = y + (height - imgS) / 2;
      
      // set line width = 3:
      gd.setLineStroke(LineStroke.SPECIFIED_WIDTH, 2);
      
      if (isEnabled && (hover || pressed))
      {
         if (pressed)
         {
            gd.setColor(brighter(NORMAL));
         }
         else
         {
            gd.setColor(brighter(NORMAL, .67f));
         }
         gd.fillEllipse(imgX + 1, imgY + 1, imgS - 2, imgS - 2);
      }
      
      if (!isEnabled)
      {
         gd.setColor(pal[DISABLED]);
      }
      else if (rb.isSelected())
      {
         gd.setColor(pal[NORMAL]);
      }
      else
      {
         gd.setColor(brighter(FOREGROUND));
      }
      
      drawRadioButton(gd, imgX, imgY, imgS, imgS, rb.isSelected());
      
      // restore line width:
      gd.setLineStroke(LineStroke.DEFAULT, 1);
      
      // draw button text
      String label = rb.getText();
      FontDetails<?>  fnt = gf.font().font;
      int fntNum = gf.resolveFontNum();
      
      gd.setGuiFont(fnt);
      int wt = rb.getTextWidthNative(label, fntNum);
      int ht = rb.getTextHeightNative(label, fntNum);
      int xt = imgX + imgS + 2;
      int yt = y + height / 2;
      gd.setFontStyle(fnt.style);
      // enabled state controls text drawing
      if (!isEnabled)
      {
         gd.setColor(pal[DISABLED]);
         gd.drawStringCentered(label, xt, yt);
      }
      else
      {
         gd.setColor(pal[FOREGROUND]);
         
         MnemonicInfo info = rb.getMnemonic();
         
         if (info != null && info.getMnemIdx() != null)
         {
            NativePoint tp = new NativePoint(xt, yt);
            gd.drawStringWithMnemonic(label, tp, info, gf, rb, true);
         }
         else
         {
            gd.drawStringCentered(label, xt, yt);
         }
      }
      
      if (focused)
      {
         // set dotted line style
         gd.setLineStroke(LineStroke.DOTTED_SMALL);
         // set dotted rectangle width
         int drWidth = (xt + wt) <= width ? wt + 1 : width - xt;
         int drHeight = Math.min(ht, height);
         // draw dotted rectangle depending on centered flag and item height
         gd.drawRect(xt - 1, y + (height - drHeight) / 2, drWidth, drHeight);
         // restore default line style
         gd.setLineStroke(LineStroke.DEFAULT);
      }
   }
   
   @Override
   public int getAlertBoxPadding()
   {
      return 20;
   }
   
//   @Override
//   public void drawAlertBox(GuiDriver gd,
//                            int x,
//                            int y,
//                            int width,
//                            int height,
//                            int paragraphWidth,
//                            ImageWrapper icon,
//                            String text,
//                            FontDetails font)
//   {
//      
//   }
   
   /**
    * Obtain the extra distance in a radio button. 
    *
    * @return  the extra distance in a radio button.
    */
   @Override
   public int getRadioButtonExtraDist()
   {
      return 3;
   }
   
   /**
    * Obtain the distance before text, including the radio button (text offset).
    *
    * @return  the distance before text, including the radio button.
    */
   @Override
   public int getRadioButtonImageSpace()
   {
      return 20;
   }
   
   /**
    * Obtains the space needed by a checkbox widget.
    *
    * @return  the space needed by a checkbox widget.
    */
   @Override
   public int getCheckBoxSpace()
   {
      return 2;
   }
   
   /**
    * Obtain the size of image on a checkbox widget.
    *
    * @return size of image on a checkbox widget.
    */
   @Override
   public int getCheckBoxSize()
   {
      return 15;
   }
   
   /**
    * Obtain the size of the scroll button on the direction perpendicular to scroll direction.
    *
    * @return  size of the scroll button on the direction perpendicular to scroll direction.
    */
   @Override
   public int getScrollBarButtonWidth()
   {
      return 12;  // no buttons!
   }
   
   /**
    * Obtain the thumb/slider width in a scrollbar widget.
    *
    * @return  the thumb/slider width in a scrollbar widget.
    */
   @Override
   public int getScrollBarThumbWidth()
   {
      return getScrollBarGaugeWidth();
   }
   
   /**
    * Obtain the minimum size of the thumb/slider of a scrollbar widget.
    *
    * @return  the minimum size of the thumb/slider of a scrollbar widget.
    */
   @Override
   public int getScrollBarThumbMinSize()
   {
      return getScrollBarGaugeWidth();
   }
   
   /**
    * Obtain the width of the gauge of a scrollbar widget.
    *
    * @return  width of the gauge of a scrollbar widget.
    */
   @Override
   public int getScrollBarGaugeWidth()
   {
      return 12;
   }
   
   /**
    * Obtain the size of the scroll button on the direction of the scroll.
    *
    * @return  the size of the scroll button on the direction of the scroll.
    */
   @Override
   public int getScrollBarButtonHeight()
   {
      return 0; // no buttons!
   }
   
   /**
    * Renders the gauge (background) of the scrollbar widget. 
    *
    * @param   gd
    *          The {@code GuiDriver} used for drawing operations.
    * @param   gc
    *          The {@code GuiColorResolver} used color selection.
    * @param   x
    *          The horizontal location (in pixel units) of the widget to be painted/component.
    * @param   y
    *          The vertical location (in pixel units) of the widget to be painted/component.
    * @param   width
    *          The width of the painted widget/component. 
    * @param   height
    *          The height of the painted widget/component.
    * @param   horizontal
    *          The direction of the scrollbar, {@code true} for horizontal.
    */
   @Override
   public void drawScrollBarGauge(GuiDriver gd,
                                  GuiColorResolver gc,
                                  int x,
                                  int y,
                                  int width,
                                  int height,
                                  boolean horizontal)
   {
      int d = getScrollBarGaugeWidth();
      
      // clear the area's heads
      gd.setColor(gc.bgColor); // pal[BACKGROUND]); 
      gd.fillRect(x, y, d, d);
      if (horizontal)
      {
         gd.fillRect(x + width - d, y, d, d);
      }
      else
      {
         gd.fillRect(x, y + height - d, d, d);
      }
      
      gd.setColor(brighter(DISABLED));
      gd.fillEllipse(x, y, d, d);
      if (horizontal)
      {
         gd.fillEllipse(x + width - d, y, d, d);
         gd.fillRect(x + d / 2, y, width - d, d);
      }
      else
      {
         gd.fillEllipse(x, y + height - d, d, d);
         gd.fillRect(x, y + d / 2, d, height - d);
      }
   }
   
   /**
    * Renders the thumb/slider of the scrollbar widget.
    *
    * @param   gd
    *          The {@code GuiDriver} used for drawing operations.
    * @param   gc
    *          The {@code GuiColorResolver} used color selection.
    * @param   x
    *          The horizontal location (in pixel units) of the widget to be painted/component.
    * @param   y
    *          The vertical location (in pixel units) of the widget to be painted/component.
    * @param   width
    *          The width of the painted widget/component. 
    * @param   height
    *          The height of the painted widget/component.
    * @param   pressed
    *          If the mouse is dragging the thumb of the scrollbar.
    * @param   hover
    *          Is mouse hovering this component ?
    * @param   horizontal
    *          The direction of the scrollbar, {@code true} for horizontal.
    */
   @Override
   public void drawScrollBarThumb(GuiDriver gd,
                                  GuiColorResolver gc,
                                  int x,
                                  int y,
                                  int width,
                                  int height,
                                  boolean pressed,
                                  boolean hover,
                                  boolean horizontal)
   {
      int d = getScrollBarThumbWidth();
      
      gd.setColor(
            pressed ?  pal[HIGHLIGHTED] :
            hover ?    brighter(HIGHLIGHTED) :
            /*normal*/ pal[NORMAL]
      );
      gd.fillEllipse(x, y, d, d);
      
      if (horizontal)
      {
         gd.fillEllipse(x + width - d, y, d, d);
         gd.fillRect(x + d / 2, y, width - d, d);
      }
      else
      {
         gd.fillEllipse(x, y + height - d, d, d);
         gd.fillRect(x, y + d / 2, d, height - d);
      }
   }
   
   /**
    * Obtain the offset in a 3D bordered widget.
    *
    * @return  the offset in a 3D bordered widget.
    */
   @Override
   public int get3DBorderOffset()
   {
      return 0;
   }
   
   /**
    * Renders a button of the scrollbar widget. 
    *
    * @param   gd
    *          The {@code GuiDriver} used for drawing operations.
    * @param   gc
    *          The {@code GuiColorResolver} used color selection.
    * @param   x
    *          The horizontal location (in pixel units) of the widget to be painted/component.
    * @param   y
    *          The vertical location (in pixel units) of the widget to be painted/component.
    * @param   width
    *          The width of the painted widget/component. 
    * @param   height
    *          The height of the painted widget/component.
    * @param   pos
    *          The position of the button (UP/DOWN, LEFT/RIGHT).
    * @param   horizontal
    *          The direction of the scrollbar, {@code true} for horizontal.
    * @param   enabled
    *          If the button is enabled.
    * @param   pressed
    *          If the button is down (pressed by mouse).
    * @param   highlighted
    *          If the mouse is hovering over the button.
    */
   @Override
   public void drawScrollBarButton(GuiDriver gd,
                                   GuiColorResolver gc,
                                   int x,
                                   int y,
                                   int width,
                                   int height,
                                   ScrollBar.Position pos,
                                   boolean horizontal,
                                   boolean enabled,
                                   boolean pressed,
                                   boolean highlighted)
   {
      // NO OP - no buttons
   }
   
   /**
    * Obtain the character used to replace text in password fields. 
    *
    * @return  the password character for this theme is bullet char (U+2022).
    */
   @Override
   public char getPasswordCharacter()
   {
      return '\u273B'; //'\u25CF';
   }
   
   
   /**
    * Renders the border (and possibly the background) of the fill-in widget. 
    *
    * @param   gd
    *          The {@code GuiDriver} used for drawing operations.
    * @param   gc
    *          The {@code GuiColorResolver} used color selection.
    * @param   x
    *          The horizontal location (in pixel units) of the widget to be painted/component.
    * @param   y
    *          The vertical location (in pixel units) of the widget to be painted/component.
    * @param   width
    *          The width of the painted widget/component. 
    * @param   height
    *          The height of the painted widget/component.
    * @param   border3D
    *          If fill-in widget has a 3d border. Some flat {@code Theme}s may ignore this.
    * @param   hasFocus
    *          The fill-in widget is focused.
    * @param   hover
    *          The mouse is hovering over the fill-in widget.
    */
   @Override
   public void drawFillInBorder(GuiDriver gd,
                                GuiColorResolver gc,
                                int x,
                                int y,
                                int width,
                                int height,
                                boolean border3D,
                                boolean hasFocus,
                                boolean hover)
   {
      // TODO: add support for transparent editors
      
      gd.setColor(pal[hasFocus ? HIGHLIGHTED : NORMAL]);
      gd.drawLine(x, y + height - 1, x + width,  y + height - 1);
      
      if (hasFocus || hover)
      {
         gd.drawLine(x, y + height - 2, x + width, y + height - 2);
      }
   }
   
   /**
    * Renders caret of the fill-in widget.
    *
    * @param   fillIn
    *          The {@code FillInGuiImpl} widget whose caret is to be rendered.
    * @param   gd
    *          The {@code GuiDriver} used for drawing operations.
    * @param   x1
    *          The left extremity of the caret. 
    * @param   y1
    *          The top extremity of the caret.
    * @param   x2
    *          The right extremity of the caret. 
    * @param   y2
    *          The bottom extremity of the caret.
    */
   @Override
   public void drawFillInCaret(FillInGuiImpl fillIn,
                               GuiDriver gd,
                               int x1,
                               int y1,
                               int x2,
                               int y2)
   {
      gd.setXORComposite();
      setCompositeXorColor(gd);
      gd.drawLine(x1, y1, x2, y2);
      gd.resetComposite();
   }
   
//   @Override
//   public int getComboBoxFieldExtraSpace(boolean horiz, boolean is3D)
//   {
//      return 0;
//   }
//   
//   @Override
//   public int getComboBoxEntryShift()
//   {
//      return 0;
//   }
//   
//   @Override
//   public int getComboBoxButtonSize()
//   {
//      return 0;
//   }
//   
//   @Override
//   public void drawComboBoxEntryField(ComboBoxGuiImpl combo,
//                                      GuiDriver gd,
//                                      GuiColorResolver gc,
//                                      int x,
//                                      int y,
//                                      int width,
//                                      int height,
//                                      FontDetails<?> font,
//                                      boolean pressed,
//                                      boolean enabled,
//                                      boolean hover)
//   {
//      
//   }
//   
//   @Override
//   public void drawComboBoxFieldBorder(ComboBoxGuiImpl combo,
//                                       GuiDriver gd,
//                                       int x,
//                                       int y,
//                                       int width,
//                                       int height,
//                                       boolean pressed,
//                                       boolean enabled,
//                                       boolean hover)
//   {
//      
//   }
//   
//   @Override
//   public void drawComboBoxButton(ComboBoxGuiImpl combo,
//                                  GuiDriver gd,
//                                  int x,
//                                  int y,
//                                  int width,
//                                  int height,
//                                  boolean pressed,
//                                  boolean enabled,
//                                  boolean hover)
//   {
//      
//   }
   
   /**
    * Renders the background of a selection list. 
    *
    * @param   gd
    *          The {@code GuiDriver} used for drawing operations.
    * @param   x
    *          The horizontal location (in pixel units) of the widget to be painted/component.
    * @param   y
    *          The vertical location (in pixel units) of the widget to be painted/component.
    * @param   width
    *          The width of the painted widget/component. 
    * @param   height
    *          The height of the painted widget/component.
    */
   @Override
   public void drawSelectionListBackground(GuiDriver gd,
                                           GuiColorResolver gc,
                                           int x,
                                           int y,
                                           int width,
                                           int height)
   {
      // fill the background
      gd.setColor(pal[BACKGROUND]); //gc.bgColor);
      gd.fillRect(x, y, width, height);
   }
   
//   @Override
//   public void drawSelectionListItem(ComboBoxGuiImpl box,
//                                     GuiDriver gd,
//                                     GuiColorResolver gc,
//                                     int x,
//                                     int y,
//                                     int width,
//                                     int height,
//                                     String text,
//                                     int fontSize,
//                                     boolean enabled,
//                                     boolean highlighted,
//                                     boolean selected)
//   {
//      
//   }
//   
//   @Override
//   public int getComboBoxDropDownOffset()
//   {
//      return 0;
//   }
//   
//   @Override
//   public int getMenuLeftPad()
//   {
//      return 0;
//   }
//   
//   @Override
//   public int getMenuRightPad()
//   {
//      return 0;
//   }
   
   /**
    * Obtain the insets of a popup menu. They are used for drawing the box/ 3D border around the
    * popup menu.
    *
    * @return  the insets of a popup menu. 
    */
   @Override
   public NativeInsets getMenuInsets()
   {
      return new NativeInsets(1, 1, 1, 1);
   }
   
   /**
    * Obtain the width of the check mark to for togglable menus. 
    *
    * @return  the width of the check mark in menu items.
    */
   @Override
   public int getMenuCheckmarkWidth()
   {
      return 32;
   }
   
   /**
    * Obtain the width of the arrow marking menus with submenus (including additional space). 
    *
    * @return  the width of the arrow mark for sub menus.
    */
   @Override
   public int getMenuSmArrowWidth()
   {
      return 32;
   }
   
   /**
    * Obtain the minimum size of a titlebar element for this Theme, including any border. 
    *
    * @param   horiz
    *          If {@code true} returns the minimum width, if {@code false} returns minimum height.
    *
    * @return  The minimum size of a titlebar element on specified direction.
    */
   @Override
   public int getMenuBarElementMinSize(boolean horiz)
   {
      return horiz ? 32 : 20; // NOT used. For compatibility only
   }
   
   /**
    * Renders the background of a menu widget. 
    * @param   gd
    *          The {@code GuiDriver} used for drawing operations.
    * @param   x
    *          The horizontal location (in pixel units) of the widget to be painted/component.
    * @param   y
    *          The vertical location (in pixel units) of the widget to be painted/component.
    * @param   width
    *          The width of the painted widget/component. 
    * @param   height
    *          The height of the painted widget/component.
    * @param   popup
    *          {@code true} for popup menus.
    */
   @Override
   public void drawMenuBackground(GuiDriver gd,
                                  int x,
                                  int y,
                                  int width,
                                  int height,
                                  boolean popup)
   {
      if (!popup)
      {
         // this is a menu bar, NOT used. For compatibility only
         gd.setColor(pal[BACKGROUND]);
         gd.fillRect(x, y, width, height);
         return;
      }
      
      gd.setColor(pal[BACKGROUND]);
      gd.fillRect(x + 1, y + 1, width - 2, height - 2);
   
      gd.setColor(darker(BACKGROUND));
      gd.drawRect(x, y, x + width - 1, y + height - 1);
   }
   
   /**
    * Renders the separator line in a menu widget. 
    *
    * @param   gd
    *          The {@code GuiDriver} used for drawing operations.
    * @param   x
    *          The horizontal location (in pixel units) of the widget to be painted/component.
    * @param   y
    *          The vertical location (in pixel units) of the widget to be painted/component.
    * @param   width
    *          The width of the painted widget/component. 
    * @param   height
    *          The height of the painted widget/component.
    */
   @Override
   public void drawMenuSeparator(GuiDriver gd, int x, int y, int width, int height)
   {
      gd.setColor(darker(BACKGROUND));
      gd.drawLine(x, y + height / 2, x + width, y + height / 2);
   }
   
   /**
    * Obtain the fixed height of a separator in a popup menu widget.
    *
    * @return  the height of a separator in a popup menu.
    */
   @Override
   public int getMenuSeparatorHeight()
   {
      return 17;
   }
   
   /**
    * Obtain the height of a {@link MenuElement} in a menu widget.
    *
    * @return  the height of a normal item in a menu widget.
    */
   @Override
   public int getMenuItemHeight(MenuElement menuItem)
   {
      return 32;
   }
   
   /**
    * Renders the foreground content of menu item in a popup menu. 
    *
    * @param   item
    *          The {@link MenuItemGuiImpl} or {@link SubMenuGuiImpl} to be drawn.
    * @param   gd
    *          The {@code GuiDriver} used for drawing operations.
    * @param   x
    *          The horizontal location (in pixel units) of the widget to be painted/component.
    * @param   y
    *          The vertical location (in pixel units) of the widget to be painted/component.
    * @param   width
    *          The width of the painted widget/component. 
    * @param   height
    *          The height of the painted widget/component.
    * @param   textHeight
    *          The height of the text.
    * @param   focused
    *          {@code true} if this item is highlighted/focused.
    */
   @Override
   public void drawMenuItemContent(AbstractWidget item,
                                   GuiDriver gd,
                                   GuiFontResolver gf,
                                   int x,
                                   int y,
                                   int width,
                                   int height,
                                   int textHeight,
                                   boolean focused)
   {
      boolean enabled = item.isEnabled();
      int checkmarkWidth = getMenuCheckmarkWidth();
      String title;
      
      // select the color for both text and checkmark / submenu arrow 
      gd.setColor(pal[enabled ? FOREGROUND : DISABLED]);
      
      if (item instanceof MenuItemGuiImpl)
      {
         MenuItemGuiImpl mi = (MenuItemGuiImpl) item;
         MenuItemConfig cfg = mi.config();
         // the space for toggle box. Printed for all menu-items in sub-menus or pop-up menus
         if (cfg.toggleBox && cfg.checked)
         {
            int cmW = 28;
            int cmH = 28;
            drawCheckmark(gd, (checkmarkWidth - cmW) / 2, (height - cmH) / 2, cmW, cmH, true);
         }
      
         title = mi.getPreparedTitle();
      }
      else // SUbMenuGuiImpl case
      {
         title = ((SubMenuGuiImpl) item).getPreparedTitle();
      }
      
      int textYoffset = y + height / 2;
      NativePoint np = new NativePoint(x + checkmarkWidth + getMenuLeftPad(), 0);
      int tw = width - checkmarkWidth - getMenuLeftPad() - getMenuSmArrowWidth();
      NativeRectangle clip = AbstractWidget.clipRectangle(np, new NativeDimension(tw, height));
      gd.draw(np, clip, () ->
      {
         gd.setGuiFont(gf.font().font);
         gd.setFontStyle(gf.font().font.style);
         gd.drawStringCentered(title, 0, textYoffset);
      });
      
      if (item instanceof MenuItemGuiImpl)
      {
         String accel = ((MenuItemGuiImpl) item).config().accelerator;
         if (accel != null && !accel.isEmpty())
         {
            // accelerators may be drawn with different color
            gd.setColor(pal[enabled ? FOREGROUND : DISABLED]);
            // TODO: this is right aligned, it should be right aligned instead
            String accText = Keyboard.keyName(accel);
            int accWidth = item.getTextWidth(accText, gf.resolveFontNum());
            int accOffset = width - accWidth - getMenuSmArrowWidth();
            gd.drawStringCentered(accText, accOffset, textYoffset);
         }
      }
      else
      {
         ImageWrapper<BufferedImage> arrow = getPngImage(
               getBinaryResourceLocation() +
                     (enabled ? "/menu-submenu-en.png" : "/menu-submenu-dis.png"));
         if (arrow != null && arrow.getImage() != null)
         {
            gd.drawImage(arrow, width - getMenuSmArrowWidth(), (height - arrow.getHeight()) / 2,
                         arrow.getWidth(), arrow.getHeight(), 0, 0, false, false, true, false);
         }
      }
   }
   
   /**
    * Renders the background of an item in a menu widget. 
    *
    * @param   item
    *          The {@link MenuItemGuiImpl} or {@link SubMenuGuiImpl} to be drawn.
    * @param   gd
    *          The {@code GuiDriver} used for drawing operations.
    * @param   x
    *          The horizontal location (in pixel units) of the widget to be painted/component.
    * @param   y
    *          The vertical location (in pixel units) of the widget to be painted/component.
    * @param   width
    *          The width of the painted widget/component. 
    * @param   height
    *          The height of the painted widget/component.
    * @param   hover
    *          {@code true} if this item is highlighted/focused.
    */
   @Override
   public void drawMenuItemBackground(AbstractWidget item,
                                      GuiDriver gd,
                                      int x,
                                      int y,
                                      int width,
                                      int height,
                                      boolean hover)
   {
      if (Menu.isMenubarElement(item))
      {
         System.err.println(
               "WARNING: Use Theme.drawMenuBarElement() instead to render a Menu Bar element");
         return;
      }
      
      if (hover && item.isEnabled())
      {
         gd.setColor(darker(BACKGROUND));
         gd.fillRect(0, 0, width, height);
      }
   }
   
   /**
    * Draws an element of a menu in a menubar.
    *
    * @param   me
    *          The {@code MenuElement} widget to be painted.
    * @param   gd
    *          The {@code GuiDriver} used for drawing operations.
    * @param   gf
    *          GUI font resolver.
    * @param   x
    *          The horizontal location (in pixel units) of the widget to be painted/component.
    * @param   y
    *          The vertical location (in pixel units) of the widget to be painted/component.
    * @param   width
    *          The width of the painted widget/component. 
    * @param   height
    *          The height of the painted widget/component.
    * @param   hover
    *          {@code true} if this item is highlighted/focused.
    */
   @Override
   public void drawMenuBarElement(MenuElement me,
                                  GuiDriver gd,
                                  GuiFontResolver gf,
                                  int x,
                                  int y,
                                  int width,
                                  int height,
                                  boolean hover)
   {
      // Not implemented. For compatibility only
   }

//   @Override
//   public void drawLabel(LabelGuiImpl label,
//                         GuiDriver gd,
//                         GuiColorResolver gc,
//                         GuiFontResolver gf,
//                         int x,
//                         int y,
//                         int width,
//                         int height,
//                         boolean emptyMode)
//   {
//      
//   }
//   
//   @Override
//   public void drawLabelUnderline(GuiDriver gd,
//                                  int x,
//                                  int y,
//                                  int width,
//                                  int height,
//                                  ColorRgb fgColor)
//   {
//      
//   }
   
   /**
    * Highlight the currently active row in a {@link BrowseGuiImpl} widget.
    *
    * @param   gd
    *          The {@code GuiDriver} used for drawing operations.
    * @param   x
    *          The horizontal location (in pixel units) of the widget to be painted/component.
    * @param   y
    *          The vertical location (in pixel units) of the widget to be painted/component.
    * @param   width
    *          The width of the painted widget/component. 
    * @param   height
    *          The height of the painted widget/component. 
    */
   @Override
   public void drawBrowseRowHighlight(BrowseGuiImpl browse,
                                      GuiDriver gd,
                                      int x,
                                      int y,
                                      int width,
                                      int height)
   {
      for (int i = 0; i < browse.getCacheSize(); i++)
      {
         if (browse.getCachedRowNumber(i) != browse.getCurrentRow())
         {
            continue; // not the right row
         }
         
         // draw current row marker
         int left = 0;
         if (browse.config().box)
         {
            left += BORDER_THICKNESS;
         }
         
         int markersColumnWidth = 0;
         
         // marker column should check the browse widget config for existence
         BrowseColumnGuiImpl markersColumn = browse.getCurrentMarkersColumn();
         if (browse.hasRowMarkers() && markersColumn != null)
         {
            NativeDimension dim = markersColumn.physicalDimension();
            if (dim != null)
            {
               markersColumnWidth = dim.width;
            }
         }
         
         left += markersColumnWidth;
         int top = 0;
         
         if (browse.config().box)
         {
            top += BORDER_THICKNESS;
         }
         
         if (browse.hasTitle())
         {
            top += browse.getTitleHeight() + BORDER_THICKNESS;
         }
         
         top += browse.getTopRowOffset(i);
         
         int viewportWidth = browse.getViewportDimension().width;
         viewportWidth -= markersColumnWidth;
         
         int columnsWidth = browse.getTotalColumnsWidth();
         columnsWidth -= markersColumnWidth;
         
         // honor scrolling
         int visibleWidth = columnsWidth + browse.getScrollableColumnSet().physicalLocation().x;
         int rowWidth = Math.min(visibleWidth, viewportWidth);
         
         gd.setLineStroke(LineStroke.DOTTED_LARGE);
         gd.setXORComposite();
         setCompositeXorColor(gd);
         gd.drawRect(left, top, rowWidth - 1, browse.getRowHeight() - 1);
         gd.resetComposite();
         gd.setLineStroke(LineStroke.DEFAULT);
         
         return;
      }
   }
   
//   @Override
//   public void drawBrowseColumnDrag(BrowseGuiImpl browse,
//                                    GuiDriver gd,
//                                    int x,
//                                    int y,
//                                    int width,
//                                    int height)
//   {
//
//   }
//   
//   @Override
//   public void drawBrowseResizingColumn(BrowseGuiImpl browse,
//                                        GuiDriver gd,
//                                        int left,
//                                        int top,
//                                        int right,
//                                        int bottom)
//   {
//      
//   }
   
   /**
    * Draw the title panel of a {@link BrowseGuiImpl} widget.
    *
    * @param   browse
    *          The {@code BrowseGuiImpl} widget.
    * @param   gd
    *          The {@code GuiDriver} used for drawing operations.
    * @param   gc
    *          The {@code GuiColorResolver} used color selection.
    * @param   gf
    *          The {@code GuiFontResolver} to be used.
    * @param   x
    *          The horizontal location (in pixel units) of the widget to be painted/component.
    * @param   y
    *          The vertical location (in pixel units) of the widget to be painted/component.
    * @param   width
    *          The width of the painted widget/component. 
    * @param   height
    *          The height of the painted widget/component. 
    */
   @Override
   public void drawBrowseTitlePanel(BrowseGuiImpl browse,
                                    GuiDriver gd,
                                    GuiColorResolver gc,
                                    GuiFontResolver gf,
                                    int x,
                                    int y,
                                    int width,
                                    int height)
   {
      boolean focused = browse.isDrawnAsFocused();
      
      gd.setColor(pal[NORMAL]);
      gd.fillRect(x, y, width, height);
   
      String title = browse.config().title;
      if (title == null || title.isEmpty())
      {
         return;
      }
      
      // calculate text position
      int textWidth = browse.getTextWidth(title, gf.resolveFontNum());
      x += (width - textWidth) / 2;
      
      // draw title string
      gd.setColor(pal[focused ? BACKGROUND : FOREGROUND]);
      gd.setGuiFont(gf.font().font);
      gd.drawStringCentered(title, x, browse.getTitleHeight() / 2);
   }
   
   /**
    * Draw the label on a {@link BrowseColumnGuiImpl} widget.
    *
    * @param   column
    *          The {@code BrowseGuiImpl} widget.
    * @param   gd
    *          The {@code GuiDriver} used for drawing operations.
    * @param   x
    *          The horizontal location (in pixel units) of the widget to be painted/component.
    * @param   y
    *          The vertical location (in pixel units) of the widget to be painted/component.
    * @param   width
    *          The width of the painted widget/component. 
    * @param   height
    *          The height of the painted widget/component. 
    */
   @Override
   public void drawBrowseColumnLabel(BrowseColumnGuiImpl column,
                                     GuiDriver gd,
                                     int x,
                                     int y,
                                     int width,
                                     int height)
   {
      BrowseColumnConfig cfg = column.config();
      BrowseGuiImpl browse = column.getBrowse();
      String[] labelLines = column.getLabelLines();
      
      drawBrowseColumnLabelRect(column, gd, 0, false, column.isMarkersColumn(), false);
      
      if (labelLines.length == 0)
      {
         return;
      }
      
      // draw label text
      NativePoint textClipPoint = column.getLabelTextClipPoint();
      NativeRectangle textClipRectangle = column.getLabelTextClipRectangle(textClipPoint);
      
      for (int i = 0; i < labelLines.length; i++)
      {
         int lineNum = i;
         
         gd.draw(textClipPoint, textClipRectangle, () ->
               drawBrowseCellText(column,
                                  gd,
                                  true,
                                  false,
                                  labelLines[labelLines.length - lineNum - 1],
                                  textClipRectangle,
                                  browse.getLabelLinesNum() - lineNum - 1,
                                  -1));
      }
   
      if (column.config().sortAscending != null)
      {
         int arrowTopOffset = (browse.getLabelsHeight() - SORT_ARROW_HEIGHT) / 2 +
                              BORDER_THICKNESS;
         int textWidth = column.getColumnLabelTextWidth();
         
         int sortAreaWidth = column.getSortAreaWidth(); // sort arrow + gap + sort number
         int labelAreaWidth = column.getLabelAreaWidth();
         
         int leftSideWidth = textClipPoint.x +
               (Browse.isRightAligned(column) ?
                     textClipRectangle.width() :
                     Math.min(textWidth, textClipRectangle.width()));
         int rightSideWidth = sortAreaWidth + SORT_AREA_RIGHT_CLIP;
         int gap = labelAreaWidth - leftSideWidth - rightSideWidth;
         
         int sortAreaLeftClip = column.isDrawnAsPressed() ?
               SORT_AREA_LEFT_CLIP - LABEL_PRESSED_OFFSET :
               SORT_AREA_LEFT_CLIP;
         gap = Math.max(sortAreaLeftClip, Math.min(gap, SORT_ARROW_MAX_LEFT_OFFSET));
         
         NativePoint arrowClipPoint = new NativePoint(
               Math.max(leftSideWidth + gap, BORDER_THICKNESS + SORT_AREA_LEFT_CLIP),
               arrowTopOffset);
         
         // Draw sort arrow
         NativeDimension arrowDim = new NativeDimension(
               Math.min(SORT_ARROW_WIDTH,
                        labelAreaWidth - BORDER_THICKNESS - SORT_AREA_LEFT_CLIP - SORT_AREA_RIGHT_CLIP),
               SORT_ARROW_HEIGHT);
         NativeRectangle arrowClipRect = AbstractWidget.clipRectangle(arrowClipPoint, arrowDim);
         gd.draw(arrowClipPoint, arrowClipRect, () ->
         {
            ImageWrapper<?> arrowImage = browse.getSortArrowImage(cfg.sortAscending);
            gd.drawImage(arrowImage, 0, 0,
                         SORT_ARROW_WIDTH, SORT_ARROW_HEIGHT,
                         0, 0, false, false, false, false);
         });
         
         // Draw sort number
         if (cfg.sortNumber != null && cfg.sortNumber > 0)
         {
            int sortNumTopOffset = arrowTopOffset - (SORT_NUMBER_HEIGHT - SORT_ARROW_HEIGHT) / 2;
            
            NativePoint sortPos = new NativePoint(0, 0);
            NativeDimension sortDim = new NativeDimension(
                  column.getLabelAreaWidth() - SORT_AREA_RIGHT_CLIP,
                  column.physicalDimension().height);
            NativeRectangle sortNumClipRect = AbstractWidget.clipRectangle(sortPos, sortDim);
            gd.draw(sortPos, sortNumClipRect, () ->
            {
               ImageWrapper<?> numberImage = browse.getSortNumImage(cfg.sortNumber);
               gd.drawImage(numberImage,
                            arrowClipPoint.x + SORT_ARROW_WIDTH + GAP_BETWEEN_SORT_ARROW_AND_NUMBER,
                            sortNumTopOffset,
                            SORT_NUMBER_WIDTH,
                            SORT_NUMBER_HEIGHT,
                            0, 0, false, false, false, false);
            });
         }
      }
   }
   
   @Override
   public void drawBrowseMarkersColumn(BrowseColumnGuiImpl column,
                                       GuiDriver gd,
                                       int x,
                                       int y,
                                       int width,
                                       int height)
   {
      BrowseGuiImpl browse = column.getBrowse();
      int down = browse.getDown();
      int rowHeight = browse.getRowHeight();
   
      for (int i = 0; i < down; i++)
      {
         int cachedRowNumber = browse.getCachedRowNumber(i);
         boolean selected = browse.isRowSelected(cachedRowNumber);
         int topOffset = browse.getTopRowOffset(i);
      
         drawBrowseColumnLabelRect(column, gd, topOffset, true, false, selected);
      
         // current row - draw row marker
         if (cachedRowNumber == browse.getCurrentRow())
         {
            NativePoint p = new NativePoint(BrowseGuiImpl.BORDER_THICKNESS,
                                            topOffset + BrowseGuiImpl.BORDER_THICKNESS);
            NativeDimension d = new NativeDimension(
                  column.physicalDimension().width - 2 * BrowseGuiImpl.BORDER_THICKNESS,
                  rowHeight - BrowseGuiImpl.BORDER_THICKNESS);
            NativeRectangle clip =
                  AbstractWidget.clipRectangle(p, new NativeDimension(d.width - 1, d.height - 1));
         
            gd.draw(p, clip, () ->
            {
               ImageWrapper markerImg = getBrowseMarkerArrow(selected);
               int offsetLeft = (d.width - markerImg.getWidth()) / 2 - 1;
               int offsetTop = (d.height - markerImg.getHeight()) / 2 - 1;
            
               gd.drawImage(markerImg,
                            offsetLeft, offsetTop, markerImg.getWidth(), markerImg.getHeight(),
                            0, 0, false, false, false, false);
            });
         }
      }
   }
   
   /**
    * Draw a normal column {@link BrowseColumnGuiImpl} widget (without label).
    *
    * @param   column
    *          The {@code BrowseGuiImpl} widget.
    * @param   gd
    *          The {@code GuiDriver} used for drawing operations.
    * @param   x
    *          The horizontal location (in pixel units) of the widget to be painted/component.
    * @param   y
    *          The vertical location (in pixel units) of the widget to be painted/component.
    * @param   width
    *          The width of the painted widget/component. 
    * @param   height
    *          The height of the painted widget/component. 
    */
   @Override
   public void drawBrowseNormalColumn(BrowseColumnGuiImpl column,
                                      GuiDriver gd,
                                      int x,
                                      int y,
                                      int width,
                                      int height)
   {
      BrowseGuiImpl browse = column.getBrowse();
   
      // fill whole column background (except the label that is already drawn)
      gd.setColor(getColumnBackgroundColor(column));
      gd.fillRect(x, y, width, height);
   
      // draw cells
      int down = browse.getDown();
      int cacheSize = browse.getCacheSizeWithIntermediate();
      for (int i = 0; i < down; i++)
      {
         if (i < cacheSize)
         {
            String text = TOGGLE_BOX.equals(column.config().widgetType) ||
                  column.config().hyperlinked ?
                  null :
                  browse.getCellTextWithIntermediate(column, i);
            drawBrowseCell(column, gd, i, text);
         }
         else
         {
            drawBrowseCell(column, gd, i, null);
         }
      }
   
      // vertical separator
      if (browse.config().separators)
      {
         gd.setColor(getBrowseCellSeparatorColor(browse));
         gd.drawLine(x + width - 1, y, x + width - 1, y + height);
      }
   
      // draw current toggle-box cell outline *over* separators
      if (TOGGLE_BOX.equals(column.config().widgetType) &&
            browse.isCurrentEditingColumn(column) &&
            browse.isCurrentRowInView())
      {
         int viewportRow = browse.getViewportRow();
         gd.setColor(pal[FOREGROUND]);
         gd.setLineStroke(LineStroke.DEFAULT);
         gd.drawRect(0,
                     browse.getTopRowOffset(viewportRow),
                     column.physicalDimension().width - BrowseGuiImpl.BORDER_THICKNESS,
                     browse.getRowHeight() - BrowseGuiImpl.BORDER_THICKNESS);
      }
   
      // draw hyperlinks
      if (column.config().hyperlinked)
      {
         for (Widget widget : column.widgets())
         {
            widget.draw();
         }
      }
   }
//   
//   @Override
//   public void drawToolTip(GuiDriver gd,
//                           int x,
//                           int y,
//                           int width,
//                           int height,
//                           FontDetails fnt,
//                           String text)
//   {
//      
//   }
   
   /**
    * Draw the title panel of a {@link BrowseGuiImpl} widget.
    *
    * @param   frame
    *          The {@code FrameGuiImpl} widget.
    * @param   gd
    *          The {@code GuiDriver} used for drawing operations.
    * @param   gc
    *          The {@code GuiColorResolver} used color selection.
    * @param   gf
    *          The {@code GuiFontResolver} to be used.
    * @param   x
    *          The horizontal location (in pixel units) of the widget to be painted/component.
    * @param   y
    *          The vertical location (in pixel units) of the widget to be painted/component.
    * @param   width
    *          The width of the painted widget/component. 
    * @param   height
    *          The height of the painted widget/component. 
    */
   @Override
   public void drawFrameTitle(FrameGuiImpl frame,
                              GuiDriver gd,
                              GuiColorResolver gc,
                              GuiFontResolver gf,
                              int x,
                              int y,
                              int width,
                              int height)
   {
      // draw the frame title background depending on focused state
      gd.setColor(pal[frame.hasFocus() ? HIGHLIGHTED : NORMAL]);
      gd.fillRect(x, y, width, height);
   
      String title = frame.config().title;
      if (title == null)
      {
         return;
      }
      
      // calculate centered text position
      int textWidth = frame.getTextWidthNative(title, gf.resolveFontNum());
      int tX = x + (width - textWidth) / 2 - 1;
      int tY = y + (height - 1) / 2 - 1;
      
      // draw title string
      gd.setColor(pal[FOREGROUND]);
      gd.setGuiFont(gf.font().font);
      gd.drawStringCentered(title, tX, tY);
   }
   
//   @Override
//   public NativePoint getRowToggleOffset(BrowseColumnGuiImpl column, boolean editor)
//   {
//      return null;
//   }
//   
//   @Override
//   public NativeDimension getRowToggleSize(BrowseColumnGuiImpl column, boolean editor)
//   {
//      return null;
//   }
   
   /**
    * Draw the line border {@link LineBorderGuiImpl} widget.
    *
    * @param   border
    *          The {@code FrameGuiImpl} widget.
    * @param   gd
    *          The {@code GuiDriver} used for drawing operations.
    * @param   gc
    *          The {@code GuiColorResolver} used color selection.
    * @param   x
    *          The horizontal location (in pixel units) of the widget to be painted/component.
    * @param   y
    *          The vertical location (in pixel units) of the widget to be painted/component.
    * @param   width
    *          The width of the painted widget/component. 
    * @param   height
    *          The height of the painted widget/component. 
    */
   @Override
   public void drawLineBorder(LineBorderGuiImpl border,
                              GuiDriver gd,
                              GuiColorResolver gc,
                              int x,
                              int y,
                              int width,
                              int height)
   {
      // TODO: no line borders ?
   }
   
   /**
    * Draws a {@link RectangleGuiImpl} widget.
    *
    * @param   rect
    *          The {@code RectangleGuiImpl} widget.
    * @param   gd
    *          The {@code GuiDriver} used for drawing operations.
    * @param   gc
    *          The {@code GuiColorResolver} used color selection.
    * @param   x
    *          The horizontal location (in pixel units) of the widget to be painted/component.
    * @param   y
    *          The vertical location (in pixel units) of the widget to be painted/component.
    * @param   width
    *          The width of the painted widget/component. 
    * @param   height
    *          The height of the painted widget/component. 
    */
   @Override
   public void drawRectangle(RectangleGuiImpl rect,
                             GuiDriver gd,
                             GuiColorResolver gc,
                             int x,
                             int y,
                             int width,
                             int height)
   {
      int ROUNDNESS = 5; // rounding diameter in pixels
      RectangleConfig cfg = rect.config();
      if (cfg.groupBox)
      {
         // in group box, fill, roundness and line thickness are ignored
         gd.setColor(pal[NORMAL]);
         gd.drawRoundRect(x, y, width - 1, height - 1, ROUNDNESS - 2);
         return;
      }
      
      int lt = Math.max(cfg.edgePixels, 0); // line thickness
      
      if (cfg.filled)
      {
         // fill only the inner part of the rectangle, leaving around space for line thickness 
         gd.setColor(gc.bgColor);
         if (cfg.rounded)
         {
            gd.fillRoundRect(x + lt - 1, y + lt - 1,
                             width - 2 * lt + 1, height - 2 * lt + 1, ROUNDNESS);
         }
         else
         {
            gd.fillRect(x + lt, y + lt, width - 2 * lt, height - 2 * lt);
         }
      }
      
      gd.setColor(gc.bgColor); //pal[NORMAL]); //gc.fgColor);
      gd.setLineStroke(LineStroke.SPECIFIED_WIDTH, lt);
      if (cfg.rounded)
      {
         gd.drawRoundRect(x + lt / 2, y + lt / 2, width - lt, height - lt, ROUNDNESS);
      }
      else
      {
         gd.drawRect(x + lt / 2, y + lt / 2, width - lt, height - lt);
      }
      gd.setLineStroke(LineStroke.DEFAULT, 1);
   }
   
   /**
    * Draws a {@link ThreeDBorderGuiImpl} widget.
    *
    * @param   border
    *          The {@code RectangleGuiImpl} widget.
    * @param   gd
    *          The {@code GuiDriver} used for drawing operations.
    * @param   x
    *          The horizontal location (in pixel units) of the widget to be painted/component.
    * @param   y
    *          The vertical location (in pixel units) of the widget to be painted/component.
    * @param   width
    *          The width of the painted widget/component.
    * @param   height
    *          The height of the painted widget/component.
    */
   @Override
   public void draw3DBorder(ThreeDBorderGuiImpl border,
                            GuiDriver gd,
                            int x,
                            int y,
                            int width,
                            int height)
   {
      // let if flat
      gd.setColor(Color.RED);
      gd.drawRect(x, y, x + width - 1, y + height - 1);
   }
   
//   @Override
//   public void drawEditorContent(EditorGuiImpl editor,
//                                 GuiDriver gd,
//                                 GuiFontResolver gf,
//                                 GuiColorResolver gc,
//                                 int x,
//                                 int y,
//                                 int width,
//                                 int height,
//                                 NativePoint selStart,
//                                 NativePoint selEnd,
//                                 int dx,
//                                 int dy)
//   {
//      
//   }
//   
//   @Override
//   public void drawEditorCaret(EditorGuiImpl editor, GuiDriver gd, int x1, int y1, int x2, int y2)
//   {
//      
//   }
//   
//   @Override
//   public void drawSelectionFocus(GuiDriver gd, int x, int y, int width, int height)
//   {
//      
//   }
//   
//   @Override
//   public int getBrowseLabelOffsetLeft(boolean rightAligned)
//   {
//      return 0;
//   }
//   
//   @Override
//   public int getBrowseLabelOffsetRight()
//   {
//      return 0;
//   }
//   
//   @Override
//   public int getBrowseDataOffsetLeft(boolean rightAligned)
//   {
//      return 0;
//   }
//   
//   @Override
//   public void drawFileChooserThumbnailBackground(GuiDriver gd,
//                                                  int x,
//                                                  int y,
//                                                  int width,
//                                                  int height,
//                                                  boolean selected,
//                                                  boolean hover,
//                                                  boolean focus)
//   {
//      
//   }
//   
//   @Override
//   public ColorRgb getFileChooserForegroundColor(GuiDriver gd,
//                                                 boolean focused,
//                                                 boolean selected,
//                                                 boolean hover,
//                                                 boolean filename,
//                                                 boolean infoLabel,
//                                                 boolean infodata)
//   {
//      return null;
//   }
//   
//   @Override
//   public TextLookAndFeel getTextLookAndFeel(GuiDriver gd,
//                                             GuiColorResolver gc,
//                                             boolean enabledTextProperty,
//                                             boolean verticallyCenteredText)
//   {
//      return null;
//   }
//   
//   @Override
//   public TextLookAndFeel getTextLookAndFeel(GuiColorResolver gcd, GuiColorResolver gce)
//   {
//      return null;
//   }
   
   /**
    * Get foreground color of the selected cell type.
    *
    * @param   column
    *          The parent {@code BrowseColumnGuiImpl} of the cell. 
    * @param   label
    *          {@code true} for label cell, {@code false} for normal cell. 
    * @param   selected
    *          {@code true} if the row is selected.
    * @param   viewportRowIndex
    *          0-based number of the target row in the viewport (for cells only). -1 for general
    *          cell foreground color.
    *
    * @return  foreground color of the selected cell type.
    */
   @Override
   public ColorRgb getBrowseCellForegroundColor(BrowseColumnGuiImpl column,
                                                boolean label,
                                                boolean selected,
                                                int viewportRowIndex)
   {
      ColorRgb textColor;
      if (selected)
      {
         textColor = new ColorRgb(pal[BACKGROUND]); 
               //ColorManager.getSysColor(SysColor.COLOR_CAPTIONTEXT);
      }
      else if (label)
      {
         BrowseGuiImpl browse = column.getBrowse();
         textColor = browse.resolveColor(column.config().labelFgColor,
                                         column.config().ehLabelFgColor,
                                         browse.config().labelFgColor,
                                         browse.config().ehLabelFgColor,
                                         true,
                                         true);
      }
      else
      {
         BrowseGuiImpl browse = column.getBrowse();
         GuiCellAttributes cellAttr = null;
         if (viewportRowIndex >= 0)
         {
            cellAttr = browse.getCellAttr(viewportRowIndex, column.config().ordinal);
         }
      
         if (cellAttr != null && cellAttr.fgColor != null)
         {
            textColor = ColorManager.getFgColor(browse.window(), cellAttr.fgColor);
         }
         else
         {
            textColor = browse.resolveColor(column.config().columnFgColor,
                                            column.config().ehFgColor,
                                            browse.config().columnFgColor,
                                            browse.config().ehFgColor,
                                            true,
                                            false);
         }
      }
   
      return textColor;
   }
   
//   @Override
//   public ColorRgb getColumnBackgroundColor(BrowseColumnGuiImpl column)
//   {
//      return null;
//   }
//   
//   @Override
//   public ColorRgb getColumnLabelBackgroundColor(BrowseColumnGuiImpl column)
//   {
//      return null;
//   }
   
   /**
    * Get the cell separator color.
    *
    * @param   browse
    *          The browse widget.
    *
    * @return  cell separator color.
    */
   @Override
   public ColorRgb getBrowseCellSeparatorColor(BrowseGuiImpl browse)
   {
      return browse.config().ehSeparatorsColor != null
            ? browse.config().ehSeparatorsColor
            : browse.getSeparatorColor();
   }

//   @Override
//   public void drawSliderBackground(SliderGuiImpl slider,
//                                    GuiDriver gd,
//                                    GuiColorResolver gc,
//                                    int x,
//                                    int y,
//                                    int w,
//                                    int h)
//   {
//      
//   }
//   
//   @Override
//   public void drawSliderTrack(SliderGuiImpl slider,
//                               GuiDriver gd,
//                               GuiColorResolver gc,
//                               int x,
//                               int y,
//                               int w)
//   {
//      
//   }
//   
//   @Override
//   public void drawSliderThumb(SliderGuiImpl slider,
//                               GuiDriver gd,
//                               GuiColorResolver gc,
//                               int x,
//                               int y,
//                               int w,
//                               int h)
//   {
//      
//   }
//   
//   @Override
//   public void drawSliderTicMarks(SliderGuiImpl slider,
//                                  GuiDriver gd,
//                                  GuiColorResolver gc,
//                                  int x,
//                                  int y,
//                                  int length)
//   {
//      
//   }
//   
//   @Override
//   public void drawSliderFrame(SliderGuiImpl slider,
//                               GuiDriver gd,
//                               GuiColorResolver gc,
//                               int x,
//                               int y,
//                               int w,
//                               int h)
//   {
//      
//   }
//   
//   @Override
//   public void drawSliderValue(SliderGuiImpl slider,
//                               GuiDriver gd,
//                               GuiColorResolver gc,
//                               GuiFontResolver gf,
//                               int x,
//                               int y,
//                               int w,
//                               int h)
//   {
//      
//   }
   
   /**
    * Get the hyperlink font basing on the regular text font. E.g. hyperlinks may have the
    * same font as the regular text, but underlined.
    *
    * @param  windowId
    *         ID of the currentWindow.
    * @param  sourceFont
    *         Regular font.
    *
    * @return hyperlink font.
    */
   @Override
   public FontDetails getHyperlinkFont(WidgetId windowId, FontDetails sourceFont)
   {
      FontDetails res = sourceFont.copy();
      FontStyle fs = FontStyle.createStyle(res.style.isBold(), res.style.isItalic(), true);
   
      if (res.fontAlias != null)
      {
         res.fontAlias.key = FontManager.getFontKey(res.fontAlias.fontName,
                                                    res.fontAlias.pointSize,
                                                    res.style.isBold(),
                                                    res.style.isItalic(),
                                                    true);
         res.fontAlias.style = fs;
         res.fontAlias.font = null;
      }
      
      res.key = FontManager.getFontKey(res.fontName,
                                       res.pointSize,
                                       res.style.isBold(),
                                       res.style.isItalic(),
                                       true);
      res.style = fs;
      res = FontManager.registerFontDetails(windowId.asInt(), res);
      return res;
   }
   
   /**
    * Get the hyperlink color.
    *
    * @param  selectedRow
    *         {@code true} if the hyperlink belongs to the selected row.
    *
    * @return hyperlink color.
    */
   @Override
   public ColorRgb getHyperlinkColor(boolean selectedRow)
   {
      return selectedRow ? ColorManager.getSysColor(SysColor.COLOR_CAPTIONTEXT) : ColorRgb.BLUE;
   }
   
   /**
    * Set the selected cell background color into the {@code gd} {@link GuiDriver}.
    *
    * TODO: this is a dependency of {@link ClassicTheme}. It is being used only temporary, until
    *       the {#code broswe} support is added to this {@link Theme}. The goal of this method is
    *       to give the browse selected line a color from standard palette. 
    *
    * @param   gd
    *          The {@code GuiDriver} to set the color for separators.
    * @param   unselColor
    *          The color for unselected cells. If {@code null} then the unselected cells' 
    *          background is not drawn. 
    */
   protected void setBrowseCellBackgroundSelectionColor(GuiDriver gd, ColorRgb unselColor)
   {
      gd.setColor(brighter(NORMAL));
   }
   
   /**
    * Draw label-style rectangle.
    *
    * @param   column
    *          The {@code BrowseGuiImpl} widget.
    * @param   gd
    *          The {@code GuiDriver} used for drawing operations.
    * @param   topOffset
    *          Top offset, in pixels.
    * @param   markersBody
    *          {@code true} if it is a cell in the markers column.
    * @param   markersLabel
    *          {@code true} if it is the label area in the markers column.
    * @param   selectedRow
    *          Is the current row selected ?
    */
   protected void drawBrowseColumnLabelRect(BrowseColumnGuiImpl column,
                                            GuiDriver gd,
                                            int topOffset,
                                            boolean markersBody,
                                            boolean markersLabel,
                                            boolean selectedRow)
   {
      NativeDimension d = column.physicalDimension();
      BrowseGuiImpl browse = column.getBrowse();
      int width = d.width;
      int height = ((topOffset == 0) ? browse.getLabelsHeight() : browse.getRowHeight()) + 
                   BrowseGuiImpl.BORDER_THICKNESS;
      if (!markersBody)
      {
         // marker-body height does not include separator lines 
         height += BrowseGuiImpl.BORDER_THICKNESS;
      }
      
      gd.setLineStroke(LineStroke.DEFAULT); // be safe ?
      BrowseConfig browseConfig = browse.config();
      if (markersBody || markersLabel)
      {
         // marker label
         drawBrowseColumnMarker(browse, gd, selectedRow, topOffset, width, height);
   
         if (browseConfig.separators)
         {
            // set the dark border color
            gd.setColor(getBrowseCellSeparatorColor(browse));
            gd.drawLine(0, height, width, height);
         }
      }
      else
      {
         // normal label
         
         // check override color first:
         ColorRgb bkColor = null;
         
         if (column.config().ehLabelBgColor != null)
         {
            bkColor = column.config().ehLabelBgColor;
         }
         else if (browseConfig.ehLabelBgColor != null)
         {
            bkColor = browseConfig.ehLabelBgColor;
         }
         else
         {
            int overColumnColor = column.config().labelBgColor;
            if (overColumnColor >= 0)
            {
               bkColor = ColorManager.getBgColor(column.window(), overColumnColor);
            }
            else
            {
               int overBrowseColor = browseConfig.labelBgColor;
               if (overBrowseColor >= 0)
               {
                  bkColor = ColorManager.getBgColor(column.window(), overBrowseColor);
               }
            }
         }
         // background Color
         if (bkColor != null)
         {
            gd.setColor(bkColor);
         }
         else if (column.isDrawnAsPressed())
         {
            gd.setColor(pal[HIGHLIGHTED]);
         }
         else
         {
            gd.setColor(gd.getSysColor(COLOR_3DFACE));
         }
         
         // one additional pixel to the left
         int dx = column.isFirstColumn() ? 0 : -1;
         // fill entire label rect
         gd.fillRect(dx + 1, topOffset + 1, width - 2 - dx, height - 2);
         
         int w = width - 1 - dx;
         // one pixel more to the height
         int h = height - 1;
         
         drawBrowseColumnLabelBorder(column, gd, dx, topOffset, w, h);
      }
   }
   
   /**
    * Draw browse row marker column rectangle.
    *
    * @param   browse
    *          The browse widget.
    * @param   gd
    *          The {@code GuiDriver} used for drawing operations.
    * @param   selected
    *          Is the current row selected ?
    * @param   topOffset
    *          Top offset, in pixels.
    * @param   w
    *          The border width in pixels.
    * @param   h
    *          The border height in pixels.
    */
   private void drawBrowseColumnMarker(BrowseGuiImpl browse, 
                                       GuiDriver gd,
                                       boolean selected,
                                       int topOffset,
                                       int w,
                                       int h)
   {
      h--; w--;
      if (selected)
      {
         gd.setColor(pal[HIGHLIGHTED]);
      }
      else
      {
         gd.setColor(gd.getSysColor(COLOR_3DFACE));
      }
      gd.fillRect(0, topOffset, w, h);
      
      
      gd.setColor(getBrowseCellSeparatorColor(browse));
      gd.drawLine(0, topOffset + h, w, topOffset + h);
      gd.drawLine(w, topOffset,     w, topOffset + h);
   }
   
   /**
    * Draw label-style rectangle.
    *
    * @param   column
    *          The {@code BrowseColumnGuiImpl} widget.
    * @param   gd
    *          The {@code GuiDriver} used for drawing operations.
    * @param   xOffset
    *          Left offset, in pixels.
    * @param   yOffset
    *          Top offset, in pixels.
    * @param   w
    *          The border width in pixels.
    * @param   h
    *          The border height in pixels.
    */
   protected void drawBrowseColumnLabelBorder(BrowseColumnGuiImpl column,
                                              GuiDriver gd,
                                              int xOffset,
                                              int yOffset,
                                              int w,
                                              int h)
   {
      // no 3d effect, let it flat
      BrowseGuiImpl browse = column.getBrowse();
      if (browse.config().separators)
      {
         gd.setColor(getBrowseCellSeparatorColor(browse));
         gd.drawLine(xOffset,     yOffset + h, xOffset + w, yOffset + h);
         gd.drawLine(xOffset + w, yOffset,     xOffset + w, yOffset + h);
      }
   }
   
   /**
    * Draw the text inside a label or cell.
    *
    * @param   column
    *          The {@code BrowseGuiImpl} widget.
    * @param   gd
    *          The {@code GuiDriver} used for drawing operations.
    * @param   label
    *          {@code true} to draw the text inside a label. {@code false} to draw the text inside
    *          a cell.
    * @param   selectedRow
    *          {@code true} if the row the cell belongs to is selected.
    * @param   text
    *          ext to draw.
    * @param   clipRectangle
    *          Clipping rectangle.
    * @param   lineNumber
    *          0-based number of line in the label/cell.
    * @param   viewportRowIndex
    *          0-based number of the target row in the viewport (for cells only).
    */
   protected void drawBrowseCellText(BrowseColumnGuiImpl column,
                                     GuiDriver gd,
                                     boolean label,
                                     boolean selectedRow,
                                     String text,
                                     NativeRectangle clipRectangle,
                                     int lineNumber,
                                     int viewportRowIndex)
   {
      gd.setColor(getBrowseCellForegroundColor(column, label, selectedRow, viewportRowIndex));
      
      FontDetails fd = label ? column.getLabelFont() : column.getCellsFont();
      gd.setGuiFont(fd);
      
      int y = label ? column.getLabelTextTopOffset() : column.getCellTextTopOffset();
      y += fd.legacyHeight * lineNumber;
      
      int x = 0;
      if (Browse.isRightAligned(column))
      {
         x = clipRectangle.width() - 
             column.getTextWidth(text, label ? column.getLabelFont() : column.getCellsFont());
      }
      
      gd.drawString(text, x, y);
   }
   
   /**
    * Utility method that loads a image (recommended in PNG format because it is lossless and has
    * support for full 8 bit alpha channel, but not otherwise restricted to). If resource is
    * successfully loaded, it is returned in wrapper and cached for subsequent fast accesses.
    *
    * @param   imagePath
    *          The path to image resource inside the bundle (jar).
    *
    * @return  an {@code ImageWrapper} containing the requested image resource or {@code null} if
    *          some error occurred (resource could not be found or it could not be decoded).
    */
   protected ImageWrapper<BufferedImage> getPngImage(String imagePath)
   {
      ImageWrapper<BufferedImage> img = imgCache.get(imagePath);
      if (img == null)
      {
         InputStream res = MaterialTheme.class.getResourceAsStream(imagePath);
         if (res != null)
         {
            try
            {
               BufferedImage image = ImageIO.read(res);
               img = new ImageWrapper<>(image, image.getHeight(), image.getWidth());
            }
            catch (IOException e)
            {
               // TODO: create a dummy image ?
            }
         }
         
         if (img == null)
         {
            System.err.println("Failed to load resource: [" + imagePath + "].");
         }
         
         // if image can't be found, store null to avoid repeating looking up non-existent image 
         imgCache.put(imagePath, img);
      }
      return img;
   }
   
   
   /**
    * Sets the composite color for XOR operations on browse widgets.
    *
    * @param   gd
    *          the active {@code GuiDriver} used to draw the browse widget.
    */
   @Override
   protected void setCompositeXorColor(GuiDriver gd)
   {
      gd.setColor(new Color(0xC0C0C0));
   }
   
   /**
    * Draws a radio-button in the designated area. The color and line style are already configured
    * in {@code gd}.
    *
    * @param   gd
    *          The {@code GuiDriver} used for drawing operations.
    * @param   x
    *          The X coordinate of the checkmark.
    * @param   y
    *          The Y coordinate of the checkmark.
    * @param   width
    *          The width of the checkmark.
    * @param   height
    *          The height of the checkmark.
    * @param   selected
    *          Whether to draw the radio-button as selected. 
    */
   private void drawRadioButton(GuiDriver gd,
                                int x,
                                int y,
                                int width,
                                int height,
                                boolean selected)
   {
      gd.drawEllipse(x, y, width, height);
      if (selected)
      {
         gd.fillEllipse((int) (x + width / 4f + .5f), (int) (y + height / 4f + .5f),
                        (int) (width / 2f + .5f), (int) (height / 2f + .5f));
      }
   }
   
   /**
    * Draws a checkmark in the designated area. The color and line style are already configured in
    * {@code gd}.
    *
    * @param   gd
    *          The {@code GuiDriver} used for drawing operations.
    * @param   x
    *          The X coordinate of the checkmark.
    * @param   y
    *          The Y coordinate of the checkmark.
    * @param   width
    *          The width of the checkmark.
    * @param   height
    *          The height of the checkmark.
    * @param   check
    *          Whether to draw a checkmark or an indeterminate state. 
    */
   private void drawCheckmark(GuiDriver gd,
                              int x,
                              int y,
                              int width,
                              int height,
                              boolean check)
   {
      if (!check)
      {
         gd.drawLine(x, y + height / 2, x + width, y + height / 2);
         return;
      }
      float x1 =  6 / 33f;
      float y1 = 19 / 33f;
      float x2 = 12 / 33f;
      float y2 = 25 / 33f;
      float x3 = 28 / 33f;
      float y3 =  9 / 33f;
      gd.drawLine((int) (x + width * x1 + .5f), (int) (y + height * y1 + .5f),
                  (int) (x + width * x2 + .5f), (int) (y + height * y2 + .5f));
      gd.drawLine((int) (x + width * x2 + .5f), (int) (y + height * y2 + .5f),
                  (int) (x + width * x3 + .5f), (int) (y + height * y3 + .5f));
   }
   
   /**
    * Creates a new, darker {@code Color} version of {@code palEntry} in color palette.
    * <p>
    * This method uses a constant factor of 0.83 to each of the three RGB components of reference
    * to create a darker version of it. The {@code alpha} value is preserved.
    *
    * @param   palEntry
    *          The color to be used as reference for creating the new darker value.
    *
    * @return  a new {@code Color} object that is a darker version of this {@code c} with the 
    *          same {@code alpha} value.
    */
   private Color darker(int palEntry)
   {
      return darker(palEntry, .83f);
   }
   
   /**
    * Creates a new, darker {@code Color} version of {@code palEntry} in color palette.
    * <p>
    * This method uses {@code level} factor to each of the three RGB components of reference to
    * create a darker version of it. The {@code alpha} value is preserved.
    *
    * @param   palEntry
    *          The color to be used as reference for creating the new darker value.
    * @param   level
    *          The level of values to be kept. For 1, the same color is returned. As {code level}
    *          drops to 0, the darker the new color will be generated. When 0, {@link Color#BLACK}
    *          is returned.
    *
    * @return  a new {@code Color} object that is a darker version of this {@code c} with the 
    *          same {@code alpha} value.
    */
   private Color darker(int palEntry, float level)
   {
      if (palEntry < 0 || palEntry >= DEFAULT_PALETTE.length)
      {
         return Color.RED;
      }
      
      return darker(pal[palEntry], level);
   }
   
   /**
    * Creates a new, darker {@code Color} version of color {@code c}.
    * <p>
    * This method uses {@code level} factor to each of the three RGB components of  {@code c} to
    * create a darker version of it. The {@code alpha} value is preserved.
    *
    * @param   c
    *          The color to be used as reference for creating the new darker value.
    * @param   level
    *          The level of values to be kept. For 1, the same color is returned. As {code level}
    *          drops to 0, the darker the new color will be generated. When 0, {@link Color#BLACK}
    *          is returned.
    * 
    * @return  a new {@code Color} object that is a darker version of this {@code c} with the 
    *          same {@code alpha} value.
    */
   private Color darker(Color c, float level)
   {
      if (level < 0f || level >= 1f)
      {
         return c;
      }
      
      return new Color((int) (c.getRed() * level),
                       (int) (c.getGreen() * level),
                       (int) (c.getBlue() * level),
                       c.getAlpha());
   }
   
   /**
    * Creates a new, darker {@code Color} version of {@code palEntry} in color palette.
    * <p>
    * This method uses a constant factor of 0.83 to each of the three RGB components of reference
    * to create a darker version of it. The {@code alpha} value is preserved.
    *
    * @param   palEntry
    *          The color to be used as reference for creating the new darker value.
    *
    * @return  a new {@code Color} object that is a darker version of this {@code c} with the 
    *          same {@code alpha} value.
    */
   private Color brighter(int palEntry)
   {
      return brighter(palEntry, .83f);
   }
   
   /**
    * Creates a new, brighter {@code Color} version of {@code palEntry} in color palette.
    * <p>
    * This method uses {@code level} factor to each of the three RGB components of reference to
    * create a brighter version of it. The {@code alpha} value is preserved.
    *
    * @param   palEntry
    *          The color to be used as reference for creating the new brighter value.
    * @param   level
    *          The level of values to be kept. For 1, the same color is returned. As {code level}
    *          drops to 0, the brighter the new color will be generated. When 0, 
    *          {@link Color#WHITE} is returned.
    *
    * @return  a new {@code Color} object that is a brighter version of this {@code c} with the 
    *          same {@code alpha} value.
    */
   private Color brighter(int palEntry, float level)
   {
      if (palEntry < 0 || palEntry >= DEFAULT_PALETTE.length)
      {
         return Color.RED;
      }
      
      return brighter(pal[palEntry], level);
   }
   
   /**
    * Creates a new, brighter {@code Color} version of color {@code c}.
    * <p>
    * This method uses {@code level} factor to each of the three RGB components of {@code c} to
    * create a brighter version of it. The {@code alpha} value is preserved.
    *
    * @param   c
    *          The color to be used as reference for creating the new brighter value.
    * @param   level
    *          The level of values to be kept. For 1, the same color is returned. As {code level}
    *          drops to 0, the brighter the new color will be generated. When 0,
    *          {@link Color#WHITE} is returned.
    *
    * @return  a new {@code Color} object that is a brighter version of this {@code c} with the 
    *          same {@code alpha} value.
    */
   private Color brighter(Color c, float level)
   {
      if (level < 0f || level >= 1f)
      {
         return c;
      }
      
      int r = 255 - c.getRed();
      int g = 255 - c.getGreen();
      int b = 255 - c.getBlue();
   
      return new Color(255 - (int) (r * level),
                       255 - (int) (g * level),
                       255 - (int) (b * level),
                       c.getAlpha());
   }
}
