/*
** Module   : ExportFieldExpansion.java
** Abstract : Expansion for EXPORT array with dynamic indices.
**
** Copyright (c) 2017, Golden Code Development Corporation.
**
** -#- -I- --Date-- ----------------------------------Description---------------------------------
** 001 CA  20170827 Created initial version.
**     HC  20170830 Modified conversion of EXPORT to expand extent fields dynamically. This
**                  should prevent the Java source file to get out of control for large extents.
*/ 
/*
** This program is free software: you can redistribute it and/or modify
** it under the terms of the GNU Affero General Public License as
** published by the Free Software Foundation, either version 3 of the
** License, or (at your option) any later version.
**
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU Affero General Public License for more details.
**
** You may find a copy of the GNU Affero GPL version 3 at the following
** location: https://www.gnu.org/licenses/agpl-3.0.en.html
** 
** Additional terms under GNU Affero GPL version 3 section 7:
** 
**   Under Section 7 of the GNU Affero GPL version 3, the following additional
**   terms apply to the works covered under the License.  These additional terms
**   are non-permissive additional terms allowed under Section 7 of the GNU
**   Affero GPL version 3 and may not be removed by you.
** 
**   0. Attribution Requirement.
** 
**     You must preserve all legal notices or author attributions in the covered
**     work or Appropriate Legal Notices displayed by works containing the covered
**     work.  You may not remove from the covered work any author or developer
**     credit already included within the covered work.
** 
**   1. No License To Use Trademarks.
** 
**     This license does not grant any license or rights to use the trademarks
**     Golden Code, FWD, any Golden Code or FWD logo, or any other trademarks
**     of Golden Code Development Corporation. You are not authorized to use the
**     name Golden Code, FWD, or the names of any author or contributor, for
**     publicity purposes without written authorization.
** 
**   2. No Misrepresentation of Affiliation.
** 
**     You may not represent yourself as Golden Code Development Corporation or FWD.
** 
**     You may not represent yourself for publicity purposes as associated with
**     Golden Code Development Corporation, FWD, or any author or contributor to
**     the covered work, without written authorization.
** 
**   3. No Misrepresentation of Source or Origin.
** 
**     You may not represent the covered work as solely your work.  All modified
**     versions of the covered work must be marked in a reasonable way to make it
**     clear that the modified work is not originating from Golden Code Development
**     Corporation or FWD.  All modified versions must contain the notices of
**     attribution required in this license.
*/

package com.goldencode.p2j.util;

import com.goldencode.p2j.persist.*;

import java.util.*;

/**
 * TODO: finish this implementation
 * TODO: table extent field support?
 */
public class ExportFieldExpansion
implements ExtentExpansion
{
   private BaseDataType[] arr;
   
   private final integer start = new integer();
   
   private final integer end = new integer();
   
   private final character format = new character();

   /** Buffer fields to be expanded */
   private FieldReference[] fieldRefs;

   public ExportFieldExpansion(BaseDataType[] arr)
   {
      this.arr = arr;
      this.start.assign(0);
      this.end.assign(arr.length - 1);
   }

   public ExportFieldExpansion(BaseDataType[] arr, NumberType start, long end)
   {
      this.arr = arr;
      this.start.assign(start);
      this.end.assign(end);
   }
   
   public ExportFieldExpansion(BaseDataType[] arr, long start, long end)
   {
      this.arr = arr;
      this.start.assign(start);
      this.end.assign(end);
   }
   
   public ExportFieldExpansion(BaseDataType[] arr, NumberType start, long end, String format)
   {
      this.arr = arr;
      this.start.assign(start);
      this.end.assign(end);
      this.format.assign(format);
   }
   
   public ExportFieldExpansion(BaseDataType[] arr, long start, long end, String format)
   {
      this.arr = arr;
      this.start.assign(start);
      this.end.assign(end);
      this.format.assign(format);
   }

   /**
    * Constructs export-field expansion on top of buffer fields.
    *
    * @param   fieldRefs
    *          Buffer fields, must not be <code>null</code>.
    */
   public ExportFieldExpansion(FieldReference... fieldRefs)
   {
      if (fieldRefs == null)
      {
         throw new IllegalArgumentException();
      }

      this.fieldRefs = fieldRefs;
   }
   
   public FieldEntry[] expand()
   {
      // TODO: deduplicate this method implementation with ImportFieldExpansion

      // TODO: validate

      FieldEntry[] fields;

      if (fieldRefs != null)
      {
         ArrayList<FieldEntry> entries = new ArrayList<>();
         for (FieldReference ref : fieldRefs)
         {
            Integer extent = ref.getExtent();
            if (extent == null)
            {
               entries.add(new ExportField(ref));
            }
            else
            {
               for (int i = 0; i < extent; i++)
               {
                  entries.add(new ExportField(new FieldReference(ref, i)));
               }
            }
         }

         fields = entries.toArray(new FieldEntry[0]);
      }
      else
      {
         int istart = start.intValue();
         int iend = end.intValue();

         fields = new PutField[iend - istart + 1];

         for (int i = istart; i <= iend; i++)
         {
            int idx = i;
            // TODO: validate
            fields[i - istart] = new PutField(() -> arr[idx]);
         }
      }

      return fields;
   }
}
