/*
** Module   : PathResolver.java
** Abstract : Defines methods and their thrown exceptions to resolve a native system file path
**            into its host system file path.
**
** Copyright (c) 2018, Golden Code Development Corporation.
**
** -#- -I- --Date-- ---------------------------------Description----------------------------------
** 001 SBI 20180521 Initial version.
** 002 ECF 20180613 Define singleton instance of unresolved path exception. This exception is
**                  thrown very often and the stack trace is not important, so using a singleton
**                  avoids the significant overhead of capturing the stack.
*/
/*
** This program is free software: you can redistribute it and/or modify
** it under the terms of the GNU Affero General Public License as
** published by the Free Software Foundation, either version 3 of the
** License, or (at your option) any later version.
**
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU Affero General Public License for more details.
**
** You may find a copy of the GNU Affero GPL version 3 at the following
** location: https://www.gnu.org/licenses/agpl-3.0.en.html
**
** Additional terms under GNU Affero GPL version 3 section 7:
**
**   Under Section 7 of the GNU Affero GPL version 3, the following additional
**   terms apply to the works covered under the License.  These additional terms
**   are non-permissive additional terms allowed under Section 7 of the GNU
**   Affero GPL version 3 and may not be removed by you.
**
**   0. Attribution Requirement.
**
**     You must preserve all legal notices or author attributions in the covered
**     work or Appropriate Legal Notices displayed by works containing the covered
**     work.  You may not remove from the covered work any author or developer
**     credit already included within the covered work.
**
**   1. No License To Use Trademarks.
**
**     This license does not grant any license or rights to use the trademarks
**     Golden Code, FWD, any Golden Code or FWD logo, or any other trademarks
**     of Golden Code Development Corporation. You are not authorized to use the
**     name Golden Code, FWD, or the names of any author or contributor, for
**     publicity purposes without written authorization.
**
**   2. No Misrepresentation of Affiliation.
**
**     You may not represent yourself as Golden Code Development Corporation or FWD.
**
**     You may not represent yourself for publicity purposes as associated with
**     Golden Code Development Corporation, FWD, or any author or contributor to
**     the covered work, without written authorization.
**
**   3. No Misrepresentation of Source or Origin.
**
**     You may not represent the covered work as solely your work.  All modified
**     versions of the covered work must be marked in a reasonable way to make it
**     clear that the modified work is not originating from Golden Code Development
**     Corporation or FWD.  All modified versions must contain the notices of
**     attribution required in this license.
*/
package com.goldencode.p2j.util;


/**
 * Defines methods to resolve the legacy system file path to the path on the host system.
 */
public interface PathResolver
{
   /** Singleton unresolved path exception to avoid construction overhead */
   public static final UnresolvedPathException UNRESOLVED_ERROR = new UnresolvedPathException();
   
   /**
    * Resolve the given legacy path to the path on the host sytem.
    * 
    * @param    path
    *           The given legacy path to be resolved on the host system
    * 
    * @return   The resolved path on the host system
    * 
    * @throws   UnresolvedPathException
    *           If the given path can't be resolved
    */
   public String resolvePath(String path) throws UnresolvedPathException;
   
   /**
    * Defines the unresolved path exception.
    */
   @SuppressWarnings("serial")
   public static class UnresolvedPathException
   extends Exception
   {
      /**
       * Creates this instance for the given unresolved legacy system path
       */
      private UnresolvedPathException()
      {
         super();
      }
   }
}