/*
** Module   : SOAPHeaderEntry.java
** Abstract : Interface defining attributes/methods for SOAP-HEADER-ENTRYREF resource.
**
** Copyright (c) 2013-2017, Golden Code Development Corporation.
**
** -#- -I- --Date-- --------------------------------Description-----------------------------------
** 001 CA  20130216 Created initial version.
** 002 CA  20130221 Added ADM-DATA, UNIQUE-ID, NAMESPACE-URI and LOCAL-NAME support.
** 003 CA  20130927 Added LegacyResource annotation, to determine resource type.
** 004 EVL 20131008 Adding legacy attributes and methods annotation.
*/
/*
** This program is free software: you can redistribute it and/or modify
** it under the terms of the GNU Affero General Public License as
** published by the Free Software Foundation, either version 3 of the
** License, or (at your option) any later version.
**
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU Affero General Public License for more details.
**
** You may find a copy of the GNU Affero GPL version 3 at the following
** location: https://www.gnu.org/licenses/agpl-3.0.en.html
** 
** Additional terms under GNU Affero GPL version 3 section 7:
** 
**   Under Section 7 of the GNU Affero GPL version 3, the following additional
**   terms apply to the works covered under the License.  These additional terms
**   are non-permissive additional terms allowed under Section 7 of the GNU
**   Affero GPL version 3 and may not be removed by you.
** 
**   0. Attribution Requirement.
** 
**     You must preserve all legal notices or author attributions in the covered
**     work or Appropriate Legal Notices displayed by works containing the covered
**     work.  You may not remove from the covered work any author or developer
**     credit already included within the covered work.
** 
**   1. No License To Use Trademarks.
** 
**     This license does not grant any license or rights to use the trademarks
**     Golden Code, FWD, any Golden Code or FWD logo, or any other trademarks
**     of Golden Code Development Corporation. You are not authorized to use the
**     name Golden Code, FWD, or the names of any author or contributor, for
**     publicity purposes without written authorization.
** 
**   2. No Misrepresentation of Affiliation.
** 
**     You may not represent yourself as Golden Code Development Corporation or FWD.
** 
**     You may not represent yourself for publicity purposes as associated with
**     Golden Code Development Corporation, FWD, or any author or contributor to
**     the covered work, without written authorization.
** 
**   3. No Misrepresentation of Source or Origin.
** 
**     You may not represent the covered work as solely your work.  All modified
**     versions of the covered work must be marked in a reasonable way to make it
**     clear that the modified work is not originating from Golden Code Development
**     Corporation or FWD.  All modified versions must contain the notices of
**     attribution required in this license.
*/

package com.goldencode.p2j.util;

/**
 * Interface describing the features for SOAP-HEADER-ENTRY resource:
 * <ul>
 *    <li>{@link #getActor ACTOR attribute}</li>
 *    <li>{@link #isMustUnderstand MUST-UNDERSTAND attribute}</li>
 *    <li>{@link #deleteHeaderEntry DELETE-HEADER-ENTRY method}</li>
 *    <li>{@link #setActor SET-ACTOR method}</li>
 *    <li>{@link #setMustUnderstand SET-MUST-UNDERSTAND method}</li>
 *    <li>{@link #setNode SET-NODE method}</li>
 *    <li>{@link #setSerialized SET-SERIALIZED method}</li>
 * </ul>
 */
@LegacyResource(resource = LegacyResource.SOAP_HEADER_ENTRYREF)
public interface SOAPHeaderEntry
extends LocalName,
        ADMData,
        UniqueID,
        NamespaceURI
{
   /**
    * Returns the value of the actor attribute for the SOAP-HEADER-ENTRYREF object as a URL. 
    * Identifies the recipient of a header element.
    * 
    * @return   See above.
    */
   @LegacyAttribute(name = "ACTOR")
   public character getActor();

   /**
    * Indicates whether a SOAP-HEADER-ENTRYREF object is mandatory (TRUE) or optional (FALSE) for
    * the recipient to process.
    * 
    * @return   See above.
    */
   @LegacyAttribute(name = "MUST-UNDERSTAND")
   public logical isMustUnderstand();

   /**
    * Deletes the XML underlying a SOAP-HEADER-ENTRYREF object, without deleting the object.
    * 
    * @return   <code>true</code> if the operation was possible.
    */
   @LegacyMethod(name = "DELETE-HEADER-ENTRY")
   public logical deleteHeaderEntry();

   /**
    * Sets the ACTOR attribute for this SOAP-HEADER-ENTRY.
    *
    * @param    uri
    *           The URI of the SOAP actor.
    * 
    * @return   <code>true</code> if the operation was possible.
    */
   @LegacyMethod(name = "SET-ACTOR")
   public logical setActor(character uri);

   /**
    * Sets the ACTOR attribute for this SOAP-HEADER-ENTRY.
    *
    * @param    uri
    *           The URI of the SOAP actor.
    * 
    * @return   <code>true</code> if the operation was possible.
    */
   @LegacyMethod(name = "SET-ACTOR")
   public logical setActor(String uri);

   /**
    * Sets the MUST-UNDERSTAND attribute for this SOAP-HEADER-ENTRY.
    *
    * @param    val
    *           The new value of this attribute.
    * 
    * @return   <code>true</code> if the operation was possible.
    */
   @LegacyMethod(name = "SET-MUST-UNDERSTAND")
   public logical setMustUnderstand(logical val);

   /**
    * Sets the MUST-UNDERSTAND attribute for this SOAP-HEADER-ENTRY.
    *
    * @param    val
    *           The new value of this attribute.
    * 
    * @return   <code>true</code> if the operation was possible.
    */
   @LegacyMethod(name = "SET-MUST-UNDERSTAND")
   public logical setMustUnderstand(boolean val);

   /**
    * Replaces the underlying XML of a SOAP-HEADER-ENTRYREF object with the DOM XML sub-tree 
    * rooted by the X-NODEREF object.
    *
    * @param    xNoderef
    *           A X-NODEREF variable that refers to the XML that will be set for the 
    *           SOAP-HEADER-ENTRYREF object.
    * 
    * @return   <code>true</code> if the operation was possible.
    */
   @LegacyMethod(name = "SET-NODE")
   public logical setNode(handle xNoderef);

   /**
    * Sets the SOAP-header-entryref object's underlying XML from serialized XML.
    * 
    * @param    xml
    *           A LONGCHAR variable that contains the XML that will become this 
    *           SOAP-HEADER-ENTRYREF object. This parameter can be only a longchar, 4GL doesn't
    *           allow character variables.
    * 
    * @return   <code>true</code> if the operation was possible.
    */
   @LegacyMethod(name = "SET-SERIALIZED")
   public logical setSerialized(longchar xml);
}