/*
** Module   : Server.java
** Abstract : An interface holding the SERVER specific method and attribute APIs.
**
** Copyright (c) 2013-2017, Golden Code Development Corporation.
**
** -#- -I- --Date-- --------------------------------Description-----------------------------------
** 001 CS  20130211 Created initial version.
** 002 CA  20130927 Added LegacyResource annotation, to determine resource type.
** 003 EVL 20131008 Adding legacy attributes and methods annotation.
** 004 EVL 20160224 Javadoc fixes to make compatible with Oracle Java 8 for Solaris 10.
*/
/*
** This program is free software: you can redistribute it and/or modify
** it under the terms of the GNU Affero General Public License as
** published by the Free Software Foundation, either version 3 of the
** License, or (at your option) any later version.
**
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU Affero General Public License for more details.
**
** You may find a copy of the GNU Affero GPL version 3 at the following
** location: https://www.gnu.org/licenses/agpl-3.0.en.html
** 
** Additional terms under GNU Affero GPL version 3 section 7:
** 
**   Under Section 7 of the GNU Affero GPL version 3, the following additional
**   terms apply to the works covered under the License.  These additional terms
**   are non-permissive additional terms allowed under Section 7 of the GNU
**   Affero GPL version 3 and may not be removed by you.
** 
**   0. Attribution Requirement.
** 
**     You must preserve all legal notices or author attributions in the covered
**     work or Appropriate Legal Notices displayed by works containing the covered
**     work.  You may not remove from the covered work any author or developer
**     credit already included within the covered work.
** 
**   1. No License To Use Trademarks.
** 
**     This license does not grant any license or rights to use the trademarks
**     Golden Code, FWD, any Golden Code or FWD logo, or any other trademarks
**     of Golden Code Development Corporation. You are not authorized to use the
**     name Golden Code, FWD, or the names of any author or contributor, for
**     publicity purposes without written authorization.
** 
**   2. No Misrepresentation of Affiliation.
** 
**     You may not represent yourself as Golden Code Development Corporation or FWD.
** 
**     You may not represent yourself for publicity purposes as associated with
**     Golden Code Development Corporation, FWD, or any author or contributor to
**     the covered work, without written authorization.
** 
**   3. No Misrepresentation of Source or Origin.
** 
**     You may not represent the covered work as solely your work.  All modified
**     versions of the covered work must be marked in a reasonable way to make it
**     clear that the modified work is not originating from Golden Code Development
**     Corporation or FWD.  All modified versions must contain the notices of
**     attribution required in this license.
*/

package com.goldencode.p2j.util;

/**
 * Interface for SERVER methods and attributes
 * <p>
 * Exposes the following Attributes and methods:
 * <table summary="">
 * <tr>
 *    <th>attribute or method</th>
 *    <th>API</th>
 * </tr>
 * <tr>
 *    <td>CLIENT-CONNECTION-ID</td>
 *    <td>{@link #getClientConnectionId()}</td>
 * </tr>
 * <tr>
 *    <td>FIRST-ASYNC-REQUEST</td>
 *    <td>{@link #getFirstAsyncRequest()}</td>
 * </tr>
 * <tr>
 *    <td>HANDLE</td>
 *    <td>n/a</td>
 * </tr>
 * <tr>
 *    <td>LAST-ASYNC-REQUEST</td>
 *    <td>{@link #getLastAsyncRequest()}</td>
 * </tr>
 * <tr>
 *    <td>RESTART-ROWID</td>
 *    <td>n/a</td>
 * </tr>
 * <tr>
 *    <td>CANCEL-REQUESTS()</td>
 *    <td>{@link #cancelRequests()}</td>
 * </tr>
 * <tr>
 *    <td>CANCEL-REQUESTS-AFTER()</td>
 *    <td>{@link #cancelRequestsAfter}</td>
 * </tr>
 * </table>
 *
 */
@LegacyResource(resource = LegacyResource.SERVER)
public interface Server 
extends CommonHandleChain
{
   /**
    * This returns the id for the AppServer connection to this handle. For Web Services the return
    * value will be an empty string.
    * 
    * @return   See above.
    */
   @LegacyAttribute(name = "CLIENT-CONNECTION-ID")
   public character getClientConnectionId();
   
   /**
    * Returns a handle to the first asynchronous requests in the AppServer or WebService list of
    * asynchronous requests.
    * 
    * @return   See above.
    */
   @LegacyAttribute(name = "FIRST-ASYNC-REQUEST")
   public handle getFirstAsyncRequest();
   
   /**
    * Returns a handle to the last asynchronous requests in the AppServer or WebService list of
    * asynchronous requests.
    * 
    * @return   See above.
    */
   @LegacyAttribute(name = "LAST-ASYNC-REQUEST")
   public handle getLastAsyncRequest();
   
   
   /**
    * This sends a stop condition to the AppServer or Web Service.
    * <p>
    * Depending on the type of server the stop condition is sent to, it may be handled 
    * differently.
    * 
    * @return   <code>true</code> if CONNECTED() method returns <code>true</code>, 
    *           <code>false</code> otherwise.
    */
   @LegacyMethod(name = "CANCEL-REQUESTS")
   public logical cancelRequests();
   
   /**
    * Calls the {@link #cancelRequests()} method when the specified number of seconds have passed.
    * 
    * @param    timeSec 
    *           The time in seconds after the {@link #cancelRequests()} method will be called. 4GL
    *           allows only integer or int64 values for this parameter. 
    *         
    * @return   <code>false</code> if timer cannot be set or {@link #cancelRequests()}'s result 
    *           otherwise
    */
   @LegacyMethod(name = "CANCEL-REQUESTS-AFTER")
   public logical cancelRequestsAfter(integer timeSec);
   
   /**
    * Calls the {@link #cancelRequests()} method when the specified number of seconds have passed.
    * 
    * @param    timeSec 
    *           The time in seconds after the {@link #cancelRequests()} method will be called. 4GL
    *           allows only integer or int64 values for this parameter. 
    *         
    * @return   <code>false</code> if timer cannot be set or {@link #cancelRequests()}'s result 
    *           otherwise
    */
   @LegacyMethod(name = "CANCEL-REQUESTS-AFTER")
   public logical cancelRequestsAfter(int64 timeSec);
   
   /**
    * Calls the {@link #cancelRequests()} method when the specified number of seconds have passed.
    * 
    * @param    timeSec 
    *           The time in seconds after the {@link #cancelRequests()} method will be called. 4GL
    *           allows only integer or int64 values for this parameter. 
    *         
    * @return   <code>false</code> if timer cannot be set or {@link #cancelRequests()}'s result 
    *           otherwise
    */
   @LegacyMethod(name = "CANCEL-REQUESTS-AFTER")
   public logical cancelRequestsAfter(long timeSec);
}