/*
** Module   : Sizeable.java
** Abstract : Defines getters for width/height of the widget or the display
**
** Copyright (c) 2013-2017, Golden Code Development Corporation.
**
** -#- -I- --Date-- -----------------------Description------------------------
** 001 VMN 20130217 Created initial version.
** 002 EVL 20130911 Adding legacy attributes and methods annotation.
** 003 HC  20141207 Implemented runtime support for widget coordinate attributes.
*/
/*
** This program is free software: you can redistribute it and/or modify
** it under the terms of the GNU Affero General Public License as
** published by the Free Software Foundation, either version 3 of the
** License, or (at your option) any later version.
**
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU Affero General Public License for more details.
**
** You may find a copy of the GNU Affero GPL version 3 at the following
** location: https://www.gnu.org/licenses/agpl-3.0.en.html
** 
** Additional terms under GNU Affero GPL version 3 section 7:
** 
**   Under Section 7 of the GNU Affero GPL version 3, the following additional
**   terms apply to the works covered under the License.  These additional terms
**   are non-permissive additional terms allowed under Section 7 of the GNU
**   Affero GPL version 3 and may not be removed by you.
** 
**   0. Attribution Requirement.
** 
**     You must preserve all legal notices or author attributions in the covered
**     work or Appropriate Legal Notices displayed by works containing the covered
**     work.  You may not remove from the covered work any author or developer
**     credit already included within the covered work.
** 
**   1. No License To Use Trademarks.
** 
**     This license does not grant any license or rights to use the trademarks
**     Golden Code, FWD, any Golden Code or FWD logo, or any other trademarks
**     of Golden Code Development Corporation. You are not authorized to use the
**     name Golden Code, FWD, or the names of any author or contributor, for
**     publicity purposes without written authorization.
** 
**   2. No Misrepresentation of Affiliation.
** 
**     You may not represent yourself as Golden Code Development Corporation or FWD.
** 
**     You may not represent yourself for publicity purposes as associated with
**     Golden Code Development Corporation, FWD, or any author or contributor to
**     the covered work, without written authorization.
** 
**   3. No Misrepresentation of Source or Origin.
** 
**     You may not represent the covered work as solely your work.  All modified
**     versions of the covered work must be marked in a reasonable way to make it
**     clear that the modified work is not originating from Golden Code Development
**     Corporation or FWD.  All modified versions must contain the notices of
**     attribution required in this license.
*/

package com.goldencode.p2j.util;

/**
 * Defines setters and getters for the WIDTH-CHARS, WIDTH-PIXELS and HEIGHT-CHARS, HEIGHT-PIXES
 * attributes.
 */
public interface Sizeable
{
   /**
    * Queries the WIDTH-PIXELS attribute of this widget.
    *
    * @return   The width of the widget in pixels.
    */
   @LegacyAttribute(name = "WIDTH-PIXELS")
   public integer getWidthPixels();

   /**
    * Sets new value of WIDTH-PIXELS attribute.
    *
    * @param    widthPixels
    *           new value of WIDTH-PIXELS attribute
    */
   @LegacyAttribute(name = "WIDTH-PIXELS", setter = true)
   public void setWidthPixels(NumberType widthPixels);

   /**
    * Sets new value of WIDTH-PIXELS attribute.
    *
    * @param    widthPixels
    *           new value of WIDTH-PIXELS attribute
    */
   @LegacyAttribute(name = "WIDTH-PIXELS", setter = true)
   public void setWidthPixels(int widthPixels);

   /**
    * Queries the HEIGHT-PIXELS attribute of this widget.
    *
    * @return   The height of the widget in pixels.
    */
   @LegacyAttribute(name = "HEIGHT-PIXELS")
   public integer getHeightPixels();

   /**
    * Sets new value of HEIGHT-PIXELS attribute.
    *
    * @param    widthPixels
    *           new value of HEIGHT-PIXELS attribute
    */
   @LegacyAttribute(name = "HEIGHT-PIXELS", setter = true)
   public void setHeightPixels(NumberType widthPixels);

   /**
    * Sets new value of HEIGHT-PIXELS attribute.
    *
    * @param    widthPixels
    *           new value of HEIGHT-PIXELS attribute
    */
   @LegacyAttribute(name = "HEIGHT-PIXELS", setter = true)
   public void setHeightPixels(int widthPixels);

   /**
    * Get the WIDTH-CHARS Progress attribute. The value is equal to the current component char
    * width.
    *
    * @return    The integer value of the WIDTH-CHARS attribute.
    */
   @LegacyAttribute(name = "WIDTH-CHARS")
   public decimal getWidthChars();

   /**
    * Set the WIDTH-CHARS Progress attribute.
    *
    * @param    widthChars
    *           The integer value for the new WIDTH-CHARS value.
    */
   @LegacyAttribute(name = "WIDTH-CHARS", setter = true)
   public void setWidthChars(double widthChars);

   /**
    * Set the WIDTH-CHARS Progress attribute.
    *
    * @param    widthChars
    *           The integer value for the new WIDTH-CHARS value.
    */
   @LegacyAttribute(name = "WIDTH-CHARS", setter = true)
   public void setWidthChars(NumberType widthChars);

   /**
    * Get HEIGHT-CHARS attribute value.
    *
    * @return   Current value of the attribute.
    */
   @LegacyAttribute(name = "HEIGHT-CHARS")
   public decimal getHeightChars();

   /**
    * Setting the HEIGHT-CHARS Progress attribute.
    *
    * @param    heightChars
    *           The integer value for the new HEIGHT-CHARS value.
    */
   @LegacyAttribute(name = "HEIGHT-CHARS", setter = true)
   public void setHeightChars(double heightChars);

   /**
    * Setting the HEIGHT-CHARS Progress attribute.
    *
    * @param    heightChars
    *           The integer value for the new HEIGHT-CHARS value.
    */
   @LegacyAttribute(name = "HEIGHT-CHARS", setter = true)
   public void setHeightChars(NumberType heightChars);
}