/*
** Module   : SkipField.java
** Abstract : outputs new lines to a stream
**
** Copyright (c) 2006-2017, Golden Code Development Corporation.
**
** -#- -I- --Date-- --JPRM-- ------------------------------Description----------------------------
** 001 GES 20060925   @29902 Created initial version which outputs new
**                           lines to a stream.
** 002 SIY 20071001   @35323 Changed inheritance. Made class suitable for 
**                           network transfers and for code reuse. 
**                           Ref: #35314
** 003 SIY 20071003   @35352 Header cleanup.
** 004 CA  20130308          Added missing c'tor, which receives a 4GL integer value.
** 005 OM  20130527          Added int64 support.
** 006 CA  20170827          Added stub c'tor for SKIP(dec-value).
*/ 
/*
** This program is free software: you can redistribute it and/or modify
** it under the terms of the GNU Affero General Public License as
** published by the Free Software Foundation, either version 3 of the
** License, or (at your option) any later version.
**
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU Affero General Public License for more details.
**
** You may find a copy of the GNU Affero GPL version 3 at the following
** location: https://www.gnu.org/licenses/agpl-3.0.en.html
** 
** Additional terms under GNU Affero GPL version 3 section 7:
** 
**   Under Section 7 of the GNU Affero GPL version 3, the following additional
**   terms apply to the works covered under the License.  These additional terms
**   are non-permissive additional terms allowed under Section 7 of the GNU
**   Affero GPL version 3 and may not be removed by you.
** 
**   0. Attribution Requirement.
** 
**     You must preserve all legal notices or author attributions in the covered
**     work or Appropriate Legal Notices displayed by works containing the covered
**     work.  You may not remove from the covered work any author or developer
**     credit already included within the covered work.
** 
**   1. No License To Use Trademarks.
** 
**     This license does not grant any license or rights to use the trademarks
**     Golden Code, FWD, any Golden Code or FWD logo, or any other trademarks
**     of Golden Code Development Corporation. You are not authorized to use the
**     name Golden Code, FWD, or the names of any author or contributor, for
**     publicity purposes without written authorization.
** 
**   2. No Misrepresentation of Affiliation.
** 
**     You may not represent yourself as Golden Code Development Corporation or FWD.
** 
**     You may not represent yourself for publicity purposes as associated with
**     Golden Code Development Corporation, FWD, or any author or contributor to
**     the covered work, without written authorization.
** 
**   3. No Misrepresentation of Source or Origin.
** 
**     You may not represent the covered work as solely your work.  All modified
**     versions of the covered work must be marked in a reasonable way to make it
**     clear that the modified work is not originating from Golden Code Development
**     Corporation or FWD.  All modified versions must contain the notices of
**     attribution required in this license.
*/

package com.goldencode.p2j.util;

/**
 * Outputs new lines to a stream. 
 */
public class SkipField
extends NullField
{
   /**
    * Default constructor (outputs a single line end).
    */
   public SkipField()
   {
   }
   
   /**
    * Constructor for an explicit number of line ends. If <code>num</code> is too large to be 
    * stored on short, it will be truncated automatically to 16 bits.
    *
    * @param    num
    *           The number of line ends to output.
    */
   public SkipField(int64 num)
   {
      if (num.isUnknown())
      {
         ErrorManager.recordOrThrowError(564, "Unable to evaluate expression for PUT statement");
         return;
      }

      setNum(num.intValue());
      setExplicit(true);
   }
   
   /**
    * Constructor for an explicit number of line ends. If <code>num</code> is too large to be 
    * stored on short, it will be truncated automatically to 16 bits.
    *
    * @param    num
    *           The number of line ends to output.
    */
   public SkipField(long num)
   {
      super(num);
   }
   
   /**
    * Constructor for an explicit number of line ends. If <code>num</code> is too large to be 
    * stored on short, it will be truncated automatically to 16 bits.
    *
    * @param    num
    *           The number of line ends to output.
    */
   public SkipField(double num)
   {
      // TODO: implement this
   }
   
   /**
    * Constructor for an explicit number of line ends. If <code>num</code> is too large to be 
    * stored on short, it will be truncated automatically to 16 bits.
    *
    * @param    num
    *           The number of line ends to output.
    */
   public SkipField(decimal num)
   {
      // TODO: implement this
   }
   
   /**
    * Renders the contained field definition to the given stream.
    *
    * @param    out
    *           The stream to which the field should be rendered.
    */
   public void output(Stream out)
   throws ErrorConditionException
   {
      if (isExplicit())
      {
         out.putLineEnd(getNum());
      }
      else
      {
         out.putLineEnd();
      }
   }
}