/* 
** Module   : Tracer.java
** Abstract : Tracing facility entry point.
**
** Copyright (c) 2015-2017, Golden Code Development Corporation.
**
** -#- -I- --Date-- ----------------------------Description-----------------------------
** 001 HC  20151018 Initial version.
*/
/*
** This program is free software: you can redistribute it and/or modify
** it under the terms of the GNU Affero General Public License as
** published by the Free Software Foundation, either version 3 of the
** License, or (at your option) any later version.
**
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU Affero General Public License for more details.
**
** You may find a copy of the GNU Affero GPL version 3 at the following
** location: https://www.gnu.org/licenses/agpl-3.0.en.html
** 
** Additional terms under GNU Affero GPL version 3 section 7:
** 
**   Under Section 7 of the GNU Affero GPL version 3, the following additional
**   terms apply to the works covered under the License.  These additional terms
**   are non-permissive additional terms allowed under Section 7 of the GNU
**   Affero GPL version 3 and may not be removed by you.
** 
**   0. Attribution Requirement.
** 
**     You must preserve all legal notices or author attributions in the covered
**     work or Appropriate Legal Notices displayed by works containing the covered
**     work.  You may not remove from the covered work any author or developer
**     credit already included within the covered work.
** 
**   1. No License To Use Trademarks.
** 
**     This license does not grant any license or rights to use the trademarks
**     Golden Code, FWD, any Golden Code or FWD logo, or any other trademarks
**     of Golden Code Development Corporation. You are not authorized to use the
**     name Golden Code, FWD, or the names of any author or contributor, for
**     publicity purposes without written authorization.
** 
**   2. No Misrepresentation of Affiliation.
** 
**     You may not represent yourself as Golden Code Development Corporation or FWD.
** 
**     You may not represent yourself for publicity purposes as associated with
**     Golden Code Development Corporation, FWD, or any author or contributor to
**     the covered work, without written authorization.
** 
**   3. No Misrepresentation of Source or Origin.
** 
**     You may not represent the covered work as solely your work.  All modified
**     versions of the covered work must be marked in a reasonable way to make it
**     clear that the modified work is not originating from Golden Code Development
**     Corporation or FWD.  All modified versions must contain the notices of
**     attribution required in this license.
*/

package com.goldencode.trace;

import java.util.concurrent.*;

import com.goldencode.util.*;

/**
 * Tracing facility entry point.
 * Note that this is work-in-progrss. 
 */
public class Tracer
{
   /**
    * Enabled flag.
    */
   private static boolean enabled;
   
   /**
    * The max number of call stack entries to store in the trace point.
    */
   private static int callStackEntries = 30;
   
   /**
    * All the trace point entries.
    */
   private static ConcurrentLinkedQueue<TracePoint> traceEntries; 
   
   /**
    * Returns <code>true</code> if tracing is enabled.
    * 
    * @return  see above.
    */
   public static boolean enabled()
   {
      return enabled;
   }
   
   /**
    * Enables tracing.
    */
   public static synchronized void enable()
   {
      traceEntries = new ConcurrentLinkedQueue<>();
      enabled = true;
   }
   
   /**
    * Disables tracing.
    */
   public static synchronized void disable()
   {
      enabled = false;
   }
   
   /**
    * Creates a trace point and stores it in the internal list.
    * 
    * @param   msg
    *          An optional message to store in the trace point.

// TODO: these are too specific
    * @param   windowId
    * @param   x1
    * @param   y1
    * @param   x2
    * @param   y2
    
    * @param   any1
    *          Optional reference to store with the trace point.
    */
   public static void trace(String msg,
                            Integer windowId, 
                            Integer x1, 
                            Integer y1, 
                            Integer x2, 
                            Integer y2, 
                            Object any1)
   {
      traceImpl(msg, windowId, 
            x1 == null ? null : new Double(x1), y1 == null ? null : new Double(y1), 
            x2 == null ? null : new Double(x2), y2 == null ? null : new Double(y2), 
            any1, null, null);
   }
   
   /**
    * Creates a trace point and stores it in the internal list.
    * 
    * @param   msg
    *          An optional message to store in the trace point.

// TODO: these are too specific
    * @param   windowId
    * @param   x1
    * @param   y1
    * @param   x2
    * @param   y2
    
    * @param   any1
    *          Optional reference to store with the trace point.
    * @param   any2
    *          Optional reference to store with the trace point.
    */
   public static void trace(String msg,
                            Integer windowId, 
                            Integer x1, 
                            Integer y1, 
                            Integer x2, 
                            Integer y2, 
                            Object any1,
                            Object any2)
   {
      traceImpl(msg, windowId, 
            x1 == null ? null : new Double(x1), y1 == null ? null : new Double(y1), 
            x2 == null ? null : new Double(x2), y2 == null ? null : new Double(y2), 
            any1, any2, null);
   }
   
   /**
    * Creates a trace point and stores it in the internal list.
    * 
    * @param   msg
    *          An optional message to store in the trace point.

// TODO: these are too specific
    * @param   windowId
    * @param   x1
    * @param   y1
    * @param   x2
    * @param   y2
    
    * @param   any1
    *          Optional reference to store with the trace point.
    * @param   any2
    *          Optional reference to store with the trace point.
    * @param   any3
    *          Optional reference to store with the trace point.
    */
   public static void trace(String msg,
                            Integer windowId, 
                            Integer x1, 
                            Integer y1, 
                            Integer x2, 
                            Integer y2, 
                            Object any1,
                            Object any2,
                            Object any3)
   {
      traceImpl(msg, windowId, 
            x1 == null ? null : new Double(x1), y1 == null ? null : new Double(y1), 
            x2 == null ? null : new Double(x2), y2 == null ? null : new Double(y2), 
            any1, any2, any3);
   }
   
   /**
    * Creates a trace point and stores it in the internal list.
    * 
    * @param   msg
    *          An optional message to store in the trace point.

// TODO: these are too specific
    * @param   windowId
    * @param   x1
    * @param   y1
    * @param   x2
    * @param   y2
    
    * @param   any1
    *          Optional reference to store with the trace point.
    */
   public static void trace(String msg,
                            Integer windowId, 
                            Double x1, 
                            Double y1, 
                            Double x2, 
                            Double y2, 
                            Object any1)
   {
      traceImpl(msg, windowId, x1, y1, x2, y2, any1, null, null);
   }
   
   /**
    * Creates a trace point and stores it in the internal list.
    * 
    * @param   msg
    *          An optional message to store in the trace point.

// TODO: these are too specific
    * @param   windowId
    * @param   x1
    * @param   y1
    * @param   x2
    * @param   y2
    
    * @param   any1
    *          Optional reference to store with the trace point.
    * @param   any2
    *          Optional reference to store with the trace point.
    */
   public static void trace(String msg,
                            Integer windowId, 
                            Double x1, 
                            Double y1, 
                            Double x2, 
                            Double y2, 
                            Object any1,
                            Object any2)
   {
      traceImpl(msg, windowId, x1, y1, x2, y2, any1, any2, null);
   }
   
   /**
    * Creates a trace point and stores it in the internal list.
    * 
    * @param   msg
    *          An optional message to store in the trace point.

// TODO: these are too specific
    * @param   windowId
    * @param   x1
    * @param   y1
    * @param   x2
    * @param   y2
    
    * @param   any1
    *          Optional reference to store with the trace point.
    * @param   any2
    *          Optional reference to store with the trace point.
    * @param   any3
    *          Optional reference to store with the trace point.
    */
   public static void trace(String msg,
                            Integer windowId, 
                            Double x1, 
                            Double y1, 
                            Double x2, 
                            Double y2, 
                            Object any1,
                            Object any2,
                            Object any3)
   {
      traceImpl(msg, windowId, x1, y1, x2, y2, any1, any2, any3);
   }
   
   /**
    * Creates a trace point and stores it in the internal list.
    * 
    * @param   msg
    *          An optional message to store in the trace point.

// TODO: these are too specific
    * @param   windowId
    * @param   x1
    * @param   y1
    * @param   x2
    * @param   y2
    
    * @param   any1
    *          Optional reference to store with the trace point.
    * @param   any2
    *          Optional reference to store with the trace point.
    * @param   any3
    *          Optional reference to store with the trace point.
    */
   private static void traceImpl(String msg,
                                 Integer windowId, 
                                 Double x1, 
                                 Double y1, 
                                 Double x2, 
                                 Double y2, 
                                 Object any1,
                                 Object any2,
                                 Object any3)
   {
      // for performance reasons, this is not synchronized
      if (!enabled)
      {
         return;
      }
      
      TracePoint t = new TracePoint();
      t.tick = System.nanoTime();
      t.thread = Thread.currentThread().getName();
      t.msg = msg;
      t.window = windowId == null ? Integer.MIN_VALUE : windowId;
      t.x1 = x1 == null ? Integer.MIN_VALUE : x1;
      t.y1 = y1 == null ? Integer.MIN_VALUE : y1;
      t.x2 = x2 == null ? t.x1 : x2;
      t.y2 = y2 == null ? t.y1 : y2;
      t.any1 = any1;
      t.any2 = any2;
      t.any3 = any3;
      t.stack = LogHelper.getStackTrace(callStackEntries);
      
      // when the sepparator is one character, this should be relatively efficient - 
      // regexp isn't created
      String[] s = t.stack.split("\n");

      // callstack line is assumed in the format: 
      // <qualifed class name>.<method name>(<file name>.<ext>:<line>)
      String caller = s[2];
      caller = caller.substring(0, caller.indexOf("("));
      int dot = caller.lastIndexOf('.');
      
      t.clazz = caller.substring(0, dot);
      t.method = caller.substring(dot + 1);
      
      String line = s[2];
      line = line.substring(line.indexOf(':') + 1, line.indexOf(')'));
      t.line = Integer.parseInt(line);
      
      traceEntries.add(t);
   }
}