/*
** Module   : WalkState.java
** Abstract : contains resources needed by tree walkers when processing trees 
**
** Copyright (c) 2009-2017, Golden Code Development Corporation.
**
** -#- -I- --Date-- --JPRM-- ----------------Description-----------------
** 001 GES 20090522   @42542 Contains resources needed by tree walkers when
**                           processing trees.
*/
/*
** This program is free software: you can redistribute it and/or modify
** it under the terms of the GNU Affero General Public License as
** published by the Free Software Foundation, either version 3 of the
** License, or (at your option) any later version.
**
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU Affero General Public License for more details.
**
** You may find a copy of the GNU Affero GPL version 3 at the following
** location: https://www.gnu.org/licenses/agpl-3.0.en.html
** 
** Additional terms under GNU Affero GPL version 3 section 7:
** 
**   Under Section 7 of the GNU Affero GPL version 3, the following additional
**   terms apply to the works covered under the License.  These additional terms
**   are non-permissive additional terms allowed under Section 7 of the GNU
**   Affero GPL version 3 and may not be removed by you.
** 
**   0. Attribution Requirement.
** 
**     You must preserve all legal notices or author attributions in the covered
**     work or Appropriate Legal Notices displayed by works containing the covered
**     work.  You may not remove from the covered work any author or developer
**     credit already included within the covered work.
** 
**   1. No License To Use Trademarks.
** 
**     This license does not grant any license or rights to use the trademarks
**     Golden Code, FWD, any Golden Code or FWD logo, or any other trademarks
**     of Golden Code Development Corporation. You are not authorized to use the
**     name Golden Code, FWD, or the names of any author or contributor, for
**     publicity purposes without written authorization.
** 
**   2. No Misrepresentation of Affiliation.
** 
**     You may not represent yourself as Golden Code Development Corporation or FWD.
** 
**     You may not represent yourself for publicity purposes as associated with
**     Golden Code Development Corporation, FWD, or any author or contributor to
**     the covered work, without written authorization.
** 
**   3. No Misrepresentation of Source or Origin.
** 
**     You may not represent the covered work as solely your work.  All modified
**     versions of the covered work must be marked in a reasonable way to make it
**     clear that the modified work is not originating from Golden Code Development
**     Corporation or FWD.  All modified versions must contain the notices of
**     attribution required in this license.
*/

package com.goldencode.trpl; 
 
import com.goldencode.ast.*;

/**
 * Contains resources needed by tree walkers when processing specific nodes in
 * a walk.
 * <p>
 * <ul>
 *    <li> Walk services such as an ID to tree node cache.
 *    <li> Tree node being walked, read-only version.
 *    <li> Tree node being walked, editable version.
 * </ul>
 */
public class WalkState
extends BaseState
{
   /** Provides walk services such as an ID to tree node cache. */
   public final Resolver resolver;
   
   /** Tree node being walked, read-only version. */
   public final Aast here;
   
   /** Tree node being walked, editable version. */
   public final Aast copy;
   
   /**
    * Construct an instance that contains the resources for the processing of a
    * single AST node.
    *
    * @param    mgr
    *           Provides core AST services.
    * @param    engine
    *           Tree processing engine which drives and controls the walk
    *           process.
    * @param    resolver
    *           Provides walk services such as an ID to tree node cache. 
    * @param    here
    *           AST being walked, read-only version.
    * @param    copy
    *           AST being walked, editable version.
    */
   public WalkState(AstManager mgr,
                    Engine     engine,
                    Resolver   resolver,
                    Aast       here,
                    Aast       copy)
   {
      super(mgr, engine);
      this.resolver = resolver;
      this.here     = here;
      this.copy     = copy;
   }
   
   /**
    * Construct an instance that contains the resources for the processing of a
    * single AST node.
    *
    * @param    base
    *           Common state to copy.
    * @param    resolver
    *           Provides walk services such as an ID to tree node cache. 
    * @param    here
    *           AST being walked, read-only version.
    * @param    copy
    *           AST being walked, editable version.
    */
   public WalkState(BaseState base, Resolver resolver, Aast here, Aast copy)
   {
      super(base);
      this.resolver = resolver;
      this.here     = here;
      this.copy     = copy;
   }
}