/*
** Module   : EmptySet.java
** Abstract : implementation of an empty set
**
** Copyright (c) 2006-2017, Golden Code Development Corporation.
**
** -#- -I- --Date-- --JPRM-- ----------------------------Description-----------------------------
** 001 ECF 20061112   @31131 Created initial version. Implementation of an
**                           empty set.
** 002 GES 20090421   @41822 Moved to a more generic package.
** 003 ECF 20131013          Implemented generics.
*/
/*
** This program is free software: you can redistribute it and/or modify
** it under the terms of the GNU Affero General Public License as
** published by the Free Software Foundation, either version 3 of the
** License, or (at your option) any later version.
**
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU Affero General Public License for more details.
**
** You may find a copy of the GNU Affero GPL version 3 at the following
** location: https://www.gnu.org/licenses/agpl-3.0.en.html
** 
** Additional terms under GNU Affero GPL version 3 section 7:
** 
**   Under Section 7 of the GNU Affero GPL version 3, the following additional
**   terms apply to the works covered under the License.  These additional terms
**   are non-permissive additional terms allowed under Section 7 of the GNU
**   Affero GPL version 3 and may not be removed by you.
** 
**   0. Attribution Requirement.
** 
**     You must preserve all legal notices or author attributions in the covered
**     work or Appropriate Legal Notices displayed by works containing the covered
**     work.  You may not remove from the covered work any author or developer
**     credit already included within the covered work.
** 
**   1. No License To Use Trademarks.
** 
**     This license does not grant any license or rights to use the trademarks
**     Golden Code, FWD, any Golden Code or FWD logo, or any other trademarks
**     of Golden Code Development Corporation. You are not authorized to use the
**     name Golden Code, FWD, or the names of any author or contributor, for
**     publicity purposes without written authorization.
** 
**   2. No Misrepresentation of Affiliation.
** 
**     You may not represent yourself as Golden Code Development Corporation or FWD.
** 
**     You may not represent yourself for publicity purposes as associated with
**     Golden Code Development Corporation, FWD, or any author or contributor to
**     the covered work, without written authorization.
** 
**   3. No Misrepresentation of Source or Origin.
** 
**     You may not represent the covered work as solely your work.  All modified
**     versions of the covered work must be marked in a reasonable way to make it
**     clear that the modified work is not originating from Golden Code Development
**     Corporation or FWD.  All modified versions must contain the notices of
**     attribution required in this license.
*/

package com.goldencode.util;

import java.util.*;

/**
 * Implementation of a (reasonably) low overhead, empty set.  This is
 * essentially a do-nothing object to be used when a safe implementation of
 * the <code>Set</code> interface is required for an API, but the set must be
 * empty.  It overrides the minimum <code>AbstractSet</code> methods
 * necessary to function as a read-only, empty set.  Elements may not be
 * added.
 */
public final class EmptySet<E>
extends AbstractSet<E>
{
   /** Empty iterator which is stateless, so it is safe to share */
   private final Iterator<E> emptyIter = new EmptyIterator<E>();
   
   /**
    * Return an iterator that never has a next element, and which does not
    * support element removal.
    *
    * @return  No-op iterator.
    */
   public Iterator<E> iterator()
   {
      return emptyIter;
   }
   
   /**
    * Return a size of zero, to indicate the set is empty.
    *
    * @return  Zero.
    */
   public int size()
   {
      return 0;
   }
}